/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Jean Marin (XQual)                                      |
| Version:     1.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.ant;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;
import java.util.Hashtable;
import java.util.Enumeration;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CTimeoutListener;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for ant build files.
 * @author jmarin
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{ant      }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String antInstallPath;
	private String OSPlatform;
	private String javaHome;
	private String antOPTS;
	private String antARGS;

	private File workingDir;
	private File logFile;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			
			javaHome = getStringParamValue("Ant",  "JAVA_HOME");
			antInstallPath = getStringParamValue("Ant",  "ANT_HOME");
			antOPTS = getStringParamValue("Ant",  "ANT_OPTS");
			antARGS = getStringParamValue("Ant",  "ANT_ARGS");
			OSPlatform = getStringParamValue("Ant",  "OS");
			testRootPath = getStringParamValue("Ant",  "Project path");
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
			
		
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		String scriptParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/";
		workingDir = new File(scriptParentFolderPath);
		
		String 	antInterpreter = "";
		antInterpreter = antInterpreter + CFileUtils.quoteFilePath((antInstallPath.length() > 0 ? antInstallPath + "/bin/ant" + (OSPlatform.equals("Windows") ? ".bat" : ".sh") : "")) + " " ;
		antInterpreter = antInterpreter + "-debug ";
		antInterpreter = antInterpreter + "-noinput ";
		antInterpreter = antInterpreter + "-buildfile " + testName.substring(0 , testName.indexOf("target:")).trim() + ".xml ";
		antInterpreter = antInterpreter + "-keep-going ";
		antInterpreter = antInterpreter + testName.substring(testName.indexOf("target:") + 7,testName.indexOf("for:")).trim() + " ";
						
		
		// for this we need to pass the envrionement to the runner
		Hashtable<String, String> env = new Hashtable<String, String>();
		if (antInstallPath.length() > 0) env.put("ANT_HOME", antInstallPath);
		if (javaHome.length() > 0) env.put("JAVA_HOME", javaHome);
		if (antOPTS.length() > 0) env.put("ANT_OPTS", antOPTS);
		if (antARGS.length() > 0) env.put("ANT_ARGS", antARGS);
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner antRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                        antInterpreter,
										env,
                                        workingDir);

		logFile = new File (workingDir + "/XstudioLogFile.txt");
		
		short result = RESULT_FAILURE;
		
		result = antRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, logFile);
		traceln(LOG_PRIORITY_INFO, "returned from ant runner!");
		
		// get the build file as an attachment for review
		addTestcaseMessageAttachment(new File (workingDir.getAbsolutePath() + "/" + testName.substring(0 , testName.indexOf("target:")).trim() + ".xml"));
		
		
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution failed"));
			parseResultFile(executionSteps);
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution succeeded"));
			parseResultFile(executionSteps);
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}	
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		Vector<CScannedTest> antOutput = new Vector<CScannedTest>();
		File antBuildFile;
		String antCanonicalPath;
		String antRelativePath;
		String antBuildName;		
		String antFileAsString;
		String antBuildClassFullyQualifiedName;
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^build\\x2exml$",       // only build.xml files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		
		Hashtable<String, String> testTargets = new Hashtable<String, String>();
		
		for (CScannedTest scannedTest : output) {
			antCanonicalPath = scannedTest.getCanonicalPath();
			antRelativePath = scannedTest.getRelativePath();
			antBuildName = scannedTest.getName();
			
			traceln(LOG_PRIORITY_INFO, "searching in ant build.xml file !");
			
			antBuildFile = new File(testRootPathFolder + "/" + (antCanonicalPath.length() > 0 ? antCanonicalPath  + "/" : "" ) + antBuildName + ".xml");
			
			testTargets = extractProjectAndTargetFromAntBuildFile (antBuildFile);
			traceln(LOG_PRIORITY_INFO, "extracted targets : " + testTargets.size());
			
			Enumeration<String> keys = testTargets.keys();
			while (keys.hasMoreElements()) {
				String targetName = keys.nextElement();
				String  projectName= testTargets.get(targetName);
				traceln(LOG_PRIORITY_INFO, "Got target from extract  " + targetName + " for project " + projectName);
				antBuildClassFullyQualifiedName = antBuildName + " target:" + targetName + " for:" + projectName; 
				traceln(LOG_PRIORITY_INFO, "created this name test name :" + antBuildClassFullyQualifiedName);
				antOutput.add(new CScannedTest(antBuildClassFullyQualifiedName, antCanonicalPath));
			}
			
		}
		
		
		return antOutput;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// parse the result file to get the result and the execution steps
		if (!logFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "log file exists"));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
			addTestcaseMessageAttachment(logFile);
		}

		String line, message="";
		boolean errorDetected = false;

		FileInputStream fileInputStream = null;
		BufferedReader bufferedReader = null;
		String reportIndexHTMLFileName = "";
		try {
			fileInputStream = new FileInputStream(logFile);
			bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				if (line.startsWith("parsing ") || 
				line.startsWith("Project") ||
				line.startsWith("Buildfile")||
				line.startsWith("Detected")||
				line.startsWith("Apache Ant") ) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));

				} else if (line.startsWith("BUILD SUCCESSFUL")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, line));
				} else if (line.startsWith("BUILD FAILED")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, line));
				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}
			
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception whle parsing the result file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
	
	// extract the project name and all of the targets from the ant build file
	// returns it as a hastable with targetName, projectName
	private Hashtable<String, String> extractProjectAndTargetFromAntBuildFile(File antBuildFile) {
		// get a file , opens it , find the package , find teh first class and concatenante tis as a package.clasname
		String fileAsString;
		String projectName = "";
		String targetName = "";
		String line = "";
		boolean targetFound = false;
		
		Hashtable<String, String> testTargets = new Hashtable<String, String>();

		FileInputStream fileInputStream = null;
		BufferedReader bufferedReader = null;
		try {
			fileInputStream = new FileInputStream(antBuildFile);
			bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));
			
			
			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				targetName ="";
				
				if (line.indexOf("<project name=")>=0) {
					try {
						projectName = line.substring(15, line.indexOf("\">"));
						projectName = projectName.substring(0, projectName.indexOf("\"")).trim();
					} catch (Exception e) {} 
				} else if (line.indexOf("<target name=")>=0) {
					try {
						targetName = line.substring(14, line.indexOf("\">"));
						targetName = targetName.substring(0, targetName.indexOf("\"")).trim();
					} catch (Exception e) {} 
					targetFound = true;
				}
				else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
				
				if (targetFound) {
					traceln(LOG_PRIORITY_INFO, "adding target " + targetName + " for project " + projectName);
					// we add one entry in the list of test with targets
					testTargets.put(targetName , projectName);
					targetFound = false;
				}
			}
			bufferedReader.close();
			fileInputStream.close();

		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the antBuild file: " + e);
		}
		
		
		return testTargets;
	}

	
}
