/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Jean Marin (XQual)                                      |
| Version:     1.4                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.bat_with_params;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CTimeoutListener;

/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Batch files.
 * @author jmarin
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{batwithparams      }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private boolean synchronous;
	private int timeout = 600;

	private File workingDir;
	private File resultFile;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath = getStringParamValue("General",  "Test root path");
			synchronous  = getBooleanParamValue("General", "Synchronous executable");
			timeout      = getIntegerParamValue("General", "Asynchronous timeout (in seconds)");
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		String scriptParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/";
		workingDir = new File(scriptParentFolderPath);
		
		// params
        StringBuffer paramsArgument = new StringBuffer();
        for (int k=0; k<params.size(); k++) {
            CTestcaseParam param = params.get(k);
            String paramName = param.getName();
            String paramValue = param.getStringValue();
            paramsArgument.append("/" + paramName + "=" + paramValue + " ");
        }
		
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner exeRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                        testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + ".bat " +
                                        "/debug " +
                                        "/testcaseIndex=" + testcaseIndex + " " + paramsArgument,
                                        workingDir);


		
		short result = RESULT_FAILURE;
		if (synchronous) {
			result = exeRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
			if (result == RESULT_FAILURE) {
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution failed"));
				parseResultFile(executionSteps);
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			} else {
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution succeeded"));
				parseResultFile(executionSteps);
				return new CReturnStatus(RESULT_SUCCESS, executionSteps);
			}
		} else {
			result = exeRunner.requestAction(IRunner.START_PROCESS, IRunner.DO_NOT_WAIT_END_OF_EXECUTION);
			if (result == RESULT_FAILURE) {
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution failed"));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}

			// to check if the execution completed correctly, we need to check if the "test_completed.txt" has been created
		    File testCompletedFile = new File(workingDir + "/test_completed.txt");
		    short resultTimeout = CTimeoutListener.waitForFile(testCompletedFile, timeout);
			if (resultTimeout != RESULT_SUCCESS) {
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "timeout of " + timeout + " seconds to execute the test case expired"));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}

		    // file exists, remove it to complete asyncronous handshake
		    if (!testCompletedFile.delete())
		       executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Failed to cleanup test_completed.txt"));
		    
			return parseResultFile(executionSteps);
		}
		
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2ebat$",          // only *.bat files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// parse the result file to get the result and the execution steps
		resultFile = new File(workingDir + "/log.txt"); // file possibly generated by the bat script
		if (!resultFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "log file exists"));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
			addTestcaseMessageAttachment(resultFile);
		}

		String line, message="";
		boolean errorDetected = false;

		FileInputStream fileInputStream = null;
		BufferedReader bufferedReader = null;
		try {
			fileInputStream = new FileInputStream(resultFile);
			bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				if (line.indexOf("[Success]")>=0) {
					try {message = line.substring(10, line.length());} catch (Exception e) {} // [Success] length = 9
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, message));

				} else if (line.indexOf("[Failure]")>=0) {
					try {message = line.substring(10, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, message));
					errorDetected = true;

				} else if (line.indexOf("[Log]")>=0) {
					try {message = line.substring(6, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, message));

				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}
			
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception whle parsing the result file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
}
