/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.cerberus;

import java.util.Vector;

import javax.swing.JOptionPane;

import org.json.JSONArray;
import org.json.JSONObject;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.CTestExecution;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Cerberus.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{cerberus      }  ";

	private String restBaseUrl;
	private String restApiKey;
	
	private String country;
	private String environment;
	private String robot;

	private CLauncherRestBackend restBackend;
	
	private String jsonReport;
	
	private String thisTestName;
	private String thisTestcaseName;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			restBaseUrl = getStringParamValue("REST Connection", "Base URL");
			restApiKey  = getStringParamValue("REST Connection", "API Key");
			
			country     = getStringParamValue("Configuration", "Country");
			environment = getStringParamValue("Configuration", "Environment");
			robot       = getStringParamValue("Configuration", "Robot");
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// Settings up the connection to the REST API server
		CLauncherRestConnectionManager connectionManager = new CLauncherRestConnectionManager(restBaseUrl, restApiKey);
		restBackend = new CLauncherRestBackend(connectionManager);
		
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		// We run a testsuite here so that we can benefits from the Setup and Teardown optimization
		// when run() is called we'll parse the log to get information about one specific testcase
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		boolean problemDuringExecution = false;
		
		thisTestName = testName;
		StringBuffer testcaseList = new StringBuffer();
		
		// Retrieve all the testcase names that are executable.
		// This is to ensure that even if we run a full robot framework testsuite (the test in XStudio), only the executable testcases are executed
		traceln(LOG_PRIORITY_INFO, "ids of all the not executable testcases: " + notExecutableTestcaseIdVector);
		StringBuffer testListOption = new StringBuffer();
		for (int indexTest=0; indexTest<testExecutionVector.size(); indexTest++) {
			CTestExecution testExecution = testExecutionVector.get(indexTest);
			int thisTestId = testExecution.getId();
			if (thisTestId == testId) {
				traceln(LOG_PRIORITY_INFO, "checking executing testcases for test " + testId + "...");
				for (int indexTestcase=0; indexTestcase<testExecution.getTestcaseExecutionVector().size(); indexTestcase++) {
					int testcaseId = testExecution.getTestcaseExecutionVector().get(indexTestcase).getId();
					trace(LOG_PRIORITY_INFO, "> testcase " + testcaseId + ": ");
					if (!notExecutableTestcaseIdVector.contains(testcaseId)) {
						traceln(LOG_PRIORITY_INFO, "executable");
						String testcaseName = testExecution.getTestcaseExecutionVector().get(indexTestcase).getName();
						testListOption.append("&test=" + CTextUtils.escapeURIParameters(testName) + "&testcase=" + CTextUtils.escapeURIParameters(testcaseName));
						testcaseList.append(testcaseName + ",");
					} else {
						traceln(LOG_PRIORITY_INFO, "NOT executable");
					}
				}
			}
		}
		
		if (testcaseList.length() > 0)
			testcaseList.deleteCharAt(testcaseList.length()-1);
		
		// +----------------------------------------------------------+
		// | Request the execution of all the testcases within a test |
		// +----------------------------------------------------------+
		
		// --------------------------
		//          QUEUING
		// --------------------------
		String jsonContent = restBackend.addToExecutionQueue(""+campaignSessionId, testListOption.toString(), country, environment, robot);
		JSONObject rootObj = new JSONObject(jsonContent);
		
		// messageType
		String messageType = rootObj.getString("messageType");
		if (messageType.equalsIgnoreCase("ok")) {
			executionSteps.add(new CExecutionStep(RESULT_SUCCESS, "testcases " + testcaseList + " successfully added to execution queue..."));
		} else {
			executionSteps.add(new CExecutionStep(RESULT_FAILURE, "problem occured while adding testcases " + testcaseList + " to execution queue..."));
			problemDuringExecution = true;
		}
		
		// message
		executionSteps.add(new CExecutionStep(RESULT_INFO, rootObj.getString("message")));
		
		// errors
		int nbErrorTCNotAllowedOnEnv      = rootObj.getInt("nbErrorTCNotAllowedOnEnv");
		int nbErrorTCNotActive            = rootObj.getInt("nbErrorTCNotActive");
		int nbErrorEnvNotExistOrNotActive = rootObj.getInt("nbErrorEnvNotExistOrNotActive");
		int nbErrorRobotMissing           = rootObj.getInt("nbErrorRobotMissing");

		if (nbErrorTCNotAllowedOnEnv>0)			executionSteps.add(new CExecutionStep(RESULT_FAILURE, "Error(s) because of testcase(s) not allowed on environment: "	+ nbErrorTCNotAllowedOnEnv));
		if (nbErrorTCNotActive>0)				executionSteps.add(new CExecutionStep(RESULT_FAILURE, "Error(s) because of testcase(s) not active: "					+ nbErrorTCNotActive));
		if (nbErrorEnvNotExistOrNotActive>0)	executionSteps.add(new CExecutionStep(RESULT_FAILURE, "Error(s) because of environment not active: " 					+ nbErrorEnvNotExistOrNotActive));
		if (nbErrorRobotMissing>0)				executionSteps.add(new CExecutionStep(RESULT_FAILURE, "Error(s) because of robot missing: "								+ nbErrorRobotMissing));
		
		/*
		We should still be able to get the results of testcases that were executed
		if ((nbErrorTCNotAllowedOnEnv+nbErrorTCNotActive+nbErrorEnvNotExistOrNotActive+nbErrorRobotMissing)>0)
			problemDuringExecution = true;
		*/
		
		CUtils.sleep(500);
		
		// --------------------------
		//  WAITING END OF EXECUTION
		// --------------------------

		String globalTestResult = "KO";
		while (true) {
			jsonContent = restBackend.resultCI(""+campaignSessionId);
			rootObj = new JSONObject(jsonContent);
			
			globalTestResult = rootObj.getString("result");
			
			if (!globalTestResult.equalsIgnoreCase("PE")) { // result different than pending (possible values: "OK", "KO" or "PE")
				traceln(LOG_PRIORITY_INFO, "=====> Test result: " + globalTestResult);
				break;
			}
			
			CUtils.sleep(1000);
		}
		/*
		Even if the test is KO, we still be able to get the results of testcases that were executed
		if (!globalTestResult.equalsIgnoreCase("OK"))
			problemDuringExecution = true;
		*/

		// --------------------------
		//    GET DETAILED RESULTS
		// --------------------------

		jsonReport = restBackend.getTagDetails(""+campaignSessionId);
		System.out.println("- - - - getTagDetails ==> " + jsonReport);

		// Failure only when we couldn't add testcases to the queue
		return new CReturnStatus(problemDuringExecution ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// parse jsonReport
		JSONObject rootReport = new JSONObject(jsonReport);
		JSONArray executionsArray = rootReport.getJSONArray("executions");
		for (int i=0; i<executionsArray.length(); i++) {
			JSONObject execution = executionsArray.getJSONObject(i);
			JSONObject testcase = execution.getJSONObject("testcase");
			if (testcase.getString("testcase").equalsIgnoreCase(testcaseName)) {
				System.out.println(" - - - - FOUND TESTCASE " + testcaseName + " in report");

				String start = execution.getString("start");
				System.out.println(" - - - - - > start=" + start);

				String controlStatus = execution.getString("controlStatus");
				System.out.println(" - - - - - > controlStatus=" + controlStatus);

				String controlMessage = execution.getString("controlMessage");
				System.out.println(" - - - - - > controlMessage=" + controlMessage);

				String link = execution.getString("link");
				System.out.println(" - - - - - > link=" + link);
				executionSteps.add(new CExecutionStep(start, RESULT_INFO, "link: " + link));
				
				long durationInMs = execution.getLong("durationInMs");
				System.out.println(" - - - - - > durationInMs=" + durationInMs);
				executionSteps.add(new CExecutionStep(start, RESULT_INFO, "duration: " + durationInMs + " ms"));
				
				String country = execution.getJSONObject("country").getString("description");
				System.out.println(" - - - - - > country=" + country);
				executionSteps.add(new CExecutionStep(start, RESULT_INFO, "executed in : " + country));
				
				String robotName = execution.getJSONObject("robot").getString("name");
				System.out.println(" - - - - - > robotName=" + robotName);
				String robotHost = execution.getJSONObject("robot").getString("host");
				System.out.println(" - - - - - > robotHost=" + robotHost);
				executionSteps.add(new CExecutionStep(start, RESULT_INFO, "executed with robot: " + robotName + " (on host " + robotHost + ")"));
				
				System.out.println("- - - - executionSteps = " + executionSteps);
				
				if (controlStatus.equalsIgnoreCase("OK")) {
					executionSteps.add(new CExecutionStep(start, RESULT_SUCCESS, controlMessage));
					return new CReturnStatus(RESULT_SUCCESS, executionSteps);
					
				} else if (controlStatus.equalsIgnoreCase("KO")) {
					executionSteps.add(new CExecutionStep(start, RESULT_FAILURE, controlMessage));
					return new CReturnStatus(RESULT_FAILURE, executionSteps);
					
				} else {
					executionSteps.add(new CExecutionStep(start, RESULT_INDEFINITE, controlMessage));
					return new CReturnStatus(RESULT_INDEFINITE, executionSteps);
				}
			} // if (testcase.getString("testcase").equalsIgnoreCase(testcaseName)) 
		}
		
		executionSteps.add(new CExecutionStep(RESULT_FAILURE, "testcase " + testcaseName + " not found in report generated by Cerberus!"));
		return new CReturnStatus(RESULT_FAILURE, executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Terminate"));
		
		restBackend = null;
		
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2etxt$",          // only *.txt files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		
		return output;
	}
}
