/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.10                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.cucumber_v2;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.util.LinkedHashMap;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;


/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Cucumber using the CLI.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{cucumber_v2   }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String javaInstallPath;
	private String cucumberInstallPath;
	private String additionalClasspath;
	private String optionalArguments;

	private String cucumberInterpreter;

	private File workingDir;

	File cucumberTraceFile;
	File gherkinFeatureFile;
	PrintWriter out;

	private boolean replaceFeature;

	private static final String CUCUMBER_RUNNER_CLASS = "cucumber.api.cli.Main";

	private static final String JAVA_INTERPRETER_EXE = CUtils.getExecutableName("java");
	private static final char CLASSPATH_DELIMITER = CUtils.getClasspathDelimiterChar();


	private LinkedHashMap<Integer, String> testcaseDescriptionLinkedHashMap;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) {
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");

		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath        = getStringParamValue("General", "Test root path");         // i.e. D:/tests/cucumber

			javaInstallPath     = getStringParamValue("Cucumber_v2", "Java install path");     // i.e. C:/Program Files/Java/jdk1.6.0_17
			cucumberInstallPath = getStringParamValue("Cucumber_v2", "Cucumber install path"); // i.e. C:/Cucumber
			additionalClasspath = getStringParamValue("Cucumber_v2", "Additional classpath");  // i.e. C:/tests/classes"
			optionalArguments   = getStringParamValue("Cucumber_v2", "Optional arguments");

			cucumberInterpreter = CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE) + " " +
			                   "-classpath " + CFileUtils.quoteFilePath(cucumberInstallPath + CLASSPATH_DELIMITER + additionalClasspath + CLASSPATH_DELIMITER + testRootPath) + " " +
			                   CUCUMBER_RUNNER_CLASS + " ";

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		workingDir = new File(testRootPath); // parent folder of all the test classes

		// +------------------------------------+
		// | console trace
		// +------------------------------------+

		try {cucumberTraceFile = File.createTempFile("cucumber_", ".txt"); } catch (IOException e1) {traceln(LOG_PRIORITY_SEVERE, "Couldn't create trace file!");}
		CFileUtils.deleteAndTouchFile(cucumberTraceFile);

		File junitOutputFile = new File(testRootPath + "/" + testName + ".xml");

		File htmlOutputFile = new File(testRootPath + "/" + testName + ".html");


		// -----------------------------------------------------
		// converting the test case description to .feature file
		// -----------------------------------------------------
		
		// get the test case description
		testcaseDescriptionLinkedHashMap = getTestcaseDescriptionLinkedHashMap();
		String testCaseDescription = testcaseDescriptionLinkedHashMap.get(testcaseId);
		traceln(LOG_PRIORITY_INFO, "test case description for  " + testcaseId + " description = " + testCaseDescription + "...");
		
		// by default we don't know if the description contains gherkin
		replaceFeature = false;
		if (testCaseDescription.length() > 0) {
			traceln(LOG_PRIORITY_INFO, "index of  scenario :" + testCaseDescription.indexOf("Scenario") + "...");
			traceln(LOG_PRIORITY_INFO, "index of  Then :" + testCaseDescription.indexOf("Then") + "...");
			traceln(LOG_PRIORITY_INFO, "index of  When :" + testCaseDescription.indexOf("When") + "...");
			traceln(LOG_PRIORITY_INFO, "index of  JUST FOR INFO :" + testCaseDescription.indexOf("JUST FOR INFO") + "...");

			if (testCaseDescription.indexOf("Scenario")>0 && 
				testCaseDescription.indexOf("Then")>0 && 
				testCaseDescription.indexOf("When")>0 &&
				testCaseDescription.indexOf("JUST FOR INFO") < 0) { // it seems to contains gherkin
				
				gherkinFeatureFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "_xs.feature");
				CFileUtils.deleteAndTouchFile(gherkinFeatureFile);

				// then we flush the test case description into .feature file we create
				// we choose to not replace the potentially existing one by adding '_xs' to the test name
				try {
					out = new PrintWriter(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "_xs.feature", "UTF-8");
					out.write(testCaseDescription);
					out.flush();
					out.close();
					// we then set the flag to state that this new feture file should be user rather than the one existing

					replaceFeature =true;
				} catch (IOException e1) {
					traceln(LOG_PRIORITY_SEVERE, "Couldn't create feature file!");
				}

				// just trace this in the log
				if (replaceFeature) {
						traceln(LOG_PRIORITY_INFO, "created on the spot feature file from test case description " + testcaseId + " description = " + testCaseDescription + "...");
				}
			}
		}

		// +------------------------------------+
		// | Run the JUnit test class
		// +------------------------------------+
		CRunner junitRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,
                                           cucumberInterpreter.toString() + "--plugin html:" + testName + ".html --plugin junit:" + testName + ".xml " + optionalArguments + " " +
                                           testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + (replaceFeature ? "_xs" : "") + ".feature", //  use the desired feature file  either _xs or teh one already present
                                           workingDir);

		// here we redirect the output to a file for later parsing
		short result = junitRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, cucumberTraceFile);

		// +---------------------------------------------------+
		// | Add the output console and reports as attachments
		// +---------------------------------------------------+
		addTestcaseMessageAttachment(cucumberTraceFile);
		addTestcaseMessageAttachment(junitOutputFile);
		addTestcaseMessageAttachment(htmlOutputFile);

		// return code of org.junit.runner.JUnitCore: if all tests run successfully, exit with a status of 0. Otherwise exit with a status of 1.
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution process failed"));
			parseResultFile(executionSteps); // to have the details anyway
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution process succeeded"));
			return parseResultFile(executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+

	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}

	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForCucumberTests(output,
											  testRootPathFolder,   // folder to scan
											  "^.*$",               // any folder
											  "^.*\\x2efeature$",   // only *.feature files
											  true,                 // remove extension from the file name to determine the test name
											  "/",                  // separator used in canonicalPath generated
											  "");                  // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	

	public void recursivelyScanFolderForCucumberTests(Vector<CScannedTest> output, File folder, 
												 String regexFolder, String regexFile,
												 boolean removeExtension, String separator, String canonicalPath) {
		if (folder.isDirectory()) {
			System.out.println(folder + " is well a directory");
			File[] children = folder.listFiles();
			for (int i = 0; i < children.length; i++) {
				File child = children[i];
				if (child.isDirectory()) {
					System.out.println("child (directory): " + child);
					System.out.println("regular expression folder: " + regexFolder);
					if (Pattern.matches(regexFolder, child.getName())) {
						System.out.println("recursively parsing " + child + "...");
						if (regexFile != null) { // searching for files
							recursivelyScanFolderForCucumberTests(output, child, regexFolder, regexFile, removeExtension, separator, 
																  canonicalPath.trim().length() > 0 ? canonicalPath + separator + child.getName() : child.getName());
						}
					} else {
						System.err.println("skipping folder \"" + canonicalPath + separator + child.getName() + "\"...");
					}
					
				} else { // file
					System.out.println("child (file): " + child); // myTest1.postman_collection.json
					System.out.println("regular expression file: " + regexFile);
					if (regexFile != null) {
						if (Pattern.matches(regexFile, child.getName())) {
							String testName = child.getName();
							System.out.println("testName: " + testName); // i.e. myTest1.postman_collection.json
							if (removeExtension) {
								testName = CFileUtils.getBasename(child);
							}
							
							String relativePath = canonicalPath;
							if (separator.equals("."))
								relativePath = relativePath.replaceAll("\\x2e", "/");
							else if (separator.equals("\\"))
								relativePath = relativePath.replaceAll("\\\\", "/");
							
							
							// read the content of the .feature file to store it in the test's description field
							String featurefileContent = CFileUtils.FileToString(child);
							//System.out.println("- - - - feature file content: " + featurefileContent);
							
							CScannedTest scannedTest = new CScannedTest(testName, canonicalPath, relativePath, featurefileContent); // the separate for relativePath is necessarily a /
							System.out.println("> found test \"" + canonicalPath + " : " + testName + "\"");
							
							// check that we do not already have discovered a test with the same name (all the tests will be imported in a unique folder so must have a different name)
							if (!output.contains(scannedTest)) { 
								scannedTest.addScannedTestcase(10, "default");
								output.add(scannedTest);
							}
						}
					}
				}
			}
		} else { // should never happen!
		}
	}

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// parse the result file to get the result and the execution steps
		if (!cucumberTraceFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		String line, message="";
		boolean errorDetected = false;
		int errorIndex = 1;

		try {
			FileInputStream fileInputStream = new FileInputStream(cucumberTraceFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				if (line.startsWith("JUnit version ")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));

				} else if (line.startsWith("Could not find class")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, line));
					errorDetected = true;

				} else if (line.startsWith("Exception in thread \"main\"")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, line));
					errorDetected = true;

				} else if (line.startsWith("Time: ")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));

				} else if (line.startsWith("There was") && line.endsWith("failure:")
				        || line.startsWith("There were") && line.endsWith("failures:")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, line));
					errorDetected = true;

				} else if (line.startsWith(errorIndex + ")")) {
					errorDetected = true;
					message = line.substring((errorIndex + ")").length() + 1, line.length());
					String nextLine = bufferedReader.readLine();
					if (nextLine != null)
						message = message + nextLine;
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, message));
					errorIndex++;

				} else if (line.startsWith("OK ")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, line));

				} else if (line.startsWith("Tests run")) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));

				} else if (line.indexOf("[Success]")>=0) {
					try {message = line.substring(10, line.length());} catch (Exception e) {} // [Success] length = 9
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, message));

				} else if (line.indexOf("[Failure]")>=0) {
					try {message = line.substring(10, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, message));
					errorDetected = true;

				} else if (line.indexOf("[Log]")>=0) {
					try {message = line.substring(6, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, message));

				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}

}
