/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.etasq;

import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.json.JSONArray;
import org.json.JSONObject;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for etasQ.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{etasQ         }  ";

	private String restBaseUrl;
	private String restApiKey;
	private int timeout;

	private CLauncherRestBackend restBackend;
	
	// libraries of keywords
	private final HashMap<String, String> libraryNameIdHashmap = new HashMap<String, String>();
	private final HashMap<String, String> keywordNameIdHashmap = new HashMap<String, String>();
	private final HashMap<String, String> keywordIdLibraryIdHashmap = new HashMap<String, String>();
	
	// parsing of the procedure's steps
	private String                  currentKeyword    = null;
	private HashMap<String, String> currentParameters = null; 
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			restBaseUrl = getStringParamValue("REST Connection", "Base URL");
			restApiKey  = getStringParamValue("REST Connection", "API Key");
			timeout     = getIntegerParamValue("REST Connection", "Timeout (in seconds)"); // i.e. 3600 (seconds)
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// Settings up the connection to the REST API server
		CLauncherRestConnectionManager connectionManager = new CLauncherRestConnectionManager(restBaseUrl, restApiKey);
		restBackend = new CLauncherRestBackend(connectionManager);
		
		// getting all libraries
		String jsonLibContent = restBackend.getLibraries();
		JSONArray jsonLibraries = new JSONArray(jsonLibContent);
		for (int i=0; i<jsonLibraries.length(); i++) {
			JSONObject library = jsonLibraries.getJSONObject(i);
			libraryNameIdHashmap.put(library.getString("name"), library.getString("id"));
		}
		traceln(LOG_PRIORITY_INFO, "libraries = " + libraryNameIdHashmap);

		// getting all keywords
		Iterator<String> libNameIterator = libraryNameIdHashmap.keySet().iterator();
		while (libNameIterator.hasNext()) {
			String libraryName = libNameIterator.next();
			String libraryId   = libraryNameIdHashmap.get(libraryName);
			
			String jsonKeywordContent = restBackend.getKeywords(libraryId);
			JSONArray jsonKeywords = new JSONArray(jsonKeywordContent);
			for (int i=0; i<jsonKeywords.length(); i++) {
				JSONObject keyword = jsonKeywords.getJSONObject(i);
				keywordNameIdHashmap.put(keyword.getString("name").toLowerCase(), keyword.getString("id"));
				keywordIdLibraryIdHashmap.put(keyword.getString("id"), libraryId);
			}
		}
		traceln(LOG_PRIORITY_INFO, "===================================");
		traceln(LOG_PRIORITY_INFO, "keywords name/id= " + keywordNameIdHashmap);
		traceln(LOG_PRIORITY_INFO, "keywords id/library id = " + keywordIdLibraryIdHashmap);
		traceln(LOG_PRIORITY_INFO, "===================================");
		
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		System.out.println();
		System.out.println();
		traceln(LOG_PRIORITY_INFO, "****************************************************************************************************");
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

    	String xmlContent = getTestcaseStepsLinkedHashMap().get(testcaseId);
    	traceln(LOG_PRIORITY_INFO, "xmlContent = " + xmlContent);
    	Document xmlDocument = CXmlDocumentFactory.createXMLDoc(xmlContent);
	    NodeList stepNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//object[@type='step']");
		
	    boolean failed = false;
	    
	    // for each step...
	    for (int j=0; j<stepNodeList.getLength(); j++) {
			Node stepNode = stepNodeList.item(j);
	
	    	// Step
	    	String stepDescription = CTextUtils.unescapeAttributeString(CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "@description"));
	    	stepDescription = stepDescription.trim();
	    	if (stepDescription.length()>0) {
		    	System.out.println("- - - - STEP " + (j+1));
		    	CReturnStatus stepReturnStatus = /*new CReturnStatus(); //*/executeKeyword(stepDescription, executionSteps);
		    	if (stepReturnStatus.getResult() != RESULT_SUCCESS)
		    		failed = true;
	    	}
	    	
	    	// checks
			NodeList checkNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(stepNode, ".//object[@type='check']");
			for (int k=0; k<checkNodeList.getLength(); k++) {
				Node checkNode = checkNodeList.item(k);
				
				String checkDescription = CTextUtils.unescapeAttributeString(CXmlDocumentFactory.getNodeStringValueFromXPath(checkNode, "@description"));
				checkDescription = checkDescription.trim();
				if (checkDescription.length()>0) {
					System.out.println("- - - - CHECK " + (k+1));
			    	CReturnStatus checkReturnStatus = /*new CReturnStatus(); //*/executeKeyword(checkDescription, executionSteps);
			    	if (checkReturnStatus.getResult() != RESULT_SUCCESS)
			    		failed = true;
				}
			}
	    }
	    
		return new CReturnStatus((failed ? RESULT_FAILURE : RESULT_SUCCESS), executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Terminate"));
		
		restBackend = null;
		
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	private void parseDescription(String input) {
		currentKeyword    = null;
		currentParameters = null;
		currentParameters = new HashMap<String, String>();
		
		String lines[] = input.split("\\r?\\n");
		
		if (lines.length > 0)
			currentKeyword = lines[0];
			
		if (currentKeyword == null)
			return;
		
		currentKeyword = currentKeyword.trim();
		traceln(LOG_PRIORITY_INFO, "keyword=" + currentKeyword);
		
		if (lines.length > 1) {
			for (int i=1; i<lines.length; i++) {
				String paramLine = lines[i];
				paramLine = paramLine.trim();
				
				int equalSignIndex = paramLine.indexOf("=");
				if (paramLine.charAt(0)!='-' || equalSignIndex<0)
					continue;
				
				paramLine = paramLine.substring(1, paramLine.length());
				//System.out.println("\n\n- - - -  paramLine = " + paramLine);
				
				String paramName = paramLine.substring(0, equalSignIndex-1);
				paramName = paramName.trim();
				//System.out.println("- - - -  paramName=" + paramName);
						
				String paramValue = paramLine.substring(equalSignIndex, paramLine.length());
				paramValue = paramValue.trim();
				//System.out.println("- - - -  paramValue=" + paramValue);

				currentParameters.put(paramName, paramValue);
			}
		}
		traceln(LOG_PRIORITY_INFO, "parameters=" + currentParameters);
	}
	
	
	
	
	private CReturnStatus executeKeyword(String stepDescription, Vector<CExecutionStep> executionSteps) {
    	stepDescription = stepDescription.toLowerCase();
    	parseDescription(stepDescription); // generates currentKeyword + currentParameters
    	
    	if (!keywordNameIdHashmap.containsKey(currentKeyword) || keywordNameIdHashmap.get(currentKeyword) == null) {
    		System.err.println("Error: keyword \"" + currentKeyword + "\" not found!");
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Error: keyword \"" + currentKeyword + "\" not found!"));
    		return new CReturnStatus(RESULT_FAILURE, executionSteps);
    		
    	} else {
    		System.out.println("");
    		traceln(LOG_PRIORITY_INFO, "--------------------------------------------");
    		traceln(LOG_PRIORITY_INFO, ">>> keyword \"" + currentKeyword + "\" found");
    		String keywordId = keywordNameIdHashmap.get(currentKeyword);
    		String libraryId = keywordIdLibraryIdHashmap.get(keywordId);
    		traceln(LOG_PRIORITY_INFO, "keywordId=" + keywordId);
    		traceln(LOG_PRIORITY_INFO, "libraryId=" + libraryId);
    		
    		// ----------------------------------------------------------
    		// get the template to provide when running this keyword
    		// ----------------------------------------------------------
    		System.out.println("");
    		traceln(LOG_PRIORITY_INFO, "getting the template from " + currentKeyword + "...");
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "getting the template from " + currentKeyword + "..."));
	    	JSONObject templateObject = null;
	    	JSONObject parametersObject = null;
    		try {
    			String getKeywordJsonContent = restBackend.getKeyword(libraryId, keywordId);
    	    	JSONObject keywordRootObject = new JSONObject(getKeywordJsonContent);
    	    	templateObject = keywordRootObject.getJSONObject("template");
    	    	parametersObject = templateObject.getJSONObject("parameters");
    	    	traceln(LOG_PRIORITY_INFO, "parameters=" + parametersObject.toString());
	    	} catch (Exception e) {
	    		traceln(LOG_PRIORITY_INFO, "couldn't fetch the template for keyword \"" + currentKeyword + "\"!");
		    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't fetch the template for keyword \"" + currentKeyword + "\"!"));
	    		return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    	}
	    	
    		// ----------------------------------------------------------
	    	// replace the parameters provided in XStudio in the template
    		// ----------------------------------------------------------
    		System.out.println("");
    		traceln(LOG_PRIORITY_INFO, "update the template from " + currentKeyword + "...");
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "update the template from " + currentKeyword + "..."));
	    	Iterator<String> parameterNameIterator = currentParameters.keySet().iterator();
			while (parameterNameIterator.hasNext()) {
				String paramName = parameterNameIterator.next();
				String paramObject = null;
				try {
					paramObject = parametersObject.getString(paramName);
					String paramValue = currentParameters.get(paramName);
					parametersObject.put(paramName, paramValue);
				} catch (Exception e) {
			    	traceln(LOG_PRIORITY_INFO, "param \"" + paramName + "\" not found in template for keyword \"" + currentKeyword + "\"!");
			    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "param \"" + paramName + "\" not found in template for keyword \"" + currentKeyword + "\"!"));
			    	return new CReturnStatus(RESULT_FAILURE, executionSteps);
				}
			}
			traceln(LOG_PRIORITY_INFO, "template with param value updated=" + templateObject.toString());
			
    		// ----------------------------------------------------------
			// request the execution of the keyword
    		// ----------------------------------------------------------
    		System.out.println("");
			traceln(LOG_PRIORITY_INFO, "request execution of keyword " + currentKeyword + "...");
			traceln(LOG_PRIORITY_INFO, "parameters: " + parametersObject);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "executing keyword " + currentKeyword + "..."));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "parameters: " + parametersObject));
	    	String executionId = null;
	    	try {
				String executeKeywordJsonContent = restBackend.executeKeyword(templateObject.toString());
				//System.out.println("- - - executeKeywordJsonContent="+executeKeywordJsonContent);
				JSONObject executeKeywordResponseRootObject = new JSONObject(executeKeywordJsonContent);
	    		executionId = executeKeywordResponseRootObject.getString("id");
	    	} catch (Exception e) {
	    		System.err.println("Error: keyword \"" + currentKeyword + "\" couldn't be submitted for execution!");
		    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Error: keyword \"" + currentKeyword + "\" couldn't be submitted for execution!"));
		    	return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    	}
	    	
    		// ----------------------------------------------------------
	    	// wait for the test to be executed
    		// ----------------------------------------------------------
    		System.out.println("");
	    	traceln(LOG_PRIORITY_INFO, "waiting for keyword execution " + executionId + " to finish...");
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "waiting for keyword execution " + executionId + " to finish..."));
	    	String getRunResponseStatus = "created";
	    	long timestampStart = CCalendarUtils.getCurrentTimeMillis();
	    	while (true) {
		    	try {
					String getRunJsonContent = restBackend.getRun(executionId);
					JSONObject getRunResponseRootObject = new JSONObject(getRunJsonContent);
		    		getRunResponseStatus = getRunResponseRootObject.getString("status");
		    		getRunResponseStatus = getRunResponseStatus.toLowerCase();
		    	} catch (Exception e) {
		    		System.err.println("Error: couldn't get status from execution!");
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Error: couldn't get status from execution!"));
		    		return new CReturnStatus(RESULT_FAILURE, executionSteps);
		    	}
		    	
		    	if (!getRunResponseStatus.equalsIgnoreCase("created") && !getRunResponseStatus.equalsIgnoreCase("running"))
		    		break;
		    	else
		    		CUtils.sleep(500);
		    	
		    	long timestampNow = CCalendarUtils.getCurrentTimeMillis();
		    	if ((timestampNow-timestampStart) > (timeout*1000)) {
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Error: keyword was too long to execute. Timeout elapsed!"));
		    		return new CReturnStatus(RESULT_FAILURE, executionSteps);
		    	}
	    	}

	    	traceln(LOG_PRIORITY_INFO, "result: " + getRunResponseStatus);
	    	if (getRunResponseStatus.equalsIgnoreCase("success")) {
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "result: " + getRunResponseStatus));
	    		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	    	} else {
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "result: " + getRunResponseStatus));
	    		return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    	}
    	}
	}
	
	
	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2etxt$",          // only *.txt files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		
		return output;
	}
	
	public static void main(String[] args) {
		HashMap<String, String> libraryNameIdHashmap = new HashMap<String, String>();
		HashMap<String, String> keywordNameIdHashmap = new HashMap<String, String>();
		HashMap<String, String> keywordIdLibraryIdHashmap = new HashMap<String, String>();
		
		// Settings up the connection to the REST API server
		CLauncherRestConnectionManager connectionManager = new CLauncherRestConnectionManager("http://13.37.236.155:8091/v0", "eyJ0eXAiOiJKV1QiLCJhbGciOiJIUzI1NiJ9.eyJpc3MiOiJjb20ucG9uYW50LXRlY2hub2xvZ2llcy50YXNxLWFwaSIsImlhdCI6MTY0OTQxOTg5MiwiZXhwIjoxNjU0NjAzODkyLCJzdWIiOiIifQ.VUhoNRcAAuTH1HWbrF7Acpkk2teVyl8_cgHzD6U7eA4");
		CLauncherRestBackend restBackend = new CLauncherRestBackend(connectionManager);
		
		// getting all libraries
		String jsonLibContent = restBackend.getLibraries();
		JSONArray jsonLibraries = new JSONArray(jsonLibContent);
		for (int i=0; i<jsonLibraries.length(); i++) {
			JSONObject library = jsonLibraries.getJSONObject(i);
			libraryNameIdHashmap.put(library.getString("name"), library.getString("id"));
		}
		System.out.println("- - - - libraries = " + libraryNameIdHashmap);

		
		// getting all keywords
		Iterator<String> libNameIterator = libraryNameIdHashmap.keySet().iterator();
		while (libNameIterator.hasNext()) {
			String libraryName = libNameIterator.next();
			String libraryId   = libraryNameIdHashmap.get(libraryName);
			
			String jsonKeywordContent = restBackend.getKeywords(libraryId);
			JSONArray jsonKeywords = new JSONArray(jsonKeywordContent);
			for (int i=0; i<jsonKeywords.length(); i++) {
				JSONObject keyword = jsonKeywords.getJSONObject(i);
				keywordNameIdHashmap.put(keyword.getString("name"), keyword.getString("id"));
				keywordIdLibraryIdHashmap.put(keyword.getString("id"), libraryId);
			}
		}
		
		System.out.println("- - - - keywords = " + keywordNameIdHashmap);
		System.out.println("- - - - assoc = " + keywordIdLibraryIdHashmap);
		
		// Find Word = KZUXG2LPNZGGSYTSMFZHS > IZUW4ZBAK5XXEZA
		
		/*
		{
		    "documentation": "Take a picture with the camera and find all words on it.\n        If at least one word is not found, this keyword will fail.\n        A specific symbol stick can be added to the left of all given words :\n            - '!' defines forbidden word\n                ex : words = \"!hello\", if \"hello\" is found, keyword will fail\n            - '~' defines retry word\n                ex : words = \"~hello\", if \"hello\" is found, the all process of finding words restarts\n            - '?' defines conditional word\n                ex : words = \"?hello\", if \"hello\" is found, suite variable ${CONDITIONAL} is set to True\n\n        Several words can have the same specific symbol, but the method stops to run as soon as one of these is found.\n        The priority of specific symbols is : \"!\" then \"~\" then \"?\".\n\n        *Parameters:*\n\n        ``words``: A list of words separated with one space\n\n        ``timeout_value``: After \"timeout value\" secs, the keywords will fail\n\n        ``area``: The area in which the icon is looked for, area to crop define in currentHWconf.json\n\n        `` best_match_threshold``: threshold to validate find_word match in range [1;100]\n\n        ``find_all``: if False search for at least one word from a words, if True search for all words from a words\n\n        ``trace``: if True save image all the time in report\n\n        *Return value:* None",
		    "id": "IZUW4ZBAK5XXEZA",
		    "library": {
		        "id": "KZUXG2LPNZGGSYTSMFZHS",
		        "name": "VisionLibrary",
		        "type": "library",
		        "url": "http://13.37.236.155:8091/v0/libraries/KZUXG2LPNZGGSYTSMFZHS"
		    },
		    "name": "Find Word",
		    "parameters": [
		        {
		            "name": "words",
		            "position": 1
		        },
		        {
		            "default": "10s",
		            "name": "timeout_value"
		        },
		        {
		            "default": "${None}",
		            "name": "area"
		        },
		        {
		            "default": "70",
		            "name": "best_match_threshold"
		        },
		        {
		            "default": "True",
		            "name": "find_all"
		        },
		        {
		            "default": "eng",
		            "name": "lang"
		        },
		        {
		            "default": "False",
		            "name": "trace"
		        }
		    ],
		    "template": {
		        "name": "Run request template",
		        "parameters": {
		            "area": "${None}",
		            "best_match_threshold": "70",
		            "find_all": "True",
		            "lang": "eng",
		            "timeout_value": "10s",
		            "trace": "False",
		            "words": "<value>"
		        },
		        "resource": {
		            "id": "IZUW4ZBAK5XXEZA",
		            "name": "Find Word",
		            "type": "keyword",
		            "url": "http://13.37.236.155:8091/v0/libraries/KZUXG2LPNZGGSYTSMFZHS/keywords/IZUW4ZBAK5XXEZA"
		        }
		    },
		    "type": "keyword",
		    "url": "http://13.37.236.155:8091/v0/libraries/KZUXG2LPNZGGSYTSMFZHS/keywords/IZUW4ZBAK5XXEZA"
		}
		*/
		CLauncherImpl titi = new CLauncherImpl();
		String toto =	"Find Word\n" +
						"- words = toto a  faim\n" +
						"-trace=True";
		
		titi.parseDescription(toto);
		
		
		
		
		
		
	}
	
}
