/*
+----------------------------------------------------------------------+
|                    Class: CLauncherImpl                              |
|                                                                      |
| Developper:  Sanjeeth Nayak                                          |
|              Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     0.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.fitnesse;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for FitNesse.
 * @author Eric Gavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

    // +==============================================================+
    // | Attributes                                                   |
    // +==============================================================+

    static final String TRACE_HEADER = "{fitnesse      }  ";

    static final String JAVA_INTERPRETER_EXE = CUtils.getExecutableName("java");
    static final String FITNESSE_JAR_NAME = "fitnesse-standalone.jar";
    
    private String jarInterpreter;
    
    private String javaInstallPath;
    private String testRootPath;
    private String fitnesseInstallPath;
    
    private File fitnesseTraceFile;
    
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

    public CLauncherImpl() {
            super(TRACE_HEADER);
    }

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
    
    @Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
        setSutDetails(sutId, sutName, sutVersion);
        
		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager        
		printConfiguration();
        
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        try {
        	// retrieve the parameters we need
			javaInstallPath          = getStringParamValue("General",  "Java install path");    // i.e. C:/Program Files/Java/jdk1.6.0_17
        	testRootPath        	 = getStringParamValue("General",  "Test root path");       // i.e. C:/test_repository/tests/fitnesse
        	
        	fitnesseInstallPath      = getStringParamValue("FitNesse",  "Install path");        // i.e. D:/softwares/FitNesse

			jarInterpreter = CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE);

        } catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
            return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
        }
        
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
        return new CReturnStatus(RESULT_SUCCESS);
    }


    @Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
        traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " Test:" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		File workingDir = new File(testRootPath); // parent folder of all the test classes

		// +------------------------------------+
		// | Delete previous results file
		// +------------------------------------+
		fitnesseTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "fitnesse_traces.txt");
		CFileUtils.deleteAndTouchFile(fitnesseTraceFile);

		// +------------------------------------+
		// | Run the sikuli test class
		// +------------------------------------+

        CRunner jarRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                        jarInterpreter  +" -jar " + fitnesseInstallPath + "/" + FITNESSE_JAR_NAME + 
                                        " -c " + testRootPath + (testPath.length() > 0 ? "." + testPath : "") + "." + testName + "." + testcaseName + "?test&format=text",
                                        // to run a suite we should use "?suite&format=text"
                                        workingDir);
        
        short result = jarRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, fitnesseTraceFile);
        
        return parseResultFile(executionSteps);
    }

    @Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
        return new CReturnStatus(RESULT_SUCCESS);
    }

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2etxt$",          // only *.txt files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// parse the result file to get the result and the execution steps
		if (fitnesseTraceFile == null || !fitnesseTraceFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		String line, message="";
		boolean errorDetected = false;

		try {
			FileInputStream fileInputStream = new FileInputStream(fitnesseTraceFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				if (line.substring(0, 3).indexOf(".")>=0) { // '.' = Success
					try {
						message = line.substring(2, line.length());
					} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(RESULT_SUCCESS, message));

				} else if (line.substring(0, 3).indexOf("F")>=0) { // 'F' = failure
					try {message = line.substring(2, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(RESULT_FAILURE, message));
					errorDetected = true;

				} else if (line.substring(0, 3).indexOf("X")>=0) { // 'X' = exception
					try {message = line.substring(2, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(RESULT_FAILURE, "Exception: " + message));
					errorDetected = true;

				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception whle parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception whle parsing the result file: " + e));
			errorDetected = true;
		}

		// attachments
    	addTestcaseMessageAttachment(fitnesseTraceFile);

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
}
