/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.0                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.git;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CTimeoutListener;
import java.io.IOException;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Executables.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{git     }  ";

	// parameters impacting executing at run time set by the test operator
	
	private boolean synchronous;
	private int timeout = 600;

	private String gitExePath;
	private String gitExportPath;
	private String gitRepoURL;
	
	private String gitUsername="";	
	private String gitPassword="";	
	private String gitRevision="HEAD";	
	private String gitExportOrCheckout="export";
	
	private File workingDir;
	private File logFile;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
		
			gitExePath = getStringParamValue("git", "GIT executor path (no ending slash)"); // C:\Program Files\gitgit\bin - not mandatroyr as it can be in the path
			gitExportPath = getStringParamValue("git",  "Export destination path (no ending slash)"); // c:\testscriptsandData - mandatory and should match the test root path of other tests
			gitRepoURL = getStringParamValue("git",  "GIT repo URL (no ending slash)"); //git://176.58.90.228/git/xqual/xstudio/trunk - mandatory
			
		
 
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		// get the attributes
		// the follwings are expected from test attributes - we don't want to expose credentials from one team to everybody through configurations
			//gitUsername // empty by default - not mandatory as it can cached on the host system 
			//gitPassword // empty by default - not mandatory as it can cached on the host system 
			//gitRevision //is HEAD by default but can be overidden in test attributes 
			//gitExportOrCheckout  // by default "export" , but can be "checkout"
		
				gitUsername = "";
				gitPassword = "";
				gitRevision = "";
				gitExportOrCheckout ="export";
				
			try {
				gitUsername = getStringAttributeValue(attributes, "com.xqual.git.username");
				gitPassword = getStringAttributeValue(attributes, "com.xqual.git.password");
				gitRevision = getStringAttributeValue(attributes, "com.xqual.git.revision");
				gitExportOrCheckout = getStringAttributeValue(attributes, "com.xqual.git.exportOrCheckout");
				
			} catch (Exception e) {
			}
			
			gitRevision = (gitRevision == null ? "" : gitRevision);
			gitExportOrCheckout = (gitExportOrCheckout == null ? "export": gitExportOrCheckout);
			gitExportOrCheckout = (gitExportOrCheckout.equalsIgnoreCase("checkout") ? gitExportOrCheckout : "export");
			
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + gitExportPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		workingDir = new File(gitExportPath + "\\");
		
		//make a copy to safe the export of the  checkout
		File workingDirSafe = new File(gitExportPath + ".old\\");
		
		try {
			if (workingDir.exists()) {
				traceln(LOG_PRIORITY_INFO, "Export or Checkout destination already exists and we safe it in  " + workingDirSafe.getAbsolutePath() + "...");
				// see if teh safe already exist and if so delete it 
				if (workingDirSafe.exists()) {
					CFileUtils.deleteFile(workingDirSafe);	
				}
				//make a safe of the original working dir
				CFileUtils.copyFiles(workingDir,workingDirSafe, false);
				traceln(LOG_PRIORITY_INFO, "Export or Checkout destination has been safed is in "  + workingDirSafe.getAbsolutePath() + "...");
				//and clean the workingDir
				CFileUtils.emptyFolder(workingDir);
			} else {
				// the workindir does not exist and we create it
				workingDir.mkdirs();
			}
		} catch (IOException e) {
			System.out.println("io exception: " + e);
		}
		
		traceln(LOG_PRIORITY_INFO, "before gitURLWithCredentials=" +  "...");
		
		//e.g. C:\Program Files\git\bin\git clone [--branch branch] https://github.com/mozilla/geckodriver.git c:\ginger -q 
		//git clone https://username:password@github.com/username/repository.git
		
		String gitURLWithCredentials ="";
		if (gitRepoURL.indexOf("http") > -1) {
			traceln(LOG_PRIORITY_INFO, "there's http " +  "...");
			gitURLWithCredentials = gitRepoURL.substring(0, gitRepoURL.indexOf(":") + 3);
			traceln(LOG_PRIORITY_INFO, "we extracted http " + gitURLWithCredentials+  "...");
			traceln(LOG_PRIORITY_INFO, "the username is  " + gitUsername +  "...");
			traceln(LOG_PRIORITY_INFO, "the password is  " + gitPassword +  "...");
			gitURLWithCredentials = gitURLWithCredentials + (gitUsername == null ? "" : gitUsername + (gitPassword == null ? "" : ":" + gitPassword) + "@" );
			traceln(LOG_PRIORITY_INFO, "we added username and password http " + gitURLWithCredentials +  "...");
			gitURLWithCredentials = gitURLWithCredentials + gitRepoURL.substring (gitRepoURL.indexOf(":") + 3, gitRepoURL.length() );
		}
		traceln(LOG_PRIORITY_INFO, "gitURLWithCredentials=" + gitURLWithCredentials + "...");
		
		String gitInterpreter = CFileUtils.quoteFilePath(gitExePath + "\\" + "git.exe") 
				+ " clone "
				+ (gitRevision == "" ? "" : " --branch " + CFileUtils.quoteFilePath(gitRevision)) + " "
				+ gitURLWithCredentials + " " 
				+ CFileUtils.quoteFilePath(workingDir.getAbsolutePath())
				+ " --verbose --progress" 
				+ " --single-branch --depth 1"
				+ " ";
								
		logFile = CFileUtils.createTmpFile("gitLogFile.txt");
		if (logFile.exists()) {
			CFileUtils.deleteFile(logFile);
		}
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner gitRunner = new CRunner("[ git " + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                        gitInterpreter,
                                        workingDir);

		short result = RESULT_FAILURE;

		result = gitRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, logFile);
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution failed"));
			if (logFile.exists()) {
				addTestcaseMessageAttachment(logFile);
			}
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			if (logFile.exists()) {
				addTestcaseMessageAttachment(logFile);
			}
			traceln(LOG_PRIORITY_INFO, "git succeeded to clone and we look if we need to get rid of .git. Value of gitExportOrCheckout is : " +  gitExportOrCheckout +  "...");
			if (gitExportOrCheckout.equalsIgnoreCase("export")) {
				traceln(LOG_PRIORITY_INFO, "this is an export , not a checkout " + gitExportOrCheckout+  "...");
				File dotGITHidden = new File(workingDir + "\\.git\\");
				traceln(LOG_PRIORITY_INFO, "the hiden .git file is  here : " + dotGITHidden.getAbsolutePath()+  "...");
				if (dotGITHidden.exists()) {
					traceln(LOG_PRIORITY_INFO, "it exists so we delete it ! " +  "...");
					CFileUtils.deleteFile(dotGITHidden);
					traceln(LOG_PRIORITY_INFO, "the .git doesn't exist anymore " +  "...");
				}
			}
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution succeeded"));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File gitExportPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(gitExportPath)); // OS dependant path
		if (gitExportPathFolder == null || !gitExportPathFolder.exists() || !gitExportPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  gitExportPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eexe$",          // only *.exe files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from gitExportPathFolder)
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		
		
		if (!logFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		String line, message="";
		boolean errorDetected = false;
		addTestcaseMessageAttachment(logFile);

		try {
			FileInputStream fileInputStream = new FileInputStream(logFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				if (line.indexOf("[Success]")>=0) {
					try {message = line.substring(10, line.length());} catch (Exception e) {} // [Success] length = 9
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, message));

				} else if (line.indexOf("[Failure]")>=0) {
					try {message = line.substring(10, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, message));
					errorDetected = true;

				} else if (line.indexOf("[Log]")>=0) {
					try {message = line.substring(6, line.length());} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, message));

				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}
			
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception whle parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception whle parsing the result file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
}
