/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.gradle;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CFileWriter;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;


/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Gradle.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{gradle        }  ";

	// parameters impacting executing at run time set by the test operator
	private String gradleInstallPath;
	private String optionalArguments;
	private String outputFileFormat;
	private String outputFilePath;
	private String consoleFilePath;
	private String gradleTestSuite;
	
	private File workingDir;
	private File consoleFile;
	private File outputFile;
	private File partedXmlOutputFile;
	
	Document xmlOutputDocument;
	
	private static final String GRADLE_EXE = "gradle"; // neither an .exe nor a .sh
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			gradleInstallPath = getStringParamValue("Gradle", "Gradle install path");           // i.e. "/usr/local/bin"
			optionalArguments = getStringParamValue("Gradle", "Optional arguments");            // i.e. "-Penv=qa"
			gradleTestSuite   = getStringParamValue("Gradle", "Test Suite");                    // i.e. "fasttests"
			consoleFilePath   = getStringParamValue("Gradle", "Console redirection file path"); // i.e. "/tmp/console.txt"
			
			outputFileFormat  = getStringParamValue("Gradle", "Output file format");            // i.e. "nose" or "TestNG"
			outputFilePath    = getStringParamValue("Gradle", "Output file path");              // i.e. "/tmp/nosetests.xml"
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// EXECUTE THE GRADLE SCRIPT AT INITIALIZATION OF THE LAUNCHER
		workingDir = new File(gradleInstallPath); // folder containing the test suite file

		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		consoleFile = new File(consoleFilePath);
		CFileUtils.deleteAndTouchFile(consoleFile);

		traceln(LOG_PRIORITY_INFO, "console redirection: " + consoleFile.getAbsolutePath());

		outputFile = new File(outputFilePath);
		CFileUtils.deleteAndTouchFile(outputFile);

		traceln(LOG_PRIORITY_INFO, "output: " + outputFile.getAbsolutePath());
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner gradleRunner = new CRunner("[Gradle Runner]", 
				   CFileUtils.quoteFilePath(gradleInstallPath + "/" + GRADLE_EXE) + " " + optionalArguments + " " + gradleTestSuite, 
				   workingDir);

		short result = gradleRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, consoleFile);

		traceln(LOG_PRIORITY_INFO, "--------------console-----------------");
		traceln(LOG_PRIORITY_INFO, CFileUtils.FileToString(consoleFile));
		traceln(LOG_PRIORITY_INFO, "-------------xml output---------------");
		traceln(LOG_PRIORITY_INFO, CFileUtils.FileToString(outputFile));
		traceln(LOG_PRIORITY_INFO, "-------------------------------------");

		// prepare the unique XML document that will be parsed several times
		xmlOutputDocument = CXmlDocumentFactory.createXMLDoc(outputFile);

		return new CReturnStatus(result, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "parsing xml file for results on " + testName + "...")); // test name !

		CReturnStatus returnStatus = null;
		if (outputFileFormat.equals("nose")) {
			returnStatus = parseNoseResultFile(testName, executionSteps);
			
		} else if (outputFileFormat.equals("TestNG")) {
			returnStatus = parseTestNGResultFile(testName, testcaseName, executionSteps);
		}
		
		// Attachments
		addTestcaseMessageAttachment(consoleFile); // ?

		return returnStatus;
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eqtp$",          // only *.qtp files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseNoseResultFile(String testName, Vector<CExecutionStep> executionSteps) {
		traceln(LOG_PRIORITY_INFO, "------------------------- nose ---------------------------");
		// parse the result file to get the result and the execution steps
		if (!outputFile.exists() || xmlOutputDocument == null) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		//System.err.println("FILE=" + CFileUtils.FileToString(xmlOutputFile));
		//System.err.println("DOC=" + CXmlDocumentFactory.nodeToString(xmlOutputDocument));
		Node testcaseNode = CXmlDocumentFactory.getNodeFromXPath(xmlOutputDocument, "//testcase[@name='" + testName + "']");
		if (testcaseNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Testcase not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: testcase \"" + testName + "\" not found!"));
			return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
		}
		
		traceln(LOG_PRIORITY_INFO, "Testcase \"" + testName + "\" found");
		partedXmlOutputFile = new File(workingDir.getAbsolutePath() + "/parted_" + outputFilePath);
		CFileUtils.deleteAndTouchFile(partedXmlOutputFile);

		// store the part of the XML file that corresponds to this testcase as an attachment
		String partedXmlOutput = CXmlDocumentFactory.nodeToString(testcaseNode);
		CFileWriter.write(partedXmlOutputFile, partedXmlOutput);
		addTestcaseMessageAttachment(partedXmlOutputFile);
		
		// parse the log
		String _className       = testcaseNode.getAttributes().getNamedItem("classname").getNodeValue();
		String _testcaseName    = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
		String _testcaseTime    = testcaseNode.getAttributes().getNamedItem("time").getNodeValue();
		traceln(LOG_PRIORITY_INFO, ">> testcase: class=" + _className + " name=" + _testcaseName + " time=" + _testcaseTime);
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]class/name[/b]: " + _className + "/" + _testcaseName));
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]time[/b]: " + _testcaseTime));

		traceln(LOG_PRIORITY_INFO, "Searching for testcase's system out...");
		String _outCDataLog     = "";
		Node testcaseOutLogNode = CXmlDocumentFactory.getNodeFromXPath(xmlOutputDocument, "//testcase[@name='" + testName + "']/system-out");
		if (testcaseOutLogNode != null) {
			_outCDataLog = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseOutLogNode);
			traceln(LOG_PRIORITY_INFO, ">>           system-out=" + _outCDataLog);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]system-out[/b]: " + _outCDataLog));
		}

		traceln(LOG_PRIORITY_INFO, "Searching for testcase's failure...");
		String _failureType     = "";
		String _failureMessage  = "";
		String _failureCDataLog = "";
		Node testcaseFailureLogNode = CXmlDocumentFactory.getNodeFromXPath(xmlOutputDocument, "//testcase[@name='" + testName + "']/failure"); // present only when there is a failure
		if (testcaseFailureLogNode != null) {
			_failureType     = testcaseFailureLogNode.getAttributes().getNamedItem("type").getNodeValue();
			_failureMessage  = testcaseFailureLogNode.getAttributes().getNamedItem("message").getNodeValue();
			_failureCDataLog = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseFailureLogNode);
			traceln(LOG_PRIORITY_INFO, ">>           failure type=" + _failureType + " message=" + _failureMessage);
			traceln(LOG_PRIORITY_INFO, ">>           failure=" + _failureCDataLog);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "[b]type[/b]: " + _failureType));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]message[/b]: " + _failureMessage));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]failure[/b]: " + _failureCDataLog));
		}

		traceln(LOG_PRIORITY_INFO, "Searching for testcase's error...");
		String _errorType     = "";
		String _errorMessage  = "";
		String _errorCDataLog = "";
		Node testcaseErrorLogNode = CXmlDocumentFactory.getNodeFromXPath(xmlOutputDocument, "//testcase[@name='" + testName + "']/error"); // present only when there is an error
		if (testcaseErrorLogNode != null) {
			_errorType     = testcaseErrorLogNode.getAttributes().getNamedItem("type").getNodeValue();
			_errorMessage  = testcaseErrorLogNode.getAttributes().getNamedItem("message").getNodeValue();
			_errorCDataLog = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseErrorLogNode);
			traceln(LOG_PRIORITY_INFO, ">>           error type=" + _errorType + " message=" + _errorMessage);
			traceln(LOG_PRIORITY_INFO, ">>           error=" + _errorCDataLog);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "[b]type[/b]: " + _errorType));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]message[/b]: " + _errorMessage));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b]error[/b]: " + _errorCDataLog));
		}

		return new CReturnStatus((testcaseFailureLogNode != null || testcaseErrorLogNode != null) ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
	}
	
	private CReturnStatus parseTestNGResultFile(String testName, String testcaseName, Vector<CExecutionStep> executionSteps) {
		traceln(LOG_PRIORITY_INFO, "----------------------- TestNG -----------------------------");
		// parse the result file to get the result and the execution steps
		if (!outputFile.exists() || xmlOutputDocument == null) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		//System.err.println("FILE=" + CFileUtils.FileToString(xmlOutputFile));
		//System.err.println("DOC=" + CXmlDocumentFactory.nodeToString(xmlOutputDocument));
		Node testcaseNode = CXmlDocumentFactory.getNodeFromXPath(xmlOutputDocument, "//test[@name='" + testName + "']//test-method[@name='" + testcaseName + "']");
		if (testcaseNode == null) {
			testcaseNode = CXmlDocumentFactory.getNodeFromXPath(xmlOutputDocument, "//test-method[@name='" + testName + "']");
			if (testcaseNode == null) {
				return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
			}
		}
		
		partedXmlOutputFile = new File(workingDir.getAbsolutePath() + "/parted_" + outputFilePath);
		CFileUtils.deleteAndTouchFile(partedXmlOutputFile);

		// store the part of the XML file that corresponds to this testcase as an attachment
		String partedXmlOutput = CXmlDocumentFactory.nodeToString(testcaseNode);
		CFileWriter.write(partedXmlOutputFile, partedXmlOutput);
		addTestcaseMessageAttachment(partedXmlOutputFile);

		// parse the log
    	String status    = testcaseNode.getAttributes().getNamedItem("status").getNodeValue();
    	String time      = testcaseNode.getAttributes().getNamedItem("started-at").getNodeValue();
    	String name      = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
    	String duration  = testcaseNode.getAttributes().getNamedItem("duration-ms").getNodeValue();
    	String exception = CXmlDocumentFactory.getNodeStringValueFromXPath(testcaseNode, "exception/@class");
    	Node messageNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "exception/message");
    	String message   = CXmlDocumentFactory.getNodeTextOrCDataContent(messageNode);
    	
    	short testcaseResult = RESULT_FAILURE;
    	
    	if (status.equalsIgnoreCase("PASS")) {
    		testcaseResult = RESULT_SUCCESS;
        	executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + status + "] " + name + " (" + duration + "ms)"));

    	} else if (status.equalsIgnoreCase("FAIL")) {
    		testcaseResult = RESULT_FAILURE;
        	executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + status + "] " + name + ": " + exception + " " + message + " (" + duration + "ms)"));

    	} else {
    		testcaseResult = RESULT_NOT_EXECUTED;
    		executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + status + "] " + name + " (" + duration + "ms)"));
    	}

		return new CReturnStatus(testcaseResult, executionSteps);
	}
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		System.err.println("- - - - -  start");
		Vector<CExecutionStep> executionSteps = null;
		CReturnStatus returnStatus = null;
		
		CLauncherImpl toto = new CLauncherImpl();
		toto.workingDir = new File("Y:/XStudio/src/com/xqual/xlauncher/gradle");
		
		//toto.outputFilePath = "Xunit.xml";
		//toto.outputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/gradle/Xunit.xml");

		//toto.outputFilePath = "nose.xml";
		//toto.outputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/gradle/nose.xml");
		
		toto.outputFilePath = "nose2.xml";
		toto.outputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/gradle/nose2.xml");
		toto.xmlOutputDocument = CXmlDocumentFactory.createXMLDoc(toto.outputFile);

		
		System.err.println("Parsing for test_createSeriesRemoveFolder...");
		executionSteps = new Vector<CExecutionStep>();
		returnStatus = toto.parseNoseResultFile("test_createSeriesRemoveFolder", executionSteps); // success
		System.err.println("RESULT = " + CUtils.resultToUserString(returnStatus.getResult()));
		for (int i=0; i<executionSteps.size(); i++) {
			System.err.println(CUtils.resultToString(executionSteps.get(i).getResult()) + " " + executionSteps.get(i).getMessage());
		}

		System.err.println("Parsing for test_createSeriesStringsForRanges...");
		executionSteps = new Vector<CExecutionStep>();
		returnStatus = toto.parseNoseResultFile("test_createSeriesStringsForRanges", executionSteps); // failure
		System.err.println("RESULT = " + CUtils.resultToUserString(returnStatus.getResult()));
		for (int i=0; i<executionSteps.size(); i++) {
			System.err.println(CUtils.resultToString(executionSteps.get(i).getResult()) + " " + executionSteps.get(i).getMessage());
		}
		
		
		
		
		
	}

}
