/*
+----------------------------------------------------------------------+
|                    Class: CLauncherImpl                              |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     0.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.jar;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;


/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Jar packages.
 * @author Eric Gavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

    // +==============================================================+
    // | Attributes                                                   |
    // +==============================================================+

    static final String TRACE_HEADER = "{jar           }  ";

    static final String JAVA_INTERPRETER_EXE         = CUtils.getExecutableName("java");
    
    private String jarInterpreter;
    
    private String javaInstallPath;
    private String testRootPath;
    private String generatedFilePath;

    private String jarOptions = "";
    
    private File jarTraceFile;
    private File generatedFile;
    
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

    public CLauncherImpl() {
            super(TRACE_HEADER);
    }

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
    
    @Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
        setSutDetails(sutId, sutName, sutVersion);
        
		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager        
		printConfiguration();
        
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        try {
        	// retrieve the parameters we need
			javaInstallPath          = getStringParamValue("General",  "Java install path");    // i.e. C:/Program Files/Java/jdk1.6.0_17
        	testRootPath        	 = getStringParamValue("General",  "Test root path");       // i.e. C:/test_repository/tests/jar
        	
        	generatedFilePath        = getStringParamValue("Jar",  "Generated file path");      // i.e. folder1/generated_file.txt

			jarInterpreter = CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE);
			
        } catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
            return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
        }
        
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		for (CTestAttribute attribute : attributes) {
			if (attribute.getName().equalsIgnoreCase("jar_options")) {
				jarOptions = attribute.getStringValues().get(0);
			}
		}
        return new CReturnStatus(RESULT_SUCCESS);
    }


    @Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
        traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " Test:" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		File workingDir = new File(testRootPath); // parent folder of all the test classes

		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		jarTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "jar_traces.txt");
		CFileUtils.deleteAndTouchFile(jarTraceFile);

		// stored in the same folder as the jar
		generatedFile = new File(workingDir.getAbsolutePath() + "/" + generatedFilePath);
		if (generatedFile.exists()) generatedFile.delete();
		try {
			generatedFile.createNewFile();
		} catch (IOException e) {
			traceln(LOG_PRIORITY_SEVERE, "Could not create file " + generatedFile + "...");
		}

		// +------------------------------------+
		// | Run the JUnit test class
		// +------------------------------------+

        CRunner jarRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                        jarInterpreter + " " +
                                        "-jar " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "_" + testcaseName + ".jar") + " " +
                                        jarOptions,
                                        workingDir);
        
        short result = jarRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, jarTraceFile);
        
		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
        if (jarTraceFile.length() > 0) addTestcaseMessageAttachment(jarTraceFile);
        if (generatedFile.length() > 0) addTestcaseMessageAttachment(generatedFile);

		// return code of org.junit.runner.JUnitCore: if all tests run successfully, exit with a status of 0. Otherwise exit with a status of 1.
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "JAR execution process failed"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "JAR execution process succeeded"));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
    }

    @Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
        return new CReturnStatus(RESULT_SUCCESS);
    }

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		// we should search for some *_*.jar then extract the left and right part of the '_'
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2ejar$",          // only *.jar files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	
	
}
