/*
+----------------------------------------------------------------------+
|                    Class: CLauncherImpl                              |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.3                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.jmeter;

import java.awt.Color;
import java.awt.Image;
import java.awt.geom.Ellipse2D;
import java.awt.image.BufferedImage;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Arrays;
import java.util.Calendar;
import java.util.Vector;

import javax.imageio.ImageIO;
import javax.swing.JOptionPane;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.annotations.XYAnnotation;
import org.jfree.chart.annotations.XYTextAnnotation;
import org.jfree.chart.axis.AxisLocation;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.plot.PlotOrientation;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.StandardXYItemRenderer;
import org.jfree.chart.renderer.xy.XYItemRenderer;
import org.jfree.chart.renderer.xy.XYLineAndShapeRenderer;
import org.jfree.data.xy.XYSeries;
import org.jfree.data.xy.XYSeriesCollection;
import org.jfree.ui.Align;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;




/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for JMeter.
 * @author Eric Gavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

    // +==============================================================+
    // | Attributes                                                   |
    // +==============================================================+

    static final String TRACE_HEADER = "{jmeter        }  ";

    static final String JAVA_INTERPRETER_EXE         = CUtils.getExecutableName("java");
    static final String JMETER_SERVER_LOG_NAME       = "jmeter-server.log";
    static final String JMETER_SERVER_ARGUMENTS      = "-XX:+HeapDumpOnOutOfMemoryError -Xms512m -Xmx512m -XX:NewSize=128m -XX:MaxNewSize=128m " +
    		                                           "-XX:SurvivorRatio=8 -XX:TargetSurvivorRatio=50 -XX:MaxTenuringThreshold=2 " + //-XX:MaxLiveObjectEvacuationRatio=20 " +
    		                                           //"-Dsun.rmi.dgc.client.gcInterval=600000 -Dsun.rmi.dgc.server.gcInterval=600000 " +
    		                                           "-XX:PermSize=64m -XX:MaxPermSize=64m " +
    		                                           "-jar \"ApacheJMeter.jar\"" + 
													  " -s -j "+JMETER_SERVER_LOG_NAME;
																	
    static final String JMETER_CLIENT_NAME           = CUtils.getBatchName("jmeter"); // jmeter.bat or jmeter.sh
    
    private String jMeterServerInterpreter;
    
    private String javaInstallPath;
    private String testRootPath;
	private String conf1Path;
    private String conf2Path;

    File jmxFile;
    File jtlFile;
    File logFile;
    
	File confFile1;
	File confFile2;
	
	String confFile1ServerArg;
	String confFile2ServerArg;

	String confFile1ClientArg;
	String confFile2ClientArg;
	
    private CRunner jmeterServer = null;
    private File jmeterServerTraceFile;
	private int timeoutRequest;
    private int timeoutConnection;
    
    private Float acceptableFailureRate;
    private int timeToReachMaxLoad;
    private int nbSimulatedUsers;
    private int nbRequestsPerUser;
    private int nbValuesToComputeMaxTime;

    private Vector<Vector<String>> httpRequestVector;

    public String uniqueIdentifier;
    
    private File workingDir;
    private File serverLogFile;
    private File execFile;
    private File execDir;
    
    private float average             = 0;
    private float maximum             = 0;
    private float measuredFailureRate = 0;
	
	

    //private Vector<String> additionStatusExpected;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

    public CLauncherImpl() {
            super(TRACE_HEADER);
    }

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
    
    @Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
        setSutDetails(sutId, sutName, sutVersion);
        
		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager        
		printConfiguration();
        
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        try {
        	// retrieve the parameters we need
			javaInstallPath          = getStringParamValue("General",  "Java install path");    // i.e. C:/Program Files/Java/jdk1.6.0_17
        	testRootPath        	 = getStringParamValue("General",  "Test root path");       // i.e. C:/test_repository/tests/jmeter
        	conf1Path                = getStringParamValue("JMeter",   "Configuration file 1");  
            conf2Path                = getStringParamValue("JMeter",   "Configuration file 2");
			
			execDir                  = new File(getStringParamValue("JMeter",  "JMeter install directory") + "/bin");
            serverLogFile            = new File(getStringParamValue("JMeter",  "JMeter install directory") + "/bin/" + JMETER_SERVER_LOG_NAME);
            execFile                 = new File(getStringParamValue("JMeter",  "JMeter install directory") + "/bin/" + JMETER_CLIENT_NAME);
            
			confFile1 = (conf1Path==null || conf1Path.trim().length()==0) ? null : new File(testRootPath + "/" + conf1Path) ;
			confFile2 = (conf2Path==null || conf2Path.trim().length()==0) ? null : new File(testRootPath + "/" + conf2Path );
			
			confFile1ServerArg = (confFile1 != null && confFile1.exists()) ? "-S " + CFileUtils.quoteFilePath(confFile1.getPath()) + " " : " ";
			confFile2ServerArg = (confFile2 != null && confFile2.exists()) ? "-S " + CFileUtils.quoteFilePath(confFile2.getPath()) + " " : " ";
			
			confFile1ClientArg = (confFile1 != null && confFile1.exists()) ? "-q " + CFileUtils.quoteFilePath(confFile1.getPath()) + " " : " ";
			confFile2ClientArg = (confFile2 != null && confFile2.exists()) ? "-q " + CFileUtils.quoteFilePath(confFile2.getPath()) + " " : " ";
			
			jMeterServerInterpreter = CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE) + " " + JMETER_SERVER_ARGUMENTS + " " +
					                                           "-Duser.dir=" + testRootPath + " " + confFile1ServerArg + confFile2ServerArg;;
        } catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
            return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
        }
        
    
		// +------------------------------------+
		// | Run the JMeter server
		// +------------------------------------+
		jmeterServerTraceFile = new File("jmeter_server_traces.txt");
		if (jmeterServerTraceFile.exists()) jmeterServerTraceFile.delete();
		try {
			jmeterServerTraceFile.createNewFile();
		} catch (IOException e) {
			traceln(LOG_PRIORITY_SEVERE, "Could not create file " + jmeterServerTraceFile);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Could not create file " + jmeterServerTraceFile));
            return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
		}

		if (serverLogFile.exists()) serverLogFile.delete();
		jmeterServer = new CRunner("[JMeter Server] ",
		                           jMeterServerInterpreter,
		                           execDir);

		short result = jmeterServer.requestAction(IRunner.START_PROCESS, IRunner.DO_NOT_WAIT_END_OF_EXECUTION, jmeterServerTraceFile);
		if (result == RESULT_SUCCESS) {
			traceln(LOG_PRIORITY_INFO, "jmeter server launched successfully");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "initialize: jmeter server successfully launched!"));
		} else {
			traceln(LOG_PRIORITY_SEVERE, "couldn't run jmeter server!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "initialize: couldn't run jmeter server! Maybe a server is already running"));
			return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
		}

		// let the time to the server to initialize (this is done only once per campaign session)
		CUtils.sleep(3000);
        
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");

		uniqueIdentifier = testName + "_" + testId;
		
		printAttributes(attributes);
		
		// reset it for each test
		/*
		additionStatusExpected = new Vector<String>();
		if (attributes != null) {
			for (int i=0; i<attributes.size(); i++) {
				CTestAttribute attribute  = attributes.get(i);
				String attributeName  = attribute.getName();
				short attributeType   = attribute.getType();
				if (attributeType == IType.TYPE_STRING && attributeName.equalsIgnoreCase("JMeter list of additional HTTP status codes authorized")) {
					additionStatusExpected = attribute.getStringValues();
					traceln(LOG_PRIORITY_INFO, "additional HTTP status codes authorized: " + additionStatusExpected + "...");
				}
			}
		}
		*/
		
        // clean all the files related to this test in the working directory 
        if (execDir.isDirectory()) {
        	File[] files = this.execDir.listFiles();
            for (int i=0; i<files.length; i++) {
                if(files[i].getName().contains(testName)) {
            		traceln(LOG_PRIORITY_INFO, "deleting file " + files[i] + "...");
                    files[i].delete();
                }
            }
        }
        
        jmxFile = new File(testRootPath + "/" + testPath + "/" + testName + ".jmx"); // the test
        if (!jmxFile.exists()) traceln(LOG_PRIORITY_SEVERE, ".jmx file " + jmxFile + " does not exist!");

        jtlFile = new File(testRootPath + "/" + testPath + "/" + testName + ".jtl"); // the XML results file

        logFile = new File(testRootPath + "/" + testPath + "/" + testName + ".log"); // the log

        return new CReturnStatus(RESULT_SUCCESS);
    }


    @Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
        traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " Test:" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");

        // best effort
		if (serverLogFile.exists()) serverLogFile.delete();

        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

        if (execFile.exists()) {
            // search for the test script in <testRootPath>/<testPath>/<testName>.jmx
            CRunner exeRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                            execFile + " " +
                                            "-n -t " + CFileUtils.quoteFilePath(jmxFile.getPath()) + " " +
                                            "-j " + CFileUtils.quoteFilePath(logFile.getPath()) + " " +
                                            "-l " + CFileUtils.quoteFilePath(jtlFile.getPath()) + " " +
                                            "-r " + 
                                            confFile1ClientArg + confFile2ClientArg,
                                            execDir);
            
            exeRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
            CUtils.sleep(500);
            if (!jtlFile.exists()) {
            	jtlFile = new File(testRootPath + "/" + testPath + "/" + testName + ".xml");
            	if (!jtlFile.exists()) {
                	jtlFile = new File(testRootPath + "/" + testPath + "/" + testName + ".csv"); // in rare cases, we saw some JMeter configuration that generate .csv containing XML!
                	if (!jtlFile.exists()) {
                		return new CReturnStatus(RESULT_FAILURE, executionSteps);
                	}
            	}
            }

			try {
	            short resultParsing = parseResults(executionSteps);
				if (resultParsing==RESULT_FAILURE) {
	                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Execution failed"));
	                return new CReturnStatus(RESULT_FAILURE, executionSteps);
	            } else {
	                return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	            }
			} catch (FileNotFoundException e) {
                traceln(LOG_PRIORITY_SEVERE, "couldn't find the result file");
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Couldn't find the result file"));
                return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
            } catch (IOException e) {
                traceln(LOG_PRIORITY_SEVERE, "couldn't read the result file");
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Couldn't read the result file"));
                return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
            }
			
        } else {
            traceln(LOG_PRIORITY_SEVERE, "Could not find JMeter binary. Please, check it is correctly installed in " + execDir.toString());
            executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Could not find JMeter binary. Please, check it is correctly installed in " + execDir.toString()));
            return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
        }

    }

    @Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        
        // cleanup of generated files
        if (jtlFile != null && jtlFile.exists()) CFileUtils.deleteFile(jtlFile);
        if (logFile != null && logFile.exists()) CFileUtils.deleteFile(logFile);
        
        return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

	@Override
	public CReturnStatus terminate() {
		traceln(LOG_PRIORITY_INFO, "terminate killing jmeter server...");
		// +------------------------------------+
		// | kill the JMeter server
		// +------------------------------------+
		jmeterServer.killProcess();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}



	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eqtp$",          // only *.qtp files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	
	


    public short parseResults(Vector<CExecutionStep> executionSteps) throws FileNotFoundException, IOException {
        String currentLine, status, time;
        int index=0, nbFailures=0, sum=0, max=0;
        
        Vector<Float> maxTimeVector  = new Vector<Float>();
        Vector<Number> timeVector    = new Vector<Number>();
        Vector<Number> averageVector = new Vector<Number>();

        if (jtlFile.exists()){ 
			traceln(LOG_PRIORITY_INFO, "Result file " + jtlFile + " opened");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Result file Found: " + jtlFile));
			addTestcaseMessageAttachment(jtlFile);
			addTestcaseMessageAttachment(logFile);
		} else{
			traceln(LOG_PRIORITY_INFO, "Result file " + jtlFile + " not opened !!!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Result filenot Found!!!!!: " + jtlFile));
			return RESULT_FAILURE;
		}
        
		try {
            FileReader fileReader         = new FileReader(jtlFile);
            BufferedReader bufferedReader = new BufferedReader(fileReader);
            
            while((currentLine = bufferedReader.readLine()) != null) {
                currentLine = new String(currentLine.getBytes(), "UTF-8");
                
                if(currentLine.contains("<httpSample") || currentLine.contains("<sample")) { // if a <httpSample> tag is found, it's a result line
                    index++;

                    // +-------------------+
                	// | status: rm        |
                	// +-------------------+
                    status = currentLine.replaceAll("^.*?rm=\"(.*?)\" .*?$", "$1"); // remove what's surrounding the "rm" attribute
                    if (!status.equals("OK")) nbFailures++;
                    
                	// +-------------------+
                	// | execution time: t |
                	// +-------------------+
                    time = currentLine.replaceAll("^.*?t=\"(.*?)\" .*?$", "$1"); // remove what's surrounding the "t" attribute
                    if (Double.parseDouble(time) > max) maxTimeVector.add(Float.parseFloat(time));
                    timeVector.add(Double.parseDouble(time));
                    sum += Integer.parseInt(time);
                    averageVector.add(((float)sum/(float)index));
                }
            }
            
            int nbTimes = timeVector.size();
            float[] maxArray = new float[nbTimes];
            for (int i=0; i<timeVector.size(); i++) {
                maxArray[i] = timeVector.get(i).floatValue();
            }
            Arrays.sort(maxArray);
            
            float maxi = 0;
            nbTimes = maxArray.length > nbValuesToComputeMaxTime ? nbValuesToComputeMaxTime : maxArray.length;
            
            for (int i=maxArray.length-1; i>(maxArray.length-1-nbTimes); i--) {
                maxi += maxArray[i];
            }
            maxi    = maxi/nbTimes;
            maximum = maxi;
            traceln(LOG_PRIORITY_INFO, "Maximum = " + maximum);
            
            bufferedReader.close();
            fileReader.close();
            
            // +--------------------------------------------+
            // |  generate the chart from these statistics  |
            // +--------------------------------------------+
            XYSeries timeSeries    = new XYSeries("Execution time per request (1 dot = 1 request)");
            XYSeries averageSeries = new XYSeries("Trend of the average execution time of 1 request");
            for (int i=0; i<timeVector.size(); i++) {
                timeSeries.add(i, timeVector.get(i));
                averageSeries.add(i, averageVector.get(i));
            }

            XYSeriesCollection dataset = new XYSeriesCollection();
            dataset.addSeries(averageSeries);
            dataset.addSeries(timeSeries);
            
            NumberAxis rangeAxis1 = new NumberAxis("Time (ms)");
            NumberAxis rangeAxis2 = new NumberAxis("Requests (#)");
            
            XYItemRenderer renderer1 = new StandardXYItemRenderer();

            XYPlot xyplot = new XYPlot(dataset, rangeAxis2, rangeAxis1, renderer1);
            xyplot.setRangeAxisLocation(AxisLocation.BOTTOM_OR_LEFT);

            // create the chart
            JFreeChart chart = ChartFactory.createXYLineChart("Trend of the duration and averages for the test " + uniqueIdentifier, "Requests #", "Time (ms)", dataset, PlotOrientation.VERTICAL, true, true, true);
			
			chart.setBackgroundPaint(Color.white);
            XYPlot plot = (XYPlot) chart.getPlot();
            XYLineAndShapeRenderer renderer = new XYLineAndShapeRenderer();
            
            // average: we draw red lines
            renderer.setSeriesLinesVisible(0, true);
            renderer.setSeriesShapesVisible(0, false);
            renderer.setSeriesPaint(0, Color.red);

            // times:   we draw blue dots (1px ellipse)
            renderer.setSeriesLinesVisible(1, false);
            renderer.setSeriesShape(1, new Ellipse2D.Float(0, 0, 1, 2));
            renderer.setSeriesShapesVisible(1, true);
            renderer.setSeriesPaint(1, Color.blue);
            
            // annotation for the average
            String averageStr = averageVector.lastElement().toString();
            if(!averageStr.contains(".")) averageStr += ".";
            XYAnnotation annotation = new XYTextAnnotation("Average = " + Integer.parseInt(averageStr.substring(0, averageStr.indexOf("."))) + "ms", index*0.9, Float.parseFloat(averageStr)*1.1);
            plot.addAnnotation(annotation);
            
            // annotation for the maximum
            annotation = new XYTextAnnotation("Max (" + nbValuesToComputeMaxTime + " biggest values) = " + maxi + "ms", index*0.9, maxi*1.02);
            plot.addAnnotation(annotation);
            
            plot.setBackgroundAlpha(new Float(0.2));
            try {
                Image image = ImageIO.read(new File("export/resources/images/thumbnail_jmeter.gif"));
                chart.setBackgroundImage(image);
                chart.setBackgroundImageAlpha(new Float(0.8));
                chart.setBackgroundImageAlignment(Align.TOP_LEFT);
            } catch (Exception e) {}
            
            
            plot.setRenderer(renderer);
            BufferedImage bufferedImage = chart.createBufferedImage(1024, 768);
            Calendar calendar = CCalendarUtils.getTimezonedInstance();
            String filename = uniqueIdentifier + "_" + CCalendarUtils.calendarToStringForFileName(calendar) +".png";
			//String filename = CCalendarUtils.calendarToStringForFileName(calendar) +".png";
            File imageFile = null;
            try {
                imageFile = new File(testRootPath + "/" + filename);
                ImageIO.write(bufferedImage, "png", imageFile);
                addTestcaseMessageAttachment(imageFile);
                addTestcaseMessageAttachment(serverLogFile);
				/*
                addTestcaseMessageAttachment(new File(workingDir.getPath() + "/" + uniqueIdentifier + ".jmx"));
                addTestcaseMessageAttachment(new File(workingDir.getPath() + "/" + uniqueIdentifier + ".jtl"));
                addTestcaseMessageAttachment(new File(workingDir.getPath() + "/" + uniqueIdentifier + ".log"));
				*/
            } catch (Exception e) {
                traceln(LOG_PRIORITY_SEVERE, "Couldn't save " + imageFile);
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Couldn't save " + imageFile));
            }
        } catch (FileNotFoundException e) {
            traceln(LOG_PRIORITY_SEVERE, "Couldn't find " + jtlFile);
            executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Couldn't find " + jtlFile));
        }
        
        // if we found at least once the tag <httpSample> we return the average time else 0
        if (index>0)
            average = ((float)sum/(float)index);
        else
            average = (new Float(0));

        measuredFailureRate = ((((float)nbFailures*100)/(this.nbRequestsPerUser*this.nbSimulatedUsers)));
		return RESULT_SUCCESS;
    }
	
    
    
    
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
    public static void main(String[] args) {
    	CLauncherImpl toto = new CLauncherImpl();
    	//toto.jtlFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\jmeter\\test3.jtl");
    	toto.jtlFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\jmeter\\test3b.jtl");
    	toto.testRootPath = "Y:\\XStudio\\src\\com\\xqual\\xlauncher\\jmeter\\";
    	toto.uniqueIdentifier = "graph";
    	
    	Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
    	try {
			toto.parseResults(executionSteps);
		} catch (Exception e) {
			e.printStackTrace();
		}
    }
    
}
