/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.0                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.katalon_v2;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;


/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for katalon.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{katalon       }  ";

	static final String XSTUDIO_OUTPUT_FOLDER = "xstudio_output";
	
	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String projectRelativePath;
	private String windowsCommand;

	private String katalonEngine;
	private String katalonPropertiesFile;
	private String katalonExecutionProfile;
	private String katalonApiKey;
	private String katalonBrowserType;
	private int katalonRetries = 0;
	private boolean katalonOnlyRetryOnFailed;

	private String katalonInterpreter;

	private File workingDir;

	private String overridenProjectRelativePath;
	
	File katalonHtmlReportFile;
	File katalonCsvReportFile;
	File katalonXmlJUnitReportFile;
	File katalonJsonReportFile;
	File katalonExecution0logFile;
	File katalonExecutionPropertiesFile;

	File katalonTraceFile;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) {
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");

		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need from the configuration
			testRootPath        = getStringParamValue("General", "Test root path");
			projectRelativePath = getStringParamValue("General", "Project relative path");
			windowsCommand      = getStringParamValue("General", "Windows command");

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// optional parameter
		try {
			katalonApiKey = getStringParamValue("katalon", "API key (opt.)");
		} catch (CParamParsingException e) {
		}

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// if an attribute is named "com.xqual.katalon.projectRelativePath" then it can overwrite the global "project relative path" configuration parameter
		overridenProjectRelativePath = null;
        for (int k=0; k<attributes.size(); k++) {
        	CTestAttribute attribute = attributes.get(k);
            String attributeName     = attribute.getName();
            String attributeValue    = attribute.getStringValue();
            
            if (attributeName.equalsIgnoreCase("com.xqual.katalon.projectRelativePath")) {
            	overridenProjectRelativePath = attributeValue;
            	traceln(LOG_PRIORITY_INFO, "Attribute overriding the project relative path found ! updated project path = " + overridenProjectRelativePath);
            } else {
            	traceln(LOG_PRIORITY_INFO, "Attribute " + attributeName + " not relevant here");
            }
        }

		try {
			katalonEngine            = getStringParamValue("katalon", "Katalon engine path");
			katalonPropertiesFile    = getStringParamValue("katalon", "Properties File");
			katalonExecutionProfile  = getStringParamValue("katalon", "Execution profile");
			katalonBrowserType       = getStringParamValue("katalon", "Browser Type");
			katalonRetries           = getIntegerParamValue("katalon", "Number of retries");
			katalonOnlyRetryOnFailed = getBooleanParamValue("katalon", "Only Retry failed Test Cases");

			//create the default interpreter for all tests
			//katalonInterpreter = katalonEngine + "katalon" + (katalonPropertiesFile.length() > 0 ? " -propertiesFile=" + testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "/" + katalonPropertiesFile : "") + " -noSplash " + " -runMode=console " + " -consoleLog " + " -summaryReport ";
			katalonInterpreter = katalonEngine + "katalon" + 
					(katalonPropertiesFile.length() > 0 ? " -propertiesFile=" + testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "/" + katalonPropertiesFile : "") + // e.g. "<testRootPath>/<testPath>/<TestName>/<katalonPropertiesFile>"
					" -noSplash " + 
					" -runMode=console " + 
					" -summaryReport ";

			katalonInterpreter = katalonInterpreter + " -projectPath=" + CFileUtils.quoteFilePath(testRootPath + "/" + (overridenProjectRelativePath != null ? overridenProjectRelativePath : projectRelativePath)); // e.g. "<testRootPath>/<projectRelativePath>"
			katalonInterpreter = katalonInterpreter + " -testSuitePath=" + CFileUtils.quoteFilePath((testPath.length()>0 ? testPath + "/" : "") + testName);  // e.g. "<testPath>/<TestName>"

			katalonInterpreter = katalonInterpreter + (katalonExecutionProfile.length() > 0 ? " -executionProfile=" + katalonExecutionProfile : " -executionProfile=" + CFileUtils.quoteFilePath("default"));
			katalonInterpreter = katalonInterpreter + (katalonBrowserType.length() > 0 ? " -browserType=" + CFileUtils.quoteFilePath(katalonBrowserType) : " -browserType=" + CFileUtils.quoteFilePath("Chrome (headless)")); // this is an arbitrary choice - could be invalide for some customers

			katalonInterpreter = katalonInterpreter + (katalonRetries > 0 ? " -retry=" + katalonRetries : " -retry=0" );
			katalonInterpreter = katalonInterpreter + (katalonRetries > 0 ? (katalonOnlyRetryOnFailed == true ? " -retryFailedTestCases=true" : " ") : " " ); // by default we expect that only failed test case be retried

			katalonInterpreter = katalonInterpreter + " -reportFolder=" + XSTUDIO_OUTPUT_FOLDER; // we force the value to a well known so that we make it easier for configuration and to work on
			katalonInterpreter = katalonInterpreter + " -reportFileName=XStudio"; // we force the value to a well known so that we make it easier for configuration and to work on

			if (katalonApiKey != null && katalonApiKey.trim().length() > 0) {
				katalonInterpreter = katalonInterpreter + " -apiKey=" + katalonApiKey.trim();
			}
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		workingDir = new File(katalonEngine); // parent folder of all the test classes - in katalon case this is where the katalon engine is ... different from most other launchers

		// we allow to set some specific parameters for the tests that are not dealing solely with web apps - can be Android or IOS
		StringBuffer paramsArgument = new StringBuffer();
        for (int k=0; k<params.size(); k++) {
            CTestcaseParam param = params.get(k);
            String paramName     = param.getName();
            String paramValue    = param.getStringValue();
									if (paramsArgument.length()>0) {
										paramsArgument.append(",");
									}
            paramsArgument.append("-" + paramName + "=" + "\"" + paramValue + "\"");
        }

		traceln(LOG_PRIORITY_INFO, paramsArgument.toString());
		
		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		
		File projectFile = new File(testRootPath + "/" + (overridenProjectRelativePath != null ? overridenProjectRelativePath : projectRelativePath));
		File projectFolder = projectFile.getParentFile();
		traceln(LOG_PRIORITY_INFO, "Project folder: " + projectFolder.getAbsolutePath());
		
		katalonTraceFile = new File(projectFolder.getAbsolutePath() + "/" + XSTUDIO_OUTPUT_FOLDER + "/" + testName +".log"); // the output of the console
		CFileUtils.deleteAndTouchFile(katalonTraceFile);

		katalonHtmlReportFile = new File(projectFolder.getAbsolutePath() + "/" + XSTUDIO_OUTPUT_FOLDER + "/" + "xstudio.html"); // katalon HMTL report
		CFileUtils.deleteAndTouchFile(katalonHtmlReportFile);

		katalonCsvReportFile = new File(projectFolder.getAbsolutePath() + "/" + XSTUDIO_OUTPUT_FOLDER + "/" + "xstudio.csv"); // katalon CSV report
		CFileUtils.deleteAndTouchFile(katalonCsvReportFile);

		katalonXmlJUnitReportFile =  new File(projectFolder.getAbsolutePath() + "/" + XSTUDIO_OUTPUT_FOLDER + "/" + "JUnit_Report.xml"); // katalon Junit
		CFileUtils.deleteAndTouchFile(katalonXmlJUnitReportFile);

		katalonJsonReportFile =  new File(projectFolder.getAbsolutePath() + "/" + XSTUDIO_OUTPUT_FOLDER + "/" + "JSON_Report.json"); // katalon json
		CFileUtils.deleteAndTouchFile(katalonJsonReportFile);

		katalonExecution0logFile =  new File(projectFolder.getAbsolutePath() + "/" + XSTUDIO_OUTPUT_FOLDER + "/" + "execution0.log"); // katalon execution
		CFileUtils.deleteAndTouchFile(katalonExecution0logFile);

		katalonExecutionPropertiesFile =   new File(projectFolder.getAbsolutePath() + "/" + XSTUDIO_OUTPUT_FOLDER + "/" + "execution.properties"); // katalon properties file
		CFileUtils.deleteAndTouchFile(katalonExecutionPropertiesFile);

		// +------------------------------------+
		// | Run the katalon test class
		// +------------------------------------+

		katalonInterpreter = windowsCommand + " /C " + CFileUtils.quoteFilePath(katalonInterpreter);

		CRunner katalonRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName, 
										    katalonInterpreter.toString() +
                                            (paramsArgument.length()>0 ? " " + paramsArgument.toString() : ""),
                                            workingDir);

		// here we redirect the output to a file for later parsing
		short result = katalonRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, katalonTraceFile);

		// return code of katalon runner: if all tests run successfully, exit with a status of 0. Otherwise exit with a status of 1.
		//rapide user guide 5.4  section 2.3.2.1
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "XStudio Test case failed"));
			parseResultFile(executionSteps); // to have the details anyway
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "XStudio Test case succeeded"));
			return parseResultFile(executionSteps);
		}
	}


	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+

	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}

	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // only folderwith name 'Test Suites'
							  "^.*\\x2ets$",           // only *.ts files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector < CExecutionStep > executionSteps) {
		// parse the result file to get the result and the execution steps

		boolean errorDetected = true;
		String line = "";

		int errorCount = 0;
		int testcaseTried = 0;
		int testcaseFailures = 0;

		// be default we expect that the session will be a success
		boolean succeededOnSLA = true;

		// we check that the result file is available
		if (!katalonXmlJUnitReportFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Junit file from Katalon test suite execution not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Junit file from Katalon test suite execution not found!"));
			if (katalonTraceFile.exists()) {
				addTestcaseMessageAttachment(katalonTraceFile);
			}
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Found the Junit file from Katalon test suite execution : " + katalonXmlJUnitReportFile.toString()));

			addTestcaseMessageAttachment(katalonXmlJUnitReportFile);
			addTestcaseMessageAttachment(katalonHtmlReportFile);
			addTestcaseMessageAttachment(katalonCsvReportFile);
			addTestcaseMessageAttachment(katalonJsonReportFile);
			addTestcaseMessageAttachment(katalonExecution0logFile);
			addTestcaseMessageAttachment(katalonExecutionPropertiesFile);
			addTestcaseMessageAttachment(katalonTraceFile);
		}
		// then parse it

		//  we search something like <testsuite errors="0" time="0.001" tests="14" skipped="0" failures="14" name="[CS_40] ci_2018-03-16 16:05:33">))
		try {
			FileInputStream fileInputStream = new FileInputStream(katalonXmlJUnitReportFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();

				//We look for a pattern of <testsuite as showed below
				// <?xml version="1.0" encoding="UTF-8"?><testsuite errors="0" time="0.001" tests="14" skipped="0" failures="14" name="[CS_40] ci_2018-03-16 16:05:33">
				if (line.indexOf("<testsuites") > -1) {
					System.out.println("Found the summary line " + line);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Summary line is : " + line));
					errorDetected = false;

					//Then if found we extract and cast to int the number of errors, triies, failed and skipped testcases
					errorCount = extractCountsFromResultLine(line, "errors=", "failures=");
					System.out.println("Katalon Test case errors --> " + errorCount);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case errors --> " + errorCount));

					testcaseTried = extractCountsFromResultLine(line, "tests=", "time=");
					System.out.println("Katalon Test case tried  --> " + testcaseTried);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case tried  --> " + testcaseTried));

					testcaseFailures = extractCountsFromResultLine(line, "failures=", "tests=");
					System.out.println("KatalonTest case failed  --> " + testcaseFailures);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case failed  --> " + testcaseFailures));
				}

				if (line.indexOf("<testcase name=") > -1) {
					System.out.println("Found the katalon test case line " + line);
					if (line.indexOf("status=\"PASSED\"") < 0) {
						System.out.println("Katalon testcase failed : " + line);
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Katalon Test case failed : " + line));
						errorDetected = true;
					}
					else {
						System.out.println("Katalon  testcase OK : " + line);
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Katalon Test case succeeded : " + line));
					}
				}

			}

			// the below resources closures should go into Finaly but we don't care as the launcher is instanciated and killed rapidly
			bufferedReader.close();
			fileInputStream.close();

			// if we have tried some testcases
			if (testcaseTried + testcaseFailures + errorCount > 0) {
				System.out.println("SLA analyis for Katalon test cases: " + testcaseTried + " trie(s) , " + testcaseFailures +" failure(s) , " + errorCount + " error(s)" );

				// then we ensure that we have no failures
				if (testcaseFailures + errorCount > 0) {
					succeededOnSLA = false;
					System.out.println("Test failed -there were failures or errors");
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Failed : " + testcaseFailures + " , Errors :" + errorCount + " Katalon test case(s)"));
				}
				else {
					System.out.println("Test succeeded");
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "SLA analyis : "  + testcaseTried + " trie(s) , " + testcaseFailures + " failure(s) , " + errorCount + " error(s) for katalon tests cases"));
				}


			}

		} catch(Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			errorDetected = true;
		}

		// unless we met some errors and till the SLA is match then the test case (hence the test in our context) is a success
		if (!errorDetected && succeededOnSLA) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "The test was executed and all katalon test case passed "));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		} else {
			// otherwise the the test case is failed
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "The test was executed but some katalon test cases failed "));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	// utility to extract and parse an integer
	private int extractCountsFromResultLine(String line, String firstSubString, String nextSubString) {

		int indexOfAfterSearchedString = line.indexOf(firstSubString) + firstSubString.length();
			System.out.println("indexOfAfterSearchedString  --> " + indexOfAfterSearchedString);
		int indexOfFirstDoubleQuote = line.indexOf("\"", indexOfAfterSearchedString);
			System.out.println("indexOfFirstDoubleQuote  --> " + indexOfFirstDoubleQuote);
		int indexOfSecondDoubleQuote = line.indexOf("\"", indexOfFirstDoubleQuote + 1);
			System.out.println("indexOfSecondDoubleQuote  --> " + indexOfSecondDoubleQuote);

		return (Integer.parseInt(line.substring(indexOfFirstDoubleQuote +1, indexOfSecondDoubleQuote )));

	}
}
