/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.marathon;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for JUnit.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{marathon      }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String marathonHome;
	private String javaInstallPath;
	private String marathonClassPath;

	private String marathonInterpreter;

	private File workingDir;

	private static final String JAVA_INTERPRETER_EXE = CUtils.getExecutableName("java");
	private static final String MARATHON_RUNNER_CLASS = "net.sourceforge.marathon.Main";

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath        = getStringParamValue("General", "Test root path");       // i.e. Y:/XStudio/build/lib/marathon-1.2.1.1/examples

			marathonHome        = getStringParamValue("Marathon", "Marathon home");       // i.e. Y:/XStudio/build/lib/marathon-1.2.1.1
			marathonClassPath   = getStringParamValue("Marathon", "Marathon classpath");  // i.e. Y:/XStudio/build/lib/marathon-1.2.1.1/marathon.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/vldocking_2.1.1.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/rmi-lite.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/forms-1.0.7/forms-1.0.7.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/jaccess-1.3/jaccess.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/junit3.8.2/junit.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/jython-2.2/jython.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/looks-2.0.4/looks-2.0.4.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/jedit-textArea.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/jline-0.9.93.jar;Y:/XStudio/build/lib/marathon-1.2.1.1/Support/vldocking_2.1.5C.jar
			javaInstallPath     = getStringParamValue("Marathon", "Java install path");   // i.e. C:/Progra~1/Java/jdk1.6.0_17

			marathonInterpreter = CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE) + " " +
            				   "-classpath " + CFileUtils.quoteFilePath(marathonClassPath) + " " +
   			                   "-Dmarathon.home=\"" + marathonHome + ".\" " +
   			                   "-Dpython.home=\"" + marathonHome + ".\" " +
			                   MARATHON_RUNNER_CLASS + " " +
			                   "-batch " +
			                   "-xml " + CFileUtils.quoteFilePath(testRootPath + "/marathon_report.xml") + " " +
			                   testRootPath; // project folder
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		workingDir = new File(testRootPath); // parent folder of all the test classes

		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		File marathonTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "marathon_traces.txt");
		CFileUtils.deleteAndTouchFile(marathonTraceFile);

		// +------------------------------------+
		// | Run the JUnit test class
		// +------------------------------------+
		CRunner marathonRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,
                                           marathonInterpreter.toString() + " " + (testPath.length()>0 ? testPath + "." : "") + testName,
                                           workingDir);
		// here we redirect the output to a file for later parsing
		short result = marathonRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, marathonTraceFile);

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(marathonTraceFile);
		addTestcaseMessageAttachment(new File(testRootPath + "/marathon_report.xml"));

		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution process failed"));
			parseResultFile(executionSteps); // to have the details anyway
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution process succeeded"));
			return parseResultFile(executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eclass$",        // only *.class files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		short result = RESULT_FAILURE;

	    int nbSkipped = 0;
		int nbFails   = 0;
		int nbSuccess = 0;

		// parse the result file to get the result and the execution steps
		File resultFile = new File(testRootPath + "/marathon_report.xml");
		if (!resultFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		// parse the result file to get the result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(resultFile);
	    Node rootXmlNode = xmlDocument.getFirstChild();
	    NodeList resultNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(rootXmlNode, "//testcase");
	    for (int j=0; j<resultNodeList.getLength(); j++) {
	    	Node resultNode = resultNodeList.item(j);
	    	String status    = resultNode.getAttributes().getNamedItem("status").getNodeValue();
	    	String time      = resultNode.getAttributes().getNamedItem("time").getNodeValue();
	    	String name      = resultNode.getAttributes().getNamedItem("name").getNodeValue();

	    	if (status.equalsIgnoreCase("0")) {
	        	executionSteps.add(new CExecutionStep(time, RESULT_SUCCESS, name + " (" + time + ")"));
	        	nbSuccess++;

	    	} else if (status.equalsIgnoreCase("1")) {
	        	executionSteps.add(new CExecutionStep(time, RESULT_FAILURE, name + " (" + time + ")"));
	        	nbFails++;

	    	} else if (status.equalsIgnoreCase("2")) {
	        	executionSteps.add(new CExecutionStep(time, RESULT_UNKNOWN, name + " (" + time + ")"));
	        	nbSkipped++;

	    	} else {
	    		executionSteps.add(new CExecutionStep(time, RESULT_UNKNOWN, name + " (" + time + ")"));
	    	}
	    }

	    if (nbFails == 0 && nbSkipped == 0 && nbSuccess > 0) {
	    	result = RESULT_SUCCESS;

	    } else if (nbFails == 0 && nbSuccess == 0) { // whatever nbSkipped
	    	result = RESULT_NOT_EXECUTED;
	    }

		return new CReturnStatus(result, executionSteps);
	}
}
