/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
|              Rusi Dimov                                              |
| Version:     1.2                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.mocha;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.json.JSONArray;
import org.json.JSONObject;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{Mocha         }  ";

	private String testRootPath;
	private String testLevel;
	private String mochaInstallPath;
	private String reporter;
	private String options;

	private File workingDir;
	private File mochaTraceFile;
	
	private String jsonContent = null;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		try {
			// retrieve the parameters we need
			testRootPath     = getStringParamValue("General", "Test root path");       // i.e. "D:/Tests/mocha"
			testLevel        = getStringParamValue("General", "Test level");           // i.e. "Test"
			
			mochaInstallPath = getStringParamValue("Mocha", "Mocha install path");     // i.e. "C:/Program Files/mocha"
			reporter         = getStringParamValue("Mocha", "Reporter").toLowerCase(); // i.e. "mocha-json-file-reporter"
			options          = getStringParamValue("Mocha", "Mocha options");

			workingDir = new File(mochaInstallPath);
			if (!workingDir.exists() || !workingDir.isDirectory()) {
				traceln(LOG_PRIORITY_SEVERE, "Mocha install path does not exist or is not a folder: " + mochaInstallPath);
				return new CReturnStatus(RESULT_FAILURE);
			}
			
			return new CReturnStatus(RESULT_SUCCESS);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			
			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}


	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
				
		jsonContent = null;
		short result = RESULT_FAILURE;
		// +--------------------------+
		// |    Test and test cases   |
		// +--------------------------+
		if (testLevel.equalsIgnoreCase("Test and test cases")) { // Run the test then split all the results on each test case
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST LEVEL = Test and test cases");
			System.err.println("- - - RUNNING THE .JS FILE " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".js"));
			System.err.println("+----------------------------------------------+");

			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

			mochaTraceFile = CFileUtils.createTmpRandomFile("mocha_");
			System.err.println("- - - TEST+TESTCASE MOCHA TRACE FILE: " + mochaTraceFile);
			CRunner mochaRunner = new CRunner("[" + testId + "] "+ testPath + testName,
											  "mocha -R " + reporter + " " + options + " " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".js"),
                                              workingDir);

			// here we redirect the output to a file for later parsing
			result = mochaRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, mochaTraceFile);
			
			jsonContent = getJSONContent(); // fill in jsonContent from the file "reporter.json" or mochaTraceFile (depending on the reporter)
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST+TESTCASE JSON CONTENT: " + jsonContent);
			System.err.println("+----------------------------------------------+");

			//if (result == RESULT_FAILURE) {
			//	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution process failed"));
			//	return new CReturnStatus(RESULT_FAILURE, executionSteps);
			//} else {
			//	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution process succeeded"));
				return new CReturnStatus(RESULT_SUCCESS, executionSteps);
			//}
		}
		
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// +------------------------------------+
		// | Run the Mocha test                 |
		// +------------------------------------+
		CRunner mochaRunner = null;
		short result = RESULT_SUCCESS;

		// +--------------------------+
		// |           Test           |
		// +--------------------------+
		if (testLevel.equalsIgnoreCase("Test")) { // Run the test (and report all sub results in default testcase's steps)
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST LEVEL = Test");
			System.err.println("- - - RUNNING THE .JS FILE " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".js"));
			System.err.println("+----------------------------------------------+");

			mochaTraceFile = CFileUtils.createTmpRandomFile("mocha_");
			System.err.println("- - - TESTE MOCHA TRACE FILE: " + mochaTraceFile);
			mochaRunner = new CRunner("[" + testId + "] "+ testPath + testName,
									  "mocha -R " + reporter + " " + options + " " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".js"),
	                                  workingDir);
			
			// here we redirect the output to a file for later parsing
			result = mochaRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, mochaTraceFile);

			jsonContent = getJSONContent(); // fill in jsonContent from the file "reporter.json" or mochaTraceFile (depending on the reporter)
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST JSON CONTENT: " + jsonContent);
			System.err.println("+----------------------------------------------+");

		// +--------------------------+
		// |        Test case         |
		// +--------------------------+
		} else if (testLevel.equalsIgnoreCase("Test case")) { // Run the testcase (and report all sub results in default testcase's steps)
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST LEVEL = Test cases");
			System.err.println("- - - RUNNING THE .JS FILE " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".js"));
			System.err.println("+----------------------------------------------+");

			mochaTraceFile = CFileUtils.createTmpRandomFile("mocha_");
			System.err.println("- - - TESTCASE MOCHA TRACE FILE: " + mochaTraceFile);
			mochaRunner = new CRunner("[" + testId + "] "+ testPath + testName + testcaseName,
									  "mocha -R " + reporter + " " + options + " " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "/" + testcaseName + ".js"),
									  workingDir);
			
			// here we redirect the output to a file for later parsing
			result = mochaRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, mochaTraceFile);
			
			jsonContent = getJSONContent(); // fill in jsonContent from the file "reporter.json" or mochaTraceFile (depending on the reporter)
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TESTCASE JSON CONTENT: " + jsonContent);
			System.err.println("+----------------------------------------------+");

			
		} else { // The test has already been executed - JUST report all sub results to each testcase)
			// don't do anything
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST LEVEL = Test and test cases");
			System.err.println("- - - DON'T RUN ANYTHING");
			System.err.println("+----------------------------------------------+");

		}

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		if (mochaTraceFile.exists())
			addTestcaseMessageAttachment(mochaTraceFile); // if "Test and Test cases" mode is used, this will be upload only once (on the first testcase which is ok)!
		
		//if (result == RESULT_FAILURE) { // we necessarily executed something so either in "Test" or "Test case" mode
		//	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution process failed"));
		//	parseResultFile(jsonContent, executionSteps); // to have the details anyway
		//	return new CReturnStatus(RESULT_FAILURE, executionSteps);
		//	
		//} else { // we did not executed anything OR we executed something and it worked
			if (testLevel.equalsIgnoreCase("Test and test cases")) { // we did not executed anything
				return parseResultFile(jsonContent, testcaseName, executionSteps); // focus on parsing data for this specific testcase
			} else {                                                 // we executed something and it worked
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution process succeeded"));
				return parseResultFile(jsonContent, executionSteps);
			}
		//}
	}
	
	
	
	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		// maybe there is a call available to get the available models and mCases?
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private String getJSONContent() {
		System.err.println("+----------------------------------------------+");
		System.err.println("- - - RETURN JSON CONTENT");
		System.err.println("+----------------------------------------------+");

		if (reporter.equalsIgnoreCase("mocha-json-file-reporter")) {
			File jsonFile = new File(testRootPath + "/reporter.json");
			addTestcaseMessageAttachment(jsonFile);
			return CFileUtils.FileToString(jsonFile);
			
		} else if (reporter.equalsIgnoreCase("JSON")) {
			// addTestcaseMessageAttachment(mochaTraceFile); // attached separately to differentiate the different mode
			return CFileUtils.FileToString(mochaTraceFile);
			
		} else {
			return null;
		}
	}
	
	private CReturnStatus parseResultFile(String jsonContent, Vector<CExecutionStep> executionSteps) {
		return parseResultFile(jsonContent, null, executionSteps);
	}

	private CReturnStatus parseResultFile(String jsonContent, String testcaseName, Vector<CExecutionStep> executionSteps) {
		//System.out.println("jsonContent = " + jsonContent);

		System.err.println("+----------------------------------------------+");
		System.err.println("- - - PARSING WITH TESTCASENAME = " + testcaseName);
		System.err.println("+----------------------------------------------+");

		boolean allSuccess        = true;
		boolean allNotExecuted    = true;
		boolean atLeastOneFailure = false;

		try {
	    	JSONObject jsonObject = new JSONObject(jsonContent);
	    	JSONObject stats = jsonObject.getJSONObject("stats");
	    	if (stats != null) {
		    	System.err.println("suites    = " + stats.getInt("suites"));
		    	System.err.println("tests     = " + stats.getInt("tests"));
		    	System.err.println("passes    = " + stats.getInt("passes"));
		    	System.err.println("pending   = " + stats.getInt("pending"));
		    	System.err.println("failures  = " + stats.getInt("failures"));
		    	System.err.println("duration  = " + stats.getInt("duration"));
		    	
		    	if (testcaseName == null) {
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "suites = " + stats.getInt("suites")));
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "tests = " + stats.getInt("tests")));
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "passes = " + stats.getInt("passes")));
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "pending = " + stats.getInt("pending")));
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "failures = " + stats.getInt("failures")));
		    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "duration = " + stats.getInt("duration")));
		    	}
	    	}

	    	// search for NOT_EXECUTED tests (those in the "tests" array that have no duration)
	    	JSONArray testList = jsonObject.getJSONArray("tests");
	    	if (testList != null) {
		    	for (int i=0; i<testList.length(); i++) {
	        		JSONObject test = (JSONObject) testList.get(i);
	        		if (test != null) {
		        		String testTitle = test.getString("title");
		        		if (testcaseName == null || testcaseName.equalsIgnoreCase(testTitle.trim())) {
			        		String testFullTitle = test.getString("fullTitle");
			        		int testDuration     = -1;
			        		try {testDuration = test.getInt("duration");} catch (Exception e) {}
			
				    		if (testDuration == -1) {
				    			System.err.println("PENDING: " + testFullTitle);
				    			executionSteps.add(new CExecutionStep(RESULT_NOT_EXECUTED, testFullTitle + " (" + testDuration + "ms)"));
				    			allSuccess = false;
				    		}
		        		}
	        		} else { 
	    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No pending test found!"));
	        		}
		    	} // for each test
	    	} else {
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No pending test found!"));
	    	}
	    	
	    	// search for FAILED tests (those in the "failures" array)
	    	testList = jsonObject.getJSONArray("failures");
	    	if (testList != null) {
		    	for (int i=0; i<testList.length(); i++) {
	        		JSONObject test = (JSONObject) testList.get(i);
	        		if (test != null) {
		        		String testTitle = test.getString("title");
		        		if (testcaseName == null || testcaseName.equalsIgnoreCase(testTitle.trim())) {
			        		String testFullTitle = test.getString("fullTitle");
			        		int testDuration     = test.getInt("duration");
			        		
			        		String errMessage = null;
			        		String errStack   = null;
			        		try {
								JSONObject err = test.getJSONObject("err");
								errMessage = err.getString("message");
								errStack   = err.getString("stack");
							} catch (Exception e) {
							}
			        		
			    			System.err.println("FAILED : " + testFullTitle + " (" + testDuration + ")");
			    			executionSteps.add(new CExecutionStep(RESULT_FAILURE, testFullTitle + " (" + testDuration + "ms)"));
			    			if (errMessage != null)
			    				executionSteps.add(new CExecutionStep(RESULT_INFO, "Error: " + errMessage));
			    			if (errStack != null)
			    				executionSteps.add(new CExecutionStep(RESULT_INFO, "Stack: " + errStack));
			    			
			    			allSuccess = false;
			    			allNotExecuted = false;
			    			atLeastOneFailure = true;
		        		}
	        		} else {
	    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No failed test found!"));
	        		}
		    	} // for each test
	    	} else {
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No failed test found!"));
	    	}
	    	
	    	// search for SUCCESFUL tests (those in the "passes" array)
	    	testList = jsonObject.getJSONArray("passes");
	    	if (testList != null) {
		    	for (int i=0; i<testList.length(); i++) {
	        		JSONObject test = (JSONObject) testList.get(i);
	        		if (test != null) {
		        		String testTitle = test.getString("title");
		        		if (testcaseName == null || testcaseName.equalsIgnoreCase(testTitle.trim())) {
			        		String testFullTitle = test.getString("fullTitle");
			        		int testDuration     = test.getInt("duration");
	
			    			System.err.println("PASSES : " + testFullTitle + " (" + testDuration + ")");
			    			executionSteps.add(new CExecutionStep(RESULT_SUCCESS, testFullTitle + " (" + testDuration + "ms)"));
			    			allNotExecuted = false;
		        		}
	        		} else {
	    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No successful test found!"));
	        		}
		    	} // for each test
	    	} else {
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No successful test found!"));
	    	}
    	} catch (Exception e) {
    		traceln(LOG_PRIORITY_SEVERE, "couldn't parse the stats: " + e);
			return null;
		}
		
		if (allNotExecuted) {
			return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
			
		} else if (atLeastOneFailure) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps); 

		} else if (allSuccess) {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps); 
		}
		
		return new CReturnStatus(RESULT_INDEFINITE, executionSteps); 
	}

	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl toto = new CLauncherImpl();
		

		System.err.println("========================================================");
		File stst = new File("Y:/XStudio/src/com/xqual/xlauncher/mocha/sample1.txt");
		String ststContent = CFileUtils.FileToString(stst);
		CReturnStatus returnStatus = toto.parseResultFile(ststContent, new Vector<CExecutionStep>());
		System.err.println(returnStatus);
	
		System.err.println("========================================================");

	}
	
}
