/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.0                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.neoload;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.CZipWriter;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{NeoLoad       }  ";

	private String projectRootPath;
	private String neoLoadInstallPath;
	private String neoLoadOptions;
	private boolean attachHtmlReport;

	private File workingDir;
	
	private File neoLoadTraceFile;
	private File neoLoadXmlResultFile;
	private File neoLoadHtmlResultFile;
	private File neoLoadHtmlZipFile;

	private static final String NEOLOAD_TESTRUNNER_EXE = CUtils.getExecutableName("NeoLoadCmd");

	/*
	C:\Program Files\NeoLoad 5.1\bin\NeoLoadCmd.exe -project "C:\Users\owner\Documents\NeoLoad Projects\ProjectName\ProjectName.nlp" 
	                                                -launch scenario1 -noGUI -report projects/jpetstore/reports/report.xml
	
	<NeoLoadInstallPath>\bin\NeoLoadCmd.exe -project <projectRootPath>/<testPath>/<testName>.nlp" 
	                                        -launch <testcaseName> -noGUI -report tmp.xml
	*/
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		try {
			// retrieve the parameters we need
			projectRootPath    = getStringParamValue("General",  "Project root path");                             // i.e. "C:/Users/Eric/Documents/NeoLoad Projects"
			
			neoLoadInstallPath = getStringParamValue("NeoLoad",  "NeoLoad install path");                          // i.e. "C:/Program Files/NeoLoad 5.1"
			neoLoadOptions     = getStringParamValue("NeoLoad",  "Options");                                       // i.e. -noGUI
			attachHtmlReport   = getBooleanParamValue("NeoLoad", "Attach HTML report (may consume lot of space)"); // i.e. false

			workingDir = new File(neoLoadInstallPath + "/bin");
			
			if (!workingDir.exists() || !workingDir.isDirectory()) {
				traceln(LOG_PRIORITY_SEVERE, "NeoLoad install path does not exist or is not a folder: " + neoLoadInstallPath);
				return new CReturnStatus(RESULT_FAILURE);
			}
			
			return new CReturnStatus(RESULT_SUCCESS);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			
			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}


	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		neoLoadTraceFile      = CFileUtils.createTmpRandomFile("neoload_console_", "txt");
		neoLoadXmlResultFile  = CFileUtils.createTmpRandomFile("neoload_result_",  "xml");
		neoLoadHtmlResultFile = CFileUtils.createTmpRandomFile("neoload_result_",  "html");
		neoLoadHtmlZipFile    = CFileUtils.createTmpRandomFile("neoload_result_",  "zip");

		//System.err.println("- - - - XML = " + neoLoadXmlResultFile.getAbsolutePath());
		//System.err.println("- - - - HTML = " + neoLoadHtmlResultFile.getAbsolutePath());
		/*
		<NeoLoadInstallPath>\bin\NeoLoadCmd.exe -project <projectRootPath>/<testPath>/<testName>.nlp" 
        -launch <testcaseName> -noGUI -report tmp.xml, tmp.html
        */
		// +------------------------------------+
		// | Run the NeoLoad test               |
		// +------------------------------------+
		CRunner neoLoadRunner = new CRunner("[" + testId + "] "+ testPath + testName + testcaseName,
											CFileUtils.quoteFilePath(neoLoadInstallPath + "/bin/" + NEOLOAD_TESTRUNNER_EXE) + " " +
											"-project " + CFileUtils.quoteFilePath(projectRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".nlp") + " " +
											"-launch " + CFileUtils.quoteFilePath(testcaseName) + " " +
											neoLoadOptions + " " +
											"-report " + neoLoadXmlResultFile + "," + neoLoadHtmlResultFile,
											workingDir);


		// here we redirect the output to a file for later parsing
		short result = neoLoadRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, neoLoadTraceFile);

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(neoLoadTraceFile);
		addTestcaseMessageAttachment(neoLoadXmlResultFile);
		
		// zip the Html structure and attach it
		if (attachHtmlReport && neoLoadHtmlResultFile.exists()) {
			CZipWriter zipWriter = new CZipWriter(neoLoadHtmlZipFile.getAbsolutePath());
			zipWriter.addToZip("",       neoLoadHtmlResultFile.getAbsolutePath());
			zipWriter.addFolderToZip("", neoLoadHtmlResultFile.getParentFile().getAbsolutePath() + "/" + CFileUtils.getBasename(neoLoadHtmlResultFile) + "_files");
			zipWriter.addFolderToZip("", neoLoadHtmlResultFile.getParentFile().getAbsolutePath() + "/images");
			zipWriter.closeZip();
			addTestcaseMessageAttachment(neoLoadHtmlZipFile);
		}
		
		// addTestcaseMessageAttachment check if the file exists
		addTestcaseMessageAttachment(new File(neoLoadHtmlResultFile.getParentFile().getAbsolutePath() + "/images/summary.png"));
		addTestcaseMessageAttachment(new File(neoLoadHtmlResultFile.getParentFile().getAbsolutePath() + "/images/errors.png"));
		addTestcaseMessageAttachment(new File(neoLoadHtmlResultFile.getParentFile().getAbsolutePath() + "/images/throughput.png"));

		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(RESULT_FAILURE, "execution process failed"));
			parseResultFile(neoLoadXmlResultFile, executionSteps); // to have the details anyway
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(RESULT_SUCCESS, "execution process succeeded"));
			return parseResultFile(neoLoadXmlResultFile, executionSteps);
		}
	}
	
	
	
	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		// maybe there is a call available to get the available models and mCases?
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File resultsXmlFile, Vector<CExecutionStep> executionSteps) {
		short status = RESULT_SUCCESS;
		
		if (!resultsXmlFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps); 
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: result file found"));
		}

		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(resultsXmlFile);
	    
	    // -------------------
	    // Title
	    // -------------------
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "[b]Target[/b]:"));
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "Project: "  + CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/project/@name")));
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "Scenario: " + CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/project/scenario/@name")));

	    // -------------------
	    // statistics
	    // -------------------
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "[b]Statictics[/b]:"));
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "Total pages: "               + CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/statistics/statistic[@name='total_pages']/@value")));
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "Total requests: "            + CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/statistics/statistic[@name='total_hits']/@value")));
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "Total users launched: "      + CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/statistics/statistic[@name='total_users_launched']/@value")));
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "Total iteration completed: " + CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/statistics/statistic[@name='total_iterations_completed']/@value")));
	    executionSteps.add(new CExecutionStep(RESULT_INFO, "Total throughput: "          + CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/statistics/statistic[@name='total_throughput']/@value") + " MB"));

	    String requestErrors = CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/statistics/statistic[@name='total_errors']/@value");
	    if (!requestErrors.equals("0"))
	    	executionSteps.add(new CExecutionStep(RESULT_FAILURE, "Total request errors: "      + "[firebrick]" + requestErrors + "[/firebrick]"));
	    else
	    	executionSteps.add(new CExecutionStep(RESULT_SUCCESS, "No request errors"));
	    
	    String actionErrors  = CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "/report/summary/statistics/statistic[@name='total_logical_actions_errors']/@value");
	    if (!actionErrors.equals("0"))
	    	executionSteps.add(new CExecutionStep(RESULT_FAILURE, "Total action errors: "       + "[firebrick]" + actionErrors + "[/firebrick]"));
	    else
	    	executionSteps.add(new CExecutionStep(RESULT_SUCCESS, "No action errors"));

	    if (!requestErrors.equals("0") || !actionErrors.equals("0")) {
	    	status = RESULT_FAILURE;
	    }
	    
	    // -------------------
	    // Errors + Alerts
	    // -------------------

	    /*
	    <requests-errors-summary/>
	     
	    <logical-actions-errors-summary/>
	      
	    <thirdparty-data-errors-summary/>
	    
	    */

	    executionSteps.add(new CExecutionStep(RESULT_INFO, "[b]Alerts[/b]:"));
	    NodeList alertNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "/report/summary/alerts-summary/alert-summary");
	    for (int i=0; i<alertNodeList.getLength(); i++) {
	    	Node alertNode = alertNodeList.item(i);
	    	if (alertNode != null) {
		    	NamedNodeMap attributes = alertNode.getAttributes();
		    	if (attributes != null) {
		    		String critical  = attributes.getNamedItem("criticalCounter").getNodeValue();
		    		String warning   = attributes.getNamedItem("warningCounter").getNodeValue();
		    		String monitored = attributes.getNamedItem("counter").getNodeValue();
		    		
		    	    if (!critical.equals("0"))
		    	    	executionSteps.add(new CExecutionStep(RESULT_FAILURE, attributes.getNamedItem("name").getNodeValue() + " - Critical: [firebrick]" + critical + "[/firebrick]"));

		    	    if (!warning.equals("0"))
		    	    	executionSteps.add(new CExecutionStep(RESULT_FAILURE, attributes.getNamedItem("name").getNodeValue() + " - Warning: [firebrick]" + warning + "[/firebrick]"));

		    	    if (!monitored.equals("0"))
		    	    	executionSteps.add(new CExecutionStep(RESULT_FAILURE, attributes.getNamedItem("name").getNodeValue() + " - Monitored: [firebrick]" + monitored + "[/firebrick]"));
		    	}
	    	}
	    }

	    return new CReturnStatus(status, executionSteps); 
	}


	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl toto = new CLauncherImpl();

		System.err.println("========================================================");
		File resultFile = new File("Y:/XStudio/src/com/xqual/xlauncher/neoload/sample/24/myReport.xml");
		CReturnStatus returnStatus = toto.parseResultFile(resultFile, new Vector<CExecutionStep>());
		System.err.println(returnStatus);
	
		System.err.println("========================================================");

	}
	
}
