/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developer:   Russell Ball (russell.ball@civil.lmco.com)              |
| 			   template Eric Gavaldo (XQual)                           |
| Launcher:	   Axe 3.5 (Odin www.odin.co.uk)                           |
| Version:     1.2                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.odin_axe;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Odin Axe.
 * @author rball
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{axe         }  ";

	private static final String TRACE_SERVER_FILE_NAME = "selenium_server_traces.txt";

	// parameters impacting the execution
	private String AxeInstallPath;
    private String projectName;
	private String projectPath;
	private String resultsPath;
	private String scriptsPath;
	private String configPath;

	private File seleniumWorkingDir;
	private File seleniumServerTraceFile;
	private String javaBinPath;
	private String seleniumServerJarPath;
	private String seleniumServerOptions;
	private boolean SeleniumWDEnabled;

	private String suiteName;
	private File workingDir;

	private String runConfigName;
	private String runConfigFile;
	private String oldrunConfigName;

	private boolean resultsEnabled;
	private String testIdentifier;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
	
	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);
		
		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");

		// check the configuration sent by the manager
		printConfiguration();
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			// Axe Run
			runConfigName = getStringParamValue("Axe Config", "Default Axe Run config file");
			resultsEnabled = getBooleanParamValue("Axe Config", "Input Results Only");
			
			// Axe Project
			AxeInstallPath = getStringParamValue("Axe Project", "Axe install path");
			projectName    = getStringParamValue("Axe Project", "Project name");
			projectPath    = getStringParamValue("Axe Project", "Project path");
			resultsPath    = projectPath + getStringParamValue("Axe Project", "Results path");
			scriptsPath    = projectPath + getStringParamValue("Axe Project", "Scripts path");
			configPath     = projectPath + getStringParamValue("Axe Project", "Config path");

            //Selenium Settings
			seleniumServerJarPath = getStringParamValue("Selenium Settings", "Server Jar Path");
			seleniumServerOptions = getStringParamValue("Selenium Settings", "Server Options");
			javaBinPath           = getStringParamValue("Selenium Settings", "Java Bin Path");
			SeleniumWDEnabled     = getBooleanParamValue("Selenium Settings", "Selenium Web Driver");
			
			suiteName  = sutName;
			
			workingDir              = new File(resultsPath + "/" + suiteName + "/");
			seleniumWorkingDir      = new File(projectPath + "/");
			seleniumServerTraceFile = new File(TRACE_SERVER_FILE_NAME);
			runConfigFile           = configPath + "/" + runConfigName + ".RunConfig.xml"; 
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		if (suiteName !=null){
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		} else {
			CUtils.showErrorMessage(null, 
					"Error: SUT (" + suiteName + ") is empty", 
					"Please go to the <b>results</b> and <b>scripts</b> folders in Axe project (" + projectPath + ") and check if these exist",
					JOptionPane.ERROR_MESSAGE);
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	
	
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun sutName =" + suiteName + " testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);

	}
	
	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run sutName =" + suiteName + " testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		testIdentifier = testName;
	    if (!resultsEnabled) 
			{
			if (!SeleniumWDEnabled)
				{
				// +----------------------------------------------------------------+
				// | Start the Selenium Server                                      |
				// +----------------------------------------------------------------+
				traceln(LOG_PRIORITY_INFO, ">>>");
				traceln(LOG_PRIORITY_INFO, ">>> starting the Selenium Server...");
				traceln(LOG_PRIORITY_INFO, ">>>");
		
				int count = 4444;
				short resultSelenium = RESULT_FAILURE;
				while (resultSelenium != RESULT_SUCCESS && count<4454) { // max 10 tries
					traceln(LOG_PRIORITY_INFO, "trying to start selenium server on port " + count + "...");
					CRunner seleniumServer = new CRunner("[Selenium Server] ",
														 CFileUtils.quoteFilePath(javaBinPath + "/java") + " -jar " + CFileUtils.quoteFilePath(seleniumServerJarPath) + " -port " + count + " " + seleniumServerOptions,
														 seleniumWorkingDir);
			
					// let the time to the server to initialize (this is done only once per campaign session)
					resultSelenium = seleniumServer.requestAction(IRunner.START_PROCESS, IRunner.DO_NOT_WAIT_END_OF_EXECUTION, seleniumServerTraceFile);
					CUtils.sleep(5000);
					count++;
				}
				if (resultSelenium == RESULT_SUCCESS)
					traceln(LOG_PRIORITY_INFO, "Selenium server launched successfully");
				else
					traceln(LOG_PRIORITY_SEVERE, "Selenium server failed to launch!");
				
				if (seleniumServerTraceFile.exists())
					addTestcaseMessageAttachment(seleniumServerTraceFile);
				}			
					// +------------------------------------+
					// | Generate the run.xml script        |
					// +------------------------------------+
					traceln(LOG_PRIORITY_INFO, ">>>");
					traceln(LOG_PRIORITY_INFO, ">>> generating the run.xml script...");
					traceln(LOG_PRIORITY_INFO, ">>>");
					StringBuffer outputBuffer = new StringBuffer();
					outputBuffer.append("<?xml version=\"1.0\" encoding=\"utf-8\"?>\n");
					outputBuffer.append("<RunSettings>\n");
					outputBuffer.append("<Option Name=\"CheckDependency\" Value=\"False\" />\n"); 
					outputBuffer.append("<Option Name=\"RunConfiguration\" Value=\"" + runConfigFile + "\" />\n");
					outputBuffer.append("<Option Name=\"RunName\" Value=\"" + testName + "\" />\n");	
					outputBuffer.append("<Tests>\n");   
					outputBuffer.append("<Test>" + testName + "</Test>\n"); //only have one test at a time - let xstudio handle the campaigns
					outputBuffer.append("</Tests>\n"); 
					outputBuffer.append("</RunSettings>\n");

					File runXmlFile = new File(resultsPath + "/" + suiteName + "/run.xml");
					CFileUtils.deleteFile(runXmlFile);
					try {
						runXmlFile.createNewFile();
						Writer output = new BufferedWriter(new FileWriter(runXmlFile));
						output.write(outputBuffer.toString());
						output.close();
					} catch (IOException e) {
						traceln(LOG_PRIORITY_SEVERE, "Could not create file \"" + runXmlFile + "\"");
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Could not create file \"" + runXmlFile + "\""));
						return new CReturnStatus(RESULT_FAILURE, executionSteps);
					}
					traceln(LOG_PRIORITY_INFO, runXmlFile + " has been generated");

					// +------------------------------------+
					// | Interpret the script               |
					// +------------------------------------+
					traceln(LOG_PRIORITY_INFO, ">>>");
					traceln(LOG_PRIORITY_INFO, ">>> starting the Axe script...");
					traceln(LOG_PRIORITY_INFO, ">>>");
					
					CRunner AxeRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
													CFileUtils.quoteFilePath(scriptsPath + "/" + suiteName + "/" + testName + ".exe"), 
													workingDir);
					short result = AxeRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
					if (result == RESULT_FAILURE) {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_NOT_EXECUTED, "script failed or missing [" + scriptsPath + "/" + suiteName + "/" + testName + ".exe]"));
						return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
					}
		}
		
		// +------------------------------------+
		// | Results Only                       |
		// +------------------------------------+

		File resultFile = new File(resultsPath + "/" + suiteName + "/results.xml");
		traceln(LOG_PRIORITY_INFO, "Result File = " + resultFile);
		
		if (resultFile != null && resultFile.exists()) {
		    
		    // Attach image files to log
			File imagePathFolder = new File(resultsPath + "/" + suiteName + "/images");
			traceln(LOG_PRIORITY_INFO, "Image Path = " + imagePathFolder);
			
			if (imagePathFolder.exists() && imagePathFolder.isDirectory()) {
				String[] children = imagePathFolder.list();
				for (int i = 0; i < children.length; i++) {
					if (children[i].startsWith(testName)) { 
						String imageFilePath = imagePathFolder + "/" + children[i];
                        File imageFile = new File(imageFilePath);

                        String xstudioImageFilePath = imagePathFolder + "/xStudio_" + children[i];
						File xstudioImage = new File(xstudioImageFilePath);
						
						traceln(LOG_PRIORITY_INFO, "Standard Image File = " + imageFilePath);
						traceln(LOG_PRIORITY_INFO, "XStudio Image File  = " + xstudioImageFilePath);
						
						try {
							CFileUtils.copyFiles(imageFile, xstudioImage, true);
						} catch (IOException e) {
							traceln(LOG_PRIORITY_SEVERE, "Could not create file \"" + xstudioImage + "\"");
							executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Could not create file \"" + xstudioImage + "\""));
							return new CReturnStatus(RESULT_FAILURE, executionSteps);
						}
						addTestcaseMessageAttachment(xstudioImage);
						traceln(LOG_PRIORITY_INFO, "xStudio Image File added to log = " + xstudioImageFilePath);
					}
				}
			}
			traceln(LOG_PRIORITY_INFO, "Parsing Result File = " + resultFile);
			return parseResultFile(resultFile, executionSteps, resultsPath, suiteName,testIdentifier);
			
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "results.xml file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		traceln(LOG_PRIORITY_INFO, "terminating...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eqtp$",          // only *.qtp files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+
	public static CReturnStatus parseResultFile(File resultFile, Vector<CExecutionStep> executionSteps, String resultsPath, String suiteName, String testIdent) {	
		Document xmlDocument = CXmlDocumentFactory.createXMLDoc(resultFile);	
		boolean errorDetected = false;
		
		short result = RESULT_UNKNOWN;
		
		String testFinish = "";
		NodeList testNodes = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//run/test[id='" + testIdent + "']");
		if(testNodes.getLength() > 0){
			for (int i = testNodes.getLength()-1 ; i < testNodes.getLength(); i++) {
				Node testNode = testNodes.item(i);
			
				String testId          = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "id");
				String testTableName   = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "table/@name");
				String testTableFile   = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "table/@file");
				String testDescription = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "description");
				//testStart       	   = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "start");
				testFinish      	   = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "finish");
				String testTime        = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "time");
				//String testResult      = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "result");
				String testSuccess     = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "success");
				String testCount       = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "testcount");
				String testPassCount   = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "passcount");
				String testFailCount   = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "failcount");
				
			    String testImage = "null";
				if (!testSuccess.equalsIgnoreCase("True")) {
					errorDetected           = true;
					testImage               = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "image");
					//String testSubtestIndex = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "image/@subtest_index");
					//String testStepIndex    = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "image/@step_index");
				}
				
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), testSuccess.equalsIgnoreCase("True") ? RESULT_SUCCESS : RESULT_FAILURE, "Test Start - Time " + testTime));
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO,                                                            "Test Name: " + testId));
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                           "Test Desc: " + testDescription));
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                           "Table Name: " + testTableName + " | Test File: " + testTableFile));
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                           "Sub-tests: " + testCount + " | Passed: " + testPassCount + " | Failed: " + testFailCount));

				NodeList basestateNodes = CXmlDocumentFactory.getNodeObjListFromXPath(testNode, "basestate");
				for (int h = 0; h < basestateNodes.getLength(); h++) {
					Node basestateNode = basestateNodes.item(h);

					//String baseStateIndex   = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "index");
					String baseStateAction  = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "action");
					String baseStateStart   = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "start");
					String baseStateFinish  = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "finish");
					String baseStateTime    = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "time");
					String baseStateResult  = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "result");
					String baseStateSuccess = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "success");
					String baseStateInfo    = CXmlDocumentFactory.getNodeStringValueFromXPath(basestateNode, "info");
					
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), baseStateSuccess.equalsIgnoreCase("True") ? RESULT_SUCCESS : RESULT_FAILURE, "Success: " + baseStateSuccess));
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                                "Action: " + baseStateAction + " | Start: " + baseStateStart + " | Finish: " + baseStateFinish + " | Time: " + baseStateTime + " | Result: " + baseStateResult));
								
					if (!baseStateSuccess.equalsIgnoreCase("True")) {
						result = RESULT_FAILURE;
						errorDetected = true;
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO , "Info: " + baseStateInfo));
					}
				}
			
				NodeList subtestNodes = CXmlDocumentFactory.getNodeObjListFromXPath(testNode, "subtest");
				for (int j = 0; j < subtestNodes.getLength(); j++) {
					Node subtestNode = subtestNodes.item(j);

					String subtestIndex       = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "index");
					String subtestId          = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "id");
					String subtestTableName   = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "table/@name");
					String subtestTableFile   = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "table/@file");
					String subtestDescription = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "description");
					String subtestStart       = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "start");
					String subtestFinish      = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "finish");
					String subtestTime        = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "time");
					String subtestResult      = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "result");
					String subtestSuccess     = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "success");
					String subtestCount       = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "testcount");
					String subtestPassCount   = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "passcount");
					String subtestFailCount   = CXmlDocumentFactory.getNodeStringValueFromXPath(subtestNode, "failcount");

					if (!subtestSuccess.equalsIgnoreCase("True")) {
						result = RESULT_FAILURE;
						errorDetected = true;

						Pattern defectlogged = Pattern.compile("[\\s#:]*(\\d{4})");
						Matcher matcher = defectlogged.matcher(subtestDescription);
						while (matcher.find()) {
							String defectId = matcher.group(1);
							executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, defectId + " [Known Issue]"));
						}
					}
					
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), subtestSuccess.equalsIgnoreCase("True") ? RESULT_SUCCESS : RESULT_FAILURE, subtestIndex + " - Sub-test: " + subtestId));
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                              "Start: " + subtestStart + " | Finish: " + subtestFinish + " | Time: " + subtestTime));
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                              "Description: " + subtestDescription));
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                              "Table: " + subtestTableName + "| File: " + subtestTableFile));
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO ,                                                              "Steps: " + subtestCount + " | Passed: " + subtestPassCount + " | Failed: " + subtestFailCount));
							
					NodeList stepNodes = CXmlDocumentFactory.getNodeObjListFromXPath(subtestNode, "step");
					for (int k = 0; k < stepNodes.getLength(); k++) {
						Node stepNode = stepNodes.item(k);
						//String stepIndex   = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "index");
						String stepObject  = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "object");
						String stepAction  = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "action");
						String stepData    = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "data");
						String stepSuccess = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "success");
						
						// Step results Fail only
						String stepInfo     = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "info");
						//String stepErrorNo  = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "error_no");
						String stepErrorMsg = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "error_msg");
						//String stepStart    = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "start");
						String stepFinish   = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "finish");
						String stepResultsDetails = "";
						
						if (!stepSuccess.equalsIgnoreCase("True")) {
							errorDetected = true;
							result = RESULT_FAILURE;
							if (subtestResult.equalsIgnoreCase("Aborted")) {
								stepResultsDetails = stepObject + "  |  " + stepAction + "  |  " + stepData + "  |  " + stepInfo + "  |  " + stepErrorMsg + "   |  " + resultsPath + "/" + testImage;
							} else {
								stepResultsDetails = stepObject + "  |  " + stepAction + "  |  " + stepData + "  |  " + stepInfo + "  |  " + stepErrorMsg;
							}
						} else {
							result = RESULT_SUCCESS;
							stepResultsDetails = stepObject + "  |  " + stepAction + "  |  " + stepData;
						}
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), stepSuccess.equalsIgnoreCase("True") ? RESULT_SUCCESS : RESULT_FAILURE, stepResultsDetails)); 
					} // for each step
				} // for each subtest
			} // for each test
		} else {
    		errorDetected = false;
    		result = RESULT_NOT_EXECUTED;
    		
    		executionSteps.add(new CExecutionStep(result, "Test Missing [" + testIdent + "]"));
		}
		
		if (!errorDetected){
		    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(),result, "Test Finish"));
			return new CReturnStatus(result, executionSteps);
			
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(),errorDetected ? RESULT_FAILURE : RESULT_SUCCESS, "Test Finish"));
			return new CReturnStatus(errorDetected ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
		}
	}	
}
