/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (egavaldo@xqual.com)                       |
|              YehudaM                                                 |
| Version:     1.13                                                    |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.offline_generic;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.LinkedHashMap;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CLauncherUtils;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CLauncherCommonParsers;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Executables.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{off. generic  }  ";

	
	// if you change one of those, change that too in the offline_generic.xml file
	static final String FORMAT_JUNIT                            = "JUnit";
	static final String FORMAT_TESTNG                           = "TestNg";
	static final String FORMAT_MOCKITO                          = "Mockito";
	static final String FORMAT_EASYMOCK                         = "EasyMock";
	static final String FORMAT_GITLAB_ALT1                      = "GitLab (alt.1)";	
	static final String FORMAT_GITLAB_ALT2                      = "GitLab (alt.2)";
	static final String FORMAT_TABULAR_TEST_RESULT              = "Tabular (test-result)";
	static final String FORMAT_TABULAR_TEST_TESTCASE_RESULT     = "Tabular (test-testcase-result)";
	static final String FORMAT_TABULAR_TEST_ERRORCOUNT          = "Tabular (test-error#)";
	static final String FORMAT_TABULAR_TEST_TESTCASE_ERRORCOUNT = "Tabular (test-testcase-error#)";
    
	
	// parameters impacting executing at run time set by the test operator
	private String scriptPath;
	private String resultFilePath;
	private String resultFileFormat;
	
	private File resultFile;

	private File workingDir;

	LinkedHashMap<Integer, Short> hashmapCurrentTestcaseStatus;
	
	LinkedHashMap<Integer, Short> hashmapTestStatus     = new LinkedHashMap<Integer, Short>();
	LinkedHashMap<Integer, Short> hashmapTestcaseStatus = new LinkedHashMap<Integer, Short>();
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		// check if there are some post processing parameters
		readPostProcessingParameters();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			scriptPath       = getStringParamValue("General",  "Script path (opt.)"); // i.e. "C:/my_scripts/script1.bat"
			resultFilePath   = getStringParamValue("General",  "Result file path");   // i.e. "C:/my_scripts/results.txt"
			resultFileFormat = getStringParamValue("General",  "Result file format"); // i.e. "TestNg"
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// If a script is provided and exists, execute it
		scriptPath = scriptPath.trim();
		File scriptFile = new File(scriptPath);
		if (scriptPath.length()>0 && scriptFile.exists() && scriptFile.isFile()) {
			CRunner exeRunner = new CRunner("Initialization script",
	                                        CFileUtils.quoteFilePath(scriptPath),
	                                        scriptFile.getParentFile());
	
			short result = exeRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
			traceln(LOG_PRIORITY_INFO, "result=" + result);
			if (result == RESULT_FAILURE) {
				traceln(LOG_PRIORITY_INFO, "The script execution returned an error code > 0 which very likely indicates it failed to execute properly!");

				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Failed to execute script"));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
		} else {
			// this is not mandatory to execute a script first
		}
		
		// check the script generated a result file
		resultFilePath = resultFilePath.trim();
		resultFile = new File(resultFilePath);
		if (!resultFile.exists()) {
			traceln(LOG_PRIORITY_INFO, "The result folder/file \"" + resultFilePath + "\" couldn't be found!");
			
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find a result folder/file"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");

		hashmapCurrentTestcaseStatus = new LinkedHashMap<Integer, Short>();
				
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + " testcaseName=" + testcaseName + " additionalInfo=" + additionalInfo);

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		CReturnStatus returnStatus = null;
		
		if (resultFileFormat.equalsIgnoreCase(FORMAT_JUNIT)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the JUnit result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = CLauncherCommonParsers.parseJUnitXmlFile(resultFile, null, testcaseName, executionSteps);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;
			
		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_TESTNG)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the TestNg result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = parseTestNgResultFile(resultFile, executionSteps, testName, testcaseName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;
				
		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_MOCKITO) || resultFileFormat.equalsIgnoreCase(FORMAT_EASYMOCK)) {
			System.err.println("- - - - MOCKITO: folder=" + resultFile);
			if (!resultFile.isDirectory()) { // expect a FOLDER
				traceln(LOG_PRIORITY_INFO, "The result folder \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the Mockito/EasyMock result folder: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			File testResultFile = new File(resultFile, "TEST-" + testName + ".xml");
			System.err.println("- - - - MOCKITO: result file =" + testResultFile);
			if (!testResultFile.exists() || !testResultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + testResultFile + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the Mockito/EasyMock result file: " + testResultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}

			returnStatus = parseMockitoEasyMockResultFile(testResultFile, executionSteps, testName, testcaseName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;

		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_GITLAB_ALT1)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the GitLab (alt.1) result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = parseGitLabAlt1ResultFile(resultFile, executionSteps, testName, testcaseName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;

		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_GITLAB_ALT2)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the GitLab (alt.2) result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = parseGitLabAlt2ResultFile(resultFile, executionSteps, testName, testcaseName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;

		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_TABULAR_TEST_RESULT)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the TestNg result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = parseTabularTestResultResultFile(resultFile, executionSteps, testName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;
			
		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_TABULAR_TEST_TESTCASE_RESULT)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the TestNg result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = parseTabularTestTestcaseResultResultFile(resultFile, executionSteps, testName, testcaseName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;
			
		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_TABULAR_TEST_ERRORCOUNT)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the TestNg result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = parseTabularTestErrorCountResultFile(resultFile, executionSteps, testName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;
			
		} else if (resultFileFormat.equalsIgnoreCase(FORMAT_TABULAR_TEST_TESTCASE_ERRORCOUNT)) {
			if (!resultFile.isFile()) { // expect a FILE
				traceln(LOG_PRIORITY_INFO, "The result file \"" + resultFilePath + "\" couldn't be found!");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the TestNg result file: " + resultFile));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
			
			returnStatus = parseTabularTestTestcaseErrorCountResultFile(resultFile, executionSteps, testName, testcaseName);
			hashmapCurrentTestcaseStatus.put(testcaseId, returnStatus.getResult());
			hashmapTestcaseStatus.put(testcaseId, returnStatus.getResult());
			return returnStatus;
			
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Unknown result file format"));
			hashmapCurrentTestcaseStatus.put(testcaseId, RESULT_FAILURE);
			hashmapTestcaseStatus.put(testcaseId, RESULT_FAILURE);
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		
		hashmapTestStatus.put(testId, CLauncherUtils.testcasesResultsToTestResult(hashmapCurrentTestcaseStatus.values().iterator(), true));
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		// +-------------------------------------+
		// | post-processing                     |
		// +-------------------------------------+
		if (postProcessingEnabled) {
			traceln(LOG_PRIORITY_INFO, "post-processing required");
			if (hashmapTestStatus == null)     hashmapTestStatus     = new LinkedHashMap<Integer, Short>();
			if (hashmapTestcaseStatus == null) hashmapTestcaseStatus = new LinkedHashMap<Integer, Short>();
			
			downloadAndRunPostProcessingScript(hashmapTestStatus, hashmapTestcaseStatus, new LinkedHashMap<Integer, String>(),
											   "", "");
		} else {
			traceln(LOG_PRIORITY_INFO, "no post-processing required");
		}
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eqtp$",          // only *.qtp files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseTestNgResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName, String testcaseName) {
		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(inputFile);
	    Node rootXmlNode = xmlDocument.getFirstChild();
	    if (rootXmlNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Invalid result file!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: invalid result file!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }

	    //System.err.println("- - - - xml content child number: " + rootXmlNode.getChildNodes().getLength());
	    //System.err.println("- - - - xml content: " + CXmlDocumentFactory.nodeToString(rootXmlNode));
	    
	    Node testcaseNode = CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, "//test[@name='" + testName + "']/*/test-method[@name='" + testcaseName + "']");
	    if (testcaseNode == null) {
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the testcase in the result file"));
	    	return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }

		String strStatus = testcaseNode.getAttributes().getNamedItem("status").getNodeValue();
		String time      = testcaseNode.getAttributes().getNamedItem("started-at").getNodeValue();
    	String name      = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
    	String duration  = testcaseNode.getAttributes().getNamedItem("duration-ms").getNodeValue();
    	String exception = CXmlDocumentFactory.getNodeStringValueFromXPath(testcaseNode, "exception/@class");
    	Node messageNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "exception/message");
    	String message   = CXmlDocumentFactory.getNodeTextOrCDataContent(messageNode);
    	
    	
		short status = RESULT_FAILURE;
		
		if (strStatus.equals("PASS")) {
			executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + strStatus + "] " + name + " (" + duration + "ms)"));
			status = RESULT_SUCCESS;
			
		} else if (strStatus.equals("FAIL")) {
			executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + strStatus + "] " + name + ": " + exception + "\n" + message + "\n(" + duration + "ms)"));
			status = RESULT_FAILURE;
			
		} else if (strStatus.equals("SKIP")) {
			executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + strStatus + "] " + name + " (" + duration + "ms)"));
			status = RESULT_NOT_EXECUTED;
			
		} else {
			executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + strStatus + "] " + name + " (" + duration + "ms)"));
			status = RESULT_FAILURE;
		}

		return new CReturnStatus(status, executionSteps);
	}
	
	
	private CReturnStatus parseTabularTestResultResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName) {
		String line = "";
		boolean resultFound = false;
		short status = RESULT_NO_RESULT;
		
		try {
			FileInputStream fileInputStream = new FileInputStream(inputFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				
				String[] lineSplit = line.split("\\W+"); // not an alphanumeric character
				if (lineSplit.length != 2) {
					continue; // probably the first line (header) or something
				}

				if (lineSplit[0].equals(testName)) {
					String strStatus = lineSplit[1]; // Success, Failure, Indefinite, NotExecuted, NotExecutable
					status = CUtils.resultUserStringToShort(strStatus);
					resultFound = true;
					break;
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			status = RESULT_FAILURE;
		}

		if (resultFound) {
			return new CReturnStatus(status, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "No valid status found"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}
	
	private CReturnStatus parseTabularTestTestcaseResultResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName, String testcaseName) {
		String line = "";
		boolean resultFound = false;
		short status = RESULT_NO_RESULT;
		
		try {
			FileInputStream fileInputStream = new FileInputStream(inputFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				
				String[] lineSplit = line.split("\\W+"); // not an alphanumeric character
				if (lineSplit.length != 3) {
					continue; // probably the first line (header) or something
				}

				if (lineSplit[0].equals(testName) && lineSplit[1].equals(testcaseName)) {
					String strStatus = lineSplit[2]; // Success, Failure, Indefinite, NotExecuted, NotExecutable
					status = CUtils.resultUserStringToShort(strStatus);
					resultFound = true;
					break;
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			status = RESULT_FAILURE;
		}

		if (resultFound) {
			return new CReturnStatus(status, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "No valid status found"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}
	
	
	private CReturnStatus parseTabularTestErrorCountResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName) {
		String line = "";
		boolean resultFound = false;
		short status = RESULT_NO_RESULT;
		
		try {
			FileInputStream fileInputStream = new FileInputStream(inputFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);

				String[] lineSplit = line.split("\\W+"); // not an alphanumeric character
				if (lineSplit.length != 2) {
					continue; // probably the first line (header) or something
				}

				if (lineSplit[0].equals(testName)) {
					String strErrorCount = lineSplit[1];
					int errorCount = 1;
					try {
						errorCount = Integer.parseInt(strErrorCount);
						status = (errorCount == 0 ? RESULT_SUCCESS : RESULT_FAILURE);
						resultFound = true;
						break;
						
					} catch (Exception e) {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "No valid error count"));
						status = RESULT_FAILURE;
						break;
					}
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			status = RESULT_FAILURE;
		}

		if (resultFound) {
			return new CReturnStatus(status, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "No valid status found"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}
	
	private CReturnStatus parseTabularTestTestcaseErrorCountResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName, String testcaseName) {
		String line = "";
		boolean resultFound = false;
		short status = RESULT_NO_RESULT;
		
		try {
			FileInputStream fileInputStream = new FileInputStream(inputFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);

				String[] lineSplit = line.split("\\W+"); // not an alphanumeric character
				if (lineSplit.length != 3) {
					continue; // probably the first line (header) or something
				}

				if (lineSplit[0].equals(testName) && lineSplit[1].equals(testcaseName)) {
					String strErrorCount = lineSplit[2];
					int errorCount = 1;
					try {
						errorCount = Integer.parseInt(strErrorCount);
						status = (errorCount == 0 ? RESULT_SUCCESS : RESULT_FAILURE);
						resultFound = true;
						break;
						
					} catch (Exception e) {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "No valid error count"));
						status = RESULT_FAILURE;
						break;
					}
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			status = RESULT_FAILURE;
		}

		if (resultFound) {
			return new CReturnStatus(status, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "No valid status found"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}
	
	private CReturnStatus parseMockitoEasyMockResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName, String testcaseName) {
		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(inputFile);
	    Node rootXmlNode = xmlDocument.getFirstChild();
	    if (rootXmlNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Invalid result file!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: invalid result file!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }
	    
		int nbTestcases =  CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuite/@tests");
		int nbError      = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuite/@errors");
		int nbSkipped    = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuite/@skipped");
		int nbFailure    = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuite/@failures");
		String timestamp = CXmlDocumentFactory.getNodeStringValueFromXPath(rootXmlNode, "/testsuites/testsuite/@timestamp");

		System.err.println("- nbTestcases = " + nbTestcases);
		System.err.println("- nbError     = " + nbError);
		System.err.println("- nbSkipped   = " + nbSkipped);
		System.err.println("- nbFailure   = " + nbFailure);
		System.err.println("- timestamp   = " + timestamp);
				
		if (nbError > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported with an error"));
		} else if (nbFailure > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported as failed"));
		} else if (nbSkipped > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported as skipped"));
		} else {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "All testcases reported as succeeded"));
		}

	    Node testcaseNode = CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, "/testsuite[@name='" + testName + "']/testcase[@name='" + testcaseName + "']");
	    if (testcaseNode == null) {
	    	executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Couldn't find the test or the testcase in the result file"));
	    	return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }

	    // --------------
	    // testcase found
	    // --------------
		short status = RESULT_SUCCESS;

		// failure
	    Node testcaseFailureNode = CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, "/testsuite[@name='" + testName + "']/testcase[@name='" + testcaseName + "']/failure");
	    if (testcaseFailureNode != null) { // Failure detected
	    	status = RESULT_FAILURE;
	    	
	    	try {
				String failureMessage = testcaseFailureNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase failed: message=" + failureMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String failureOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseFailureNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase failed: output=" + failureOutput));
			} catch (Exception e) {
			}
	    }
	    
		// error
	    Node testcaseErrorNode = CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, "/testsuite[@name='" + testName + "']/testcase[@name='" + testcaseName + "']/error");
	    if (testcaseErrorNode != null) { // Error detected
	    	status = RESULT_FAILURE;
	    	
	    	try {
				String errorMessage = testcaseErrorNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase errored: message=" + errorMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String errorOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseErrorNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase errored: output=" + errorOutput));
			} catch (Exception e) {
			}
	    }
		
		// skipped
	    Node testcaseSkippedNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "skipped");
	    if (testcaseSkippedNode != null) { // Skipped detected
	    	status = RESULT_NOT_EXECUTABLE;
	    	
	    	try {
				String skippedMessage = testcaseSkippedNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_NOT_EXECUTABLE, "Testcase skipped: message=" + skippedMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String skippedOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseSkippedNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_NOT_EXECUTABLE, "Testcase skipped: output=" + skippedOutput));
			} catch (Exception e) {
			}
	    }
	    
	    // system-out
	    Node testcaseSystemOutputNode = CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, "/testsuite[@name='" + testName + "']/testcase[@name='" + testcaseName + "']/system-out");
	    if (testcaseNode != null) { // Failure detected
	    	try {
				String systemOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseSystemOutputNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_INFO, "System output=" + systemOutput));
			} catch (Exception e) {
			}
	    }
	    return new CReturnStatus(status, executionSteps);
	}
	
	private CReturnStatus parseGitLabAlt1ResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName, String testcaseName) {
		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(inputFile);
	    Node rootXmlNode = xmlDocument.getFirstChild();
	    if (rootXmlNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Invalid result file!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: invalid result file!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }
	    
		int nbTestcases = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@tests");
		int nbFailure   = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@failures");
		int nbError     = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@errors");
		int nbSkipped   = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@skipped");
		String timestamp = CXmlDocumentFactory.getNodeStringValueFromXPath(rootXmlNode, "/testsuites/testsuite/@timestamp");
		
		System.err.println("- nbTestcases = " + nbTestcases);
		System.err.println("- nbError     = " + nbError);
		System.err.println("- nbSkipped   = " + nbSkipped);
		System.err.println("- nbFailure   = " + nbFailure);
		System.err.println("- timestamp   = " + timestamp);
		
		/*
		if (nbError > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported with an error"));
		} else if (nbFailure > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported as failed"));
		} else if (nbSkipped > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported as skipped"));
		} else {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "All testcases reported as succeeded"));
		}
		*/
		
		/*
		String xPathToTestcase = "/testsuites/testsuite/testcase[@name='" + testName + "\\x5b" + testcaseName + "\\x5d']"; // does not work for any reason !!!
		System.out.println("- - - - xPathToTestcase = " + xPathToTestcase);
		
	    Node testcaseNode = CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, xPathToTestcase); // test="test_network_fifo" and testcase="test_A_1"
	    if (testcaseNode == null) {
	    	System.out.println("- - - - couldn't find " + xPathToTestcase);
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Couldn't find the test or the testcase in the result file"));
	    	return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }
	    */
		
		Node testcaseNode = null;
		NodeList testcaseNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(rootXmlNode, "/testsuites/testsuite/testcase");
		for (int i=0; i<testcaseNodeList.getLength(); i++) {
			testcaseNode = testcaseNodeList.item(i);
			String testcaseNodeName = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
			
			// testcaseNodeName="test_network_fifo[test_A_1]"
			if (testcaseNodeName.equals(testName + "[" + testcaseName + "]")) { // test="test_network_fifo"  testcase="test_A_1"
				System.out.println("found " + testcaseName);
				break;
			}
		}
		
	    if (testcaseNode == null) {
	    	System.err.println("did not find " + testcaseName);
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: couldn't find this testcase " + "[" + testName + "_" + testcaseName + "]" + " in the XML"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }
	    
	    String executionTime = CXmlDocumentFactory.getNodeStringValueFromXPath(testcaseNode, "@time");
	    System.out.println("executionTime " + executionTime);
	    executionSteps.add(new CExecutionStep(timestamp, RESULT_INFO, "execution time: " + executionTime));
 
	    // --------------
	    // testcase found
	    // --------------
		short status = RESULT_SUCCESS;

		// failure
	    Node testcaseFailureNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "failure");
	    if (testcaseFailureNode != null) { // Failure detected
	    	status = RESULT_FAILURE;
	    	
	    	try {
				String failureMessage = testcaseFailureNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase failed: message=" + failureMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String failureOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseFailureNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase failed: output=" + failureOutput));
			} catch (Exception e) {
			}
	    }
	    
		// error
	    Node testcaseErrorNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "error");
	    if (testcaseErrorNode != null) { // Error detected
	    	status = RESULT_FAILURE;
	    	
	    	try {
				String errorMessage = testcaseErrorNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase errored: message=" + errorMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String errorOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseErrorNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase errored: output=" + errorOutput));
			} catch (Exception e) {
			}
	    }
		
		// skipped
	    Node testcaseSkippedNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "skipped");
	    if (testcaseSkippedNode != null) { // Skipped detected
	    	status = RESULT_NOT_EXECUTABLE;
	    	
	    	try {
				String skippedMessage = testcaseSkippedNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_NOT_EXECUTABLE, "Testcase skipped: message=" + skippedMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String skippedOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseSkippedNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_NOT_EXECUTABLE, "Testcase skipped: output=" + skippedOutput));
			} catch (Exception e) {
			}
	    }
	    
	    // system-out
	    Node testcaseSystemOutputNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "system-out");
	    if (testcaseNode != null) { // Failure detected
	    	try {
				String systemOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseSystemOutputNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_INFO, "System output=" + systemOutput));
			} catch (Exception e) {
			}
	    }
	    return new CReturnStatus(status, executionSteps);
	}
	
	
	
	
	
	private CReturnStatus parseGitLabAlt2ResultFile(File inputFile, Vector<CExecutionStep> executionSteps, String testName, String testcaseName) {
		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(inputFile);
	    Node rootXmlNode = xmlDocument.getFirstChild();
	    if (rootXmlNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Invalid result file!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: invalid result file!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }
	    
		int nbTestcases = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@tests");
		int nbFailure   = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@failures");
		int nbError     = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@errors");
		int nbSkipped   = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "/testsuites/testsuite/@skipped");
		String timestamp = CXmlDocumentFactory.getNodeStringValueFromXPath(rootXmlNode, "/testsuites/testsuite/@timestamp");
		
		System.err.println("- nbTestcases = " + nbTestcases);
		System.err.println("- nbError     = " + nbError);
		System.err.println("- nbSkipped   = " + nbSkipped);
		System.err.println("- nbFailure   = " + nbFailure);
		System.err.println("- timestamp   = " + timestamp);
		
		/*
		if (nbError > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported with an error"));
		} else if (nbFailure > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported as failed"));
		} else if (nbSkipped > 0) {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "at least one testcase in this test reported as skipped"));
		} else {
			executionSteps.add(new CExecutionStep(RESULT_INFO, "All testcases reported as succeeded"));
		}
		*/
		
		Node testcaseNode = null;
		NodeList testcaseNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(rootXmlNode, "/testsuites/testsuite/testcase");
		for (int i=0; i<testcaseNodeList.getLength(); i++) {
			testcaseNode = testcaseNodeList.item(i);
			String testcaseNodeName = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
			
			// testcaseNodeName="test_network_fifo[test_A_1]"
			if (testcaseNodeName.endsWith("[" + testName + "_" + testcaseName + "]")) { // test="test_A"  testcase="1"
				System.out.println("found " + testcaseName);
				break;
			}
		}
		
	    if (testcaseNode == null) {
	    	System.err.println("did not find " + testcaseName);
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: couldn't find this testcase " + "[" + testName + "_" + testcaseName + "]" + " in the XML"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }
	    
	    String executionTime = CXmlDocumentFactory.getNodeStringValueFromXPath(testcaseNode, "@time");
	    System.out.println("executionTime " + executionTime);
	    executionSteps.add(new CExecutionStep(timestamp, RESULT_INFO, "execution time: " + executionTime));
 
	    // --------------
	    // testcase found
	    // --------------
		short status = RESULT_SUCCESS;

		// failure
	    Node testcaseFailureNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "failure");
	    if (testcaseFailureNode != null) { // Failure detected
	    	status = RESULT_FAILURE;
	    	
	    	try {
				String failureMessage = testcaseFailureNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase failed: message=" + failureMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String failureOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseFailureNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase failed: output=" + failureOutput));
			} catch (Exception e) {
			}
	    }
	    
		// error
	    Node testcaseErrorNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "error");
	    if (testcaseErrorNode != null) { // Error detected
	    	status = RESULT_FAILURE;
	    	
	    	try {
				String errorMessage = testcaseErrorNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase errored: message=" + errorMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String errorOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseErrorNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, "Testcase errored: output=" + errorOutput));
			} catch (Exception e) {
			}
	    }
		
		// skipped
	    Node testcaseSkippedNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "skipped");
	    if (testcaseSkippedNode != null) { // Skipped detected
	    	status = RESULT_NOT_EXECUTABLE;
	    	
	    	try {
				String skippedMessage = testcaseSkippedNode.getAttributes().getNamedItem("message").getNodeValue();
				executionSteps.add(new CExecutionStep(timestamp, RESULT_NOT_EXECUTABLE, "Testcase skipped: message=" + skippedMessage));
			} catch (DOMException e) {
			}
	    	
	    	try {
				String skippedOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseSkippedNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_NOT_EXECUTABLE, "Testcase skipped: output=" + skippedOutput));
			} catch (Exception e) {
			}
	    }
	    
	    // system-out
	    Node testcaseSystemOutputNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "system-out");
	    if (testcaseNode != null) { // Failure detected
	    	try {
				String systemOutput = CXmlDocumentFactory.getNodeTextOrCDataContent(testcaseSystemOutputNode);
				executionSteps.add(new CExecutionStep(timestamp, RESULT_INFO, "System output=" + systemOutput));
			} catch (Exception e) {
			}
	    }
	    return new CReturnStatus(status, executionSteps);
	}
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl launcher = new CLauncherImpl();
		CReturnStatus returnStatus;
		
		/*
		launcher.resultFileFormat = FORMAT_MOCKITO;
		launcher.resultFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\offline_generic\\sample\\mockito\\");
		
		//File inputFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\offline_generic\\sample\\mockito\\TEST-com.afklm.simone.test.mock.TechnologyServiceEasyMockTest.xml");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		//CReturnStatus returnStatus = launcher.parseMockitoResultFile(inputFile, executionSteps, "com.afklm.simone.test.mock.TechnologyServiceEasyMockTest", "testAddParentTechnology");
		CReturnStatus returnStatus = null;
		returnStatus = launcher.run(0, "", "com.afklm.simone.test.mock.TechnologyServiceEasyMockTest", 0, 0, "testAddParentTechnology", null, null);
		System.err.println("- - - - " + returnStatus);
		returnStatus = launcher.run(0, "", "com.afklm.simone.test.mock.PersonServiceMockitoTest", 0, 0, "testDeleteDevelopedProjects", null, null);
		System.err.println("- - - - " + returnStatus);
		returnStatus = launcher.run(0, "", "com.afklm.simone.test.mock.PersonServiceMockitoTest", 0, 0, "testDeletePersonPicture", null, null);
		System.err.println("- - - - " + returnStatus);
		*/
		

		/*
		launcher.resultFileFormat = FORMAT_GITLAB_ALT1;
		launcher.resultFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\offline_generic\\sample\\gitlab\\input.xml");
		launcher.hashmapCurrentTestcaseStatus = new LinkedHashMap<Integer, Short>();
		
		returnStatus = launcher.run(0, "", "test_network_fifo", 0, 0, "test_A_1", null, null);
		System.err.println("- - - - " + returnStatus);
		
		returnStatus = launcher.run(0, "", "test_network_fifo", 0, 0, "test_B_3c", null, null); // failure
		System.err.println("- - - - " + returnStatus);
		
		returnStatus = launcher.run(0, "", "test_network_fifo", 0, 0, "test_C_4", null, null);
		System.err.println("- - - - " + returnStatus);
		*/
		
		

		launcher.resultFileFormat = FORMAT_GITLAB_ALT2;
		/*
		launcher.resultFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\offline_generic\\sample\\gitlab\\input.xml");
		launcher.resultFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\offline_generic\\sample\\gitlab\\gitlab_xml_skipped.xml")		launcher.hashmapCurrentTestcaseStatus = new LinkedHashMap<Integer, Short>();
		
		returnStatus = launcher.run(0, "", "test_A", 0, 0, "1", null, null);
		System.err.println("- - - - " + returnStatus);
		
		returnStatus = launcher.run(0, "", "test_B", 0, 0, "3c", null, null); // failure
		System.err.println("- - - - " + returnStatus);
		
		returnStatus = launcher.run(0, "", "test_C", 0, 0, "4", null, null);
		System.err.println("- - - - " + returnStatus);
		*/
		
		
		launcher.resultFile = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\offline_generic\\sample\\gitlab\\gitlab_xml_skipped.xml");
		launcher.hashmapCurrentTestcaseStatus = new LinkedHashMap<Integer, Short>();
		
		returnStatus = launcher.run(0, "", "resnet10", 0, 0, "eob", null, null); // skipped
		System.err.println("- - - - " + returnStatus);
		
		returnStatus = launcher.run(0, "", "test_A", 0, 0, "1", null, null); // skipped
		System.err.println("- - - - " + returnStatus);
		
		returnStatus = launcher.run(0, "", "test_B", 0, 0, "13c", null, null); // success
		System.err.println("- - - - " + returnStatus);

		returnStatus = launcher.run(0, "", "test_B", 0, 0, "1b", null, null); // failure
		System.err.println("- - - - " + returnStatus);
		
		returnStatus = launcher.run(0, "", "test_not_existing", 0, 0, "not_existing", null, null); // not found
		System.err.println("- - - - " + returnStatus);

	
	
	
	}
}









