/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.10                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.postman;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for postman.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{postman       }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String commandExecutor;

	private String postmanLocalEnvFileFullName; // if not provided we just ignore it , otherwise we add '.postman_environment.Json'  (e.g. xqualdemo.postman_environment) in teh current folder
	private String postmanGlobalEnvFullName; // if not provided we just ignoare it , otherwise we add ".postman_globals.json "
	private String postmanNewmanOptions; // if not provide we don't care
	private final int postmanRetries = 1;

	private String postmanInterpreter;

	private File workingDir;
	
	
	File postmanHtmlReportFile;
	File postmanXmlJUnitReportFile;
	File postmanJsonReportFile;
	File postmanTraceFile;
	File postmanNewmanReportPath;


	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) {
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");

		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need from the configuration
			testRootPath        = getStringParamValue("General", "Test root path");
			commandExecutor = getStringParamValue("General", "Command executor");

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		try {
			
			postmanLocalEnvFileFullName = getStringParamValue("Postman", "Environment Local variable file");  //Environment Local variable file - XStudio will add '.postman_environment.Json' to this full path and file name
			postmanGlobalEnvFullName  = getStringParamValue("Postman", "Workspace Global variable file");  //Workspace Global variable file - XStudio will add '.postman_globals.json' to this full path and file name
			postmanNewmanOptions  = getStringParamValue("Postman", "Newman options");
			
			
			postmanNewmanReportPath = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") +"newman/");
			
			//create the default interpreter for all tests
			//e.g. newman run Calculator.postman_collection.json -e xqualdemo.postman_environment.json -r html,cli,json,junit  --no-color --disable-unicode
			postmanInterpreter =  CFileUtils.quoteFilePath("newman run") + " " +
			testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".postman_collection.json " +
					
			(postmanLocalEnvFileFullName.length() > 0 ? " --environment " + postmanLocalEnvFileFullName + ".postman_environment.json " : "") +
			(postmanGlobalEnvFullName.length()    > 0 ? "--globals "      + postmanGlobalEnvFullName + ".postman_globals.json " : "") +
			
			"--reporters " + "html,cli,json,junit "  +
			postmanNewmanOptions;
			
			
			// we allow to set some specific attributes  for the test - getting a specific dta file and setting a number of retries
			StringBuffer attributesArgument = new StringBuffer();
			
			for (int k=0; k<attributes.size(); k++) {
				CTestAttribute attribute = attributes.get(k);
				String attributeName = attribute.getName();
				
				traceln(LOG_PRIORITY_INFO, "In for loop :" + attributeName);
		
				if (attributeName.equals("com.xqual.postman.datafile")) {
					attributesArgument.append (" --iteration-data " + attribute.getStringValue());
				}
				else if (attributeName.equals("com.xqual.postman.retries")) {
					attributesArgument.append (" --iteration-count " + attribute.getValue().toString ());
				}
			}	
			
			if (attributesArgument.length() > 0) {
				postmanInterpreter = postmanInterpreter + " "  + attributesArgument.toString();
			}

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();


		workingDir = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "")); // parent folder of all the test classes - in postman case this is where the postman engine is ... different from most other launchers

		// we allow to set some specific parameters for the test case - the folder of teh collection which allows to limit the set of test to be ran
		StringBuffer paramsArgument = new StringBuffer();
        for (int k=0; k<params.size(); k++) {
            CTestcaseParam param = params.get(k);
            String paramName     = param.getName();

			traceln(LOG_PRIORITY_INFO, "In for loop :" + paramName);
		
			if (paramName.equals("com.xqual.postman.folder")) {
					paramsArgument.append (" --folder " + param.getStringValue());
			}
			
			
			if (paramsArgument.length() > 0) {
				postmanInterpreter = postmanInterpreter + " "  + paramsArgument.toString();
			}
        }

		traceln(LOG_PRIORITY_INFO, paramsArgument.toString());
		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		postmanTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "newman/" + testName +".log"); // the output of the console e
		CFileUtils.deleteAndTouchFile(postmanTraceFile);



		// +------------------------------------+
		// | Run the postman test class
		// +------------------------------------+

		postmanInterpreter = (commandExecutor.length() > 0 ? commandExecutor + " " : "")  + postmanInterpreter;
		
		//postmanInterpreter = "CMD /C " + CFileUtils.quoteFilePath("newman run") + " C:\\myPostManCollections\\Calculator.postman_collection.json" + " " + "--environment C:\\myPostManCollections\\xqualdemo.postman_environment.json" + " "  + "--reporters html,cli,json,junit";
	
		CRunner postmanRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName, postmanInterpreter, workingDir);
		
				// here we redirect the output to a file for later parsing
		short result = postmanRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, postmanTraceFile);

		// return code of postman runner: if all tests run successfully, exit with a status of 0. Otherwise exit with a status of 1.
		if (result == RESULT_FAILURE) {
			traceln(LOG_PRIORITY_INFO, "Failure case  : " );
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "XStudio Test case failed"));
			parseResultFile(executionSteps);
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			traceln(LOG_PRIORITY_INFO, "Success case  : " );
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "XStudio Test case succeeded"));
			return parseResultFile(executionSteps);
		}
	}


	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+

	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}

	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,                     // folder to scan
							  "^.*$",                                 // any folder
							  "^.*\\x2epostman_collection\\x2ejson$", // only *.postman_collection.json files
							  true,                                   // remove extension from the file name to determine the test name
							  ".postman_collection.json",             // extension to be removed
							  "/",                                    // separator used in canonicalPath generated
							  "");                                    // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

		private CReturnStatus parseResultFile(Vector < CExecutionStep > executionSteps) {
		// parse the result file to get the result and the execution steps

		boolean errorDetected = true;
		String line = "";

		int errorCount = 0;
		int testcaseTried = 0;
		int testcaseFailures = 0;

		// be default we expect that the session will be a success
		boolean succeededOnSLA = true;
		
		traceln(LOG_PRIORITY_INFO, "newman report path  : " + postmanNewmanReportPath.getAbsolutePath());
		String regexFile;
		String fileName;
		File[] children = postmanNewmanReportPath.listFiles();
		for (int i = 0; i < children.length; i++) {
			File child = children[i];
			traceln(LOG_PRIORITY_INFO, "file found in reports  : " + child.getName());
			regexFile = "^.*\\x2exml$";
			if (Pattern.matches(regexFile, child.getName())) {
				fileName = child.getName();
				traceln(LOG_PRIORITY_INFO, "we found the XML " + fileName);
				postmanXmlJUnitReportFile = new File(postmanNewmanReportPath, fileName);
				traceln(LOG_PRIORITY_INFO, "Junit file from postman : " + postmanXmlJUnitReportFile.getAbsolutePath());
			}
			 regexFile = "^.*\\x2ejson$";
			if (Pattern.matches(regexFile, child.getName())) {
				fileName = child.getName();
				traceln(LOG_PRIORITY_INFO, "we found the JSON " + fileName);
				postmanJsonReportFile = new File(postmanNewmanReportPath, fileName);
			}
			 regexFile = "^.*\\x2ehtml$";
			if (Pattern.matches(regexFile, child.getName())) {
				fileName = child.getName();
				traceln(LOG_PRIORITY_INFO, "we found the HTML " + fileName);
				postmanHtmlReportFile = new File(postmanNewmanReportPath, fileName);
			}
		}
		traceln(LOG_PRIORITY_INFO, "we are going to parse the  junit file  the XML " );
		// we check taht the result file  si available
		if (!postmanXmlJUnitReportFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Junit file from postman test suite execution not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Junit file from postman test suite execution not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Found the Junit file from postman test suite execution : " + postmanXmlJUnitReportFile.toString()));
			addTestcaseMessageAttachmentFolder (postmanNewmanReportPath);
		}
		// then parse it

		//  we search something like <testsuite errors="0" time="0.001" tests="14" skipped="0" failures="14" name="[CS_40] ci_2018-03-16 16:05:33">))
		try {
			FileInputStream fileInputStream = new FileInputStream(postmanXmlJUnitReportFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();

				//We look for a pattern of <testsuite as showed below
				// <?xml version="1.0" encoding="UTF-8"?><testsuite errors="0" time="0.001" tests="14" skipped="0" failures="14" name="[CS_40] ci_2018-03-16 16:05:33">
				
				if (line.indexOf("<testsuites ") > -1) {
					System.out.println("Found hthe global summary line " + line);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Collection  is : " + extractStringFromResultLine(line,"name=",">")));
					errorDetected = false;
				}
				
				
				if (line.indexOf("<testsuite ") > -1) {
					System.out.println("Found a summary line " + line);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Folder / test is : " + extractStringFromResultLine(line,"name=","errors=")));
					errorDetected = false;

					//Then if found we extract and cast to int the number of errors, triies, failed and skipped testcases
					errorCount = extractCountsFromResultLine(line, "errors=", "failures=");
					testcaseTried = extractCountsFromResultLine(line, "tests=", "id=");
					testcaseFailures = extractCountsFromResultLine(line, "failures=", "tests=");

					System.out.println("postman Test case errors --> " + errorCount);
					System.out.println("postman Test case tried  --> " + testcaseTried);
					System.out.println("postmanTest case failed  --> " + testcaseFailures);
					
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case errors : " + errorCount + " ;tried : " + testcaseTried + " ;failed  : " + testcaseFailures));
				}

				if (line.indexOf("<testcase ") > -1) {
					System.out.println("Found the postman test case line " + line);
					if (line.indexOf("classname=\"") > -1) {
						System.out.println("postman testcase failed : " + line);
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "postman Test case failed : " + extractStringFromResultLine(line,"name=","classname=")));
						errorDetected = true;
					}
					else {
						System.out.println("postman  testcase OK : " + line);
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postman Test case succeeded : " + extractStringFromResultLine(line,"name=","/>")));
					}
				}

			}

			// the below resource clousre should go into Finaly but we shoudl don't care as the launcher is  instanciated and killed  rapidly
			bufferedReader.close();
			fileInputStream.close();

	
		}
		catch(Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			errorDetected = true;
		}

		//unless we met some errors and till the SLA is match then teh test case ( hence teh test in our context) is a success
		if (!errorDetected && succeededOnSLA) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Tests executed and all postman test case passed "));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		} else {
			//otherwise the the test case is failed
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Test executed but some postman test cases failed "));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	// utility to extract and parse int an integrer value between 0 and whatever
	private int extractCountsFromResultLine(String line, String firstSubString, String nextSubString) {

		int indexOfAfterSearchedString = line.indexOf(firstSubString) + firstSubString.length();
			System.out.println("indexOfAfterSearchedString  --> " + indexOfAfterSearchedString);
		int indexOfFirstDoubleQuote = line.indexOf("\"", indexOfAfterSearchedString);
			System.out.println("indexOfFirstDoubleQuote  --> " + indexOfFirstDoubleQuote);
		int indexOfSecondDoubleQuote = line.indexOf("\"", indexOfFirstDoubleQuote + 1);
			System.out.println("indexOfSecondDoubleQuote  --> " + indexOfSecondDoubleQuote);

		return (Integer.parseInt(line.substring(indexOfFirstDoubleQuote +1, indexOfSecondDoubleQuote )));

	}

	// utility to extract and parse a string between 2 attributes
	private String extractStringFromResultLine(String line, String firstSubString, String nextSubString) {

		int indexOfAfterSearchedString = line.indexOf(firstSubString) + firstSubString.length();
			System.out.println("indexOfAfterSearchedString  --> " + indexOfAfterSearchedString);
		int indexOfFirstDoubleQuote = line.indexOf("\"", indexOfAfterSearchedString);
			System.out.println("indexOfFirstDoubleQuote  --> " + indexOfFirstDoubleQuote);
		int indexOfSecondDoubleQuote = line.indexOf("\"", indexOfFirstDoubleQuote + 1);
			System.out.println("indexOfSecondDoubleQuote  --> " + indexOfSecondDoubleQuote);

		return (line.substring(indexOfFirstDoubleQuote +1, indexOfSecondDoubleQuote ));

	}

	
}