/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Jean Marin (XQual)                                      |
| Version:     1.4                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/

package com.xqual.xlauncher.protractor;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Hashtable;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.json.JSONArray;
import org.json.JSONObject;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CNetworkInfo;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;


/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for ant build files.
 * @author jmarin
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{protractor  }  ";
	
	static final String TRACE_SELENIUM_START    = "selenium_start_traces.txt";
	static final String TRACE_SELENIUM_SHUTDOWN = "selenium_shutdown_traces.txt";

	static String PROTRACTOR_EXECUTABLE = "";
	
	private static final String JAVA_INTERPRETER_EXE = CUtils.getExecutableName("java");

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String javaHome;
	
	private String nodeJsInstallPath;
	private String npmInstallPath;
	private String chromeDriverPath;
	private String geckoDriverPath;
	private String seleniumServerPath;
	private String seleniumAddress;
	private String seleniumPort;
	private String seleniumOptions;

	private File workingDir;
	
	private File logFile;
	private File reportJSONFile;
	
	private Hashtable<String, String> environment;
	
	CRunner seleniumServer;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath    = getStringParamValue("General",  "Test root path");
			javaHome        = getStringParamValue("General",  "Java Home");
			
			nodeJsInstallPath	= getStringParamValue("Protractor",  "Node.js install path");
			npmInstallPath		= getStringParamValue("Protractor",  "Npm install path");
			chromeDriverPath	= getStringParamValue("Protractor",  "Chrome driver path");
			geckoDriverPath		= getStringParamValue("Protractor",  "Gecko driver path");
			seleniumServerPath	= getStringParamValue("Protractor",  "Selenium server path");
			seleniumAddress		= getStringParamValue("Protractor",  "Selenium address");
			seleniumPort		= getStringParamValue("Protractor",  "Selenium port");
			seleniumOptions		= getStringParamValue("Protractor",  "Selenium options");
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// +------------------------------------+
		// | Set environment variables
		// +------------------------------------+
		environment = new Hashtable<String, String>();
		if (javaHome != null && javaHome.length() > 0) environment.put("JAVA_HOME", javaHome);
		if (nodeJsInstallPath != null && nodeJsInstallPath.length() > 0) environment.put("Path", System.getenv("Path") + ";" + nodeJsInstallPath);
		
    	CNetworkInfo networkInfo = new CNetworkInfo();
    	short osType = networkInfo.getOsType();
    	
		switch (osType) {
			case CNetworkInfo.OS_WIN32: 
			case CNetworkInfo.OS_WIN64: 
				PROTRACTOR_EXECUTABLE = npmInstallPath + "\\" + "protractor.cmd";
    			break;
    		case CNetworkInfo.OS_LINUX: 
    		case CNetworkInfo.OS_MAC:
    			PROTRACTOR_EXECUTABLE = npmInstallPath + "/" + "protractor";
    			break;
    		case CNetworkInfo.OS_UNKNOWN:
    		default:
    			traceln(LOG_PRIORITY_SEVERE, "OS couldn't be determined!");
    			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "OS couldn't be determined!"));
    			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// +------------------------------------+
		// | Run the Selenium server
		// +------------------------------------+
		File seleniumStartTracesFile = new File(TRACE_SELENIUM_START);
		CFileUtils.deleteAndTouchFile(seleniumStartTracesFile);

		workingDir = new File(npmInstallPath);
		
		StringBuffer seleniumServerCommand = new StringBuffer();
		seleniumServerCommand.append(CFileUtils.quoteFilePath(javaHome + "/bin/" + JAVA_INTERPRETER_EXE) + " ");
		seleniumServerCommand.append("-Dwebdriver.chrome.driver=" + chromeDriverPath + " ");
		seleniumServerCommand.append("-Dwebdriver.gecko.driver=" + geckoDriverPath + " ");
		seleniumServerCommand.append("-jar " + seleniumServerPath + " ");
		seleniumServerCommand.append("-port " + seleniumPort + " ");
		seleniumServerCommand.append(seleniumOptions);

		// if there is already a selenium server running that's not a problem has 2 server cannot listen on the same port this will fail nicely
		seleniumServer = new CRunner("[Selenium Server] ",
									 seleniumServerCommand.toString() + " " /*+ seleniumServerOptions*/,
									 workingDir);

		short result = seleniumServer.requestAction(IRunner.START_PROCESS, IRunner.DO_NOT_WAIT_END_OF_EXECUTION, seleniumStartTracesFile);
		if (result == RESULT_SUCCESS) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "initialize: selenium server successfully launched!"));
			traceln(LOG_PRIORITY_INFO, "selenium server launched successfully");
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "initialize: couldn't run selenium server! Maybe a server is already running"));
			traceln(LOG_PRIORITY_SEVERE, "couldn't run selenium server!");
			// Maybe a selenium server is already running... so ignore it.
		}

		addSessionMessageAttachment(seleniumStartTracesFile);
		
		// let the time to the server to initialize (this is done only once per campaign session)
		CUtils.sleep(7*1000); // 7 seconds

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		String scriptParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/";
		workingDir = new File(scriptParentFolderPath);

		String protractorInterpreter = CFileUtils.quoteFilePath(PROTRACTOR_EXECUTABLE) + " " + CFileUtils.quoteFilePath(testName + ".js") + " ";
		protractorInterpreter = protractorInterpreter + "--troubleshoot --logLevel DEBUG --resultJsonOutputFile jsonReport.json ";
		protractorInterpreter = protractorInterpreter + (seleniumAddress.length() > 0 ? "--seleniumAddress http://" + seleniumAddress + (seleniumPort.length() > 0 ? ":" + seleniumPort + "/wd/hub" : "/wd/hub") : "");

		logFile = new File (workingDir + "/XStudioLogFile.txt");
		CFileUtils.deleteAndTouchFile(logFile);
		
		reportJSONFile = new File (workingDir + "/jsonReport.json");
		CFileUtils.deleteAndTouchFile(reportJSONFile);

		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner protractorRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
											   protractorInterpreter,
											   environment,
											   workingDir);
		
		short result = protractorRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, logFile);
		traceln(LOG_PRIORITY_INFO, "returned from protractor runner: " + result);

		if (!logFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps); // nothing to parse
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "log file exists"));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
			addTestcaseMessageAttachment(logFile);
		}

		if (!reportJSONFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "JSON report file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: JSON report file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps); // nothing to parse
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "JSON report file exists"));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: JSON report file found"));
			addTestcaseMessageAttachment(reportJSONFile);
		}

		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "execution returned a code > 0"));
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution succeeded"));
		}
		
		return parseResultFile(executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		
		// +------------------------------------+
		// | Run the Selenium server
		// +------------------------------------+
		traceln(LOG_PRIORITY_SEVERE, "Shutting down selenium server...");
		seleniumServer.killProcess();

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		traceln(LOG_PRIORITY_INFO, "-------------------------------------");
		traceln(LOG_PRIORITY_INFO, "Parsing the log file & json report...");
		traceln(LOG_PRIORITY_INFO, "-------------------------------------");

		// parse the log file to get the result and the execution steps
		String line;
		boolean errorDetected = true;
		
		FileInputStream fileInputStream = null;
		BufferedReader bufferedReader = null;
		try {
			fileInputStream = new FileInputStream(logFile);
			bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				if (line.startsWith("ran") || 
					line.indexOf ("failures,") > -1 ||
					line.indexOf("Protractor version:") > -1 ||
					line.indexOf("running") > -1 ||
					line.indexOf("passed") > -1 ||
					line.indexOf("failed") > -1 ) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));
				}
				
				if (line.indexOf(", 0 failure") > -1) {
					errorDetected = false;
				}
			}
			
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception whle parsing the result file: " + e));
			errorDetected = true;
		}

		
		// parse the json report file to get the details so we can add useful information in the message list
		//System.err.println("- - - - json file: " + CFileUtils.FileToString(reportJSONFile));

		try {
			JSONArray jsonArray = new JSONArray(CFileUtils.FileToString(reportJSONFile));
			traceln(LOG_PRIORITY_INFO, "step#" + jsonArray.length());
			for (int i=0; i<jsonArray.length(); i++) {
				traceln(LOG_PRIORITY_INFO, "- step" + i);
				JSONObject stepDetails = jsonArray.getJSONObject(i);
				String description = (String)stepDetails.get("description");
				int duration = (int)stepDetails.get("duration");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Step: " + description + " (executed in " + duration + "ms)"));
				
				JSONArray assertionArray = stepDetails.getJSONArray("assertions");
				for (int j=0; j<assertionArray.length(); j++) {
					traceln(LOG_PRIORITY_INFO, "- assertion" + j);
					JSONObject assertionDetails = assertionArray.getJSONObject(j);
					boolean passed = (boolean)assertionDetails.get("passed");
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), passed ? RESULT_SUCCESS : RESULT_FAILURE, " > Passed: " + passed));
					if (!passed) errorDetected = true;
				}
			}
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "Couldn't parse the json: " + e);
		}
		
		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
	
	
	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		Vector<CScannedTest> protractorOutput = new Vector<CScannedTest>();
		File protractorBuildFile;
		String protractorCanonicalPath;
		String protractorRelativePath;
		String protractorBuildName;		
		String protractorFileAsString;
		String protractorConfFullyQualifiedName;
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependprotractor path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2ejs$",           // only *.js files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		
			
		for (CScannedTest scannedTest : output) {
			protractorCanonicalPath = scannedTest.getCanonicalPath();
			protractorRelativePath = scannedTest.getRelativePath();
			protractorBuildName = scannedTest.getName();
			
			traceln(LOG_PRIORITY_INFO, "searching in js file if this is a conf file!");
			
			protractorBuildFile = new File(testRootPathFolder + "/" + (protractorCanonicalPath.length() > 0 ? protractorCanonicalPath  + "/" : "" ) + protractorBuildName + ".js");
			
			if (thisIsAProtractorConfFile (protractorBuildFile)) {
				protractorConfFullyQualifiedName = protractorBuildName; 
				protractorOutput.add(new CScannedTest(protractorConfFullyQualifiedName, protractorCanonicalPath));
			}			
		}
		
		
		return protractorOutput;
	}
	
	// extract the project name and all of the targets from the ant build file
	// returns itas a hastable with targetName, projectName
	private boolean thisIsAProtractorConfFile(File protractorBuildFile) {
		// get a file , opens it , find the package , find teh first class and concatenprotractore tis as a package.clasname

		String line = "";
		boolean confFileFound = false;
		
		FileInputStream fileInputStream = null;
		BufferedReader bufferedReader = null;
		try {
			fileInputStream = new FileInputStream(protractorBuildFile);
			bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));
			
			
			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
			
				if (line.indexOf("exports.config =")>=0) {
					confFileFound = true;
				}
				else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
				
			}
			bufferedReader.close();
			fileInputStream.close();

		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the protractorBuild file: " + e);
		}
		
		
		return confFileFound;
	}

}
