/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developer:   Wind River Systems, Inc.                                |
| Version:     see "TRACE_VERSION" constant                            |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.pytest;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Vector;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;

/**
 * This class allows the running of pytest tests from within XStudio.
 * It bases on the the general Python launcher extended it to:
 *   - launch the pytest module, and 
 *   - expect the pytest-specific output 
 * 
 * @author Wind River Systems, Inc.
 *
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {
	/** Trace constant used for logging */
	private static final String TRACE_HEADER = "{pytest        }  ";
	/** 
	 * Major Changes:
	 *      0.5->0.6:   change to SDK 3.1 beta 2
	 * 		0.4->0.5:	change to SDK 3.1 beta 1
	*/
	private static final String TRACE_VERSION = "0.6";
	/** Regular expression for a single line within the pytest output summarizing the test run.
	 * Examples:
	 * ========================== 1 passed in 0.02 seconds =========================== // old version
	 * ============================== 1 passed in 0.02s ============================== // new version
	 * shows 1 test case ran successfully, within 20ms
	 * 
	 * ======================== no tests ran in 0.00 seconds =========================
	 * shows a failure in case no test cases were found, and
	 * 
	 * ========================== 1 failed in 0.05 seconds ===========================
	 * shows 1 test case run with a failure, taking pytest 50ms to execute.
	 * 
	 * ===================== 1 failed, 1 passed in 0.05 seconds ======================
	 * shows 1 test case passed, the other failed, within 50ms 
	*/
	private static final Pattern STATE_REGEX = Pattern.compile("=* (.*) in ([\\p{Alnum}.]*) *s(.*) =*");
	
	/** The pytest result line will have status information, passed is the one we are looking for in success case */
	private static final String SUCCESS = "passed";
	private static final String FAILED = "failed";
	private static final String NOTESTS = "no tests ran";
	
	/** These three status maps are used to later fill the configuration information in terminate */
	private final LinkedHashMap<Integer, Short> fHashmapTestStatus     = new LinkedHashMap<Integer, Short>();
	private final LinkedHashMap<Integer, Short> fHashmapTestcaseStatus = new LinkedHashMap<Integer, Short>();
	private final LinkedHashMap<Integer, String> fHashmapTestcaseMessage = new LinkedHashMap<Integer, String>();
	
	/** BEGIN Parameters from the Python launcher */
	protected String testRootPath;
	protected int timeout = 600;
	protected String pythonInstallPath;
	
	protected File workingDir;
	
	private static final String PYTHON_INTERPRETER_EXE = CUtils.getExecutableName("python");
	
	protected String attribs[][];
	protected String logfilename = "";
	
	/** END parameters from the Python launcher */
	
	public CLauncherImpl(){
		super(TRACE_HEADER);
	}
	/**
	 * This is a copy of the Python launcher initialize function, including a call to the read post processing method 
	*/
	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		/** check the configuration sent by the manager */
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath = getStringParamValue("General", "Test root path");
			timeout = getIntegerParamValue("General", "Asynchronous timeout (in seconds)");
			pythonInstallPath = getStringParamValue("Python", "Python install path");
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		readPostProcessingParameters();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}
	/**
	 * PostRun is identical to Python launcher
	 */
	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}
	@Override
	 /**
	  * On top of the Python launcher terminate: Do post post-processing using the collected maps of status
	 */
	 public CReturnStatus terminate() {
		  if (postProcessingEnabled) {
		      traceln(LOG_PRIORITY_INFO, "post-processing required");
		      String agentId = "";
		      String agentName = "";
		      downloadAndRunPostProcessingScript(fHashmapTestStatus, fHashmapTestcaseStatus, fHashmapTestcaseMessage, agentId, agentName);
		  } else {
		      traceln(LOG_PRIORITY_INFO, "no post-processing required");
		  }

		  Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		  executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		  return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	 }
	/**
	 * Pre-run method: Identical to the Python launcher
	 */
	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");

		attribs = new String[attributes.size()][2];
		for (int i = 0; i < attributes.size(); i++) {
			traceln(LOG_PRIORITY_INFO, "attribute (" + i + ")" + attributes.elementAt(i).getName() + " = " + attributes.elementAt(i).getValue());
			attribs[i][0] = attributes.elementAt(i).getName();
			attribs[i][1] = attributes.elementAt(i).getValue().toString();
		}

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}
	/**
	 * Run method is similar to Python launcher:
	 *  - adds status to an internal map of status for later postprocessing
	 */
	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		// Build attributes list
		traceln(LOG_PRIORITY_INFO, "building attribute list...");

		int testTimeout = timeout;
		String attriblist = "";
		for (int i = 0; i < attribs.length; i++) {
			// Find if timeout attribute is used. If so, replace timeout with custom value
			if (attribs[i][0].equals("timeout")) {
				testTimeout = Integer.parseInt(attribs[i][1]);
				traceln(LOG_PRIORITY_INFO, "Reset timeout to " + testTimeout);
			}
			attriblist = attriblist.concat(" /" + attribs[i][0] + "=\"" + attribs[i][1] + "\"");
		}
		traceln(LOG_PRIORITY_INFO, "attribute list = " + attriblist);
		
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + " testcaseIndex=" + testcaseIndex + attriblist + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, TRACE_HEADER + "(" + TRACE_VERSION + "): initializing."));

		String scriptParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/";
		workingDir = new File(scriptParentFolderPath);
		logfilename = String.format("%s_%s.txt", testName, testcaseIndex);

		/** For now we have empty messages */
		fHashmapTestcaseMessage.put(testcaseId, "");
		File logFile = new File(workingDir + "/" + logfilename);
		CReturnStatus status = runPython(testId, testPath, testName, testcaseId, testcaseIndex, testcaseName, executionSteps, params, testTimeout, logFile);
		if (status.getResult() != CReturnStatus.RESULT_SUCCESS) {
			fHashmapTestcaseStatus.put(testcaseId, status.getResult());
			/** Make the whole test fail the same way */
			fHashmapTestStatus.put(testId, status.getResult());	
			return status;
		}
		if (!logFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file " + logFile.getAbsolutePath() + " not found!"));
			status = new CReturnStatus(RESULT_FAILURE, executionSteps);
			fHashmapTestcaseStatus.put(testcaseId, status.getResult());
			/** Make the whole test fail the same way */
			fHashmapTestStatus.put(testId, status.getResult());
			return status;
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file " + logFile.getAbsolutePath() + " found"));
		}
		addTestcaseMessageAttachment(logFile);

		status = parseResultFile(logFile, executionSteps);
		fHashmapTestcaseStatus.put(testcaseId, status.getResult());
		/** If there is no overall test status, or if the existing one is success: use the new one */
		if (fHashmapTestStatus.get(testId) == null || fHashmapTestStatus.get(testId).equals(CReturnStatus.RESULT_SUCCESS))
			fHashmapTestStatus.put(testId, status.getResult());
		return status;
	}

	/**
	 * Enable the scanner feature, like the Python launcher does
	 */
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	/**
	 *  From the Python launcher: scan for Python files to execute
	*/
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2epy$",           // only *.py files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	/** 
	 * Split the parameter into multiple parameters.
	 * This was necessary w/ ProcessBuilder wrapping space-infested arguments with quotes.
	 *   parameter was -lab x, but the ProcessBuilder made that into command "-lab x"
	 * 
	 * @param parameter
	 * @return
	 */
	private List<String> splitUserParameters(String parameter){
		List<String> toBeReturned = new ArrayList<String>();
		/** Split by spaces, but allow for spaces to be escaped */
		for (String part: parameter.replaceAll("\\\\ ","<spacereplaced>").split(" ")) {
			/** Now make escaped spaces normal spaces. Note that ProcessBuilder will quote the argument */
			toBeReturned.add(part.replaceAll("<spacereplaced>", " "));
		}
		return toBeReturned;
		
	}
	/**
	 * We run Python quite differently to the standard Python launcher:
	 * - Using a processbuilder to construct the command
	 * - Doing away with lots of code around CRunner
	 * - Not using flags like /debug, /testcaseIndex
	 */
	private CReturnStatus runPython(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CExecutionStep>executionSteps, Vector<CTestcaseParam> parameters, int testTimeout, File logFile){
		/** Depending on the testcaseName we attach it to the python file or not */
		String fileAndMethodName = testName + ".py";
		if (!testcaseName.equals("default"))
			fileAndMethodName += "::" + testcaseName;
		
		/** Allow the user to specify the actual python executable: this allows for using python3 over python (2) for example. */
		String pythonExe = pythonInstallPath;
		if (!new File(pythonExe).isFile())
			pythonExe += File.separator + PYTHON_INTERPRETER_EXE;
		
		/** Create a command-line to execute. pytest comes as a module to Python, so one of the more generic errors will be that module not being installed. */
		ProcessBuilder builder = new ProcessBuilder(pythonExe, 
				"-m",
				"pytest",
				testRootPath + (testPath.length() > 0 ? File.separator + testPath : "") + File.separator + fileAndMethodName);
		/** tests usually need other components, let's make sure the test root path is known to python */
		String pythonPath = testRootPath; 
		if (builder.environment().containsKey("PYTHONPATH"))
			pythonPath = builder.environment().get("PYTHONPATH") + File.pathSeparator + pythonPath;
		builder.environment().put("PYTHONPATH", pythonPath);
		/** 
		  Honor the used parameters and attributes with 2 operations: 
		  - add a parameter to the pytest commandline, or
		  - set a variable to the environment
		*/
		for (CTestcaseParam parameter: parameters) {
			if (parameter.getName().startsWith("cmdline_parameter"))
				builder.command().addAll(splitUserParameters(parameter.getValue().toString()));
			else if (parameter.getName().startsWith("environment") && parameter.getValue().toString().contains("="))
				builder.environment().put(parameter.getValue().toString().split("=")[0], parameter.getValue().toString().split("=")[1]);
		}
		for (int i=0; i< attribs.length; i++) {
			if (attribs[i][0].startsWith("cmdline_parameter"))
				builder.command().addAll(splitUserParameters(attribs[i][1]));
			else if (attribs[i][0].startsWith("environment") && attribs[i][1].contains("="))
				builder.environment().put(attribs[i][1].split("=")[0], attribs[i][1].split("=")[1]);
		}
			
		builder.redirectErrorStream(true);
		/** Run within workingDir */
		builder.directory(workingDir);
		/** have all output be stored in a file, which will always be uploaded to the campaign result for tracing */
		builder.redirectOutput(logFile);
        String message = "Launching command: " + Arrays.toString(builder.command().toArray(new String[builder.command().size()]));
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, message));
        traceln(LOG_PRIORITY_INFO, message);
		Process process;
		try {
			process = builder.start();
		} catch (IOException e1) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Process could not be started : " + e1.getMessage()));
			return new CReturnStatus (RESULT_FAILURE, executionSteps);
		}
		/** Assume the process fails, and be proven otherwise */
		int rc = -1;
		double quitAfterMS = timeout * 1000;
		while (true) {
	        try {
	            rc = process.exitValue();
	            traceln(LOG_PRIORITY_INFO, "Command had an exit value of: " + rc);
	            break;
	        }
	        catch (IllegalThreadStateException itse) {
	        	try {
					Thread.sleep(50);
					quitAfterMS-=50;
					/** If it's time: destroy the process so we get out of here in the next iteration */
					if (quitAfterMS <= 0)
						process.destroy();
				} catch (InterruptedException e) {
		            traceln(LOG_PRIORITY_INFO, "Thread interrupted during sleep: " + e.getMessage());
				}
	        }
	    }
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), rc ==0 ? RESULT_SUCCESS : RESULT_FAILURE, "Command had an exit value of: " + rc));
		/**  Continue even if the return code is non-zero: parsing the result becomes even more important for failures, and RESULT_FAILURE would stop the execution here */
		return new CReturnStatus (RESULT_SUCCESS, executionSteps);
	}
	/**
	 * Check the result file
	 * Use a simple strategy: result is an error (unknown) by default
	 * success is only possible of the exact right success line is found
	 *  
	 */
	private CReturnStatus parseResultFile(File resultFile, Vector<CExecutionStep> executionSteps) {
		short status = CReturnStatus.RESULT_UNKNOWN;
		BufferedReader bufferedReader = null;
		try {
			bufferedReader = new BufferedReader(new InputStreamReader(new FileInputStream(resultFile)));
			String line = null;
			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				Matcher matcher = STATE_REGEX.matcher(line);
				if (matcher.matches() && matcher.groupCount() == 3){
					/** Any mention of failure or no tests ran will trigger the overall result to be failure */
					boolean failure = matcher.group(1).contains(FAILED) || matcher.group(2).contains(NOTESTS);
					/** Test has not failed? OK, check if it actually succeeded. Anything else is also a failure */
					if (!failure)
						failure = !matcher.group(1).contains(SUCCESS);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(),failure ? RESULT_FAILURE: RESULT_SUCCESS, "Found result of test run: " + line));
					return new CReturnStatus(failure ? RESULT_FAILURE: RESULT_SUCCESS, executionSteps);
				}
			}
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
		} finally {
			if (bufferedReader != null)
				try {
					bufferedReader.close();
				} catch (IOException e) {
					traceln(LOG_PRIORITY_SEVERE, "exception while closing reader: " + e);
				}
		}

		return new CReturnStatus(status, executionSteps);
	}
	/**
	 * Used for testing
	 * 
	 * @param args
	 */
	public static void main(String[] args){
		CLauncherImpl toto = new CLauncherImpl();
	
		//============================== 1 passed in 0.02s ============================== // new version
		File sample = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\pytest\\sample\\test_subtraktion_1_success.txt");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		CReturnStatus returnStatus = toto.parseResultFile(sample, executionSteps);
		System.out.println("- - - - RESULT test_subtraktion_1_success: " + returnStatus);
		
		//========================== 1 passed in 0.02 seconds =========================== // old version
		sample = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\pytest\\sample\\test_subtraktion_1_success_oldformat.txt");
		executionSteps = new Vector<CExecutionStep>();
		returnStatus = toto.parseResultFile(sample, executionSteps);
		System.out.println("- - - - RESULT test_subtraktion_1_success_oldformat: " + returnStatus);

		//========================== no tests ran in 0.00s ===========================
		sample = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\pytest\\sample\\test_subtraktion_1_norun.txt");
		executionSteps = new Vector<CExecutionStep>();
		returnStatus = toto.parseResultFile(sample, executionSteps);
		System.out.println("- - - - RESULT test_subtraktion_1_norun: " + returnStatus);

		//========================== 1 failed in 0.05s ===========================
		sample = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\pytest\\sample\\test_subtraktion_1_failed.txt");
		executionSteps = new Vector<CExecutionStep>();
		returnStatus = toto.parseResultFile(sample, executionSteps);
		System.out.println("- - - - RESULT test_subtraktion_1_failed: " + returnStatus);

		//========================== 1 failed, 1 passed in 0.05s ===========================
		sample = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\pytest\\sample\\test_subtraktion_1_mix.txt");
		executionSteps = new Vector<CExecutionStep>();
		returnStatus = toto.parseResultFile(sample, executionSteps);
		System.out.println("- - - - RESULT test_subtraktion_1_mix: " + returnStatus);

	}
}
