/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Yossi Kimron                                            |
|              Eric Gavaldo                                            |
| Version:     1.7                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
 */
package com.xqual.xlauncher.python;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CTimeoutListener;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Python.
 * @author Yossi Kimron
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{python        }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private int timeout = 600;
	private String pythonInstallPath;
	private File workingDir;
	
	private static final String PYTHON_INTERPRETER_EXE = CUtils.getExecutableName("python");
	
	String attribs[][];
	String params[][];
	
	String attriblist = "";
	String paramlist = "";
	
	private String logfilename = "";

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath = getStringParamValue("General", "Test root path");
			timeout = getIntegerParamValue("General", "Asynchronous timeout (in seconds)");
			pythonInstallPath = getStringParamValue("Python", "Python install path");
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");

		// +------------------------------------+
		// | Build attributes list
		// +------------------------------------+
		traceln(LOG_PRIORITY_INFO, "building attribute list...");
		attribs = new String[attributes.size()][2];
		for (int i = 0; i < attributes.size(); i++) {
			traceln(LOG_PRIORITY_INFO, "attribute (" + i + ")" + attributes.elementAt(i).getName() + " = " + attributes.elementAt(i).getValue());
			attribs[i][0] = attributes.elementAt(i).getName();
			attribs[i][1] = attributes.elementAt(i).getValue().toString();
		}
		attriblist = "";
		for (int i = 0; i < attribs.length; i++) {
			attriblist = attriblist.concat(" /" + attribs[i][0] + "=\"" + attribs[i][1] + "\"");
		}
		traceln(LOG_PRIORITY_INFO, "attribute list = " + attriblist);

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> parameters, String additionalInfo) {
		// +------------------------------------+
		// | Overload of the config's timeout
		// +------------------------------------+
		int testTimeout = timeout;
		for (int i = 0; i < attribs.length; i++) {
			// Find if timeout attribute is used. If so, replace timeout with custom value
			if (attribs[i][0].equals("timeout")) {
				testTimeout = Integer.parseInt(attribs[i][1]);
				traceln(LOG_PRIORITY_INFO, "Reset timeout to " + testTimeout);
			}
		}
		
		// +------------------------------------+
		// | Build parameters list
		// +------------------------------------+
		traceln(LOG_PRIORITY_INFO, "building parameter list...");
		params = new String[parameters.size()][2];
		for (int i = 0; i < parameters.size(); i++) {
			traceln(LOG_PRIORITY_INFO, "parameter (" + i + ")" + parameters.elementAt(i).getName() + " = " + parameters.elementAt(i).getValue());
			params[i][0] = parameters.elementAt(i).getName();
			params[i][1] = parameters.elementAt(i).getValue().toString();
		}
		paramlist = "";
		for (int i = 0; i < params.length; i++) {
			paramlist = paramlist.concat(" /" + params[i][0] + "=\"" + params[i][1] + "\"");
		}
		traceln(LOG_PRIORITY_INFO, "parameter list = " + paramlist);
		
		
		
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + " testcaseIndex=" + testcaseIndex + attriblist + paramlist + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		String scriptParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/";
		workingDir = new File(scriptParentFolderPath);
		logfilename = String.format("%s_%s.txt", testName, testcaseIndex);

		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner pythonRunner = new CRunner("[" + testId + "] " + testPath + ":" + testName + "." + testcaseIndex, 
										   CFileUtils.quoteFilePath(pythonInstallPath + "/" + PYTHON_INTERPRETER_EXE) + " "
		                                   + CFileUtils.quoteFilePath(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + ".py") + " " + "/debug " + "/testcaseIndex=" + testcaseIndex + " /log=" + CFileUtils.quoteFilePath(logfilename)
		                                   + attriblist
		                                   + paramlist,
		                                   workingDir);

		short result = pythonRunner.requestAction(IRunner.START_PROCESS, IRunner.DO_NOT_WAIT_END_OF_EXECUTION);
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "script interpretation failed"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		// to check if the execution completed correctly, we need to check if
		// the "test_completed.txt" has been created
		File testCompletedFile = new File(workingDir + "/test_completed.txt");
		short resultTimeout = CTimeoutListener.waitForFile(testCompletedFile, testTimeout);
		if (resultTimeout != RESULT_SUCCESS) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "timeout of " + testTimeout + " seconds to execute the test case expired"));
			
			// try to delete test_completed.txt in case it exists
			testCompletedFile.delete();
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		// file exists, remove it to complete asyncronous handshake
		if (!testCompletedFile.delete())
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_UNKNOWN, "Failed to cleanup test_completed.txt"));

		File logFile = new File(workingDir + "/" + logfilename);
		if (!logFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}
		addTestcaseMessageAttachment(logFile);

		return parseResultFile(logFile, executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2epy$",           // only *.py files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File resultFile, Vector<CExecutionStep> executionSteps) {
		String line, message, timestamp;
		boolean errorDetected = false;

		try {
			FileInputStream fileInputStream = new FileInputStream(resultFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));
			// BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream, "UTF-8"));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);

				// Requested format: 2010-01-23 19:00:09 [Success] Any
				// message...
				if (line.indexOf("[Success]") >= 0) {
					timestamp = line.substring(0, line.indexOf("[Success]"));
					message = line.substring(line.indexOf("[Success]") + 10, line.length());
					executionSteps.add(new CExecutionStep(timestamp, RESULT_SUCCESS, message));

				} else if (line.indexOf("[Failure]") >= 0) {
					timestamp = line.substring(0, line.indexOf("[Failure]"));
					message = line.substring(line.indexOf("[Failure]") + 10, line.length());
					executionSteps.add(new CExecutionStep(timestamp, RESULT_FAILURE, message));
					errorDetected = true;

				} else if (line.indexOf("[Log]") >= 0) {
					timestamp = line.substring(0, line.indexOf("[Log]"));
					message = line.substring(line.indexOf("[Log]") + 6, line.length());
					executionSteps.add(new CExecutionStep(timestamp, RESULT_INFO, message));

				} else {
					// traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception whle parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception whle parsing the result file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
}