/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     0.6                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.qftest;

import java.io.File;
import java.io.IOException;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for AutoIt.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{qftest        }  ";

	String QFTEST_EXE;
	
	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String qftestInstallPath;
	private String options;

	private File workingDir;

	//private File traceFile;
	private File tempFolder;
	private File logFile;
	private File reportXmlFolder;
	private File reportXmlFile;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
		
		String os = System.getProperty("os.name");
		if (os.startsWith("Windows")) {
			QFTEST_EXE = CUtils.getExecutableName("qftestc");
		} else { // "Linux", "Mac OS X"
			QFTEST_EXE = CUtils.getExecutableName("qftest"); // there's no qftestc in linux or MacOSX
		}
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath      = getStringParamValue("General", "Test root path");       // i.e. Y:/Externals/qftest/scripts

			qftestInstallPath = getStringParamValue("QF-Test", "QF-Test install path"); // i.e. C:/Program Files/qfs/qftest/qftest-3.4.11
			options           = getStringParamValue("QF-Test", "Options");              // i.e. -batch -compact
			
			workingDir = new File(qftestInstallPath + "/bin");

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		//testAdditionalInfo = additionalInfo;

		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		//testcaseAdditionalInfo = additionalInfo;

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		//traceFile       = new File(workingDir + "/" + TRACES_TESTCASE); // we cannot redirect to a place that will be created a few milliseconds later		

		try {logFile = File.createTempFile("log_", ".grz"); } catch (IOException e1) {traceln(LOG_PRIORITY_SEVERE, "Couldn't create log file!");}

		tempFolder = logFile.getParentFile();
		
		reportXmlFolder = new File(tempFolder.getAbsolutePath() + "/report_" + testName + "_" + testcaseName);
		CFileUtils.deleteFile(reportXmlFolder);
		
		reportXmlFile   = new File(tempFolder.getAbsolutePath() + "/report_" + testName + "_" + testcaseName + "/log_" + testName + "_" + testcaseName + "_report.xml");

		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
        CRunner qftestTestcaseRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
								           CFileUtils.quoteFilePath(qftestInstallPath + "/bin/" + QFTEST_EXE) + " " +
								           options + " " +

										   "-runlog " + CFileUtils.quoteFilePath(CTextUtils.left(logFile.getAbsolutePath(), logFile.getAbsolutePath().length()-4)) + " " + // grz file without the extension
										   "-report " + CFileUtils.quoteFilePath(reportXmlFolder.getAbsolutePath()) + " " +
										   
								           "-test " + CFileUtils.quoteFilePath(testcaseName) + " " +
								           CFileUtils.quoteFilePath(testRootPath + "/" + testPath + "/" + testName + ".qft"),
                                           workingDir);
        short result = qftestTestcaseRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION/*, traceFile*/); // nothing in the console so nothing in the trace file
		/*
        if (result == RESULT_SUCCESS) {
			System.err.println("- - -  success: result = " + result);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "script returned a success"));
		} else {
			System.err.println("- - -  failure result = " + result);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "script returned a failure"));
		}
		*/
        /*
		if (traceFile != null && traceFile.exists()) {
			traceln(LOG_PRIORITY_INFO, "trace file exists");
			addTestcaseMessageAttachment(traceFile);
		}
		*/
        
		if (logFile != null && logFile.exists()) {
			traceln(LOG_PRIORITY_INFO, "log file exists");
			addTestcaseMessageAttachment(logFile);
		}
		
		if (reportXmlFile != null && reportXmlFile.exists()) {
			traceln(LOG_PRIORITY_INFO, "report XML file exists");
			addTestcaseMessageAttachment(reportXmlFile);
			return parseResultFile(reportXmlFile, executionSteps);
			
		} else {
			traceln(LOG_PRIORITY_SEVERE, "report does not exist!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't find XML report to parse!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");

		//the file should be already deleted by the agent's thread
		//deleteFile(new File(TRACES_TESTCASE));

		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eqft$",          // only *.qft files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	

	@Override
	public void recursivelyScanFolderForTests(Vector<CScannedTest> output, File folder, 
												 String regexFolder, String regexFile,
												 boolean removeExtension, String separator, String currentRelativePath) {
		if (folder.isDirectory()) {
			File[] children = folder.listFiles();
			if (children != null) {
				for (int i = 0; i < children.length; i++) {
					File child = children[i];
					if (child.isDirectory()) {
						recursivelyScanFolderForTests(output, child, regexFolder, regexFile, removeExtension, separator, 
													  currentRelativePath.trim().length() > 0 ? currentRelativePath + separator + child.getName() : child.getName());
					} else { // file
						if (Pattern.matches(regexFile, child.getName())) {
							String testName = child.getName();
							if (removeExtension) {
								testName = CFileUtils.getBasename(child);
							}
							
							CScannedTest scannedTest = new CScannedTest(testName, currentRelativePath);
							System.out.println("> found test \"" + currentRelativePath + " : " + testName + "\"");
							
							// check that we do not already have discovered a test with the same name (all the tests will be imported in a unique folder so must have a different name)
							if (!output.contains(scannedTest)) {
								// parse the .qft file to discover the testcases
								Document xmlDocument = CXmlDocumentFactory.createXMLDoc(child);
								NodeList testcaseNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//TestCase");
								for (int j = 0; j < testcaseNodeList.getLength(); j++) {
									Node testcaseNode = testcaseNodeList.item(j);
									String testcaseName = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
									System.out.println("   > found testcase \"" + j + " : " + testcaseName + "\"");
									scannedTest.addScannedTestcase((j+1), testcaseName);
								}
								output.add(scannedTest);
							}
						}
					}
				}
			}
		} else { // should never happen!
		}
	}
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File xmlReportFile, Vector<CExecutionStep> executionSteps) {
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Parsing the reports..."));

		//trick to workaround a problem with the format generated (invalid)
		String xmlContent = CFileUtils.FileToString(xmlReportFile);
		System.out.println("report content = " + xmlContent);
		Document xmlDocument = CXmlDocumentFactory.createXMLDoc(xmlContent);

		boolean atLeastOneFailure = false;
		
		NodeList nodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//testcase");
			
		if (nodeList != null && nodeList.getLength() > 0) {
			System.out.println("nb testcases = " + nodeList.getLength());
			for (int i=0; i<nodeList.getLength(); i++) {
				try {
					Node testcaseNode = nodeList.item(i); 
					String testcaseName = testcaseNode.getAttributes().getNamedItem("name").getNodeValue();
					
					Node summaryNode = CXmlDocumentFactory.getNodeFromXPath(testcaseNode, "summary");
					NamedNodeMap attributes = summaryNode.getAttributes();
					int nbErrors         = Integer.parseInt(attributes.getNamedItem("errors").getNodeValue());
					int nbExpectedErrors = Integer.parseInt(attributes.getNamedItem("expectederrors").getNodeValue());
					int nbExceptions     = Integer.parseInt(attributes.getNamedItem("exceptions").getNodeValue());
					int nbWarnings       = Integer.parseInt(attributes.getNamedItem("warnings").getNodeValue());
					String time          = attributes.getNamedItem("time").getNodeValue();
					
					executionSteps.add(new CExecutionStep(time, nbErrors > 0 ?     RESULT_FAILURE : RESULT_SUCCESS, "errors: " + nbErrors));
					executionSteps.add(new CExecutionStep(time, nbExceptions > 0 ? RESULT_FAILURE : RESULT_SUCCESS, "exceptions: " + nbExceptions));
					executionSteps.add(new CExecutionStep(time, nbWarnings > 0 ?   RESULT_FAILURE : RESULT_SUCCESS, "warnings: " + nbWarnings));

					boolean failed = (nbErrors-nbExpectedErrors > 0 || nbExceptions > 0);
					
					Node testcaseMessageNode = CXmlDocumentFactory.getNodeFromXPath(summaryNode, "//message");
					String testcaseMessage = "";
					if (testcaseMessageNode != null) {
						testcaseMessage = testcaseMessageNode.getFirstChild().getNodeValue();
					}
					
					if (nbErrors-nbExpectedErrors > 0 || nbExceptions > 0) {
						System.out.println(" ===> RESULT_FAILURE");
						atLeastOneFailure = true;
						executionSteps.add(new CExecutionStep(time, RESULT_FAILURE, "[FAILED] " + testcaseMessage));
						
					} else if (nbWarnings > 0) {				
						System.out.println(" ===> RESULT_INDEFINITE");
						executionSteps.add(new CExecutionStep(time, RESULT_INDEFINITE, "[WARNING] " + testcaseMessage));
						
					} else {
						System.out.println(" ===> RESULT_SUCCESS");
						executionSteps.add(new CExecutionStep(time, RESULT_SUCCESS, "[SUCCESS] " + testcaseMessage));
					}
				} catch (Exception e) {
					e.printStackTrace();
					continue;
				}
			}
		} else {
			atLeastOneFailure = true;
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't parse the results xml"));
		}

		return new CReturnStatus(atLeastOneFailure ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
	}
	
	

	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
/*	
	public static void main(String[] args) {
		File inputFile = new File("C:/Documents and Settings/egavaldo/Desktop/Nouveau1.xml");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl myLauncher = new CLauncherImpl();
		
		myLauncher.parseResultFile(inputFile, executionSteps);



		File logFile = null;
		String testName = "test1";
		String testcaseName = "testcase1";
		try {logFile = File.createTempFile("log_", ".grz"); } catch (IOException e1) {}

		File tempFolder = logFile.getParentFile();
		
		File reportXmlFolder = new File(tempFolder.getAbsolutePath() + "/report_" + testName + "_" + testcaseName);
		CFileUtils.deleteFile(reportXmlFolder);
		
		File reportXmlFile   = new File(tempFolder.getAbsolutePath() + "/report_" + testName + "_" + testcaseName + "/log_" + testName + "_" + testcaseName + "_report.xml");
		
		System.err.println("- - - - logFile         = " + logFile);
		System.err.println("- - - - tempFolder      = " + tempFolder);
		System.err.println("- - - - reportXmlFolder = " + reportXmlFolder);
		System.err.println("- - - - reportXmlFile   = " + reportXmlFile);
		System.err.println("- - - - logRun          = " + CTextUtils.left(logFile.getAbsolutePath(), logFile.getAbsolutePath().length()-4));
	}
*/
/*
	public static void main(String[] args) {
		CLauncherImpl launcher = new CLauncherImpl();
		launcher.testRootPath = "Y:\\XStudio\\src\\test\\junit\\xlauncher\\recursivelyScanFolderForTests\\regexfile_not_null";
		Vector<CScannedTest> output = launcher.scanPhysicalTests();
		
		System.err.println("- - - -  tests found = " + output);
	}
*/
}
