/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
|              Yoann Le Bohec                                          |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.qtp;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.CZipWriter;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for QTP.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{qtp/uft       }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String wscriptExePath;
	
	private String resultPrefix;
	private boolean qtpVisible;
	private String qtpWindowState;
	private String qtpView;
	private boolean qtpQuit;
	private String qtpDefaultResult;
	
	private short shortDefaultResult;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath   = getStringParamValue("General", "Test root path");              // i.e. Y:/XStudio/build/tmp/eclipse_classes
			wscriptExePath = getStringParamValue("General", "Windows cscript.exe path");    // i.e. C:/WINDOWS/system32/cscript.exe

			resultPrefix     = getStringParamValue("QTP", "Results folders name prefix");                   // i.e. Res
			qtpVisible       = getBooleanParamValue("QTP", "QTP Visible");                                  // i.e. true
			qtpWindowState   = getStringParamValue("QTP", "QTP window state");                              // i.e. Maximized
			qtpView          = getStringParamValue("QTP", "QTP View");                                      // i.e. ExpertView
			qtpQuit          = getBooleanParamValue("QTP", "Quit QTP after test");                          // i.e. true
			qtpDefaultResult = getStringParamValue("QTP", "Default result when no verification in script"); // i.e. true
			
			if (qtpDefaultResult.equalsIgnoreCase("Success")) {
				shortDefaultResult = RESULT_SUCCESS;
			} else if (qtpDefaultResult.equalsIgnoreCase("Failure")) {
				shortDefaultResult = RESULT_FAILURE;
			} if (qtpDefaultResult.equalsIgnoreCase("Indefinite")) {
				shortDefaultResult = RESULT_INDEFINITE;
			} if (qtpDefaultResult.equalsIgnoreCase("Not Executed")) {
				shortDefaultResult = RESULT_NOT_EXECUTED;
			} 
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		System.out.println("- - - - - testId        = " + testId);
		System.out.println("- - - - - testPath      = " + testPath);
		System.out.println("- - - - - testName      = " + testName);
		System.out.println("- - - - - testcaseId    = " + testcaseId);
		System.out.println("- - - - - testcaseIndex = " + testcaseIndex);
		System.out.println("- - - - - testcaseName  = " + testcaseName);
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

    	String vbScriptPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/runTest_" + testName  + ".vbs";
    	vbScriptPath = vbScriptPath.replaceAll("/", "\\\\");
    	
		String vbTestPath   = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName;
		vbTestPath = vbTestPath.replaceAll("/", "\\\\"); // UFT CANNOT CLOSE PROPERLY THE TEST IF THERE ARE SOME / IN IT !!!
		
		// +------------------------------------+
		// | Generate the VB script
		// +------------------------------------+
		StringBuffer outputBuffer = new StringBuffer();
		
		outputBuffer.append("Dim qtApp 'As QuickTest.Application\n");
		outputBuffer.append("Dim qtTest 'As QuickTest.Test\n");
		outputBuffer.append("Set qtApp = CreateObject(\"QuickTest.Application\")\n"); 
		outputBuffer.append("If qtApp.launched <> True Then\n"); // launch QTP if not already opened
		outputBuffer.append("   qtApp.Launch\n");
		outputBuffer.append("End If\n");
		outputBuffer.append("qtApp.Visible = " + (qtpVisible ? "True" : "False") + "\n");	
		outputBuffer.append("qtApp.WindowState = \"" + qtpWindowState + "\"\n");   
		outputBuffer.append("qtApp.ActivateView \"" + qtpView + "\"\n"); 
		// outputBuffer.append("qtApp.Options.Run.ImageCaptureForTestResults = \"OnError\"\n");
		// outputBuffer.append("qtApp.Options.Run.RunMode = \"Fast\"\n");
		// outputBuffer.append("qtApp.Options.Run.ViewResults = False\n");
		outputBuffer.append("qtApp.Open \"" + vbTestPath + "\", True\n"); // open the test in read-only mode 
		outputBuffer.append("Set qtTest = qtApp.Test\n");
		// outputBuffer.append("qtTest.Settings.Run.OnError = "NextStep"\n"); // Instruct QuickTest to perform next step when error occurs
		outputBuffer.append("qtTest.Run\n"); 
		outputBuffer.append("WScript.StdOut.Write \"Status is: \" & qtTest.LastRunResults.Status\n"); 
		outputBuffer.append("qtTest.Close\n"); // close the test
		if (qtpQuit) outputBuffer.append("qtApp.Quit\n");  // close QTP
		// outputBuffer.append("Set qtResultsOpt = Nothing\n"); 
		outputBuffer.append("Set qtTest = Nothing\n");
		outputBuffer.append("Set qtApp = Nothing\n");
	
		/*
		To specify where to output results:
		Set qtResultsOpt = CreateObject("QuickTest.RunResultsOptions") 
		qtResultsOpt.ResultsLocation = "D:\Result"
		qtTest.Run qtResultsOpt,True 'Run the test and wait until end of the test run
		Set qtResultsOpt = Nothing
		*/
		
		/*
		to associate libraries
		Set qtLibraries = App.Test.Settings.Resources.Libraries
		'If the library file "libraary.vbs" is not assiciates with the Test then associate it
		If qtLibraries.Find("D:\libraary.vbs") = -1 Then
		    qtLibraries.Add "D:\libraary.vbs", 1 
		End If
		Set qtLibraries = Nothing
		*/
		
		
		/* Test options:
		Dim App 'As Application
		Set App = CreateObject("QuickTest.Application")
		App.Launch
		App.Visible = True
		App.Test.Settings.Launchers("Web").Active = False
		App.Test.Settings.Launchers("Web").Browser = "IE"
		App.Test.Settings.Launchers("Web").Address = "http://www.rediff.com"
		App.Test.Settings.Launchers("Web").CloseOnExit = True
		App.Test.Settings.Launchers("Windows Applications").Active = True
		App.Test.Settings.Launchers("Windows Applications").Applications.RemoveAll
		App.Test.Settings.Launchers("Windows Applications").RecordOnQTDescendants = True
		App.Test.Settings.Launchers("Windows Applications").RecordOnExplorerDescendants = False
		App.Test.Settings.Launchers("Windows Applications").RecordOnSpecifiedApplications = True
		App.Test.Settings.Run.IterationMode = "rngAll"
		App.Test.Settings.Run.StartIteration = 1
		App.Test.Settings.Run.EndIteration = 1
		App.Test.Settings.Run.ObjectSyncTimeOut = 20000
		App.Test.Settings.Run.DisableSmartIdentification = False
		App.Test.Settings.Run.OnError = "Dialog"
		App.Test.Settings.Resources.DataTablePath = "<Default>"
		App.Test.Settings.Resources.Libraries.RemoveAll
		App.Test.Settings.Web.BrowserNavigationTimeout = 60000
		App.Test.Settings.Web.ActiveScreenAccess.UserName = ""
		App.Test.Settings.Web.ActiveScreenAccess.Password = ""
		*/
		
		/* QTP options:
		Dim App 'As Application
		Set App = CreateObject("QuickTest.Application")
		App.Launch
		App.Visible = True
		App.Options.DisableVORecognition = False
		App.Options.AutoGenerateWith = False
		App.Options.WithGenerationLevel = 2
		App.Options.TimeToActivateWinAfterPoint = 500
		App.Options.SaveLoadAndMonitorData = True
		App.Options.Run.RunMode = "Fast"
		App.Options.Run.ViewResults = True
		App.Options.Run.StepExecutionDelay = 0
		App.Options.Run.MovieCaptureForTestResults = "Never"
		App.Options.Web.AddToPageLoadTime = 10
		App.Options.Web.RecordCoordinates = False
		App.Options.Web.RecordMouseDownAndUpAsClick = False
		App.Options.Web.RecordAllNavigations = False
		App.Options.Web.RecordByWinMouseEvents = ""
		App.Options.Web.BrowserCleanup = False
		App.Options.Web.RunOnlyClick = False
		App.Options.Web.RunMouseByEvents = True
		App.Options.Web.RunUsingSourceIndex = True
		App.Options.Web.EnableBrowserResize = True
		App.Options.Web.PageCreationMode = "URL"
		App.Options.Web.CreatePageUsingUserData = "Get Post"
		App.Options.Web.CreatePageUsingNonUserData = ""
		App.Options.Web.CreatePageUsingAdditionalInfo = True
		App.Options.Web.FrameCreationMode = "URL"
		App.Options.Web.CreateFrameUsingUserData = "Get Post"
		App.Options.Web.CreateFrameUsingNonUserData = ""
		App.Options.Web.CreateFrameUsingAdditionalInfo = True
		App.Options.WindowsApps.AttachedTextRadius = 35
		App.Options.WindowsApps.AttachedTextArea = "TopLeft"
		App.Options.WindowsApps.ExpandMenuToRetrieveProperties = True
		App.Options.WindowsApps.NonUniqueListItemRecordMode = "ByName"
		App.Options.WindowsApps.RecordOwnerDrawnButtonAs = "PushButtons"
		App.Options.WindowsApps.ForceEnumChildWindows = 0
		App.Options.WindowsApps.ClickEditBeforeSetText = 0
		App.Options.WindowsApps.VerifyMenuInitEvent = 0
		App.Options.TextRecognitionOrder = "APIThenOCR"
		App.Folders.RemoveAll
		*/

		
		File vbScriptFile = new File(vbScriptPath);
		CFileUtils.deleteFile(vbScriptFile);
		try {
			vbScriptFile.createNewFile();
		    Writer output = new BufferedWriter(new FileWriter(vbScriptFile));
		    output.write(outputBuffer.toString());
		    output.close();
		} catch (IOException e) {
			traceln(LOG_PRIORITY_SEVERE, "Could not create file \"" + vbScriptFile + "\"");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Could not create file \"" + vbScriptFile + "\""));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		traceln(LOG_PRIORITY_INFO, vbScriptFile + " has been generated");

		
		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		File qtpExecutionTraces = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "qtp_traces.txt");
		CFileUtils.deleteAndTouchFile(qtpExecutionTraces);
		
		
		CRunner qtpRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,
		                                wscriptExePath.toString() + " \"" + vbScriptPath + "\"");

		// here we redirect the output to a file for later parsing
		short result = qtpRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, qtpExecutionTraces);

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(qtpExecutionTraces);

		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution process failed"));
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution process succeeded"));
		}
		
		return parseResultFile(new File(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName), executionSteps); // to have the details anyway
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	private void recursivelyScanFolderForTests(Vector<CScannedTest> output, File folder, String regexFolder, String separator, String currentRelativePath) {
		if (folder.isDirectory()) {
			File[] children = folder.listFiles();
			if (children != null) {
				for (int i = 0; i < children.length; i++) {
					File child = children[i];
					if (child.isDirectory()) {
						if (Pattern.matches(regexFolder, child.getName())) {
							
							if (!child.getName().startsWith("Res") && !child.getName().startsWith("Report") && !child.getName().startsWith("Action") && !child.getName().startsWith("SnapShots")) {
								String testName = child.getName();
								CScannedTest scannedTest = new CScannedTest(testName, currentRelativePath);
								System.out.println("> found test \"" + currentRelativePath + separator + testName + "\"");
								
								// check that we do not already have discovered a test with the same name (all the tests will be imported in a unique folder so must have a different name)
								if (!output.contains(scannedTest)) { 
									// search for testcases here !
									scannedTest.addScannedTestcase(10, "default");
									output.add(scannedTest);
								}
	
								recursivelyScanFolderForTests(output, child, regexFolder, separator, currentRelativePath.trim().length() > 0 ? currentRelativePath + separator + child.getName() : child.getName());
							}
	
						} else {
							System.err.println("skipping folder \"" + currentRelativePath + separator + child.getName() + "\"...");
						}
						
					}
				}
			}
		} else { // should never happen!
		}
	}
	
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File testFolder, Vector<CExecutionStep> executionSteps) {
		traceln(LOG_PRIORITY_INFO, "parsing result folder " + testFolder + "...");

		short result = RESULT_FAILURE;

	    int nbWarning = 0;
		int nbFails   = 0;
		int nbSuccess = 0;
		
		if (testFolder == null || !testFolder.exists() || !testFolder.isDirectory()) {
			traceln(LOG_PRIORITY_INFO, "couldn't find the folder " + testFolder + "...");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't find the folder"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		File[] childFolders = testFolder.listFiles();
		if (childFolders == null) {
			traceln(LOG_PRIORITY_INFO, "empty test folder " + testFolder);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "empty test folder"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);			
		}
		int index = -1;
		File resultsFile = null;
		boolean missingReport = false;
		for (File folder: childFolders) {
			System.out.println("-> " + folder);
			String folderName = folder.getName();
			if (folder.isDirectory() && folderName.startsWith(resultPrefix)) {
				//traceln(LOG_PRIORITY_INFO, "found a results folder candidate: " + folder);
				try {
					int currentIndex = Integer.parseInt(folderName.substring(resultPrefix.length()));
					traceln(LOG_PRIORITY_INFO, "current index: " + folderName.substring(resultPrefix.length()) + "  --->  " + currentIndex);
					if (currentIndex > index) { // only keeps the latest one
						index = currentIndex;
						
						// old naming
						resultsFile = new File(testFolder.getAbsolutePath() + "/" + resultPrefix + currentIndex + "/Report/Results.xml");
						if (resultsFile.exists()) {
							traceln(LOG_PRIORITY_INFO, "        > old naming report exists: " + index);
							missingReport = false; // we ensure the last report we found exists (if some previous reports does not exist we don't care about it)
						} else {
							
							// new naming
							resultsFile = new File(testFolder.getAbsolutePath() + "/" + resultPrefix + currentIndex + "/Report/run_results.xml");
							if (resultsFile.exists()) {
								traceln(LOG_PRIORITY_INFO, "        > new naming report exists: " + index);
								missingReport = false; // we ensure the last report we found exists (if some previous reports does not exist we don't care about it)
							} else {
								traceln(LOG_PRIORITY_INFO, "        > report does NOT exists: " + index);
								missingReport = true;
							}
						}
					}
				} catch (NumberFormatException e) {
					traceln(LOG_PRIORITY_INFO, "the rest of the name is not an integer: " + folderName);
				}
			}
		}
		if (index == -1) {
			traceln(LOG_PRIORITY_INFO, "couldn't find appropriate result folder");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't find appropriate result folder"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);				
		}
		
		File latestResultsFolder = new File(testFolder.getAbsolutePath() + "/" + resultPrefix + index);
		traceln(LOG_PRIORITY_INFO, "latest results folder found: " + latestResultsFolder);
		
		if (missingReport) {
			traceln(LOG_PRIORITY_INFO, "report missing for this result folder");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "report missing for this result folder"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);	
		}
		
		// search potential HTML reports in resx
		File[] htmlFiles = latestResultsFolder.listFiles();
		if (htmlFiles != null) {
			for (File htmlFile: htmlFiles) {
				//System.err.println(" extension = " + CFileUtils.getExtension(htmlFile));
				String extension = CFileUtils.getExtension(htmlFile);
				if (extension != null && extension.equalsIgnoreCase("html")) {
			    	traceln(LOG_PRIORITY_INFO, "attaching html report " + htmlFile + "...");
			    	addTestcaseMessageAttachment(htmlFile);
				}
			}
		}
	
		// zip the full "Report" folder
		try {
			File reportFolder = new File(testFolder.getAbsolutePath() + "/" + resultPrefix + index + "/Report");
			File zippedReport = new File(System.getProperty("java.io.tmpdir") + "/qtp_full_report.zip");
			CFileUtils.deleteFile(zippedReport);
			CZipWriter zipWriter = new CZipWriter(zippedReport.getAbsolutePath());
			zipWriter.zipFolder(reportFolder.getAbsolutePath());
			addTestcaseMessageAttachment(zippedReport);
		} catch (Exception e1) {
			e1.printStackTrace();
		}
		

		// parse the result file to get the result and the execution steps
		addTestcaseMessageAttachment(resultsFile);
		traceln(LOG_PRIORITY_INFO, "reading " + resultsFile + "...");
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(resultsFile);
	    
	    // +-----------------------------------------+
	    // |                OLD FORMAT               |
	    // +-----------------------------------------+
	    if (resultsFile.getName().equalsIgnoreCase("Results.xml")) {

		    // attach all the excel sheets used for this test
		    NodeList tableNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//NodeArgs[@eType='Table']/BtmPane[@vType='Table']/Path");
		    if (tableNodeList != null) {
			    for (int i=0; i<tableNodeList.getLength(); i++) {
			    	Node resultNode = tableNodeList.item(i);
			    	String tableName = resultNode.getFirstChild().getNodeValue();
			    	File tableFile = new File(testFolder.getAbsolutePath() + "/" + resultPrefix + index + "/Report/" + tableName);
			    	traceln(LOG_PRIORITY_INFO, "attaching excel sheet " + tableFile + "...");
			    	addTestcaseMessageAttachment(tableFile);
			    }
		    }
	
		    NodeList iterationNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//DIter");
		    if (iterationNodeList != null) {
			    if (iterationNodeList.getLength() > 0) {
				    for (int j=0; j<iterationNodeList.getLength(); j++) {
				    	Node iterationNode = iterationNodeList.item(j);
				    	
				    	String iterationIndex = iterationNode.getAttributes().getNamedItem("iterID").getNodeValue();
				    	System.err.println("============ Iteration " + iterationIndex + " =============");
				    	
						NodeList resultNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(iterationNode, "Action//Step/NodeArgs[@eType='Replay' or @eType='Context' or @eType='User']");
					    for (int k=0; k<resultNodeList.getLength(); k++) {
					    	String type   = "";
					    	String status = "";
					    	String time   = "";
					    	String desc   = "";
					    	
					    	Node resultNode = resultNodeList.item(k);
					    	
					    	try {type = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "@eType");} catch (Exception e) {}
					    	
					    	try {status = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "@status");} catch (Exception e) {}
					    	
					    	Node timeNode = CXmlDocumentFactory.getNodeFromXPath(resultNode, "preceding-sibling::Time[1]");
					    	try {time   = timeNode.getFirstChild().getNodeValue();} catch (Exception e) {}
					    		
					    	Node descNode = CXmlDocumentFactory.getNodeFromXPath(resultNode, "Disp");
					    	try {desc   = descNode.getFirstChild().getNodeValue();} catch (Exception e) {}
					    	
					    	//System.err.println("- Result: " + status + " : " + type + " : " + desc);
					    	
					    	if (status.equals("Passed")) {
					    		if (!desc.startsWith("LoadFunctionLibrary")) { // hide the possibly very numerous successfull LoadFunctionLibrary
						    		System.err.println(k + " SUCCESS: " + iterationIndex + " " + type + " " + desc);
						        	executionSteps.add(new CExecutionStep(time, RESULT_SUCCESS, "#" + iterationIndex + " " + type + " " + desc));
						        	nbSuccess++;
					    		}
			
					    	} else if (status.equals("Failed")) {
					    		System.err.println(k + " FAILURE: " + iterationIndex + " " + type + " " + desc);
					        	executionSteps.add(new CExecutionStep(time, RESULT_FAILURE, "#" + iterationIndex + " " + type + " " + desc));
					        	nbFails++;
			
					    	} else if (status.equals("Warning")) {
					    		System.err.println(k + " RELATIVE: " + iterationIndex + " " + type + " " + desc);
					        	executionSteps.add(new CExecutionStep(time, RESULT_INDEFINITE, "#" + iterationIndex + " " + type + " " + desc));
					        	nbWarning++;
	
					    	} else if (status.equals("Done")) {
					    		// ignore
			
					    	} else { // something else
					    		System.err.println(k + " UNKNOWN: " + status + ": " + iterationIndex + " " + type + " " + desc);
					    		executionSteps.add(new CExecutionStep(time, RESULT_NOT_EXECUTED, "#" + iterationIndex + " " + type + " " + desc));
					    	}
					    }	    	
				    }
			    } else {
					NodeList resultNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//Action//Step/NodeArgs[@eType='Replay' or @eType='Context' or @eType='User']");
				    for (int k=0; k<resultNodeList.getLength(); k++) {
				    	String type   = "";
				    	String status = "";
				    	String time   = "";
				    	String desc   = "";
				    	
				    	Node resultNode = resultNodeList.item(k);
				    	
				    	try {type = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "@eType");} catch (Exception e) {}
				    	
				    	try {status = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "@status");} catch (Exception e) {}
				    	
				    	Node timeNode = CXmlDocumentFactory.getNodeFromXPath(resultNode, "preceding-sibling::Time[1]");
				    	try {time   = timeNode.getFirstChild().getNodeValue();} catch (Exception e) {}
				    		
				    	Node descNode = CXmlDocumentFactory.getNodeFromXPath(resultNode, "Disp");
				    	try {desc   = descNode.getFirstChild().getNodeValue();} catch (Exception e) {}
				    	
				    	//System.err.println("- Result: " + status + " : " + type + " : " + desc);
				    	
				    	if (status.equals("Passed")) {
				    		if (!desc.startsWith("LoadFunctionLibrary")) { // hide the possibly very numerous successfull LoadFunctionLibrary
					    		System.err.println(k + " SUCCESS: " + type + " " + desc);
					        	executionSteps.add(new CExecutionStep(time, RESULT_SUCCESS, "#0 " + type + " " + desc));
					        	nbSuccess++;
				    		}
		
				    	} else if (status.equals("Failed")) {
				    		System.err.println(k + " FAILURE: " + type + " " + desc);
				        	executionSteps.add(new CExecutionStep(time, RESULT_FAILURE, "#0 " + type + " " + desc));
				        	nbFails++;
		
				    	} else if (status.equals("Warning")) {
				    		System.err.println(k + " RELATIVE: " + type + " " + desc);
				        	executionSteps.add(new CExecutionStep(time, RESULT_INDEFINITE, "#0 " + type + " " + desc));
				        	nbWarning++;
		
				    	} else if (status.equals("Done")) {
				    		// ignore
		
				    	} else { // something else
				    		System.err.println(k + " UNKNOWN: " + type + " " + desc);
				    		executionSteps.add(new CExecutionStep(time, RESULT_NOT_EXECUTED, "#0 " + type + " " + desc));
				    	}
				    }	    		    	
			    }
		    } else {
		    	nbFails++;
		    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_NOT_EXECUTED, "No DIter nodes in the report!"));
		    }
		    
		    result = shortDefaultResult;
		    System.err.println("===> nbSuccess = " + nbSuccess);
		    System.err.println("===> nbFails   = " + nbFails);
		    System.err.println("===> nbWarning = " + nbWarning);
		    if (nbFails == 0 && nbWarning == 0 && nbSuccess > 0) {
		    	System.err.println("===> SUCCESS");
		    	result = RESULT_SUCCESS;
		    } else if (nbFails > 0) {
		    	System.err.println("===> FAILURE");
		    	result = RESULT_FAILURE;
		    } else {
		    	System.err.println("===> DEFAULT");
		    }
	    
	    // +-----------------------------------------+
	    // |                NEW FORMAT               |
	    // +-----------------------------------------+
		} else if (resultsFile.getName().equalsIgnoreCase("run_results.xml")) {
			
		    // attach all the excel sheets used for this test
			/* Is there an equivalent in the new format?
		    NodeList tableNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//NodeArgs[@eType='Table']/BtmPane[@vType='Table']/Path");
		    if (tableNodeList != null) {
			    for (int i=0; i<tableNodeList.getLength(); i++) {
			    	Node resultNode = tableNodeList.item(i);
			    	String tableName = resultNode.getFirstChild().getNodeValue();
			    	File tableFile = new File(testFolder.getAbsolutePath() + "/" + resultPrefix + index + "/Report/" + tableName);
			    	traceln(LOG_PRIORITY_INFO, "attaching excel sheet " + tableFile + "...");
			    	addTestcaseMessageAttachment(tableFile);
			    }
		    }
		    */
	

			//NodeList resultNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//ReportNode[@type='Action']/ReportNode[@type='Iteration']//ReportNode[@type='User' ]");
			//NodeList resultNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//ReportNode[@type='Iteration']/ReportNode[@type='Action']//ReportNode[@type='User' ]"); // UFT 12.53
			NodeList resultNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//ReportNode[@type='Iteration' or @type='Action']//ReportNode[@type='User' ]");
		    for (int k=0; k<resultNodeList.getLength(); k++) {
		    	String name   = "";
		    	String status = "";
		    	String time   = "";
		    	String desc   = "";
		    	
		    	Node resultNode = resultNodeList.item(k);
		    	
		    	try {name   = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "Data/name");} catch (Exception e) {}
		    	try {status = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "Data/Result");} catch (Exception e) {}
		    	try {time   = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "Data/StartTime");} catch (Exception e) {}
		    	try {desc   = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "Data/Description");} catch (Exception e) {}
		    	
		    	//System.err.println("- Result: " + status + " : " + type + " : " + desc);
		    	
		    	if (status.equals("Passed")) {
		    		if (!desc.startsWith("LoadFunctionLibrary")) { // hide the possibly very numerous successfull LoadFunctionLibrary
			    		System.err.println(k + " SUCCESS: " + name + " " + desc);
			        	executionSteps.add(new CExecutionStep(time, RESULT_SUCCESS, name + " " + desc));
			        	nbSuccess++;
		    		}

		    	} else if (status.equals("Failed")) {
		    		System.err.println(k + " FAILURE: " + name + " " + desc);
		        	executionSteps.add(new CExecutionStep(time, RESULT_FAILURE, name + " " + desc));
		        	nbFails++;

		    	} else if (status.equals("Warning")) {
		    		System.err.println(k + " RELATIVE: " + name + " " + desc);
		        	executionSteps.add(new CExecutionStep(time, RESULT_INDEFINITE, name + " " + desc));
		        	nbWarning++;

		    	} else if (status.equals("Done")) {
		    		// ignore

		    	} else { // something else
		    		System.err.println(k + " UNKNOWN: " + status + ": " + name + " " + desc);
		    		executionSteps.add(new CExecutionStep(time, RESULT_NOT_EXECUTED, name + " " + desc));
		    	}
		    }

		    
		    result = shortDefaultResult;
		    System.err.println("===> nbSuccess = " + nbSuccess);
		    System.err.println("===> nbFails   = " + nbFails);
		    System.err.println("===> nbWarning = " + nbWarning);
		    if (nbFails == 0 && nbWarning == 0 && nbSuccess > 0) {
		    	System.err.println("===> SUCCESS");
		    	result = RESULT_SUCCESS;
		    } else if (nbFails > 0) {
		    	System.err.println("===> FAILURE");
		    	result = RESULT_FAILURE;
		    } else {
		    	System.err.println("===> DEFAULT");
		    }
		}

		return new CReturnStatus(result, executionSteps);
	}
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl launcher = new CLauncherImpl();
		launcher.resultPrefix = "Res";
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		// old format: Results.xml
		// -----------------------
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/Schedule Path SSRs by Doctor"), executionSteps);
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/qtp_full_report_[T-4]_Auto BatchClose Processor_results"), executionSteps); // HUGE REPORTS !!!!!
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/qtp_full_report_[T-26]_TFS 2304_Remove_TID_Check_for_Refunds_results"), executionSteps);
		
		// new format: run_results.xml
		// ---------------------------
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/qtp_full_report_[T-246]_Signoff_Now_result"), executionSteps);
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/qtp_full_report_[T-114]_DCC_Certification_Suite_run_results"), executionSteps);
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/Test3"), executionSteps);
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/Other1"), executionSteps);
		launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/Other2"), executionSteps);
		//launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/qtp/sample/Other3"), executionSteps);
		
		// Scanner
		// -------
		/*
		launcher.testRootPath = "Y:\\XStudio\\src\\com\\xqual\\xlauncher\\qtp\\";
		Vector<CScannedTest> scannedTests = launcher.scanPhysicalTests();
		System.err.println("- - - - scannedTests = " + scannedTests);
		*/
		System.out.println(executionSteps);
	}
	
}
