/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.6                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.random;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.CTestExecution;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for a fake launcher.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{random        }  ";

	// parameters impacting executing at run time set by the test operator
	private int maxTestcaseDelay;
	private boolean preRunAndPostRunTakeTime;
	private int preRunDelay;
	private int postRunDelay;

	Vector<CTestAttribute> attributes = null;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		printSutCustomFields();
		
		//System.err.println("----------- Test Prerequisites -----------");
		//System.out.println(getTestPrerequisiteLinkedHashMap() + "\n");

		//System.err.println("----------- Test Descriptions ------------");
		//System.out.println(getTestDescriptionLinkedHashMap() + "\n");
		
		//System.err.println("----------- Test Custom Fields -----------");
		//System.out.println("> Boolean:\n" + testBooleanCustomFieldsHashtable + "\n");
		//System.out.println("> Integer:\n" + testIntegerCustomFieldsHashtable + "\n");
		//System.out.println("> String:\n" + testStringCustomFieldsHashtable + "\n");
		//System.out.println("> Html:\n" + testHtmlCustomFieldsHashtable + "\n");
		//System.out.println("> Choice:\n" + testStringChoiceCustomFieldsHashtable + "\n");

		
		//System.err.println("---------- Testcase Descriptions ---------");
		//System.out.println(getTestcaseDescriptionLinkedHashMap() + "\n");

		//System.err.println("------------- Testcase Steps -------------");
		//System.out.println(getTestcaseStepsLinkedHashMap() + "\n");
		
		//System.err.println("------------ Testcase Editor -------------");
		//System.out.println(getTestcaseEditorTypeLinkedHashMap() + "\n");
		
		//System.err.println("--------- Testcase Custom Fields ---------");
		//System.out.println("> Boolean:\n" + testcaseBooleanCustomFieldsHashtable + "\n");
		//System.out.println("> Integer:\n" + testcaseIntegerCustomFieldsHashtable + "\n");
		//System.out.println("> String:\n" + testcaseStringCustomFieldsHashtable + "\n");
		//System.out.println("> Html:\n" + testcaseHtmlCustomFieldsHashtable + "\n");
		//System.out.println("> Choice:\n" + testcaseStringChoiceCustomFieldsHashtable + "\n");

		System.err.println("--------- Test attributes (including overloading) ---------");
		System.err.println("> test-attributes:\n" + this.getTestAttributeValuesHashMap() + "\n");

		System.err.println("--------- Testcase params (including overloading) ---------");
		System.err.println("> testcase-params:\n" + this.getTestcaseParamValuesHashMap() + "\n");

		try {
			// retrieve the parameters we need
			maxTestcaseDelay         = getIntegerParamValue("Timing", "max time between testcases (ms)");
			preRunAndPostRunTakeTime = getBooleanParamValue("Timing", "prerun and postrun take time to execute");
			preRunDelay              = getIntegerParamValue("Timing", "time to execute prerun (ms)");
			postRunDelay             = getIntegerParamValue("Timing", "time to execute postrun (ms)");
			
			// addSessionMessageAttachment(new File("C:\\test_repository\\data\\attachment_session_1.xml"));

			// Retrieve all the testcase names that are executable.
			// This can be usefull only with launcher that run complete testsuites
			// In those case, we probably need to provide the test framework, the list of executable testcases (i.e. based on "ready for automated run" or assets rules)
			traceln(LOG_PRIORITY_INFO, "ids of all the not executable testcases: " + notExecutableTestcaseIdVector);
			for (int indexTest=0; indexTest<testExecutionVector.size(); indexTest++) {
				CTestExecution testExecution = testExecutionVector.get(indexTest);
				int thisTestId = testExecution.getId();
				traceln(LOG_PRIORITY_INFO, "checking executing testcases for test " + thisTestId + "...");
				for (int indexTestcase=0; indexTestcase<testExecution.getTestcaseExecutionVector().size(); indexTestcase++) {
					int testcaseId = testExecution.getTestcaseExecutionVector().get(indexTestcase).getId();
					trace(LOG_PRIORITY_INFO, "> testcase " + testcaseId + ": ");
					if (!notExecutableTestcaseIdVector.contains(testcaseId)) {
						traceln(LOG_PRIORITY_INFO, "executable");
					} else {
						traceln(LOG_PRIORITY_INFO, "NOT executable");
					}
				}
			}
			
			return new CReturnStatus(RESULT_SUCCESS, null);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");

			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		this.attributes = attributes;

		printAttributes(attributes);

		printTestCustomFields(testId);
		
		if (preRunAndPostRunTakeTime) {
			CUtils.sleep(preRunDelay);
		}
		
		// not yet implemented as we're missing a table table_test_exec_attach
		// addTestMessageAttachment(new File("C:\\test_repository\\data\\attachment_test_1.xml"));

		return new CReturnStatus(RESULT_SUCCESS, null);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		long rand;
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + " [" + testName + "] testcaseIndex=" + testcaseIndex + " testcaseName=" + testcaseName + "...");

		printParams(params);

		printTestcaseCustomFields(testcaseId);
		
		try {
			// because attachments are deleted afterwards
			System.out.println("- - - - restoring attachments");
			CFileUtils.copyFiles(new File("C:\\test_repository\\data\\attachments_backup"), new File("C:\\test_repository\\data\\"), true);
		} catch (IOException e) {
			//e.printStackTrace();
		}

		
		// retrieve the static and dynamic attributes of the current test
		/*
		try {
			Vector<Integer> listOfNumbers = getIntegerAttributeValues(attributes, "list_of_numbers");
			Vector<String> listOfStrings = getStringAttributeValues(attributes, "list_of_strings");
			traceln(LOG_PRIORITY_INFO, "list of numbers dynamically passed by the operator " + listOfNumbers);
			traceln(LOG_PRIORITY_INFO, "list of strings dynamically passed by the operator " + listOfStrings);
		} catch (CAttributeParsingException e1) {
			e1.printStackTrace();
		}
		*/

		// +--------------------------------------+
		// |       simulate time spending         |
		// +--------------------------------------+
		rand = Math.round(Math.random()*maxTestcaseDelay); // from 0 to maxTestcaseDelay
		CUtils.sleep(100 + rand);


		// +--------------------------------------+
		// |            attachments               |
		// +--------------------------------------+
		System.out.println("- - - - add attachment");
		addTestcaseMessageAttachment(new File("C:\\test_repository\\data\\attachment_testcase_1.xml"));
		//addTestcaseMessageAttachment(new File("C:\\test_repository\\data\\attachment_testcase_2.xml"));
		
		// +--------------------------------------+
		// |     auto-extractable attachments     |
		// +--------------------------------------+
		/*
		addTestcaseMessageAttachment(new File("C:\\test_repository\\data\\report.html.zip"));
		addTestcaseMessageAttachment(new File("C:\\test_repository\\data\\report1.html.zip")); // wrong index file
		addTestcaseMessageAttachment(new File("C:\\test_repository\\data\\report2.zip")); // not auto-extractable
		*/
		
		// +--------------------------------------+
		// |        on-the-fly attachments        |
		// +--------------------------------------+
		/*addTestcaseMessageAttachment(new File("C:\\test_repository\\data\\IconAgent.png"));
		addTestcaseMessageAttachment(new File("C:\\test_repository\\data\\IconAgentSelected.png"));*/

		// +--------------------------------------+
		// |          simulate result             |
		// +--------------------------------------+
		return new CReturnStatus(getRandomTestcaseResult(), getRandomMessages());
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		if (preRunAndPostRunTakeTime) {
			CUtils.sleep(postRunDelay);
		}

		// not yet implemented as we're missing a table table_test_exec_attach
		// addTestMessageAttachment(new File("C:\\test_repository\\data\\attachment_test_2.xml"));

		return new CReturnStatus(RESULT_SUCCESS, null);
	}

	@Override
	public CReturnStatus terminate() {
		//addSessionMessageAttachment(new File("C:\\test_repository\\data\\attachment_session_2.xml"));

		return new CReturnStatus(RESULT_SUCCESS, null);
	}

	
	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
    	File testRootPathFolder = new File("C:\\test_repository");
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,     // folder to scan
							  "^.*$",                 // any folder
							  "^.*\\x2etxt$",         // only *.txt files
							  true,                   // remove extension from the file name to determine the test name
							  "/",                    // separator used in canonicalPath generated
							  "");                    // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	
	
	
	
	
	private short getRandomTestcaseResult() {
		short[] possibleResults = new short[] {RESULT_SUCCESS, RESULT_SUCCESS, RESULT_SUCCESS, RESULT_SUCCESS, 
		                                       RESULT_FAILURE, RESULT_INDEFINITE, RESULT_NOT_EXECUTED, RESULT_NOT_EXECUTABLE};
		
		//short[] possibleResults = new short[] {RESULT_SUCCESS, RESULT_NOT_EXECUTABLE, RESULT_NOT_EXECUTABLE, RESULT_NOT_EXECUTABLE, RESULT_NOT_EXECUTABLE};

		//short[] possibleResults = new short[] {RESULT_SUCCESS, RESULT_NOT_EXECUTED, RESULT_NOT_EXECUTED, RESULT_NOT_EXECUTED, RESULT_NOT_EXECUTED};
		
		// simulate different type of results
		long rand = Math.round(Math.random()*(possibleResults.length-1)); // Math.round(0.000001 - 6.99999999) = 0 - 7
		return possibleResults[(int)rand];
	}
	
	private short getRandomMessageResult() {
		short[] possibleResults = new short[] {RESULT_SUCCESS, RESULT_SUCCESS, RESULT_FAILURE, RESULT_INDEFINITE, RESULT_NOT_EXECUTED, RESULT_NOT_EXECUTABLE, RESULT_UNKNOWN, RESULT_INFO};
		
		// simulate different type of results
		long rand = Math.round(Math.random()*(possibleResults.length-1));
		return possibleResults[(int)rand];
	}
	
	private Vector<CExecutionStep> getRandomMessages() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		long nbMessages = Math.round(Math.random()*20);
		//executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "screenshot1::: [img]OTF_IconAgent.png[/img]"));
		for (int i=0; i<nbMessages; i++) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), getRandomMessageResult(), "message #" + i));
		}
		//executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "screenshot2::: [img]OTF_IconAgentSelected.png[/img]"));

		return executionSteps;
	}

	
	
}
