/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
|              kurt Callebaut                                          |
|              Radim Brixi                                             |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.ranorex;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.DOMException;
import org.w3c.dom.Document;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.CZipWriter;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Ranorex native tests.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{ranorex       }  ";

	static final int MAX_DELAY_IN_SECONDS = 30;
	
	private String testRootPath;
	private String runType;
	private boolean cleanup;
	
	public String testParentFolderPath;
	
	
	// +==============================================================+
	// | Constructors

	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			testRootPath = getStringParamValue("General",  "Test root path");                     // i.e. C:/Documents and Settings/keynux/Mes documents/RanorexStudio Projects/Samples
			runType      = getStringParamValue("Ranorex",  "Run type");                           // i.e. Tests
			cleanup      = getBooleanParamValue("Ranorex", "First clean up existing reports");
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		testParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/"; // needs to be done here as retrieveRxlogFile() needs it

		// i.e. C:/Documents and Settings/keynux/Mes documents/RanorexStudio Projects/Samples/WordPad Test Sample     /bin/debug/Word Pad Test      .exe
		// i.e. C:/Documents and Settings/keynux/Mes documents/RanorexStudio Projects/Samples/WordPad Test SampleVBNET/bin/debug/Word Pad Test      .exe
		// i.e. C:/Documents and Settings/keynux/Mes documents/RanorexStudio Projects/Samples/WordpadTestSamplePy     /bin/debug/WordpadTestSamplePy.exe   /testcase:AddNewEntry

		//      \______________________________ testRootPath _______________________________/\________ canonical path__________/ \____ testName ___/                 \_ tcName _/

		//      \______________________________________________ testParentFolderPath __________________________________________/

		// cleanup old results
		if (cleanup) {
			File[] reportFiles = getAllTheDataFiles(testParentFolderPath);
			if (reportFiles != null) {
				for (int i=0; i<reportFiles.length; i ++) {
					CFileUtils.deleteFile(reportFiles[i]);
				}
			}
		}
		
		// +------------------------------------+
		// | Run the Ranorex executable
		// +------------------------------------+

		File workingDir = new File(testParentFolderPath);
		CRunner exeRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
		                                CFileUtils.quoteFilePath(testParentFolderPath + testName + ".exe") + (runType.equals("Tests") ? "" : " /testcase:" + testcaseName),
                                        workingDir);

		short result = exeRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution failed or an error has been returned")); // if an error is returned. NOT only if the exe is not found. We must continue
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution succeeded"));
		}

		File rxlogFile = retrieveRxlogFile(testName); // test again if the file exists
		if (rxlogFile == null || !rxlogFile.exists()) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_NOT_EXECUTED, "rxlog file for test " + testName + " not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		addTestcaseMessageAttachment(rxlogFile);

		Vector<File> screenshotFiles = retrieveScreenshots(testName);
		addTestcaseMessageAttachments(screenshotFiles);
		

		// zip containing only the required folders and files
		//File zip = createFilteredZipArchive(testParentFolderPath, testName, rxlogFile, ".zip");
		//addTestcaseMessageAttachment(zip);
		
		// zip containing only the required folders and files and just renamed with .rxz extension so that we can associate it with the report command line reader
		/*
		File rxz = createFilteredZipArchive(testParentFolderPath, testName, rxlogFile, ".rxz");
		addTestcaseMessageAttachment(rxz);
		*/
		
		// zip containing ALL the folders and files
		try {
			File reportFolder = new File(testParentFolderPath);
			File zippedReport = new File(System.getProperty("java.io.tmpdir") + "/ranorex_full_report.zip");
			CFileUtils.deleteFile(zippedReport);
			CZipWriter zipWriter = new CZipWriter(zippedReport.getAbsolutePath());
			zipWriter.zipFolder(reportFolder.getAbsolutePath());
			addTestcaseMessageAttachment(zippedReport);
		} catch (Exception e1) {
			e1.printStackTrace();
		}

		return parseResultFile(rxlogFile, executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		traceln(LOG_PRIORITY_INFO, "terminate...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	// zip only what we need - to be tested
	/*
	public File createFilteredZipArchive(String srcFolder, String testName, File rxlogFile, String outputExtension) {
		String rxlogFileName= rxlogFile.getName();
		if (rxlogFileName.endsWith(".data")) {
			rxlogFileName = rxlogFileName.substring(0, rxlogFileName.length() - 5);
		}
		if (rxlogFileName.endsWith(".rxlog")) {
			rxlogFileName = rxlogFileName.substring(0, rxlogFileName.length() - 6);
		}
		String rxlogFileNameWithoutExtension=rxlogFileName;
		if (rxlogFileNameWithoutExtension.contains(testName)==false){
			traceln(LOG_PRIORITY_INFO,"Error in filename. " + testName + " does not correspond to " + rxlogFileNameWithoutExtension);
		}
		
		String zipFilePath = System.getProperty("java.io.tmpdir") + "/" + rxlogFileNameWithoutExtension + outputExtension;
		CZipWriter zipWriter = new CZipWriter(zipFilePath);
		zipWriter.setRegexFolderNameToInclude("images_" + rxlogFileNameWithoutExtension);
		zipWriter.setRegexFileNameToInclude("RanorexReport|" + rxlogFileNameWithoutExtension);
		zipWriter.zipFolder(srcFolder);
		return new File(zipFilePath);
    }
    */
	
	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eexe$",          // only *.exe files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private File retrieveRxlogFile(String testName) {
		return retrieveRxlogFile(testName, false);
	}
	private File retrieveRxlogFile(String testName, boolean skipTimeVerification) {
		File output = new File(testParentFolderPath + "/Reports/" + testName.replaceAll(" ", "") + ".rxlog.data");
		if (output.exists()) { // Ranorex >= 9 ?
			traceln(LOG_PRIORITY_INFO, "Ranorex >= 9? (with a way to configure the format) data file found...");
			return output;
		}

		output = new File(testParentFolderPath + "/" + testName.replaceAll(" ", "") + ".rxlog.data");
		if (output.exists()) { // Ranorex >= 4
			traceln(LOG_PRIORITY_INFO, "Ranorex >= 4 (with a way to configure the format) data file found...");
			return output;
		}
		
		output = new File(testParentFolderPath + "/" + testName.replaceAll(" ", "") + ".rxlog");
		if (output.exists()) { // Ranorex < 4
			traceln(LOG_PRIORITY_INFO, "Ranorex < 4 data file found...");
			return output;
		}
		
		// if we get there it's probably because the user did not configure the format of the report
		// hence the report name probably includes date, time etc.
		output = getTheMostRecentDataFile(testParentFolderPath, skipTimeVerification); //
		if (output != null) { // Ranorex >= 5 (but in last position as it's less reliable that getting from a unique static name)
			traceln(LOG_PRIORITY_INFO, "Ranorex >= 5 (with no way to configure the format) data file found...");
			return output;
		}

		traceln(LOG_PRIORITY_SEVERE, "rxlog file for test " + testName + " does not exist !");
		return null;
	}
	

	private File getTheMostRecentDataFile(String rootFolderPath) {
		return getTheMostRecentDataFile(rootFolderPath, false);
	}
	
	private File getTheMostRecentDataFile(String rootFolderPath, boolean skipTimeVerification) {
		traceln(LOG_PRIORITY_INFO, "searching for the report in " + rootFolderPath + "...");

		File rootFolder = new File(rootFolderPath);
		if (rootFolder == null || !rootFolder.exists())
			return null;
		
		File[] childDataFiles = rootFolder.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return name.endsWith(".data");
			}
		});
		traceln(LOG_PRIORITY_INFO, "number of candidates: " + childDataFiles.length + "...");
		
		// search for the most recent .data file
		long lastModifiedMillis = 0;
		File mostRecentDataFile = null;
		for (File dataFile : childDataFiles) {
			if (dataFile.lastModified() > lastModifiedMillis) {
				mostRecentDataFile = dataFile;
				lastModifiedMillis = dataFile.lastModified();
			}
		}
		
		if (mostRecentDataFile == null) {
			return null;
		}

		traceln(LOG_PRIORITY_INFO, "most recent candidate: " + mostRecentDataFile);

		// check the file is not too old (that's why this method is not 100% reliable)
		if (!skipTimeVerification && mostRecentDataFile.lastModified() < System.currentTimeMillis()-(1000*MAX_DELAY_IN_SECONDS)) {
			traceln(LOG_PRIORITY_INFO,"most recent candidate is too old!");
			return null;
		}
		
		return mostRecentDataFile;
	}
	
	
	private File[] getAllTheDataFiles(String rootFolderPath) {
		traceln(LOG_PRIORITY_INFO, "searching for all the reports in " + rootFolderPath + "...");

		File rootFolder = new File(rootFolderPath);
		File[] childDataFiles = rootFolder.listFiles(new FilenameFilter() {
			@Override
			public boolean accept(File dir, String name) {
				return name.endsWith(".data");
			}
		});
		
		return childDataFiles;
	}
	
	private Vector<File> retrieveScreenshots(String testName) {
		Vector<File> output = new Vector<File>();
		File parentFolder = new File(testParentFolderPath);
		File[] childFiles = parentFolder.listFiles();
		for (int i=0; i<childFiles.length; i++) {
			String fileName = childFiles[i].getName();
			
			if ((fileName.startsWith(testName.replaceAll(" ", "") + ".rxlog_") && fileName.endsWith("rxlog.jpg")) ||
				(fileName.startsWith(testName.replaceAll(" ", ""))             && fileName.endsWith("trace.jpg"))) {
				output.add(childFiles[i]);
			}
		}
		return output;
	}

	private CReturnStatus parseResultFile(File rxlogFile, Vector<CExecutionStep> executionSteps) {
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_NOT_EXECUTED, "Parsing the reports..."));
		boolean atLeastOneFailure = false;

		String xmlContent    = CFileUtils.FileToString(rxlogFile, "UTF-8");
		traceln(LOG_PRIORITY_INFO, "results file xmlContent = " + xmlContent);

		xmlContent = CXmlDocumentFactory.removeBOMIfAnyFromXmlContent(xmlContent);
		
		Document xmlDocument = CXmlDocumentFactory.createXMLDoc(xmlContent);
		traceln(LOG_PRIORITY_INFO, "document: " + xmlDocument);
		
		// problem during execution (i.e. can't get the data file or something)
		NodeList nodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//activity[@result!='Success']");
		if (nodeList != null) {
			traceln(LOG_PRIORITY_INFO, nodeList.getLength() + " activities in error nodes found");
			for (int i=0; i<nodeList.getLength(); i++) {
				atLeastOneFailure = true;

				Node activityErrorNode=nodeList.item(i);
				Node errorMessageNode = CXmlDocumentFactory.getNodeFromXPath(activityErrorNode, "errormessage");
				String activityErrorMessage = CXmlDocumentFactory.getNodeTextOrCDataContent(errorMessageNode);
				
				executionSteps.add(new CExecutionStep(RESULT_FAILURE, activityErrorMessage));
			}
		}
		
		
		// latest versions of Ranorex
		nodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//item");
		if (nodeList != null) {
			traceln(LOG_PRIORITY_INFO, nodeList.getLength() + " items nodes found");
			for (int i=0; i<nodeList.getLength(); i++) {
				Node itemNode = nodeList.item(i);
				NamedNodeMap attributes = itemNode.getAttributes();
				String time = "";
				String level = "";
				String category = "";
				try {
					time     = attributes.getNamedItem("time").getNodeValue();
					level    = attributes.getNamedItem("level").getNodeValue();
					category = attributes.getNamedItem("category").getNodeValue();
				} catch (DOMException e) {
					traceln(LOG_PRIORITY_SEVERE, "attributes coudn't be retrieved in the log file !");
				}
	
				short result = RESULT_FAILURE;
				if (level.equalsIgnoreCase("Error") || level.equalsIgnoreCase("Failure")) {
					result = RESULT_FAILURE;
					atLeastOneFailure = true;
				} else if (level.equalsIgnoreCase("Success") || level.equalsIgnoreCase("Info")) {
					result = RESULT_SUCCESS;
				} else {
					result = RESULT_NOT_EXECUTED;
				}
	
				NodeList traceList = CXmlDocumentFactory.getNodeObjListFromXPath(itemNode, "message");
				for (int j=0; j<traceList.getLength(); j++) {
					traceln(LOG_PRIORITY_INFO, "parsing trace " + j + "/" + traceList.getLength());
					executionSteps.add(new CExecutionStep(time, result, "[" + category + "] " + traceList.item(j).getFirstChild().getNodeValue()));
				}
			}
		} else { // old versions of Ranorex
			traceln(LOG_PRIORITY_INFO, "older version of Ranorex");
			nodeList    = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//message");
			if (nodeList != null) {
				traceln(LOG_PRIORITY_INFO, nodeList.getLength() + " message nodes found");
				for (int i=0; i<nodeList.getLength(); i++) {
					Node messageNode = nodeList.item(i);
					NamedNodeMap attributes = messageNode.getAttributes();
					String time = "";
					String level = "";
					String category = "";
					try {
						time     = attributes.getNamedItem("time").getNodeValue();
						level    = attributes.getNamedItem("level").getNodeValue();
						category = attributes.getNamedItem("category").getNodeValue();
					} catch (DOMException e) {
						traceln(LOG_PRIORITY_SEVERE, "attributes coudn't be retrieved in the log file !");
					}
		
					short result = RESULT_FAILURE;
					if (level.equalsIgnoreCase("Error") || level.equalsIgnoreCase("Failure")) {
						result = RESULT_FAILURE;
						atLeastOneFailure = true;
					} else if (level.equalsIgnoreCase("Success") || level.equalsIgnoreCase("Info")) {
						result = RESULT_SUCCESS;
					} else {
						result = RESULT_NOT_EXECUTED;
					}
		
					NodeList traceList = messageNode.getChildNodes();
					for (int j=0; j<traceList.getLength(); j++) {
						traceln(LOG_PRIORITY_INFO, "parsing trace " + j + "/" + traceList.getLength());
						Node traceNode = traceList.item(j);
						String trace = traceNode.getNodeValue();
						if (traceNode.getNodeName().equals("#cdata-section") && trace.trim().length()>0) {
							executionSteps.add(new CExecutionStep(time, result, "[" + category + "] " + trace));
						}
					}
				}
			} else {
				traceln(LOG_PRIORITY_INFO, "no message nodes found");
			}
		}
		
		return new CReturnStatus(atLeastOneFailure ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
	}


	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
    public static void main(String[] args) {
        CLauncherImpl toto = new CLauncherImpl();
        //toto.testParentFolderPath = "Y:/XStudio/src/com/xqual/xlauncher/ranorex/example2 (ranorex 4.x)";
        //File dataFile = toto.retrieveRxlogFile("Calculatrice_Addition", true);
        
        toto.testParentFolderPath = "Y:/XStudio/src/com/xqual/xlauncher/ranorex/example3 (problem with data)xxx";
        File dataFile = toto.retrieveRxlogFile("AT_CreateSharedFolder", true);
        
        Vector<CExecutionStep> execSteps = new Vector<CExecutionStep>();
        CReturnStatus returnStatus = toto.parseResultFile(dataFile, execSteps);
        
		System.out.println(execSteps);
		System.out.println(returnStatus);
		System.out.println(" ===> " + CUtils.resultToString(returnStatus.getResult()));
    }
}
