/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.10                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.rapise;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Rapise.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{rapise        }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String wscriptExePath;
	private String windowsCommand;

	private String rapiseEngine;

	private String rapiseInterpreter;

	private File workingDir;

	File rapiseTRPFile;
	File rapiseTAPFile;
	File rapiseTraceFile;
	File rapiseSummaryLogFile;


	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) {
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");

		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need from the configuration
			testRootPath        = getStringParamValue("General", "Test root path");
			wscriptExePath = getStringParamValue("General", "Windows cscript.exe path");
			windowsCommand = getStringParamValue("General", "Windows command");
			rapiseEngine = getStringParamValue("Rapise", "Rapise engine");

			//create the default interpreter for all tests
			rapiseInterpreter = windowsCommand + " /C " + wscriptExePath + " " + CFileUtils.quoteFilePath(rapiseEngine) + " ";

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		workingDir = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "")); // parent folder of all the test classes

		        StringBuffer paramsArgument = new StringBuffer();
        for (int k=0; k<params.size(); k++) {
            CTestcaseParam param = params.get(k);
            String paramName     = param.getName();
            String paramValue    = param.getStringValue();
									if (paramsArgument.length()>0) {
										paramsArgument.append(",");
									}
            paramsArgument.append("g_" + paramName + ":" + "\'" + paramValue + "\'");
        }

		traceln(LOG_PRIORITY_INFO, paramsArgument.toString());
		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		rapiseTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName +".log"); // testPath contains . and not \ or /
		CFileUtils.deleteAndTouchFile(rapiseTraceFile);

		// make sure the rapise reports are fresh
		rapiseTRPFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "last.trp"); // testPath contains . and not \ or /
		CFileUtils.deleteAndTouchFile(rapiseTRPFile);
		rapiseTAPFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "last.tap"); // testPath contains . and not \ or /
		CFileUtils.deleteAndTouchFile(rapiseTAPFile);
		rapiseSummaryLogFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") +"summary.log"); // testPath contains . and not \ or /
		CFileUtils.deleteAndTouchFile(rapiseSummaryLogFile);



		// +------------------------------------+
		// | Run the rapise test class
		// +------------------------------------+
		CRunner rapiseRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,rapiseInterpreter.toString() +
                                           CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".sstest") +
                                           (paramsArgument.length()>0 ? " " + " \"-eval:g_testSetParams={" + paramsArgument.toString() + "};\"" : ""),workingDir);


		// here we redirect the output to a file for later parsing
		short result = rapiseRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, rapiseTraceFile);

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(rapiseTraceFile);

		// return code of rapise runner: if all tests run successfully, exit with a status of 0. Otherwise exit with a status of 1.
		//rapide user guide 5.4  section 2.3.2.1
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Test case failed"));
			parseResultFile(executionSteps); // to have the details anyway
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Test case succeeded"));
			return parseResultFile(executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+

	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}

	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2esstest$",        // only *.class files
							  true,                    // remove extension from the file name to determine the test name
							  ".",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// parse the result file to get the result and the execution steps
		if (!rapiseTRPFile.exists() && !rapiseTAPFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
					addTestcaseMessageAttachment(rapiseTRPFile);
					addTestcaseMessageAttachment(rapiseTAPFile);
					addTestcaseMessageAttachment(rapiseSummaryLogFile);
		}

		String line, message="";
		boolean errorDetected = false;
		int errorIndex = 1;

		try {
				FileInputStream fileInputStream = new FileInputStream(rapiseTRPFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);

				if (line.indexOf ("status=") == 0) {
					try {message = line;} catch (Exception e) {}
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, message));

				} else if (line.indexOf("\"Fail\"") >= 0) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, line));
					errorDetected = true;

				} else if (line.indexOf("\"Pass\"") >= 0) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, line));

				} else if (line.indexOf("\"Info\"") >= 0) {
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));

				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}

}
