/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.robot_framework;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.CTestExecution;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.CZipWriter;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for RobotFramework (to execute one testsuite).
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{robot_frmwrk  }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String pythonInstallPath;
	private String optionalArguments;
	private String extension;
	
	private File workingDir;
	private File consoleFile;
	private File logFile;
	private File xmlOutputFile;
	private File htmlReportFile;
	private File zippedReportAndLogFile;

	private static final String PYTHON_INTERPRETER_EXE = CUtils.getExecutableName("python");

	private boolean firstTestcase = true;
	private String thisTestName;
	private String thisTestcaseName;
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath      = getStringParamValue("General", "Test root path");
			pythonInstallPath = getStringParamValue("Python", "Python install path");

			// these 2 parameters have been added later on with the default values corresponding to the previous version's hardcoded values "" and ".txt")
			optionalArguments = getStringParamValue("Robot Framework", "Optional arguments");
			extension         = getStringParamValue("Robot Framework", "Extension");
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		// We run a testsuite here so that we can benefits from the Setup and Teardown optimization
		// when run() is called we'll parse the log to get information about one specific testcase
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		firstTestcase = true;

		thisTestName = testName;
		if (additionalInfo != null && additionalInfo.trim().length()>0) { // test's additional infos
			thisTestName = additionalInfo.trim();
			traceln(LOG_PRIORITY_INFO, "preRun final testName=" + thisTestName);
		}

		workingDir = new File(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/"); // folder containing the test suite file

		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		consoleFile = new File(workingDir.getAbsolutePath() + "/rf_console.txt");
		CFileUtils.deleteAndTouchFile(consoleFile);
		
		logFile = new File(workingDir.getAbsolutePath() + "/rf_log.html");
		CFileUtils.deleteAndTouchFile(logFile);

		xmlOutputFile = new File(workingDir.getAbsolutePath() + "/rf_output.xml");
		CFileUtils.deleteAndTouchFile(xmlOutputFile);

		htmlReportFile = new File(workingDir.getAbsolutePath() + "/rf_report.html");
		CFileUtils.deleteAndTouchFile(htmlReportFile);
		
		// Retrieve all the testcase names that are executable.
		// This is to ensure that even if we run a full robot framework testsuite (the test in XStudio), only the executable testcases are executed
		// The the test list must be of the form:
		// --test test1 --test test2 --test test3* etc.
		// so 1 option per test
		traceln(LOG_PRIORITY_INFO, "ids of all the not executable testcases: " + notExecutableTestcaseIdVector);
		StringBuffer testListOption = new StringBuffer();
		for (int indexTest=0; indexTest<testExecutionVector.size(); indexTest++) {
			CTestExecution testExecution = testExecutionVector.get(indexTest);
			int thisTestId = testExecution.getId();
			if (thisTestId == testId) {
				traceln(LOG_PRIORITY_INFO, "checking executing testcases for test " + testId + "...");
				for (int indexTestcase=0; indexTestcase<testExecution.getTestcaseExecutionVector().size(); indexTestcase++) {
					int testcaseId = testExecution.getTestcaseExecutionVector().get(indexTestcase).getId();
					trace(LOG_PRIORITY_INFO, "> testcase " + testcaseId + ": ");
					if (!notExecutableTestcaseIdVector.contains(testcaseId)) {
						traceln(LOG_PRIORITY_INFO, "executable");
						String testcaseName = testExecution.getTestcaseExecutionVector().get(indexTestcase).getName();
						testListOption.append("--test " + CFileUtils.quoteFilePath(testcaseName) + " ");
					} else {
						traceln(LOG_PRIORITY_INFO, "NOT executable");
					}
				}
			}
		}
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner pythonRunner = new CRunner("[" + testId + "] " + testPath + ":" + thisTestName, 
				   CFileUtils.quoteFilePath(pythonInstallPath + "/" + PYTHON_INTERPRETER_EXE) + " -m robot.run " +
				   "--suite "      + CFileUtils.quoteFilePath(thisTestName) + " " + // a testsuite
				   testListOption.toString() + 
				   "--outputdir "  + CFileUtils.quoteFilePath(workingDir.getAbsolutePath()) + " " +
				   "--log "        + CFileUtils.quoteFilePath(logFile.getAbsolutePath()) + " " +
				   "--output "     + CFileUtils.quoteFilePath(xmlOutputFile.getAbsolutePath()) + " " +
				   "--report "     + CFileUtils.quoteFilePath(htmlReportFile.getAbsolutePath()) + " " +
				   optionalArguments + " " + // --variable BROWSER:%browser%
				   CFileUtils.quoteFilePath(workingDir.getAbsolutePath() + "/" + thisTestName + extension),
				   workingDir);

		short result = pythonRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, consoleFile);
		/*
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "python testsuite execution failed"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "python testsuite execution succeeded"));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);			
		}
		*/
				
		return new CReturnStatus(RESULT_SUCCESS, executionSteps); // robot framework will return a > 0 error code if the test fails but we want to parse the results to determine this
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		thisTestcaseName = testcaseName;
		if (additionalInfo != null && additionalInfo.trim().length()>0) {
			thisTestcaseName = additionalInfo.trim();
			traceln(LOG_PRIORITY_INFO, "run final testcaseName=" + thisTestcaseName);
		}

		short result = parseResultFile(xmlOutputFile, thisTestcaseName, executionSteps);
		
		// +-----------------------------------------------+
		// | Add all the attachments to the first testcase |
		// +-----------------------------------------------+
		if (firstTestcase) {
			// Add the attachments to the first testcase
			addTestcaseMessageAttachment(consoleFile);
			addTestcaseMessageAttachment(htmlReportFile);
			addTestcaseMessageAttachment(logFile);


	        File copyOutputFile = new File(System.getProperty("java.io.tmpdir") + "/" + xmlOutputFile.getName()); // copy the output file before attaching it or it will be deleted
	        CFileUtils.deleteAndTouchFile(copyOutputFile);
	        try {
	            CFileUtils.copyFiles(xmlOutputFile, copyOutputFile, true);
	        } catch (Exception e) {
	            traceln(LOG_PRIORITY_INFO, "output file copy exception: " + e);
	        }
			addTestcaseMessageAttachment(copyOutputFile); // the output xml


	        File originalRobotFile = new File(workingDir.getAbsolutePath() + "/" + thisTestName + extension); // copy the .robot before attaching it or it will be deleted
	        File copyRobotFile = new File(System.getProperty("java.io.tmpdir") + "/" + thisTestName + extension);
	        CFileUtils.deleteAndTouchFile(copyRobotFile);
	        try {
	            CFileUtils.copyFiles(originalRobotFile, copyRobotFile, true);
	        } catch (Exception e) {
	            traceln(LOG_PRIORITY_INFO, ".robot file copy exception: " + e);
	        }
			addTestcaseMessageAttachment(copyRobotFile); // the script itself .robot or .txt

			
			// zip the Html structure and attach it as the html and report log files may include some references to some images
			if (htmlReportFile.exists() && logFile.exists()) {
				// the extension + name are important as it's used when we open the zip to try to find an entry point
				// in this case, when the user will open the attachment it will see the double extension .html.zip and will try to unzip then open rf_report.html
				zippedReportAndLogFile = new File(System.getProperty("java.io.tmpdir") + "/rf_report.html.zip");
				
				CZipWriter zipWriter = new CZipWriter(zippedReportAndLogFile.getAbsolutePath());
				// zipWriter.setRegexFileNameToInclude("^(.*\\x2egif)|(.*\\x2epng)|(.*\\x2ejpg)|(.*\\x2ecss)|(.*\\x2ehtml)|(.*\\x2exml)|(.*\\x2etxt)|(" + CTextUtils.escapeStringforRegex(thisTestName) + "\\x2erobot)$");
				zipWriter.setRegexFileNameToInclude("^(.*\\x2egif)|(.*\\x2epng)|(.*\\x2ejpg)|(.*\\x2ecss)|(.*\\x2ehtml)|(.*\\x2exml)|(.*\\x2etxt)|(" + CTextUtils.escapeStringforRegex(thisTestName) + ".*)$");
				zipWriter.zipFolder(workingDir.getAbsolutePath());
				zipWriter.closeZip();
				
				addTestcaseMessageAttachment(zippedReportAndLogFile);
			}
			
			firstTestcase = false;
		}
		
		return new CReturnStatus(result, executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2etxt$",          // only *.txt files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	@Override
	public void recursivelyScanFolderForTests(Vector<CScannedTest> output, File folder, 
											  String regexFolder, String regexFile,
											  boolean removeExtension, String separator, String currentRelativePath) {
		if (folder.isDirectory()) {
			System.out.println("DIRECTORY: " + folder + "\"");
			File[] children = folder.listFiles();
			if (children != null) {
				System.out.println("contains " + children.length + " files");
				for (int i = 0; i < children.length; i++) {
					File child = children[i];
					if (child.isDirectory()) {
						recursivelyScanFolderForTests(output, child, regexFolder, regexFile, removeExtension, separator, 
													  currentRelativePath.trim().length() > 0 ? currentRelativePath + separator + child.getName() : child.getName());
					} else { // file
						System.out.println("   FILE: " + child + "\"");
						if (Pattern.matches(regexFile, child.getName())) {
							String testName = child.getName();
							if (removeExtension) {
								testName = CFileUtils.getBasename(child);
							}
							
							CScannedTest scannedTest = new CScannedTest(testName, currentRelativePath);
							System.out.println("      > found test \"" + currentRelativePath + " : " + testName + "\"");
							
							// check that we do not already have discovered a test with the same name (all the tests will be imported in a unique folder so must have a different name)
							if (!output.contains(scannedTest)) {
								// parse the .robot file to discover the testcases
								int j=1;
								boolean testcasesSection = false;
								
								try {
									BufferedReader robotFileBufferedReader = new BufferedReader(new FileReader(child));

									String readLine = "";
									System.out.println("          Reading file for testcases...");
									while ((readLine = robotFileBufferedReader.readLine()) != null) {
									    //System.out.println(readLine);
										if (readLine.equals("*** Test Cases ***")) {
											testcasesSection = true;
											continue;
										}

										if (readLine.equals("*** Keywords ***")) {
											testcasesSection = false;
											break;
										}
										
									    if (testcasesSection && readLine.trim().length() > 0 && readLine.charAt(0) != ' ' && readLine.charAt(0) != '\t') {
											System.out.println("          + found testcase \"" + j + " : " + readLine.trim() + "\"");
											scannedTest.addScannedTestcase((j++), readLine.trim());
									    }
									}
									
									robotFileBufferedReader.close();
								} catch (Exception e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}

								output.add(scannedTest);
							}
						}
					}
				}
			}
		} else { // should never happen!
		}
	}
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private String getTabulation(int level) {
		StringBuffer output = new StringBuffer();
		for (int i=0; i<level; i++) {
			//output.append("\t");
		}
		return output.toString();
	}
	
    static public String resultToUserString(int result) {
        if (result==IConstantsResults.RESULT_FAILURE) {
           return "F ";
        } else if (result==IConstantsResults.RESULT_SUCCESS) {
            return "S ";
        } else if (result==IConstantsResults.RESULT_INDEFINITE) {
            return "I ";
        } else if (result==IConstantsResults.RESULT_NOT_EXECUTED) {
            return "NE";
        } else if (result==IConstantsResults.RESULT_NOT_EXECUTABLE) {
            return "NA";
        } else if (result==IConstantsResults.RESULT_UNKNOWN) {
            return "U ";
        } else if (result==IConstantsResults.RESULT_INFO) {
            return "I ";
        } else {
           return "U ";
        }
     }
	private short parseResultFile(File xmlFile, String testcaseName, Vector<CExecutionStep> executionSteps) {
		if (!xmlFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return RESULT_FAILURE;
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: result file found"));
		}

		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(xmlFile);
	    Node xmlNode = CXmlDocumentFactory.getNodeFromXPath(xmlDocument, "//test[@name='" + testcaseName + "']");
	    if (xmlNode == null) {
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: testcase name not found in result file (probably a typo in the XStudio testcase name"));
	    	return RESULT_FAILURE;
	    }
	    short result = recursivelyParseKeywords(xmlNode, executionSteps, -1); // this populate the messages etc.

	    // BUT now we get the status from the top root node (some childs CAN have failed status and test being Successful - indeed you can tests some things that will condition how the tests continue)
	    String status = CXmlDocumentFactory.getNodeStringValueFromXPath(xmlNode, "status/@status");
	    if (status.equals("PASS")) {
	    	return RESULT_SUCCESS;
	    } else {
	    	return RESULT_FAILURE;
	    }
	}
	
	// a keyword can contain a keyword etc.
	private short recursivelyParseKeywords(Node xmlNode, Vector<CExecutionStep> executionSteps, int level) {
		System.err.println("- - - recursive");
		NodeList childNodeList = xmlNode.getChildNodes();
		System.err.println("- - -      nb childs = " + childNodeList.getLength());
		short result = RESULT_SUCCESS;
		for (int i=0; i<childNodeList.getLength(); i++) {
			Node childNode = childNodeList.item(i);
			System.err.println("- - -         > child" + i + "=" + childNode.getNodeName());

			if (childNode.getNodeName().equals("kw")) { //kw
				//String type    = childNode.getAttributes().getNamedItem("type").getNodeValue(); // kw type=xxx // WARNING there is not always some types !
				String name    = childNode.getAttributes().getNamedItem("name").getNodeValue(); // kw name=xxx
				System.err.println("+++ kw: " + name);
				String status  = "INFO";
				String endTime = "";
				String msg     = "";
				
				NodeList grandChildNodeList = childNode.getChildNodes();
				for (int j=0; j<grandChildNodeList.getLength(); j++) {
					Node grandChildNode = grandChildNodeList.item(j);
					if (grandChildNode.getNodeName().equals("status")) {
						System.err.println("    > status");
						try {
							status  = grandChildNode.getAttributes().getNamedItem("status").getNodeValue();
							endTime = grandChildNode.getAttributes().getNamedItem("endtime").getNodeValue();
						} catch (Exception e) {
						}
					} else if (grandChildNode.getNodeName().equals("msg")) {
						msg = grandChildNode.getFirstChild().getNodeValue();
						if (msg==null || msg.trim().length() == 0) {
							msg = "";
						}
					}
				}

				if (status.equals("PASS")) {
					System.err.println("    * pass");
					executionSteps.add(new CExecutionStep(endTime, RESULT_SUCCESS, getTabulation(level) + "[b][SUCCESS][/b] " + name + " {[i]" + msg + "[/i]}"));
				} else if (status.equals("INFO")) {
					System.err.println("    * info");
					executionSteps.add(new CExecutionStep(endTime, RESULT_INFO, getTabulation(level) + "[b][INFO][/b] " + name + " {[i]" + msg + "[/i]}"));
				} else {
					System.err.println("    * else");
					result = RESULT_FAILURE;
					executionSteps.add(new CExecutionStep(endTime, RESULT_FAILURE, getTabulation(level) + "[b][FAILURE][/b] " + name + " {[i]" + msg + "[/i]}"));
				}

				// recursivity
				short childResult = recursivelyParseKeywords(childNode, executionSteps, level+1);
				if (childResult == RESULT_FAILURE) {
					result = RESULT_FAILURE;
				}
				
			} else if (childNode.getNodeName().equals("doc")) { // doc
				/*
				String doc = CXmlDocumentFactory.getNodeTextOrCDataContent(childNode);
				if (doc!=null && doc.trim().length() > 0)
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, getTabulation(level) + "(" + doc + ")"));
				*/
			} else if (childNode.getNodeName().equals("msg")) { // msg
				/*
				String timeStamp = childNode.getAttributes().getNamedItem("timestamp").getNodeValue();
				//String status = childNode.getAttributes().getNamedItem("status").getNodeValue();
				String msg = CXmlDocumentFactory.getNodeTextOrCDataContent(childNode);
				if (msg!=null && msg.trim().length() > 0)
					executionSteps.add(new CExecutionStep(timeStamp, RESULT_INFO, getTabulation(level) + msg));
				*/
			} else if (childNode.getNodeName().equals("arguments")) { // arguments
				NodeList argNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(childNode, "arg");
				int nbArguments = argNodeList.getLength();
				if (nbArguments > 0) {
					//executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, getTabulation(level) + "arguments:"));
					for (int j=0; j<nbArguments; j++) {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, getTabulation(level) + "- " + CXmlDocumentFactory.getNodeTextOrCDataContent(argNodeList.item(j))));
					}
				}
			}
		}
		
		return result;
	}
	
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl toto = new CLauncherImpl();
		/*
		short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework/sample/E-Cargo_RCT_Release_1.0_18.12.2014_171224_output.xml"),
											"e-cargo_RCT_Connexion",
											executionSteps);
		*/
		short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework/sample/thales_alenia_space_robotframework_output.xml"),
											"Executing the offline analysis: Check the number of files in the output folder",
											executionSteps);
		
		
		
		System.err.println("RESULT = " + CUtils.resultToUserString(result));
		for (int i=0; i<executionSteps.size(); i++) {
			System.err.println(resultToUserString(executionSteps.get(i).getResult()) + " " + executionSteps.get(i).getMessage());
		}
	}

}
