/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.9                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.robot_framework_v2;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.CZipWriter;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for RobotFramework (to execute only one test).
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{robot_frmwrk_2}  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String pythonInstallPath;
	private String optionalArguments;
	private String runType;
	private String extension;
	
	private File workingDir;
	private File consoleFile;
	private File logFile;
	private File xmlOutputFile;
	private File htmlReportFile;
	private File zippedReportAndLogFile;
	
	private boolean firstTestcase = true;
	private String thisTestName;
	private String thisTestcaseName;
		
	private static final String PYTHON_INTERPRETER_EXE = CUtils.getExecutableName("python");
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath      = getStringParamValue("General", "Test root path");
			
			pythonInstallPath = getStringParamValue("Python", "Python install path");
			
			optionalArguments = getStringParamValue("Robot Framework", "Optional arguments");
			extension         = getStringParamValue("Robot Framework", "Extension");
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		try {
			runType = getStringParamValue("Robot Framework", "Run type");
			traceln(LOG_PRIORITY_INFO, "runType=" + runType);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "No run type in the configuration: using default value \"Tests\"...");
			runType ="Tests";
		}
		
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		firstTestcase = true;

		workingDir = new File(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/"); // folder containing the test suite file

		thisTestName = testName;
		if (additionalInfo != null && additionalInfo.trim().length()>0) { // test's additional infos
			thisTestName = additionalInfo.trim();
			traceln(LOG_PRIORITY_INFO, "preRun final testName=" + thisTestName);
		}

		// We run a testsuite here so that we can benefits from the Setup and Teardown optimization
		// when run() is called we'll parse the log to get information about one specific testcase
		if (runType.equals("Test suites")) {
			// +------------------------------------+
			// | console trace
			// +------------------------------------+
			consoleFile = new File(workingDir.getAbsolutePath() + "/rf_console.txt");
			CFileUtils.deleteAndTouchFile(consoleFile);
			
			logFile = new File(workingDir.getAbsolutePath() + "/rf_log.html");
			CFileUtils.deleteAndTouchFile(logFile);

			xmlOutputFile = new File(workingDir.getAbsolutePath() + "/rf_output.xml");
			CFileUtils.deleteAndTouchFile(xmlOutputFile);

			htmlReportFile = new File(workingDir.getAbsolutePath() + "/rf_report.html");
			CFileUtils.deleteAndTouchFile(htmlReportFile);

			// +------------------------------------+
			// | Interpret the script
			// +------------------------------------+
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "executing full testsuite " + thisTestName + "..."));

			CRunner pythonRunner = new CRunner("[" + testId + "] " + testPath + ":" + thisTestName, 
					   CFileUtils.quoteFilePath(pythonInstallPath + "/" + PYTHON_INTERPRETER_EXE) + " -m robot.run " +
					   "--suite "      + CFileUtils.quoteFilePath(thisTestName) + " " + // run a testsuite 
					   "--outputdir "  + CFileUtils.quoteFilePath(workingDir.getAbsolutePath()) + " " +
					   "--log "        + CFileUtils.quoteFilePath(logFile.getAbsolutePath()) + " " +
					   "--output "     + CFileUtils.quoteFilePath(xmlOutputFile.getAbsolutePath()) + " " +
					   "--report "     + CFileUtils.quoteFilePath(htmlReportFile.getAbsolutePath()) + " " +
					   optionalArguments + " " + // --variable BROWSER:%browser%
					   CFileUtils.quoteFilePath(workingDir.getAbsolutePath() + "/" + thisTestName + extension), 
					   workingDir);

			short result = pythonRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, consoleFile);
			traceln(LOG_PRIORITY_INFO, "testsuite execution returned");
			return new CReturnStatus(RESULT_SUCCESS /*result*/, executionSteps); // RF seems to return a > 0 error code when at least one testcase failed!
			
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		thisTestcaseName = testcaseName;
		if (additionalInfo != null && additionalInfo.trim().length()>0) {
			thisTestcaseName = additionalInfo.trim();
			traceln(LOG_PRIORITY_INFO, "run final testcaseName=" + thisTestcaseName);
		}

		if (runType.equals("Test suites")) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "parsing xml file for results on " + thisTestcaseName + "..."));

			//imagePathToBeReplaced = new Hashtable<String, String>(); // will be filled in while parsing the result file
			short result = parseResultFile(xmlOutputFile, thisTestcaseName, executionSteps);

			if (firstTestcase) {
				// Add the attachments to the first testcase
				addTestcaseMessageAttachment(consoleFile);
				addTestcaseMessageAttachment(htmlReportFile);
				addTestcaseMessageAttachment(logFile);
				
		        File copyOutputFile = new File(System.getProperty("java.io.tmpdir") + "/" + xmlOutputFile.getName()); // copy the output file before attaching it or it will be deleted
		        CFileUtils.deleteAndTouchFile(copyOutputFile);
		        try {
		            CFileUtils.copyFiles(xmlOutputFile, copyOutputFile, true);
		        } catch (Exception e) {
		            traceln(LOG_PRIORITY_INFO, "output file copy exception: " + e);
		        }
				addTestcaseMessageAttachment(copyOutputFile); // the output xml

		        File originalRobotFile = new File(workingDir.getAbsolutePath() + "/" + thisTestName + extension); // copy the .robot before attaching it or it will be deleted
		        File copyRobotFile = new File(System.getProperty("java.io.tmpdir") + "/" + thisTestName + extension);
		        CFileUtils.deleteAndTouchFile(copyRobotFile);
		        try {
		            CFileUtils.copyFiles(originalRobotFile, copyRobotFile, true);
		        } catch (Exception e) {
		            traceln(LOG_PRIORITY_INFO, ".robot file copy exception: " + e);
		        }
				addTestcaseMessageAttachment(copyRobotFile); // the script itself .robot or .txt
				
				
				// zip the Html structure and attach it as the html and report log files may include some references to some images
				if (htmlReportFile.exists() && logFile.exists()) {
					// the extension + name are important as it's used when we open the zip to try to find an entry point
					// in this case, when the user will open the attachment it will see the double extension .html.zip and will try to unzip then open rf_report.html
					zippedReportAndLogFile = new File(System.getProperty("java.io.tmpdir") + "/rf_report.html.zip");
					
					CZipWriter zipWriter = new CZipWriter(zippedReportAndLogFile.getAbsolutePath());
					zipWriter.setRegexFileNameToInclude("^(.*\\x2egif)|(.*\\x2epng)|(.*\\x2ejpg)|(.*\\x2ecss)|(.*\\x2ehtml)|(.*\\x2exml)|(.*\\x2etxt)|(" + CTextUtils.escapeStringforRegex(thisTestName) + ".*)$");
					zipWriter.zipFolder(workingDir.getAbsolutePath());
					zipWriter.closeZip();
					
					addTestcaseMessageAttachment(zippedReportAndLogFile);
				}
				
				firstTestcase = false;
			}
			
			return new CReturnStatus(result, executionSteps);
			
		} else {
			// +------------------------------------+
			// | console trace
			// +------------------------------------+
			consoleFile = new File(workingDir.getAbsolutePath() + "/rf_console.txt");
			CFileUtils.deleteAndTouchFile(consoleFile);
			
			logFile = new File(workingDir.getAbsolutePath() + "/rf_log.html");
			CFileUtils.deleteAndTouchFile(logFile);
	
			xmlOutputFile = new File(workingDir.getAbsolutePath() + "/rf_output.xml");
			CFileUtils.deleteAndTouchFile(xmlOutputFile);
	
			htmlReportFile = new File(workingDir.getAbsolutePath() + "/rf_report.html");
			CFileUtils.deleteAndTouchFile(htmlReportFile);
	
			// params
			StringBuffer paramsArgument = new StringBuffer();
			for (int k=0; k<params.size(); k++) {
				CTestcaseParam param = params.get(k);
				String paramName = param.getName();
				String paramValue = param.getStringValue();
				paramsArgument.append("--variable " + paramName + ":\"" + paramValue + "\" ");
			}
	
			// +------------------------------------+
			// | Interpret the script
			// +------------------------------------+
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "executing specific test " + thisTestcaseName + "..."));
			CRunner pythonRunner = new CRunner("[" + testcaseId + "] " + testPath + ":" + thisTestName + ":" + thisTestcaseName, 
					   CFileUtils.quoteFilePath(pythonInstallPath + "/" + PYTHON_INTERPRETER_EXE) + " -m robot.run " +
					   paramsArgument + 
					   "--test "       + CFileUtils.quoteFilePath(thisTestcaseName) + " " + // run only 1 specific test in the testsuite
					   "--outputdir "  + CFileUtils.quoteFilePath(workingDir.getAbsolutePath()) + " " +
					   "--log "        + CFileUtils.quoteFilePath(logFile.getAbsolutePath()) + " " +
					   "--output "     + CFileUtils.quoteFilePath(xmlOutputFile.getAbsolutePath()) + " " +
					   "--report "     + CFileUtils.quoteFilePath(htmlReportFile.getAbsolutePath()) + " " +
					   optionalArguments + " " + // --variable BROWSER:%browser%
					   CFileUtils.quoteFilePath(workingDir.getAbsolutePath() + "/" + thisTestName + extension), 
					   workingDir);
	
			short result = pythonRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, consoleFile);

			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "parsing xml file for results on " + thisTestcaseName + "..."));
	
			//imagePathToBeReplaced = new Hashtable<String, String>(); // will be filled in while parsing the result file
			result = parseResultFile(xmlOutputFile, thisTestcaseName, executionSteps);
			
			// Attachments
			addTestcaseMessageAttachment(consoleFile);
			addTestcaseMessageAttachment(htmlReportFile);
			addTestcaseMessageAttachment(logFile);
			
	        File copyOutputFile = new File(System.getProperty("java.io.tmpdir") + "/" + xmlOutputFile.getName()); // copy the output file before attaching it or it will be deleted
	        CFileUtils.deleteAndTouchFile(copyOutputFile);
	        try {
	            CFileUtils.copyFiles(xmlOutputFile, copyOutputFile, true);
	        } catch (Exception e) {
	            traceln(LOG_PRIORITY_INFO, "output file copy exception: " + e);
	        }
			addTestcaseMessageAttachment(copyOutputFile); // the output xml

	        File originalRobotFile = new File(workingDir.getAbsolutePath() + "/" + thisTestName + extension); // copy the .robot before attaching it or it will be deleted
	        File copyRobotFile = new File(System.getProperty("java.io.tmpdir") + "/" + thisTestName + extension);
	        CFileUtils.deleteAndTouchFile(copyRobotFile);
	        try {
	            CFileUtils.copyFiles(originalRobotFile, copyRobotFile, true);
	        } catch (Exception e) {
	            traceln(LOG_PRIORITY_INFO, ".robot file copy exception: " + e);
	        }
			addTestcaseMessageAttachment(copyRobotFile); // the script itself .robot or .txt
	
			// zip the Html structure and attach it as the html and report log files may include some references to some images
			if (htmlReportFile.exists() && logFile.exists()) {
				// the extension + name are important as it's used when we open the zip to try to find an entry point
				// in this case, when the user will open the attachment it will see the double extension .html.zip and will try to unzip then open rf_report.html
				zippedReportAndLogFile = new File(System.getProperty("java.io.tmpdir") + "/rf_report.html.zip");
				
				CZipWriter zipWriter = new CZipWriter(zippedReportAndLogFile.getAbsolutePath());
				// zipWriter.setRegexFileNameToInclude("^(.*\\x2egif)|(.*\\x2epng)|(.*\\x2ejpg)|(.*\\x2ecss)|(.*\\x2ehtml)|(.*\\x2exml)|(.*\\x2etxt)|(" + CTextUtils.escapeStringforRegex(thisTestName) + "\\x2erobot)$");
				zipWriter.setRegexFileNameToInclude("^(.*\\x2egif)|(.*\\x2epng)|(.*\\x2ejpg)|(.*\\x2ecss)|(.*\\x2ehtml)|(.*\\x2exml)|(.*\\x2etxt)|(" + CTextUtils.escapeStringforRegex(thisTestName) + ".*)$");
				zipWriter.zipFolder(workingDir.getAbsolutePath());
				zipWriter.closeZip();
				
				addTestcaseMessageAttachment(zippedReportAndLogFile);
			}
			
			return new CReturnStatus(result, executionSteps);
		}
		
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,                                   // folder to scan
							  "^.*$",                                               // any folder
							  "^.*\\x2e" + extension.replaceAll("\\x2e", "") + "$", // only *.<extension> files
							  true,                                                 // remove extension from the file name to determine the test name
							  "/",                                                  // separator used in canonicalPath generated
							  "");                                                  // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	@Override
	public void recursivelyScanFolderForTests(Vector<CScannedTest> output, File folder, 
											  String regexFolder, String regexFile,
											  boolean removeExtension, String separator, String currentRelativePath) {
		if (folder.isDirectory()) {
			System.out.println("DIRECTORY: " + folder + "\"");
			File[] children = folder.listFiles();
			if (children != null) {
				System.out.println("contains " + children.length + " files");
				for (int i = 0; i < children.length; i++) {
					File child = children[i];
					if (child.isDirectory()) {
						recursivelyScanFolderForTests(output, child, regexFolder, regexFile, removeExtension, separator, 
													  currentRelativePath.trim().length() > 0 ? currentRelativePath + separator + child.getName() : child.getName());
					} else { // file
						System.out.println("   FILE: " + child + "\"");
						if (Pattern.matches(regexFile, child.getName())) {
							String testName = child.getName();
							if (removeExtension) {
								testName = CFileUtils.getBasename(child);
							}
							
							CScannedTest scannedTest = new CScannedTest(testName, currentRelativePath);
							System.out.println("      > found test \"" + currentRelativePath + " : " + testName + "\"");
							
							// check that we do not already have discovered a test with the same name (all the tests will be imported in a unique folder so must have a different name)
							if (!output.contains(scannedTest)) {
								// parse the .robot file to discover the testcases
								int j=1;
								boolean testcasesSection = false;
								
								try {
									BufferedReader robotFileBufferedReader = new BufferedReader(new FileReader(child));

									String readLine = "";
									System.out.println("          Reading file for testcases...");
									while ((readLine = robotFileBufferedReader.readLine()) != null) {
									    //System.out.println(readLine);
										if (readLine.equals("*** Test Cases ***")) {
											testcasesSection = true;
											continue;
										}

										if (readLine.equals("*** Keywords ***")) {
											testcasesSection = false;
											break;
										}
										
									    if (testcasesSection && readLine.trim().length() > 0 && readLine.charAt(0) != ' ' && readLine.charAt(0) != '\t') {
											System.out.println("          + found testcase \"" + j + " : " + readLine.trim() + "\"");
											scannedTest.addScannedTestcase((j++), readLine.trim());
									    }
									}
									
									robotFileBufferedReader.close();
								} catch (Exception e) {
									// TODO Auto-generated catch block
									e.printStackTrace();
								}

								output.add(scannedTest);
							}
						}
					}
				}
			}
		} else { // should never happen!
		}
	}
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private String getTabulation(int level) {
		StringBuffer output = new StringBuffer();
		for (int i=0; i<level; i++) {
			//output.append("\t");
		}
		return output.toString();
	}
	
    static public String resultToUserString(int result) {
        if (result==IConstantsResults.RESULT_FAILURE) {
           return "F ";
        } else if (result==IConstantsResults.RESULT_SUCCESS) {
            return "S ";
        } else if (result==IConstantsResults.RESULT_INDEFINITE) {
            return "I ";
        } else if (result==IConstantsResults.RESULT_NOT_EXECUTED) {
            return "NE";
        } else if (result==IConstantsResults.RESULT_NOT_EXECUTABLE) {
            return "NA";
        } else if (result==IConstantsResults.RESULT_UNKNOWN) {
            return "U ";
        } else if (result==IConstantsResults.RESULT_INFO) {
            return "I ";
        } else {
            return "U ";
        }
     }
	private short parseResultFile(File xmlFile, String testcaseName, Vector<CExecutionStep> executionSteps) {
		if (!xmlFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return RESULT_FAILURE;
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: result file found"));
		}

		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(xmlFile);
	    Node xmlNode = CXmlDocumentFactory.getNodeFromXPath(xmlDocument, "//test[@name='" + testcaseName + "']");
	    if (xmlNode == null) {
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: testcase name not found in result file (probably a typo in the XStudio testcase name"));
	    	return RESULT_FAILURE;
	    }
	    short result = recursivelyParseKeywords(xmlNode, executionSteps, -1); // this populate the messages etc.

	    // BUT now we get the status from the top root node (some childs CAN have failed status and test being Successful - indeed you can tests some things that will condition how the tests continue)
	    String status = CXmlDocumentFactory.getNodeStringValueFromXPath(xmlNode, "status/@status");
	    if (status.equals("PASS")) {
	    	return RESULT_SUCCESS;
	    } else {
	    	return RESULT_FAILURE;
	    }
	}
	
	// a keyword can contain a keyword etc.
	private short recursivelyParseKeywords(Node xmlNode, Vector<CExecutionStep> executionSteps, int level) {
		System.err.println("- - - recursive");
		NodeList childNodeList = xmlNode.getChildNodes();
		System.err.println("- - -      nb childs = " + childNodeList.getLength());
		short result = RESULT_SUCCESS;
		for (int i=0; i<childNodeList.getLength(); i++) {
			Node childNode = childNodeList.item(i);
			System.err.println("- - -         > child" + i + "=" + childNode.getNodeName());
			
			if (childNode.getNodeName().equals("kw")) { //kw
				//String type    = childNode.getAttributes().getNamedItem("type").getNodeValue(); // kw type=xxx // WARNING there is not always some types !
				String name    = childNode.getAttributes().getNamedItem("name").getNodeValue(); // kw name=xxx
				System.err.println("+++ kw: " + name);
				String status  = "INFO";
				String endTime = "";
				String msg     = "";
				
				NodeList grandChildNodeList = childNode.getChildNodes();
				for (int j=0; j<grandChildNodeList.getLength(); j++) {
					Node grandChildNode = grandChildNodeList.item(j);
					if (grandChildNode.getNodeName().equals("status")) {
						System.err.println("    > status");
						try {
							status  = grandChildNode.getAttributes().getNamedItem("status").getNodeValue();
							endTime = grandChildNode.getAttributes().getNamedItem("endtime").getNodeValue();
						} catch (Exception e) {
						}
						
					} else if (grandChildNode.getNodeName().equals("msg")) {
						System.err.println("    > msg1");
						if (grandChildNode.getFirstChild() != null) { // CDATA
							msg = grandChildNode.getFirstChild().getNodeValue();
						} else {
							msg = grandChildNode.getNodeValue();
						}
						if (msg==null || msg.trim().length() == 0) {
							msg = "";
						} else {
							parseMsgNode(msg, executionSteps, level);
						}
					}
				}

				if (status.equals("PASS")) {
					System.err.println("    * pass");
					executionSteps.add(new CExecutionStep(endTime, RESULT_SUCCESS, getTabulation(level) + "[b][SUCCESS][/b] " + name + " {[i]" + msg + "[/i]}"));
				} else if (status.equals("INFO")) {
					System.err.println("    * info");
					executionSteps.add(new CExecutionStep(endTime, RESULT_INFO, getTabulation(level) + "[b][INFO][/b] " + name + " {[i]" + msg + "[/i]}"));
				} else {
					System.err.println("    * else");
					result = RESULT_FAILURE;
					executionSteps.add(new CExecutionStep(endTime, RESULT_FAILURE, getTabulation(level) + "[b][FAILURE][/b] " + name + " {[i]" + msg + "[/i]}"));
				}

				// recursivity
				short childResult = recursivelyParseKeywords(childNode, executionSteps, level+1);
				if (childResult == RESULT_FAILURE) {
					result = RESULT_FAILURE;
				}
				
			} else if (childNode.getNodeName().equals("doc")) { // doc
				/*
				String doc = CXmlDocumentFactory.getNodeTextOrCDataContent(childNode);
				if (doc!=null && doc.trim().length() > 0)
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, getTabulation(level) + "(" + doc + ")"));
				*/
			} else if (childNode.getNodeName().equals("msg")) { // msg
				/*
				String timeStamp = childNode.getAttributes().getNamedItem("timestamp").getNodeValue();
				//String status = childNode.getAttributes().getNamedItem("status").getNodeValue();
				String msg = CXmlDocumentFactory.getNodeTextOrCDataContent(childNode);
				if (msg!=null && msg.trim().length() > 0)
					executionSteps.add(new CExecutionStep(timeStamp, RESULT_INFO, getTabulation(level) + msg));
				*/
				
				// msg nodes can contain references to screenshots or images
				//<msg timestamp="20160316 17:59:11.257" html="yes" level="INFO">&lt;/td&gt;&lt;/tr&gt;&lt;tr&gt;&lt;td colspan="3"&gt;&lt;a href="../../data/Screenshots/selenium-screenshot-7.png"&gt;&lt;img src="../../data/Screenshots/selenium-screenshot-7.png" width="800px"&gt;&lt;/a&gt;</msg>
				String msg = "";
				System.err.println("    > msg2");
				if (childNode.getFirstChild() != null) { // CDATA
					msg = childNode.getFirstChild().getNodeValue();
				} else {
					msg = childNode.getNodeValue();
				}
				if (msg==null || msg.trim().length() == 0) {
					msg = "";
				} else {
					parseMsgNode(msg, executionSteps, level);
				}

			} else if (childNode.getNodeName().equals("arguments")) { // arguments
				NodeList argNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(childNode, "arg");
				int nbArguments = argNodeList.getLength();
				if (nbArguments > 0) {
					//executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, getTabulation(level) + "arguments:"));
					for (int j=0; j<nbArguments; j++) {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, getTabulation(level) + "- " + CXmlDocumentFactory.getNodeTextOrCDataContent(argNodeList.item(j))));
					}
				}
			}
		}
		
		return result;
	}
	
	private void parseMsgNode(String msgContent, Vector<CExecutionStep> executionSteps, int level) {
        int startIndex = msgContent.indexOf("<img src=\"");
        int stopIndex = 0;
        while (startIndex >= 0) {
        	stopIndex = msgContent.indexOf("\"", startIndex+12);
        	String embeddedImagePath = msgContent.substring(startIndex+10, stopIndex);
        	System.err.println("... ===> embeddedImagePath = " + embeddedImagePath); // path relative to workingDir
        	System.err.println("... ===> workingDir = " + workingDir);
        	
        	// download the image
        	File image = new File(workingDir.getAbsoluteFile() + "/" + embeddedImagePath);
        	if (image != null && image.exists() && image.isFile()) {

        		// If we had access to all the XStudio API from here we could simply call:
    			// - int attachmentId = CStudioAttachmentUtils.uploadAttachment(CTreeNodeData.TYPE_FOLDER, ITreeConstants.PARENT_ROOT_FOLDER_ID_DOCUMENT, image, true); // hidden
        		// and reference the image in the message as [img]123_image1.png[/img] (123 being the id) but unfortunately, the launcher is called asynchronously by the agent.
        		// So we have to use a trick and upload the image as an independant testcase execution attachment (asynchronoulsy) and reference it by name later on.
        		// We call it "on-the-fly" images
        		addTestcaseMessageAttachment(image);
    			
        		// Add the image in the messages as "ON_THE_FLY" images.
        		// - When messages includes standard images they are refered as [img]123_image1.png[/img] and images are stored in "table_folder_attach" table.
        		// - When messages includes ON_THE_FLY ("OTF") prefix instead of the id the renderer will recognize it and will download images from "table_exec_attach" from their names
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, getTabulation(level) + "- screenshot: [img]OTF_" + image.getName() + "[/img]"));
        	} else {
        		System.err.println("... ===> image " + image.getAbsolutePath() + " not found!");
        	}
			
    		startIndex = msgContent.indexOf("<img src=\"", startIndex+1);
        }
	}
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl toto = new CLauncherImpl();
		toto.workingDir = new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2");

		// *************** PARSING ********************
		
		//short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/E-Cargo_RCT_Release_1.0_18.12.2014_171224_output.xml"),
		//									"e-cargo_RCT_Connexion",
		//									executionSteps);

		//short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/robotframework_output.xml"),
		//									"OPLI001 Test Case",
		//									executionSteps);
		
		//short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/robotframework2_output.xml"),
		//									"OPLI001 Test Case",
		//									executionSteps);

		//short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/Robot_Framework_v2.9/robotframework_output2.xml"),
		//									"e-cargo_RCT_QuoteAndBookBARDisplayNominal",
		//									executionSteps);

		/*
		short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/rf_output.xml"),
											"EXEC",
											executionSteps);
		*/
		
		//toto.workingDir = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\robot_framework_v2\\sample\\RIDEvk36z5.d");
		//short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/RIDEvk36z5.d/output.xml"),
		//									"e-cargo_RCT_QuoteAndBookBARDisplayNominal",
		//									executionSteps);

		//toto.workingDir = new File("Y:\\XStudio\\src\\com\\xqual\\xlauncher\\robot_framework_v2\\sample\\REST");
		//short result = toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/REST/rf_output.xml"),
		//									"e-cargo_RCT_QuoteAndBookBARDisplayNominal",
		//									executionSteps);

		/*
		System.err.println("RESULT = " + CUtils.resultToUserString(result));
		for (int i=0; i<executionSteps.size(); i++) {
			System.out.println(resultToUserString(executionSteps.get(i).getResult()) + " " + executionSteps.get(i).getMessage());
		}
		*/
		
		// ****************** SCANNER *****************
		toto.testRootPath = "Y:/XStudio/src/com/xqual/xlauncher/robot_framework_v2/sample/tests";
		toto.extension = ".robot";
		Vector<CScannedTest> output = toto.scanPhysicalTests();
		
		//System.err.println("- - - ->>>  tests and testcases found = " + output);
	}

}
