/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.3                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.sahi60;

import java.io.File;
import java.util.Arrays;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;

import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for AutoIt.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{sahi60        }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String sahiInstallPath;
	private String baseUrl;
	private String browser;

	private File workingDir;

	private File testcaseTraces;
	
	private CRunner sahiTestcaseRunner;

	private String testAdditionalInfo;
	private String testcaseAdditionalInfo;

	private static final String TESTRUNNER = CUtils.getBatchName("testrunner");
	
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath        = getStringParamValue("General", "Test root path");       // i.e. Y:/Externals/sahi/userdata/scripts

			sahiInstallPath     = getStringParamValue("Sahi",    "Sahi install path");    // i.e. Y:/Externals/sahi
			baseUrl             = getStringParamValue("Sahi",    "Url");                  // i.e. http://www.google.com
			browser             = getStringParamValue("Sahi",    "Browser");              // i.e. "ie" or "ie+firefox"

			workingDir = new File(sahiInstallPath + "/userdata/bin");

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		// delete all previous session's reports so that we don't mix things up
		// once the reports are named as "xxx_2010-12-31 23:59_xxx" in Sahi we could remove this hack and use the date and time to retrieve the good one :/
		if (!deleteAllHtmlReportFiles()) {
			traceln(LOG_PRIORITY_SEVERE, "couldn't delete the former reports !");
		}

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		short result = RESULT_SUCCESS;
		return new CReturnStatus(result);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		testcaseAdditionalInfo = additionalInfo;

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		testcaseTraces = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "testcase_traces.txt");
		CFileUtils.deleteAndTouchFile(testcaseTraces);

		if (!deleteAllHtmlReportFiles()) {
			traceln(LOG_PRIORITY_SEVERE, "couldn't delete the former reports !");
		}
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		String url="";
		if (testcaseAdditionalInfo != null && testcaseAdditionalInfo.length()>0) {
			url = baseUrl + "/" + testcaseAdditionalInfo;
		} else if (testAdditionalInfo != null && testAdditionalInfo.length()>0) {
			url = baseUrl + "/" + testAdditionalInfo;
		} else {
			url = baseUrl;
		}

		String relativeTestPath = "";
		if (testRootPath.length() > 0)
			relativeTestPath = relativeTestPath + testRootPath + "/";
		if (testPath.length() > 0)
			relativeTestPath = relativeTestPath + testPath + "/";
		
		relativeTestPath = relativeTestPath + testName + ".sah";
		
		
        sahiTestcaseRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
								           CFileUtils.quoteFilePath(sahiInstallPath + "/userdata/bin/" + TESTRUNNER) + " " +
		                                   CFileUtils.quoteFilePath(relativeTestPath) + " " +
		                                   url + " " +
		                                   browser,
                                           workingDir);
        short result = sahiTestcaseRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, testcaseTraces);
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "script interpretation failed"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		File htmlReportFile = retrieveHtmlReportFile(testName);
		if (htmlReportFile != null && htmlReportFile.exists()) {
			traceln(LOG_PRIORITY_INFO, "report exists");
			addTestcaseMessageAttachment(htmlReportFile);
			addTestcaseMessageAttachment(testcaseTraces);

			return parseResultFile(htmlReportFile, executionSteps);
		} else {
			traceln(LOG_PRIORITY_SEVERE, "report does not exist!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't find Sahi report to parse!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		CFileUtils.deleteFile(testcaseTraces);

		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private File retrieveHtmlReportFile(String testName) {
		// First search in <install>/userdata/logs/playback
		File logsFolder = new File(sahiInstallPath + "/userdata/logs/playback");
		if (!logsFolder.exists()) {
			traceln(LOG_PRIORITY_WARNING, "Could not find \"" + logsFolder + "\"...");
			return null;
		}

		traceln(LOG_PRIORITY_INFO, "searching for report file in \"" + logsFolder + "\"...");
		File reportFile = recursivelySearchForHtmlReportFile(logsFolder, testName);
		if (reportFile != null) {
			traceln(LOG_PRIORITY_INFO, "report found in root folder: \"" + reportFile + "\"...");
			return reportFile;
		}

		traceln(LOG_PRIORITY_SEVERE, "no test report!");
		return null;
	}

	private File recursivelySearchForHtmlReportFile(File parentFolder, String testName) {
		traceln(LOG_PRIORITY_INFO, ">>> \"" + parentFolder + "\"...");
		//File backupReport = null;
		File[] childFiles = parentFolder.listFiles();
		if (childFiles != null) {
			// first check the files
			for (int j=0; j<childFiles.length; j++) {
				File childFile = childFiles[j];	
				if (childFile.isFile() &&                                                                  // maybe a report
			        (childFile.getName().endsWith(".htm") || childFile.getName().endsWith(".html"))) {     // html file
					System.err.println("    > file: " + childFile);

					System.err.println("- - - > " + childFile);
					if (childFile.getName().indexOf(testName) >= 0 && // name includes <TestName>
						!childFile.getName().startsWith("testcase_summary") &&
						!childFile.getName().startsWith("index")) {
						traceln(LOG_PRIORITY_INFO, "!!!! TEST REPORT FOUND !!! " + childFile);
						return childFile;
					}
					/*
					System.err.println("- - - > " + childFile);
					if (childFile.getName().startsWith("index")) {                                         // name starts with "index"
						traceln(LOG_PRIORITY_INFO, "!!!! INDEX TEST REPORT FOUND !!! " + childFile);
						backupReport = childFile;
						// backup solution do not return immediately but only if nothing else is found
					}
					*/
				}
			}
			
			// then restart with only the directories
			for (int j=0; j<childFiles.length; j++) {
				File childFile = childFiles[j];
				if (childFile.isDirectory()) {                                                             // child folder
					System.err.println("    > folder: " + childFile);
					File reportFile = recursivelySearchForHtmlReportFile(childFile, testName);
					if (reportFile != null)
						return reportFile;
				}
			}

			// once we tested all the files
			/*
			if (backupReport != null)
				return backupReport;
			*/
		}
		return null;
	}    
    
	private boolean deleteAllHtmlReportFiles() {
		System.err.println("- - - > deleting reports...");
		File logsFolder = new File(sahiInstallPath + "/userdata/logs/playback");

		if (logsFolder== null || !logsFolder.exists()) {
			traceln(LOG_PRIORITY_WARNING, "Could not find \"" + logsFolder + "\"...");
			return false;
		}

		if (logsFolder.isDirectory()) {
			File[] testFolders = logsFolder.listFiles();
			if (testFolders.length > 0) {
				for (int i=0; i<testFolders.length; i++) {
					File testFolder = testFolders[i];
					System.err.println("- - - > deleting reports and potential containers " + testFolder + "...");
					if (!CFileUtils.deleteFile(testFolder)) { // delete the complete folder or the file
						traceln(LOG_PRIORITY_WARNING, "Could not delete \"" + testFolder + "\"...");
						return false;
					}
				}
			}
		}
		return true;
	}

	private CReturnStatus parseResultFile(File htmlReportFile, Vector<CExecutionStep> executionSteps) {
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Parsing the reports..."));
		
		// first of all, let's check if there is an "images" subfolder and if so upload all the images
		File imageFolder = new File(htmlReportFile.getParent() + "/images");
		if (imageFolder.exists() && imageFolder.isDirectory()) {
			addTestcaseMessageAttachments(new Vector<File>(Arrays.asList(imageFolder.listFiles())));
		}

		//trick to workaround a problem with the format generated (invalid)
		String xmlContent = CFileUtils.FileToString(htmlReportFile)
						        .replaceAll("<br>", "")
						        .replaceAll("&", "&amp;"); // there are some '&' characters in some traces in the xml so that MAY corrupt the XML validity
        
		//trick to workaround an inconsistency between Sahi and SahiPro
		String reportContent = (xmlContent.indexOf("xml version")>=0) ? 
								xmlContent : 
								"<?xml version=\"1.0\" encoding=\"UTF-8\" ?><root>" + xmlContent + "</root>";
		
		//trick to workaround an issue in the format generated by Sahi (what's inside the <script> tag may contain invalid XML characters)
		reportContent = reportContent.replaceAll("<script>", "<script><![CDATA[");
		reportContent = reportContent.replaceAll("<script type=\"text/javascript\">", "<script><![CDATA[");
		reportContent = reportContent.replaceAll("</script>", "]]></script>");
		reportContent = reportContent.replaceAll("<META http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">", "");
		reportContent = reportContent.replaceAll("<meta http-equiv=\"content-type\" content=\"text/html; charset=UTF-8\">", "");

		//trick to workaround invalid tag for images in Sahi reports
		reportContent = reportContent.replaceAll(".png\">", ".png\"/>");
		reportContent = reportContent.replaceAll(".gif\">", ".gif\"/>");
		reportContent = reportContent.replaceAll(".jpg\">", ".jpg\"/>");
		
		// trick to workaround not terminated <input> tag
		while (reportContent.indexOf("<input") > 0) {
			System.err.println("- - - > input tag found");
			int startIndex = reportContent.indexOf("<input");
			int stopIndex = reportContent.indexOf(">", startIndex);
			String begining = reportContent.substring(0, startIndex);
			String ending = reportContent.substring(stopIndex+1);
			reportContent = begining + ending;
		}

		System.err.println("\n- - -  workarounded reportContent = " + reportContent + "\n");
		
		boolean atLeastOneFailure = false;
		
		Document xmlDocument;
		try {
			xmlDocument = CXmlDocumentFactory.createXMLDoc(reportContent);
			if (xmlDocument != null) {
				Node resultNode = CXmlDocumentFactory.getNodeFromXPath(xmlDocument, "//table[@class='summary']/tr[position()=2]"); // second line of the summary table
				
				if (resultNode != null) {
					String totalSteps  = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "td[position()=2]");
					String failures    = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "td[position()=3]");
					String errors      = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "td[position()=4]");
					String successRate = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "td[position()=5]");
					String timeTaken   = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "td[position()=6]") + " ms";
					
					traceln(LOG_PRIORITY_INFO, "- totalSteps  = " + totalSteps);
					traceln(LOG_PRIORITY_INFO, "- failures    = " + failures);
					traceln(LOG_PRIORITY_INFO, "- errors      = " + errors);
					traceln(LOG_PRIORITY_INFO, "- successRate = " + successRate);
					traceln(LOG_PRIORITY_INFO, "- timeTaken   = " + timeTaken);
					
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "total steps=" + totalSteps));
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "success rate=" + successRate));
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "time taken=" + timeTaken));
					if (!failures.equalsIgnoreCase("0") || !errors.equalsIgnoreCase("0")) {
						atLeastOneFailure = true;
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "failures=" + failures + "/errors=" + errors));
					}
				} else {
					System.err.println("- - -  report HTML is invalid! (parsing)");
					atLeastOneFailure = true;
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't parse the results xml (parsing)"));
				}
			} else {
				System.err.println("- - -  report HTML is invalid! (null document)");
				atLeastOneFailure = true;
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't parse the results xml (null document)"));
			}
		} catch (Exception e) {
			System.err.println("- - -  report HTML is invalid! (exception)");
			atLeastOneFailure = true;
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't parse the results xml (exception)"));
		}

		traceln(LOG_PRIORITY_INFO, "- executionSteps  = " + executionSteps);
		return new CReturnStatus(atLeastOneFailure ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
	}
	

	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		//File inputFile = new File("C:/Documents and Settings/egavaldo/Desktop/App andern_ie__25IX2012__12_24_42_286.html");
		//File inputFile = new File("C:/Documents and Settings/egavaldo/Desktop/App andern_ie__25IX2012__12_43_07_348.html");
		//File inputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/sahi40/try2_12Mar2014_14_06_43.htm");
		File inputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/sahi60/sample/login_chrome__87acbffc0beac048e30bc460d86c42d5e068.html");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl myLauncher = new CLauncherImpl();
		myLauncher.sahiInstallPath = "Y:/XStudio/src/com/xqual/xlauncher/sahi60/";
		
		myLauncher.parseResultFile(inputFile, executionSteps);
		

		
		
/*		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl myLauncher = new CLauncherImpl();
		myLauncher.sahiInstallPath = "Y:/XStudio/src/com/xqual/xlauncher/sahi60/";
		
		File report = myLauncher.retrieveHtmlReportFile("test");
		myLauncher.parseResultFile(report, executionSteps);
		
		//System.err.println("attachments: " + myLauncher.testcaseMessageAttachments);
*/
		
	}
}
