/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.0                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.selenese;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CLauncherCommonParsers;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Selenese.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{selenese      }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String javaInstallPath;
	private String seleneseRunnerJarPath;
	private String arguments;
	private String attachmentsPath;

	private File workingDir;

	private File seleneseHtmlReportFile;
	private File seleneseXmlReportFile;
	private File seleneseTraceFile;
	
	private static final String JAVA_INTERPRETER_EXE = CUtils.getExecutableName("java");

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath            = getStringParamValue("General", "Test root path");                  // i.e. C:/test_repository/tests/selenium/html

			javaInstallPath         = getStringParamValue("Selenese", "Java install path");              // i.e. C:/Program Files/Java/jdk1.6.0_17
			seleneseRunnerJarPath   = getStringParamValue("Selenese", "Selenese Runner JAR path");       // i.e. Y:/Externals/Selenese/selenese-runner.jarr
			arguments               = getStringParamValue("Selenese", "Arguments");                      // i.e. --driver ie --iedriver IEDriverServer.exe

			attachmentsPath         = getStringParamValue("Attachments", "Additional attachments path"); // i.e. C:/test_repository/tests/selenium/attachments
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		// WARNING: Selenese does not support well testcase name including '.' characters
		
		workingDir = new File(testRootPath); // parent folder of all the test classes
		
		seleneseHtmlReportFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "TEST-" + testcaseName + ".html");
		CFileUtils.deleteAndTouchFile(seleneseHtmlReportFile);

		seleneseXmlReportFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "TEST-" + testcaseName + ".xml");
		CFileUtils.deleteAndTouchFile(seleneseXmlReportFile);

		seleneseTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "selenium_server_traces.txt");
		CFileUtils.deleteAndTouchFile(seleneseTraceFile);

		
		if (attachmentsPath.length()>0) {
			File attachmentsPathFolder = new File(attachmentsPath);
			if (attachmentsPathFolder.exists() && attachmentsPathFolder.isDirectory()) {
				String[] children = attachmentsPathFolder.list();
				for (int i = 0; i < children.length; i++) {
					boolean returnCode = CFileUtils.deleteFile(new File(attachmentsPathFolder, children[i]));
					if (!returnCode) {
						traceln(LOG_PRIORITY_SEVERE, "Could not delete file " + new File(attachmentsPathFolder, children[i]) + "...");
					}
				}
			}
		}
		
		// +------------------------------------+
		// | Run Selenese
		// +------------------------------------+
		
		// if there is already a selenium server running that's not a problem has 2 server cannot listen on the same port this will fail nicely
		CRunner seleneseRunner = new CRunner("[Selenese runner] ",
	                                     CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE) + " -jar " + CFileUtils.quoteFilePath(seleneseRunnerJarPath) + " " + arguments + " " +
	                                     CFileUtils.quoteFilePath(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testcaseName + ".html") + " " + 
	                                     "--strict-exit-code " +
	                                     "--screenshot-dir " + CFileUtils.quoteFilePath(attachmentsPath) + " " +
	                                     "--html-result " + CFileUtils.quoteFilePath(seleneseHtmlReportFile.getParentFile().getAbsolutePath()) + " " +
	                                     "--xml-result " + CFileUtils.quoteFilePath(seleneseXmlReportFile.getParentFile().getAbsolutePath()),
	                                     workingDir);
			
			
		short result = seleneseRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, seleneseTraceFile);
		/*
		strict code returned
		   - 0: SUCCESS
		   - 2: WARNING
		   - 3: FAILURE
		   - 4: ERROR
		   - 5: UNEXECUTED
		*/
		
		// +------------------------------------+
		// | Kill Selenese ???
		// +------------------------------------+
		seleneseRunner.killProcess();

		// selenium returns 1 in case of success and 2 in case of failure
		// it retuns also 2 when it crashed (we detect this a bit later when there is no result file)
		/*
		if (result != 1 && result != 2) {
			System.err.println("- - - - - UNEXPECTED RETURN");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "selenium server refused to start!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		*/

		// +----------------------------------------------+
		// | Add the output console and report to the test
		// +----------------------------------------------+
		if (seleneseTraceFile.exists())
			addTestcaseMessageAttachment(seleneseTraceFile);

		if (seleneseHtmlReportFile.exists())
			addTestcaseMessageAttachment(seleneseHtmlReportFile);

		if (seleneseXmlReportFile.exists())
			addTestcaseMessageAttachment(seleneseXmlReportFile);
		
		if (attachmentsPath.length()>0) {
			File attachmentsPathFolder = new File(attachmentsPath);
			if (attachmentsPathFolder.exists() && attachmentsPathFolder.isDirectory()) {
				String[] children = attachmentsPathFolder.list();
				for (int i = 0; i < children.length; i++) {
					addTestcaseMessageAttachment(new File(attachmentsPathFolder, children[i]));
				}
			}
		}
		
		File extractedOutputFile = null; // don't need to extract something
		CReturnStatus returnStatus = CLauncherCommonParsers.parseJUnitXmlFile(seleneseXmlReportFile, extractedOutputFile, testcaseName, executionSteps);
		if (returnStatus.getResult() == RESULT_NOT_EXECUTED) {
			CReturnStatus returnStatusAlternativeParsing = CLauncherCommonParsers.parseJUnitXmlFile(seleneseXmlReportFile, extractedOutputFile, "test_1", executionSteps); // in some conditions, the testcase name in the XML is always test_1 !!!
			if (returnStatusAlternativeParsing.getResult() != RESULT_NOT_EXECUTED) {
				returnStatus = returnStatusAlternativeParsing;
			}
		}
		
		//addTestcaseMessageAttachment(extractedOutputFile);
		
		return returnStatus;
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2ehtml$",         // only *.html files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl toto = new CLauncherImpl();
		

		System.err.println("========================================================");
		File seleneseXmlReportFile = new File("Y:/XStudio/src/com/xqual/xlauncher/selenese/sample/TEST-LogIn_Logoff.xml");
		//String seleneseXmlReportFileContent = CFileUtils.FileToString(seleneseXmlReportFile);
		
		File extractedOutputFile = null; // don't need to extract something
		CReturnStatus returnStatus = CLauncherCommonParsers.parseJUnitXmlFile(seleneseXmlReportFile, extractedOutputFile, "LogIn_Logoff", new Vector<CExecutionStep>());
		if (returnStatus.getResult() == RESULT_NOT_EXECUTED) {
			/*
			CReturnStatus returnStatusAlternativeParsing = CLauncherCommonParsers.parseJUnitXmlFile(seleneseXmlReportFile, extractedOutputFile, "test_1", executionSteps); // in some conditions, the testcase name in the XML is always test_1 !!!
			if (returnStatusAlternativeParsing.getResult() != RESULT_NOT_EXECUTED) {
				returnStatus = returnStatusAlternativeParsing;
			}
			*/
		}
	
		System.err.println("========================================================");

	}
}
