/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.00                                               |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.selenium3_html;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Writer;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Selenium HTML.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{selenium3 Html }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String javaInstallPath;
	private String seleniumServerJarPath;
	private String browser;
	private String domainUrl;
	private String attachmentsPath;
	
	private String browserDriverFile;
	private String browserCodeName;

	private File workingDir;

	private File seleniumServerReportFile;
	private File seleniumServerTraceFile;
	private File selenium3HtmlReportFile;
	private File testsuiteHtmlFile;
	
	private String browserDriverProperty;
	private String browserLogProperty;
	
	private static final String JAVA_INTERPRETER_EXE = CUtils.getExecutableName("java");

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath            = getStringParamValue("General", "Test root path");                   // i.e. C:/test_repository/tests/selenium/html

			javaInstallPath         = getStringParamValue("Selenium - Html", "Java install path");        // i.e. C:/Program Files/Java/jdk1.6.0_17
			seleniumServerJarPath   = getStringParamValue("Selenium - Html", "Server JAR path");          // i.e. Y:/Externals/Selenium-1.0/selenium-remote-control-1.0.1/selenium-server-1.0.1/selenium-server.jar
			browser                 = getStringParamValue("Selenium - Html", "Browser");                  // i.e. googlechrome (it can also firefox or edge or iehta)
			domainUrl               = getStringParamValue("Selenium - Html", "Domain URL");               // i.e. http://www.xqual.com
			
			browserDriverFile = getStringParamValue("Selenium - Html", "Brower WebDriver file full path"); //c:/chromedriver/chromedriver.exe

			attachmentsPath         = getStringParamValue("Attachments", "Additional attachments path");  // i.e. C:/test_repository/tests/selenium/attachments

			if (browser.equalsIgnoreCase("Firefox")) {
				browserDriverProperty="webdriver.gecko.driver";
				browserLogProperty="webdriver.gecko.logfile";
				browserCodeName ="*firefox";
			}
			else if (browser.equalsIgnoreCase("Internet Explorer")) {
				browserDriverProperty="webdriver.ie.driver";
				browserLogProperty="webdriver.ie.driver.logfile";
				browserCodeName ="*iexplore";
			}
			else if (browser.equalsIgnoreCase("Edge")) {
				browserDriverProperty="webdriver.edge.driver";
				browserLogProperty="webdriver.edge.logfile";		
				browserCodeName ="*MicrosoftEdge";
			}			
			else if (browser.equalsIgnoreCase("Chrome")) {
				browserDriverProperty="webdriver.chrome.driver";
				browserLogProperty="webdriver.chrome.logfile";
				browserCodeName ="*googlechrome";				
			}
			
			
			} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		workingDir = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "")); //

		seleniumServerReportFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "report");
		if (seleniumServerReportFile.exists()) {
			CFileUtils.deleteFile(seleniumServerReportFile);
		}
		
		seleniumServerTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/report/" : "") + "selenium_server_traces.txt");
		CFileUtils.deleteAndTouchFile(seleniumServerTraceFile);

		
		if (attachmentsPath.length()>0) {
			File attachmentsPathFolder = new File(attachmentsPath);
			if (attachmentsPathFolder.exists() && attachmentsPathFolder.isDirectory()) {
				String[] children = attachmentsPathFolder.list();
				for (int i = 0; i < children.length; i++) {
					boolean returnCode = CFileUtils.deleteFile(new File(attachmentsPathFolder, children[i]));
					if (!returnCode) {
						traceln(LOG_PRIORITY_SEVERE, "Could not delete file " + new File(attachmentsPathFolder, children[i]) + "...");
					}
				}
			}
		}
		
			
		
		// +------------------------------------+
		// | Run the Selenium server
		// +------------------------------------+
		
		// if there is already a selenium server running that's not a problem has 2 server cannot listen on the same port this will fail nicely
		CRunner seleniumServer;
		
		testsuiteHtmlFile= new File (testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + ".html");

		if (!isTestSuite()) {
			// we need to create a test suite on the fly that contains only one test case
			traceln(LOG_PRIORITY_INFO, "we are tretaing a test case , not a test suite \"" + testsuiteHtmlFile + "\"");
			StringBuffer outputBuffer = new StringBuffer();
					
			outputBuffer.append("<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
			outputBuffer.append("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n");
			outputBuffer.append("<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n");
			outputBuffer.append("<head>\n");
			outputBuffer.append("    <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n");
			outputBuffer.append("    <title>Test suite</title>\n");
			outputBuffer.append("    <title>" + CTextUtils.escapeAttributeString(testcaseName) + "</title>\n");
			outputBuffer.append("</head>\n");
			outputBuffer.append("<body>\n");
			outputBuffer.append("	<table id=\"suiteTable\" cellpadding=\"1\" cellspacing=\"1\" border=\"1\" class=\"selenium\">\n");
			outputBuffer.append("		<tbody>\n");
			outputBuffer.append("			<tr><td><b>Test Suite</b></td></tr>\n");
			outputBuffer.append("				<tr><td><a href=\"./" + CTextUtils.escapeAttributeString(testName) + ".html\">" + CTextUtils.escapeAttributeString(testName) + "</a></td></tr>\n");
			outputBuffer.append("   	</tbody>\n");
			outputBuffer.append("	</table>\n");
			outputBuffer.append("</body>\n");
			outputBuffer.append("</html>\n");

			testsuiteHtmlFile = new File(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/on_the_fly.html");
			CFileUtils.deleteFile(testsuiteHtmlFile);
			try {
				testsuiteHtmlFile.createNewFile();
			    Writer output = new BufferedWriter(new FileWriter(testsuiteHtmlFile));
			    output.write(outputBuffer.toString());
			    output.close();
			} catch (IOException e) {
				traceln(LOG_PRIORITY_SEVERE, "Could not create file \"" + testsuiteHtmlFile + "\"");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Could not create file \"" + testsuiteHtmlFile + "\""));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
		
			traceln(LOG_PRIORITY_INFO, testsuiteHtmlFile + " has been generated");
		}
		traceln(LOG_PRIORITY_INFO, "the file to be used is " + CFileUtils.quoteFilePath(testsuiteHtmlFile.getAbsolutePath()));
		
		seleniumServer = new CRunner("[Selenium Server] ",
	                                CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE) +
									" -D" + browserDriverProperty + "=" + CFileUtils.quoteFilePath(browserDriverFile) +
									" -D" + browserLogProperty + "=" + CFileUtils.quoteFilePath(seleniumServerTraceFile.getAbsolutePath()) +
									" -jar " +
									CFileUtils.quoteFilePath(seleniumServerJarPath) + " " + 
	                                "-htmlSuite \"" + browserCodeName + "\" " +
									domainUrl + " " + // do not double-quote the domain URL as the URL would be detected as malformed by selenium on Linux
	                                CFileUtils.quoteFilePath(testsuiteHtmlFile.getAbsolutePath()) + " " +
									CFileUtils.quoteFilePath(seleniumServerReportFile.getAbsolutePath()) + " ",
	                                workingDir);
			
		
			
			
		short result = seleniumServer.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
		
		// +------------------------------------+
		// | Run the Selenium server
		// +------------------------------------+
		seleniumServer.killProcess();

		// selenium returns 1 in case of success and 2 in case of failure
		// it retuns also 2 when it crashed (we detect this a bit later when there is no result file)
		if (result != 1 && result != 2) {
			System.err.println("- - - - - UNEXPECTED RETURN");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "selenium server refused to start!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		// +----------------------------------------------+
		// | Add the output console and report to the test
		// +----------------------------------------------+
		if (seleniumServerReportFile.exists())
			addTestcaseMessageAttachmentFolder(seleniumServerReportFile);
		
		
		// we clean as we don't want to leave an on the fly behind us.
		testsuiteHtmlFile = new File(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/on_the_fly.html");
		CFileUtils.deleteFile(testsuiteHtmlFile);
		
		if (attachmentsPath.length()>0) {
			File attachmentsPathFolder = new File(attachmentsPath);
			if (attachmentsPathFolder.exists() && attachmentsPathFolder.isDirectory()) {
				String[] children = attachmentsPathFolder.list();
				for (int i = 0; i < children.length; i++) {
					addTestcaseMessageAttachment(new File(attachmentsPathFolder, children[i]));
				}
			}
		}

		return parseResultFile(executionSteps); // even when succeeding, it looks like selenium exits with return code > 0
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2ehtml$",         // only *.html files but find all test suite and test test
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+
	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// length() trick: it's necessary as there sounds to be a bug in java where exists() always
		// returns false or true if the file in mapped on a remote network drive
		if (!seleniumServerReportFile.exists()) { 
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "selenium server probably crashed!"));
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: report not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_NOT_EXECUTED, "run: result file found"));
		}

		String line;
		boolean errorDetected = false;
		String regexFile;
		String fileName;
		
		
		try {
			
			File[] children = seleniumServerReportFile.listFiles();
			for (int i = 0; i < children.length; i++) {
			File child = children[i];
			traceln(LOG_PRIORITY_INFO, "file found in reports  : " + child.getName());
			regexFile = "^.*\\x2ehtml$";
			if (Pattern.matches(regexFile, child.getName())) {
				fileName = child.getName();
				traceln(LOG_PRIORITY_INFO, "we found the html " + fileName);
				selenium3HtmlReportFile = new File(seleniumServerReportFile, fileName);
				}
			}
			
			
			FileInputStream fileInputStream = new FileInputStream(selenium3HtmlReportFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				if (line.startsWith("<td>result:</td>")) {
					line = bufferedReader.readLine().trim();
					String globalResult = line.substring(4, line.length()-5);
					traceln(LOG_PRIORITY_INFO, "globalResult  : " + globalResult);
					if (globalResult.equalsIgnoreCase("pass")) {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Result: " + globalResult));
					} else {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Result: " + globalResult));
						errorDetected = true;
					}

				} else if (line.startsWith("<td>totalTime:</td>")) {
					line = bufferedReader.readLine().trim();
					String totalTime = line.substring(4, line.length()-5);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "totalTime: " + totalTime));

				} else if (line.startsWith("<td>numTestTotal:</td>")) {
					line = bufferedReader.readLine().trim();
					String numTestTotal = line.substring(4, line.length()-5);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "numTestTotal: " + numTestTotal));

				} else if (line.startsWith("<td>numTestPasses:</td>")) {
					line = bufferedReader.readLine().trim();
					String numTestPasses = line.substring(4, line.length()-5);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "numTestPasses: " + numTestPasses));

				} else if (line.startsWith("<td>numTestFailures:</td>")) {
					line = bufferedReader.readLine().trim();
					int numTestFailures = Integer.parseInt(line.substring(4, line.length()-5));
					executionSteps.add(new CExecutionStep(numTestFailures > 0 ? RESULT_FAILURE: RESULT_SUCCESS, "numTestFailures: " + numTestFailures));

				} else if (line.endsWith("status_passed\"><TD>")) {
					line = bufferedReader.readLine().trim();
					String[] array = line.split(">");

					String testName = array[1].substring(1, array[1].length()-9);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "passed: " + testName));

				} else if (line.endsWith("status_failed\"><TD>")) {
					line = bufferedReader.readLine().trim();
					String[] array = line.split(">");

					String testName = array[1].substring(1, array[1].length()-9);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "failed: " + testName));

				} else {
					//traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}
			
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception whle parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else{
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
	
	
	private boolean isTestSuite() {
	
	boolean isASuite = false;
	String line;
	
		try {
			FileInputStream fileInputStream = new FileInputStream(testsuiteHtmlFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));
			
			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);
				if (line.indexOf("suiteTable") > -1) {
					isASuite = true;
				}	
			}
			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e){
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the selenium html file: " + e);
		}
	return isASuite;
	}
}
