/*
+----------------------------------------------------------------------+
|                    Class: CLauncherImpl                              |
|                                                                      |
| Developper:  Sanjeeth Nayak                                          |
|              Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     0.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.sikulix;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CNetworkInfo;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for SikuliX.
 * @author Eric gavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

    // +==============================================================+
    // | Attributes                                                   |
    // +==============================================================+

    static final String TRACE_HEADER = "{sikulix       }  ";

    private String sikuliRunner;
    
    private String testRootPath;
    private String sikuliInstallPath;
    private String scriptExtension;
    private int debugLevel;
    private String sikuliLogFilePath;
    private String userLogFilePath;
    private String optArguments;

    private File sikuliTraceFile;
    private File sikuliLogFile;
    private File userLogFile;
    
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

    public CLauncherImpl() {
            super(TRACE_HEADER);
    }

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
    
    @Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
        setSutDetails(sutId, sutName, sutVersion);
        
		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager        
		printConfiguration();
        
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        try {
        	// retrieve the parameters we need
        	testRootPath        	 = getStringParamValue("General",  "Test root path");        // i.e. C:/test_repository/tests/jar
        	
        	sikuliInstallPath        = getStringParamValue("SikuliX",  "Install path");          // i.e. D:/softwares/Sikuli
        	scriptExtension          = getStringParamValue("SikuliX",  "Script extension");      // i.e. .sikuli
        	debugLevel               = getIntegerParamValue("SikuliX", "Debug level (1-3)");     // i.e. 2
        	sikuliLogFilePath        = getStringParamValue("SikuliX",  "Sikuli Log file path");  // i.e. C:/Users/egavaldo/sikuli_log.txt
        	userLogFilePath          = getStringParamValue("SikuliX",  "User Log file path");    // i.e. C:/Users/egavaldo/user_log.txt
        	optArguments             = getStringParamValue("SikuliX",  "Opt. arguments");
        } catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
            return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
        }
        
    	CNetworkInfo networkInfo = new CNetworkInfo();
    	short osType = networkInfo.getOsType();
    	
		switch (osType) {
			case CNetworkInfo.OS_WIN32: 
			case CNetworkInfo.OS_WIN64: 
    			sikuliRunner = CFileUtils.quoteFilePath(sikuliInstallPath + "/runsikulix.cmd");
    			break;
    		case CNetworkInfo.OS_LINUX: 
    		case CNetworkInfo.OS_MAC:   
    			sikuliRunner = CFileUtils.quoteFilePath(sikuliInstallPath + "/runsikulix");
    			break;
    		case CNetworkInfo.OS_UNKNOWN:
    		default:
    			traceln(LOG_PRIORITY_SEVERE, "OS couldn't be determined!");
    			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "OS couldn't be determined!"));
    			
    			CUtils.showErrorMessage(null, 
    					"Error: Couldn't initialize the launcher", 
    					"OS couldn't be determined!",
    					JOptionPane.ERROR_MESSAGE);
    			
                return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
		}

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
        return new CReturnStatus(RESULT_SUCCESS);
    }


    @Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
        traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " Test:" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		File workingDir = new File(testRootPath); // parent folder of all the test classes

		// +------------------------------------+
		// | Delete previous results file
		// +------------------------------------+
		sikuliTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "sikuli_traces.txt");
		CFileUtils.deleteAndTouchFile(sikuliTraceFile);

	    sikuliLogFile = new File(sikuliLogFilePath);
		CFileUtils.deleteAndTouchFile(sikuliLogFile);
		
	    userLogFile = new File(userLogFilePath);
		CFileUtils.deleteAndTouchFile(userLogFile);

		// +------------------------------------+
		// | Run the sikuli test class
		// +------------------------------------+
	
		// compute the option
		StringBuffer arguments = new StringBuffer(" -- ");
		if (optArguments != null && optArguments.trim().length() > 0) {
			arguments.append(optArguments.trim());
		}
		for (int k=0; k<params.size(); k++) {
			CTestcaseParam param = params.get(k);
			String paramName = param.getName();
			String paramValue = param.getStringValue();
			arguments.append(" '" + paramName + "=" + paramValue + "'");
		}
		String argumentsStr = arguments.toString();
		String option = (argumentsStr.length()>4) ? argumentsStr : "";
		
		// run the command
        CRunner jarRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                        sikuliRunner +
                                        " -d " + debugLevel + 
                                        " -l " + CFileUtils.quoteFilePath(sikuliLogFilePath) +
                                        " -u " + CFileUtils.quoteFilePath(userLogFilePath) +
                                        " -c " + // console
                                        " -r " + CFileUtils.quoteFilePath(testRootPath + "/" + testName + "/" + testcaseName + scriptExtension) + 
                                        option, 
                                        workingDir);
        
        short result = jarRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, sikuliTraceFile);
        
		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
        if (sikuliTraceFile.exists() && sikuliTraceFile.length()>0)
        	addTestcaseMessageAttachment(sikuliTraceFile);
        
        if (sikuliLogFile.exists() && sikuliLogFile.length()>0) 
        	addTestcaseMessageAttachment(sikuliLogFile);

        if (userLogFile.exists() && userLogFile.length()>0) 
        	addTestcaseMessageAttachment(userLogFile);

        
        // the result is based on the exit code
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Sikuli execution process failed"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Sikuli execution process succeeded"));
			return parseResultFile(sikuliTraceFile, executionSteps);
		}
    }

    @Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
        return new CReturnStatus(RESULT_SUCCESS);
    }

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTestcases(output,
				  testRootPathFolder,                // folder to scan
				  "^.*$",                            // any folder
				  "^.*\\x2" + scriptExtension + "$", // only *.mds files
				  true,                              // remove extension from the file name to determine the test name
				  "/",                               // separator used in canonicalPath generated
				  "");                               // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	@Override
	public void recursivelyScanFolderForTests(Vector<CScannedTest> output, File folder, String regexFolder, String regexFile, boolean removeExtension, String separator, String currentRelativePath) {
		if (folder.isDirectory()) {
			File[] children = folder.listFiles();
			CScannedTest scannedTest = null;
			if (children != null) {
				for (int i = 0; i < children.length; i++) {
					File child = children[i];
					if (child.isDirectory()) {
						if (Pattern.matches(regexFolder, child.getName())) {
							recursivelyScanFolderForTests(output, child, regexFolder, regexFile, removeExtension, separator, 
												  currentRelativePath.trim().length() > 0 ? currentRelativePath + separator + child.getName() : child.getName());
						} else {
							System.err.println("skipping folder \"" + currentRelativePath + separator + child.getName() + "\"...");
						}
						
					} else { // file
						if (Pattern.matches(regexFile, child.getName())) {
							String testcaseName = child.getName(); // the file is a testcase
							String testName = folder.getName(); // the parent folder is the test
							
							if (scannedTest == null) {
								scannedTest = new CScannedTest(testName, CFileUtils.getBasename(new File(currentRelativePath))); // keep only the path above the parent (the test)
								System.out.println("> found test \"" + currentRelativePath + separator + testcaseName + "\"");
							}
							
							scannedTest.addScannedTestcase(0, testcaseName);
						} else {
							//System.out.println("> skipping file \"" + currentRelativePath + separator + child.getName() + "\"");
						}
					}
				}
			}
			output.add(scannedTest);
		} else { // should never happen!
		}
	}
	
	
	
	
	
	
	private CReturnStatus parseResultFile(File resultFile, Vector<CExecutionStep> executionSteps) {
		String line, timeline, message;
		String existingPattern = "dd.MM.yyyy HH:mm:ss";
		String expectedPattern = "yyyy-MM-dd HH:mm:ss";
		SimpleDateFormat formatter;
		Date myDate;
		boolean errorDetected = false;

		try {
			FileInputStream fileInputStream = new FileInputStream(resultFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));
			// BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream, "UTF-8"));

		while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				System.out.println(">" + line);

				// Current format [log (2.4.2015 22:57:59)] Any
				// Requested format: 2010-01-23 19:00:09 [Success] Any
				// message...
				if (line.indexOf("[debug]") >= 0) {
					message = line.substring(line.indexOf("[debug]") + 8, line.length());
					executionSteps.add(new CExecutionStep(RESULT_INFO, message));

				} else if (line.indexOf("[debug (") >= 0) {
					timeline = line.substring(line.indexOf(" ("), line.indexOf(")]") + 1);
					line = line.replace(timeline, "");
					timeline = timeline.trim().replace("(", "").replace(")", "");
					message = line.substring(line.indexOf("[debug]") + 8, line.length());
					formatter = new SimpleDateFormat(existingPattern);
					myDate = formatter.parse(timeline);
					formatter = new SimpleDateFormat(expectedPattern);
					timeline = formatter.format(myDate);
					executionSteps.add(new CExecutionStep(timeline, RESULT_SUCCESS, message));

				} else if (line.indexOf("[log]") >= 0) {
					message = line.substring(line.indexOf("[log]") + 6, line.length());
					executionSteps.add(new CExecutionStep(RESULT_SUCCESS, message));

				} else if (line.indexOf("[log (") >= 0) {
					timeline = line.substring(line.indexOf(" ("), line.indexOf(")]") + 1);
					line = line.replace(timeline, "");
					timeline = timeline.trim().replace("(", "").replace(")", "");
					message = line.substring(line.indexOf("[log]") + 6, line.length());
					formatter = new SimpleDateFormat(existingPattern);
					myDate = formatter.parse(timeline);
					formatter = new SimpleDateFormat(expectedPattern);
					timeline = formatter.format(myDate);
					executionSteps.add(new CExecutionStep(timeline, RESULT_SUCCESS, message));

				} else if (line.indexOf("[error]") >= 0) {
					message = line.substring(line.indexOf("[error]") + 8, line.length());
					executionSteps.add(new CExecutionStep(RESULT_FAILURE, message));
					errorDetected = true;

				} else if (line.indexOf("[error (") >= 0) {
					timeline = line.substring(line.indexOf(" ("), line.indexOf(")]") + 1);
					line = line.replace(timeline, "");
					timeline = timeline.trim().replace("(", "").replace(")", "");
					message = line.substring(line.indexOf("[error]") + 8, line.length());
					formatter = new SimpleDateFormat(existingPattern);
					myDate = formatter.parse(timeline);
					formatter = new SimpleDateFormat(expectedPattern);
					timeline = formatter.format(myDate);
					executionSteps.add(new CExecutionStep(timeline, RESULT_FAILURE, message));
					errorDetected = true;

				} else {
					// traceln(LOG_PRIORITY_SEVERE, "unknown tag!");
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception whle parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the sikuli trace file: " + e));
			errorDetected = true;
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		//File inputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/sikulix/sikuli_traces.txt");        // ok
		//File inputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/sikulix/sikuli_error.txt");         // ok
		//File inputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/sikulix/sikuli_sucess.txt");        // ok
		File inputFile = new File("Y:/XStudio/src/com/xqual/xlauncher/sikulix/sikuli_partialSucess.txt");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl myLauncher = new CLauncherImpl();
		
		CReturnStatus returnStatus = myLauncher.parseResultFile(inputFile, executionSteps);
		System.err.println("- - - - STATUS   = " + CUtils.resultToString(returnStatus.getResult()));
		System.err.println("- - - - MESSAGES = " + returnStatus.getExecutionSteps());
	}
}
