/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.0                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.soatest;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.CTestExecution;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;

/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for SOATest.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{soatest       }  ";

	// parameters impacting executing at run time set by the test operator
	private String soaTestInstallPath;
	private String soaTestWorkspace;
	private String soaTestConfiguration;
	private String soaTestOptions;

	private File workingDir;

	private File consoleFile;
	private File guiConsoleFile;
	private File xmlReportFile;
	private File copyXmlReportFile;
	private File batFile;
	
	private boolean firstTestcase = true;
	private String thisTestName;
	
	private static final String SOATEST_INTERPRETER_EXE = CUtils.getExecutableName("soatestcli");

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) {
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");

		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need from the configuration
			soaTestInstallPath       = getStringParamValue("SOATest", "SOATest install path");   // i.e. "C:/Program Files/Parasoft/SOATest/9.8"
			soaTestWorkspace         = getStringParamValue("SOATest", "SOATest workspace path"); // i.e. "C:/Users/Bob/parasoft/workspace"
			soaTestConfiguration     = getStringParamValue("SOATest", "SOATest configuration");  // i.e. "user://example configuration"
			soaTestOptions           = getStringParamValue("SOATest", "SOATest options");        // i.e. "-publish -router searchURI:host1.adobe.com"

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		batFile = CFileUtils.createTmpFile("soaRunTest.bat");
		traceln(LOG_PRIORITY_INFO, "BAT FILE = " + batFile.getAbsolutePath());

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		thisTestName = testName + ".tst";
		traceln(LOG_PRIORITY_INFO, "preRun default testName=" + thisTestName);
		if (additionalInfo != null && additionalInfo.trim().length()>0) { // test's additional infos can include the .tst file name
			thisTestName = additionalInfo.trim();
			traceln(LOG_PRIORITY_INFO, "preRun final testName=" + thisTestName);
		}

		workingDir = new File(soaTestInstallPath);

		firstTestcase = true;
		
		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		
		consoleFile = CFileUtils.createTmpRandomFile("soaconsole_", "txt");
		traceln(LOG_PRIORITY_INFO, "CONSOLE FILE = " + consoleFile.getAbsolutePath());
		
		guiConsoleFile = CFileUtils.createTmpRandomFile("soaguiconsole_", "txt");
		traceln(LOG_PRIORITY_INFO, "GUI CONSOLE FILE = " + guiConsoleFile.getAbsolutePath());

		xmlReportFile = CFileUtils.createTmpRandomFile("soareport_", "xml");
		traceln(LOG_PRIORITY_INFO, "XML REPORT FILE = " + xmlReportFile.getAbsolutePath());

		// Retrieve all the testcase names that are executable.
		// This is to ensure that only the testcases referenced in XStudio are executed within the test suite
		// The test list must be of the form:
		// –testName test1 –testName test2 –testName test3 etc.
		// so 1 option per test
		traceln(LOG_PRIORITY_INFO, "ids of all the not executable testcases: " + notExecutableTestcaseIdVector);
		StringBuffer testListOption = new StringBuffer();
		for (int indexTest=0; indexTest<testExecutionVector.size(); indexTest++) {
			CTestExecution testExecution = testExecutionVector.get(indexTest);
			int thisTestId = testExecution.getId();
			if (thisTestId == testId) {
				traceln(LOG_PRIORITY_INFO, "checking executing testcases for test " + testId + "...");
				for (int indexTestcase=0; indexTestcase<testExecution.getTestcaseExecutionVector().size(); indexTestcase++) {
					int testcaseId = testExecution.getTestcaseExecutionVector().get(indexTestcase).getId();
					trace(LOG_PRIORITY_INFO, "> testcase " + testcaseId + ": ");
					if (!notExecutableTestcaseIdVector.contains(testcaseId)) {
						traceln(LOG_PRIORITY_INFO, "executable");
						String testcaseName = testExecution.getTestcaseExecutionVector().get(indexTestcase).getName();
						testListOption.append("-testName \"" + testcaseName + "\" ");
					} else {
						traceln(LOG_PRIORITY_INFO, "NOT executable");
					}
				}
			}
		}
		
		// +------------------------------------+
		// | Run the soaTest test
		// +------------------------------------+
		
		String commandLine = CFileUtils.quoteFilePath(soaTestInstallPath + "/" + SOATEST_INTERPRETER_EXE) + " " + // for an unknown reason we need to provide the absolute path
                "-data " + CFileUtils.quoteFilePath(soaTestWorkspace) + " " +
                "-resource " + CFileUtils.quoteFilePath((testPath.length()>0 ? testPath + "/" : "") + thisTestName) + " " + // this is actually a testsuite
                "-config " + CFileUtils.quoteFilePath(soaTestConfiguration) + " " +
                testListOption + // this is the list of the tests
                soaTestOptions + " " +
                "-report " + CFileUtils.quoteFilePath(xmlReportFile.getAbsolutePath()) + " " +
                "-appconsole " + CFileUtils.quoteFilePath(guiConsoleFile.getAbsolutePath());
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "command line: " + commandLine));
		traceln(LOG_PRIORITY_INFO, "command line = " + commandLine);
		
		// generate a bat script to run the command line (not sure about why this is needed but when executin directly soatestcli.exe we encounter issue where soatestcli.exe returns an error code 5 (Access Denied))
		// executing a bat script containing the exact same call seems to make the deal
		CFileUtils.deleteFile(batFile);
		try {
			batFile.createNewFile();
		    Writer output = new BufferedWriter(new FileWriter(batFile));
		    output.write(commandLine);
		    output.close();
		} catch (IOException e) {
			traceln(LOG_PRIORITY_SEVERE, "Could not create file \"" + batFile + "\"");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Could not create bat file \"" + batFile + "\""));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		traceln(LOG_PRIORITY_INFO, batFile + " has been generated");

		// +------------------------------------+
		// | Run the soaTest test
		// +------------------------------------+
		CRunner soaTestRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,
											batFile.getAbsolutePath(), // xstudioRunTest.bat
											workingDir);

		short result = soaTestRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, consoleFile);
		traceln(LOG_PRIORITY_INFO, "result=" + result);

		// return code of soaTestcli runner: if the test runs successfully, exit with a status of 0. Otherwise exit with a status > 0
		// https://docs.parasoft.com/display/SOA9103/CLI+Exit+Codes
		
		/*
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Testsuite execution failed!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Testsuite execution succeeded"));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
		*/
		
		// for now let's ignore the return code from the bat script
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "bat file executed"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		if (firstTestcase) {
			// Add the attachments to the first testcase
			this.addTestcaseMessageAttachment(consoleFile);
			this.addTestcaseMessageAttachment(guiConsoleFile);
			
			// copy the xml file before attaching it or it will be deleted
	        copyXmlReportFile = new File(System.getProperty("java.io.tmpdir") + "/copy_" + xmlReportFile.getName()); 
	        CFileUtils.deleteAndTouchFile(copyXmlReportFile);
	        try {
	            CFileUtils.copyFiles(xmlReportFile, copyXmlReportFile, true);
	        } catch (Exception e) {
	            traceln(LOG_PRIORITY_INFO, "output file copy exception: " + e);
	        }
			addTestcaseMessageAttachment(xmlReportFile);

			// find the automatically generated html report
			String xmlReportFilePath = xmlReportFile.getAbsolutePath();
			String htmlReportFilePath = xmlReportFilePath.substring(0, xmlReportFilePath.length()-4) + ".html";
			this.addTestcaseMessageAttachment(new File(htmlReportFilePath));
			
			firstTestcase = false;
		}

		return parseResultFile(copyXmlReportFile, testcaseName, executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+

	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}

	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(soaTestWorkspace)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2etst$",          // only *.tst files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output; // the .tst file is a binary file so we can't parse it to search for testcases
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File xmlFile, /*String testName,*/ String testcaseName, Vector<CExecutionStep> executionSteps) {
	    System.out.println("-----------------------------------------------------------------");
		if (xmlFile == null || !xmlFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: result file found"));
		}
		
		//System.err.println("- - - result file: " + CFileUtils.FileToString(xmlFile));

		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(xmlFile);
	    Node rootXmlNode = xmlDocument.getFirstChild();
	    if (rootXmlNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Invalid result file!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: invalid result file!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    } else {
	    	//traceln(LOG_PRIORITY_SEVERE, "rootXmlNode ok");
	    }

		// +------------------------+
	    // | TEST SUITE             |
		// +------------------------+
	    System.out.println("========= TESTSUITE: " + testcaseName + " =======");
	    
		int nbRowTotal = 0;
		int nbRowPass  = 0;
		int nbRowFail  = 0;
		String xPathTestcase = "//TestSuite[@name=\"Scenario: "+ testcaseName + "\"]"; // we're using " instead of ' as we may have testcases containing ' in their names. Did not find a good solution to handle tests with ' AND " in their names
		System.out.println("- - - - xPathTestcase : " + xPathTestcase);
		// if "Scenario: XXXX" not found, try "Test Suite: XXXX"
		Node nodeExists = CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, xPathTestcase);
		if (nodeExists == null) {
			System.out.println("- - - - 'Scenario:' NOT found! switching to ''Test Suite:'...");
			xPathTestcase = "//TestSuite[@name=\"Test Suite: "+ testcaseName + "\"]"; // we're using " instead of ' as we may have testcases containing ' in their names. Did not find a good solution to handle tests with ' AND " in their names
		}

		
		
		try {
			nbRowPass = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, xPathTestcase + "/@pass"); // in all version of SOATest
			System.out.println("nbRowPass = " + nbRowPass);
		} catch (Exception e) {
			//traceln(LOG_PRIORITY_SEVERE, "couldn't find testsuite 'pass' attribute !");
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		try {
			nbRowFail = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, xPathTestcase + "/@fail");
			System.out.println("nbRowFails = " + nbRowFail);
		} catch (Exception e) {
			//traceln(LOG_PRIORITY_SEVERE, "couldn't find testsuite 'fail' attribute !");

			// in this case we look for the total
			try {
				nbRowTotal = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, xPathTestcase + "/@total");
				System.out.println("nbRowTotal = " + nbRowTotal);
				
				nbRowFail = nbRowTotal - nbRowPass;
				System.out.println("nbRowFails = " + nbRowFail);
			} catch (Exception e2) {
				traceln(LOG_PRIORITY_SEVERE, "couldn't find 'total' attribute !");
				traceln(LOG_PRIORITY_SEVERE, "result file parsing failed !");
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}
		}
		
	    short testcaseResult = RESULT_FAILURE;
	    if (nbRowFail == 0 && nbRowPass > 0) {
	    	testcaseResult = RESULT_SUCCESS;
	    } else if (nbRowFail == 0 && nbRowPass == 0) {
	    	testcaseResult = RESULT_NOT_EXECUTED;
	    }
	    
	    String testcaseResultMessage = "Statistics:";
	    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, testcaseResultMessage)); // to not generate a wrong number of success, failure etc. (all tests are reported later)
	    
	    testcaseResultMessage = "[b]" + nbRowPass + "[/b] row success";
	    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, testcaseResultMessage));
	    
	    testcaseResultMessage = "[b]" + nbRowFail + "[/b] row failed";
	    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, testcaseResultMessage));

	    // ********* RECURSIVE PARSING ***********
	    _recursiveParseResultFile(CXmlDocumentFactory.getNodeFromXPath(rootXmlNode, xPathTestcase), executionSteps);

		return new CReturnStatus(testcaseResult, executionSteps);
	}


	
	private void _recursiveParseResultFile(Node parentNode, Vector<CExecutionStep> executionSteps) {
		if (parentNode != null) {
			NodeList childrenNodes = parentNode.getChildNodes();
			for (int i=0; i<childrenNodes.getLength(); i++) {
				Node childrenNode = childrenNodes.item(i);
				
				String childrenNodeName = childrenNode.getNodeName();
				
				if (childrenNodeName.equalsIgnoreCase("testsuite")) {
					
			    	String innerTestSuiteName = childrenNode.getAttributes().getNamedItem("name").getNodeValue();
			    	//System.err.println("   ========= INNER TESTSUITE : " + innerTestSuiteName + " =======");
	
			    	try {
			    		int innerTestSuiteTotal = 0;
			    		int innerTestSuitePass  = 0;
			    		int innerTestSuiteFail  = 0;
			    		
						try {
							innerTestSuitePass = Integer.parseInt(childrenNode.getAttributes().getNamedItem("pass").getNodeValue()); // in all version of SOATest
							//System.out.println("   innerTestSuitePass = " + innerTestSuitePass);
						} catch (Exception e) {
							//traceln(LOG_PRIORITY_SEVERE, "couldn't find inner testsuite 'pass' attribute !");
							//return new CReturnStatus(RESULT_FAILURE, executionSteps);
						}
	
						try {
							innerTestSuiteFail = Integer.parseInt(childrenNode.getAttributes().getNamedItem("fail").getNodeValue());
							//System.out.println("   innerTestSuiteFails = " + innerTestSuiteFail);
						} catch (Exception e) {
							//traceln(LOG_PRIORITY_SEVERE, "couldn't find inner testsuite 'fail' attribute !");
	
							// in this case we look for the total
							try {
								innerTestSuiteTotal = Integer.parseInt(childrenNode.getAttributes().getNamedItem("total").getNodeValue());
								//System.out.println("   innerTestSuiteTotal = " + innerTestSuiteTotal);
								
								innerTestSuiteFail = innerTestSuiteTotal - innerTestSuitePass;
								//System.out.println("   innerTestSuiteFails = " + innerTestSuiteFail);
							} catch (Exception e2) {
								traceln(LOG_PRIORITY_SEVERE, "couldn't find 'total' attribute !");
								traceln(LOG_PRIORITY_SEVERE, "result file parsing failed !");
								//return new CReturnStatus(RESULT_FAILURE, executionSteps);
							}
						}
						
						//System.out.println("   >> result: " + innerTestSuiteFail + "," + innerTestSuitePass);
						
						short innerTestSuiteResult = RESULT_FAILURE;
						if (innerTestSuiteFail == 0 && innerTestSuitePass > 0) {
							innerTestSuiteResult = RESULT_SUCCESS;
						} else if (innerTestSuiteFail == 0 && innerTestSuitePass == 0) {
							innerTestSuiteResult = RESULT_NOT_EXECUTED;
						}
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), innerTestSuiteResult, "Inner Test Suite: [b]" + innerTestSuiteName + "[/b]"));
						
			    	} catch (Exception e) {
			    		
			    	}
	
					// recursive call
					_recursiveParseResultFile(childrenNode, executionSteps);
	
				} else if (childrenNodeName.equalsIgnoreCase("test")) {
					
			    	String innerTestName = childrenNode.getAttributes().getNamedItem("name").getNodeValue();
			    	//System.err.println("      ========= " + innerTestName + " =======");
	
			    	try {
			    		int innerTestTotal = 0;
			    		int innerTestPass  = 0;
			    		int innerTestFail  = 0;
						
			    		try {
			    			innerTestPass = Integer.parseInt(childrenNode.getAttributes().getNamedItem("pass").getNodeValue()); // in all version of SOATest
			    			//System.out.println("innerTestPass = " + innerTestPass);
			    		} catch (Exception e) {
			    			//traceln(LOG_PRIORITY_SEVERE, "couldn't find test 'pass' attribute !");
			    			//return new CReturnStatus(RESULT_FAILURE, executionSteps);
			    		}
	
			    		try {
			    			innerTestFail = Integer.parseInt(childrenNode.getAttributes().getNamedItem("fail").getNodeValue());
			    			//System.out.println("innerTestFails = " + innerTestFail);
			    		} catch (Exception e) {
			    			//traceln(LOG_PRIORITY_SEVERE, "couldn't find test 'fail' attribute !");
	
			    			// in this case we look for the total
			    			try {
			    				innerTestTotal = Integer.parseInt(childrenNode.getAttributes().getNamedItem("total").getNodeValue());
			    				//System.out.println("innerTestTotal = " + innerTestTotal);
			    				
			    				innerTestFail = innerTestTotal - innerTestPass;
			    				//System.out.println("innerTestFails = " + innerTestFail);
			    			} catch (Exception e2) {
			    				//traceln(LOG_PRIORITY_SEVERE, "couldn't find 'total' attribute !");
			    				//return new CReturnStatus(RESULT_FAILURE, executionSteps);
			    			}
			    		}
						
						//System.out.println("      >> TEST result: " + innerTestFail + "," + innerTestPass);
						
						short innerTestResult = RESULT_FAILURE;
						if (innerTestFail == 0 && innerTestPass > 0) {
							innerTestResult = RESULT_SUCCESS;
						} else if (innerTestFail == 0 && innerTestPass == 0) {
							innerTestResult = RESULT_NOT_EXECUTED;
						}
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), innerTestResult, "- Inner Test: [b]" + innerTestName + "[/b]"));
						
			    	} catch (Exception e) {
			    		
			    	}
			    	
					// recursive call
					_recursiveParseResultFile(childrenNode, executionSteps);
						
				} else if (childrenNodeName.equalsIgnoreCase("testcase")) {
					
			    	String innerTestCaseName = "";
			    	try {
						innerTestCaseName = childrenNode.getAttributes().getNamedItem("name").getNodeValue();
					} catch (Exception e1) {
						//traceln(LOG_PRIORITY_SEVERE, "couldn't find testcase 'name' attribute !");
						try {
							innerTestCaseName = childrenNode.getAttributes().getNamedItem("params").getNodeValue();
						} catch (Exception e2) {
							//traceln(LOG_PRIORITY_SEVERE, "couldn't find testcase 'params' attribute !");
						}
					}
			    	//System.err.println("            ========= " + innerTestCaseName + " =======");
	
			    	try {
			    		int innerTestCaseTotal = 0;
			    		int innerTestCasePass  = 0;
			    		int innerTestCaseFail  = 0;
	
			    		try {
			    			innerTestCasePass = Integer.parseInt(childrenNode.getAttributes().getNamedItem("pass").getNodeValue()); // in all version of SOATest
			    			//System.out.println("innerTestCasePass = " + innerTestCasePass);
			    		} catch (Exception e) {
			    			//traceln(LOG_PRIORITY_SEVERE, "couldn't find testcase 'pass' attribute !");
			    			//return new CReturnStatus(RESULT_FAILURE, executionSteps);
			    		}
	
			    		try {
			    			innerTestCaseFail = Integer.parseInt(childrenNode.getAttributes().getNamedItem("fail").getNodeValue());
			    			//System.out.println("innerTestCaseFails = " + innerTestCaseFail);
			    		} catch (Exception e) {
			    			//traceln(LOG_PRIORITY_SEVERE, "couldn't find testcase 'fail' attribute !");
	
			    			// in this case we look for the total
			    			try {
			    				innerTestCaseTotal = Integer.parseInt(childrenNode.getAttributes().getNamedItem("total").getNodeValue());
			    				//System.out.println("               innerTestCaseTotal = " + innerTestCaseTotal);
			    				
			    				innerTestCaseFail = innerTestCaseTotal - innerTestCasePass;
			    				//System.out.println("               innerTestCaseFails = " + innerTestCaseFail);
			    			} catch (Exception e2) {
			    				traceln(LOG_PRIORITY_SEVERE, "couldn't find 'total' attribute !");
			    				//return new CReturnStatus(RESULT_FAILURE, executionSteps);
			    			}
			    		}
			    		
						//System.out.println("            >> result: " + innerTestCaseFail + "," + innerTestCasePass);
						
						short innerTestCaseResult = RESULT_FAILURE;
						if (innerTestCaseFail == 0 && innerTestCasePass > 0) {
							innerTestCaseResult = RESULT_SUCCESS;
						} else if (innerTestCaseFail == 0 && innerTestCasePass == 0) {
							innerTestCaseResult = RESULT_NOT_EXECUTED;
						}
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), innerTestCaseResult, "---> Inner TestCase: [b]" + innerTestCaseName + "[/b]"));
					} catch (Exception e) {
	
					}
					
				} // if (childrenNodeName.equalsIgnoreCase("testsuite | test | testcase"))
				
			} // for (int i=0; i<childrenNodes.getLength(); i++)
		} // if (parentNode != null)
	}
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl toto = new CLauncherImpl();

		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/soatest/sample/Report.xml"),
							//"Inbound Channels",                    // test     <=> TestSuite
							"S 3390 - Incremental Auth RRN Testing", // testcase <=> Scenario
							executionSteps);

		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/soatest/sample/Report2.xml"),
							//"Example",              // test     <=> TestSuite
							"ISO Basic Test Pattern", // testcase <=> Scenario
							executionSteps);

		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/soatest/sample/soareport_160459278.xml"),
							//"Example",         // test     <=> TestSuite
							"Internal Scenario", // testcase <=> Scenario
							executionSteps);

		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/soatest/sample/soareport_61148983.xml"),
							//"Example",         // test     <=> TestSuite
							"Tfr 4684 - Batch Processor - Add LocationData to the 'TransactionLocationData' field", // testcase <=> Scenario
							//"Internal Scenario", // testcase <=> Scenario
							executionSteps);
		
		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/soatest/sample/soareport_469937533.xml"),
							//"Example",         // test     <=> TestSuite
							"Domestic Offline Authorization with DCC Settlement", // testcase <=> Scenario
							executionSteps);

		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/soatest/sample/soareport_1276274219.xml"),
							//"Example",         // test     <=> TestSuite
							"Tb 5558 - [AEON] - No Auth Code Return for offline transactions", // testcase <=> Scenario
							//"Internal Scenario", // testcase <=> Scenario
							executionSteps);

		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/soatest/sample/soareport_1040667385.xml"),
							//"Example",         // test     <=> TestSuite
							"Tfr 5198 - New field for PSW & MAS for Opera and 3C Gateway", // testcase <=> Scenario
							//"Internal Scenario", // testcase <=> Scenario
							executionSteps);

		/*
		for (int i=0; i<executionSteps.size(); i++) {
			System.err.println(CUtils.resultToUserString(executionSteps.get(i).getResult()) + "  =  " + executionSteps.get(i).getMessage());
		}
		*/
	}
}
