/*
+----------------------------------------------------------------------+
|                    Class: CLauncherImpl                              |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.2                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
 */
package com.xqual.xlauncher.sql_compare;

import java.io.File;
import java.io.FileWriter;
import java.net.URL;
import java.net.URLClassLoader;
import java.sql.Connection;
import java.sql.Driver;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.Calendar;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CDriverWrapper;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for SQL Compare.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{sql_compare   }  ";

	// parameters impacting executing at run time set by the test operator
	private String sqlScriptRootPath;

	private String connectionUrl1;
	private String username1;
	private String password1;
	private String driverJar1;
	private String driverClassPath1;
	private boolean writeAllValues1;

	private String connectionUrl2;
	private String username2;
	private String password2;
	private String driverJar2;
	private String driverClassPath2;
	private boolean writeAllValues2;

	private boolean runCommandRemotelyBeforeRunningTest;
	private String plinkExe;
	private String plinkRemoteHost;
	private String plinkUsername;
	private String plinkPassword;
	private String command;

	private Connection connection1 = null;
	private Connection connection2 = null;

	File resultLogFile1 = null;
	File resultLogFile2 = null;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			sqlScriptRootPath                   = getStringParamValue("SQL", "sql script root path");        // i.e. Y:/XStudio/src

			connectionUrl1                      = getStringParamValue("Connection1", "connection url");      // i.e. jdbc:mysql://localhost/xstudio_db
			username1                           = getStringParamValue("Connection1", "username");            // i.e. xstudio
			password1                           = getStringParamValue("Connection1", "password");            // i.e. xstudiopwd
			driverJar1                          = getStringParamValue("Connection1", "driver jar");          // i.e. Y:/mysql-connector-java-3.1.12/mysql-connector-java-3.1.12-bin.jar
			driverClassPath1                    = getStringParamValue("Connection1", "driver classpath");    // i.e. com.mysql.jdbc.Driver
			writeAllValues1                     = getBooleanParamValue("Connection1", "write all values");

			connectionUrl2                      = getStringParamValue("Connection2", "connection url");      // i.e. jdbc:mysql://localhost/xstudio_db
			username2                           = getStringParamValue("Connection2", "username");            // i.e. xstudio
			password2                           = getStringParamValue("Connection2", "password");            // i.e. xstudiopwd
			driverJar2                          = getStringParamValue("Connection2", "driver jar");          // i.e. Y:/mysql-connector-java-3.1.12/mysql-connector-java-3.1.12-bin.jar
			driverClassPath2                    = getStringParamValue("Connection2", "driver classpath");    // i.e. com.mysql.jdbc.Driver
			writeAllValues2                     = getBooleanParamValue("Connection2", "write all values");

			runCommandRemotelyBeforeRunningTest = getBooleanParamValue("Scripts", "run command remotely before running test");
			plinkExe                            = getStringParamValue("Scripts", "plink executable");        // i.e. C:/progra~1/Putty/plink.exe -batch
			plinkRemoteHost                     = getStringParamValue("Scripts", "remote host");
			plinkUsername                       = getStringParamValue("Scripts", "username");
			plinkPassword                       = getStringParamValue("Scripts", "password");
			command                             = getStringParamValue("Scripts", "command");                 // i.e. grep /~eric/ /var/log/httpd/access.log > fredlog
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		try {
			File driverFile1 = new File(driverJar1);
			URL driverUrl1 = driverFile1.toURI().toURL();
			URLClassLoader jarLoader1 = URLClassLoader.newInstance(new URL[] { driverUrl1 }, CLauncherImpl.class.getClassLoader());
			Class driverClass1 = Class.forName(driverClassPath1, true, jarLoader1);
			DriverManager.registerDriver(new CDriverWrapper((Driver) driverClass1.newInstance()));
			traceln(LOG_PRIORITY_INFO, "connecting to " + connectionUrl1 + " with " + username1 + ":" + password1 + "...");
			connection1 = DriverManager.getConnection(connectionUrl1, username1, password1);
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, e.getStackTrace().toString());
			traceln(LOG_PRIORITY_SEVERE, "connection to sql database 1 failed: " + e.getMessage());
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't connect to sql database 1"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		try {
			File driverFile2 = new File(driverJar2);
			URL driverUrl2 = driverFile2.toURI().toURL();
			URLClassLoader jarLoader2 = URLClassLoader.newInstance(new URL[] { driverUrl2 }, CLauncherImpl.class.getClassLoader());
			Class driverClass2 = Class.forName(driverClassPath2, true, jarLoader2);
			DriverManager.registerDriver(new CDriverWrapper((Driver) driverClass2.newInstance()));
			traceln(LOG_PRIORITY_INFO, "connecting to " + connectionUrl2 + " with " + username2 + ":" + password2 + "...");
			connection2 = DriverManager.getConnection(connectionUrl2, username2, password2);
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "connection to sql database 2 failed: " + e.getMessage());
			traceln(LOG_PRIORITY_SEVERE, e.getStackTrace().toString());
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't connect to sql database 2"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "successfully connected to both sql databases"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		if (runCommandRemotelyBeforeRunningTest) {
			traceln(LOG_PRIORITY_INFO, "running the command remotely...");
			CRunner pLinkRunner = new CRunner("[" + testId + "] " + testPath + ":" + testName, plinkExe + " " + plinkRemoteHost + " -l " + plinkUsername + " -pw " + plinkPassword + " " + "\"" + command + "\"");
			Calendar startTime = CCalendarUtils.getTimezonedInstance();
			short resultCommand = pLinkRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
			Calendar stopTime = CCalendarUtils.getTimezonedInstance();
			long duration = stopTime.getTimeInMillis() - startTime.getTimeInMillis();
			if (resultCommand == RESULT_FAILURE) {
				executionSteps.add(new CExecutionStep(resultCommand, "plink failed in " + CCalendarUtils.millisTimeToDurationString(duration)));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			} else {
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "plink completed in " + CCalendarUtils.millisTimeToDurationString(duration)));
			}
		}

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		File logFile = new File("sqlCompare_log" + testcaseIndex + ".txt");
		CFileUtils.deleteAndTouchFile(logFile);

		if (writeAllValues1) {
			resultLogFile1 = new File("Results1_log" + testcaseIndex + ".txt");
			CFileUtils.deleteAndTouchFile(resultLogFile1);
		}
		if (writeAllValues2) {
			resultLogFile2 = new File("Results2_log" + testcaseIndex + ".txt");
			CFileUtils.deleteAndTouchFile(resultLogFile2);
		}

		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + ":" + testName + " testcaseIndex=" + testcaseIndex + "...");
		writeLog("\n\n\n" + "Executing test: " + testName + " testcaseIndex=" + testcaseIndex, logFile);
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// +-----------------+
		// |     script 1    |
		// +-----------------+
		String scriptPath1 = sqlScriptRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + "_" + testcaseName + "_a.sql";
		File script1 = new File(scriptPath1);
		if (!script1.exists()) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, scriptPath1 + " script 1 not found"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		String scriptContent1 = CFileUtils.FileToString(script1).trim();

		// replace the placeholders with their values in the query
		for (int k=0; k<params.size(); k++) {
			CTestcaseParam param = params.get(k);
			String paramName = param.getName();
			String paramValue = param.getStringValue();
			traceln(LOG_PRIORITY_INFO, "--> replacing parameter " + paramName + " with " + paramValue + "...");
			scriptContent1 = scriptContent1.replaceAll(paramName/*.replaceAll(".", "\\x2e")*/, paramValue);
		}

		traceln(LOG_PRIORITY_INFO, "Query 1: " + scriptContent1);
		writeLog("Query 1: " + scriptContent1, logFile);
		if (writeAllValues1) writeLog("Query 1: " + scriptContent1, resultLogFile1);

		// +-----------------+
		// |     script 2    |
		// +-----------------+
		String scriptPath2 = sqlScriptRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + "_" + testcaseName + "_b.sql";
		File script2 = new File(scriptPath2);
		if (!script2.exists()) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, scriptPath2 + " script 2 not found"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		String scriptContent2 = CFileUtils.FileToString(script2).trim();

		// replace the placeholders with their values in the query
		for (int k=0; k<params.size(); k++) {
			CTestcaseParam param = params.get(k);
			String paramName = param.getName();
			String paramValue = param.getStringValue();
			traceln(LOG_PRIORITY_INFO, "--> replacing parameter " + paramName + " with " + paramValue + "...");
			scriptContent2 = scriptContent2.replaceAll(paramName/*.replaceAll(".", "\\x2e")*/, paramValue);
		}

		traceln(LOG_PRIORITY_INFO, "Query 2: " + scriptContent2);
		writeLog("Query 2 :" + scriptContent2, logFile);
		if (writeAllValues2) writeLog("Query 2: " + scriptContent2, resultLogFile2);

		Vector<String> itemVector = new Vector<String>();
		int count = 1;
		boolean error = false;
		try {

			Statement statement1 = connection1.createStatement();
			Statement statement2 = connection2.createStatement();
			ResultSet resultSet1 = statement1.executeQuery(scriptContent1);
			ResultSet resultSet2 = statement2.executeQuery(scriptContent2);

			ResultSetMetaData rsmd1 = resultSet1.getMetaData();
			int numberOfColumns1 = rsmd1.getColumnCount();
			traceln(LOG_PRIORITY_INFO, "--> query 1 returns " + numberOfColumns1 + " columns...");

			ResultSetMetaData rsmd2 = resultSet2.getMetaData();
			int numberOfColumns2 = rsmd2.getColumnCount();
			traceln(LOG_PRIORITY_INFO, "--> query 2 returns " + numberOfColumns2 + " columns...");

			boolean sameNumberOfColumn = (numberOfColumns1 == numberOfColumns2);

			if (!sameNumberOfColumn) {
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "1st query Results have " + numberOfColumns1 + " column(s) and " +
						                                                                               "2nd query Results have " + numberOfColumns2 + " column(s) => cannot compare"));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}

			// at this point, both queries have the same number of columns but not necessarily the same number of rows
			while (resultSet1.next()) {
				resultSet2.next();
				traceln(LOG_PRIORITY_INFO, "[Record at line: " + count + "]");
				writeLog("\nRecord at line: " + count + "", logFile);
				writeResults("\nRecord at line: " + count + "");

				for (int i = 1; i <= numberOfColumns1; i++) {
					String columnsName1 = rsmd1.getColumnName(i);
					String columnsName2 = rsmd2.getColumnName(i);

					String field1 = resultSet1.getString(i);
					String field2 = resultSet2.getString(i);

					if (field1 == null) field1 = "null";
					if (field2 == null) field2 = "null";

					traceln(LOG_PRIORITY_INFO, "> " + columnsName1 + " (query 1)=" + field1);
					writeLog(columnsName1 + " (query 1)=" + field1, logFile);
					if (writeAllValues1) writeLog(columnsName1 + "(query 1)=" + field1, resultLogFile1);

					traceln(LOG_PRIORITY_INFO, "> " + columnsName2 + " (query 2)=" + field2);
					writeLog(columnsName2 + " (query 2)=" + field2, logFile);
					if (writeAllValues2) writeLog(columnsName2 + " (query 2)=" + field2, resultLogFile2);

					traceln(LOG_PRIORITY_INFO, "\n");
					writeLog("", logFile);
					writeResults("");

					if (field1.equals(field2)) {
						itemVector.add(field1);
					} else {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "results differ at line #" + count + "and column #" + i + " where " + field1 + " != " + field2));
						writeLog("ERROR: results differ at line #" + count + "and column #" + i + " where " + field1 + " != " + field2, logFile);
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "first correct results: " + itemVector));
						writeLog("first correct results: " + itemVector, logFile);
						error = true;
						if (!writeAllValues1 && !writeAllValues2) {
							addTestcaseMessageAttachment(logFile);
							return new CReturnStatus(RESULT_FAILURE, executionSteps);
						}
					}
				}
				count++;
			}
			
			while (resultSet2.next()) { // if there are more rows on query 2
				traceln(LOG_PRIORITY_INFO, "[Record at line: " + count + "] (only in query 2!)");
				writeLog("\nRecord at line: " + count + " (only in query 2!)", logFile);

				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Query 2 returned more rows than query 1"));
				error = true;
				if (!writeAllValues2) {
					addTestcaseMessageAttachment(logFile);
					return new CReturnStatus(RESULT_FAILURE, executionSteps);
				} else {
					for (int i = 1; i <= numberOfColumns1; i++) {
						String columnsName2 = rsmd2.getColumnName(i);
						String field2 = resultSet2.getString(i);
						if (field2 == null) field2 = "null";

						traceln(LOG_PRIORITY_INFO, "> " + columnsName2 + " (query 2)=" + field2);
						writeLog(columnsName2 + " (query 2)=" + field2, logFile);
						if (writeAllValues2) writeLog(columnsName2 + " (query 2)=" + field2, resultLogFile2);

						traceln(LOG_PRIORITY_INFO, "\n");
						writeLog("", logFile);
						writeResults("");
					}
					count++;
				}
			}

		} catch (SQLException e) {
			traceln(LOG_PRIORITY_SEVERE, e.getMessage() + " / parsing error during comparison at entry " + count);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage() + " / parsing error during comparison at entry " + count + " (probably missing rows or missing columns in query 2)"));

			addTestcaseMessageAttachment(logFile);
			if (writeAllValues1) addTestcaseMessageAttachment(resultLogFile1);
			if (writeAllValues2) addTestcaseMessageAttachment(resultLogFile2);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		addTestcaseMessageAttachment(logFile);
		if (writeAllValues1) addTestcaseMessageAttachment(resultLogFile1);
		if (writeAllValues2) addTestcaseMessageAttachment(resultLogFile2);

		if (!error) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "results are equals\n Test sucess"));
			writeLog("results are equals\n Test sucess", logFile);
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		} else {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "post-run successful"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2esah$",          // only *.sah files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	
	
	private void writeResults(String string) {
		if (writeAllValues1) writeLog(string, resultLogFile1);
		if (writeAllValues2) writeLog(string, resultLogFile2);
	}

	private void writeLog(String s, File f) {
		try {
			FileWriter aWriter = new FileWriter(f, true);
			aWriter.write(s + System.getProperty("line.separator"));
			aWriter.flush();
			aWriter.close();
		} catch (java.io.IOException ex) {
			traceln(LOG_PRIORITY_INFO, "Problem with log file...");
		}
	}
}
