/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.2                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.tape;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{Tape          }  ";

	private String testRootPath;
	private String testLevel;
	private String tapInstallPath;
	private String options;

	private File workingDir;
	private File consoleFile;
	
	private final String textReportContent = null;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		try {
			// retrieve the parameters we need
			testRootPath     = getStringParamValue("General", "Test root path");     // i.e. "D:/Tests/tap"
			testLevel        = getStringParamValue("General", "Test level");         // i.e. "Test"
			tapInstallPath   = getStringParamValue("Tape", "Node install path");     // i.e. "C:/Program Files/tap"
			options          = getStringParamValue("Tape", "Tape options");

			workingDir = new File(tapInstallPath);
			if (!workingDir.exists() || !workingDir.isDirectory()) {
				traceln(LOG_PRIORITY_SEVERE, "Node install path does not exist or is not a folder: " + tapInstallPath);
				return new CReturnStatus(RESULT_FAILURE);
			}
			
			return new CReturnStatus(RESULT_SUCCESS);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			
			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
        
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// +------------------------------------+
		// | Run the TAP test                   |
		// +------------------------------------+
		CRunner tapeRunner = null;

		consoleFile = CFileUtils.createTmpRandomFile("tape_");
		
		if (testLevel.equalsIgnoreCase("Test")) { // Run the test (and report all sub results in default testcase's steps)
			tapeRunner = new CRunner("[" + testId + "] "+ testPath + testName + testcaseName,
								  "node " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".js"),
								  workingDir);
		} else if (testLevel.equalsIgnoreCase("Test case")) {
			tapeRunner = new CRunner("[" + testId + "] "+ testPath + testName + testcaseName,
								  "node " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testcaseName + ".js"),
								  workingDir);
		}
		
		
		// here we redirect the output to a file to attach it in XStudio
		short result = tapeRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, consoleFile);
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "return code=" + result));
		traceln(LOG_PRIORITY_INFO, "return code=" + result);
		
		
		// Report
		if (consoleFile != null && consoleFile.exists() && consoleFile.isFile()) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));

			addTestcaseMessageAttachment(consoleFile);

			return parseResultFile(consoleFile, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}
	
	
	
	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		// maybe there is a call available to get the available models and mCases?
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File resultFile, Vector<CExecutionStep> executionSteps) {
		//System.out.println("jsonContent = " + jsonContent);
		traceln(LOG_PRIORITY_INFO, "Parsing the text report file...");
		String line, message="";
		boolean errorDetected = false;
		boolean exceptionDetected = false;

		try {
			FileInputStream fileInputStream = new FileInputStream(resultFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();
				if (line.indexOf("# fail")>=0) {
					errorDetected = true;
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Failure(s) detected: " + line));
				}
				if (line.toLowerCase().contains("error")) {
					exceptionDetected = true;
				}
			}

			bufferedReader.close();
			fileInputStream.close();
		} catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception whle parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			errorDetected = true;
		}
		
		// parse it again to find the details
		if (errorDetected || exceptionDetected) {
			boolean armed = false;
			try {
				FileInputStream fileInputStream = new FileInputStream(resultFile);
				BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));
	
				while ((line = bufferedReader.readLine()) != null) {
					line = line.trim();
					System.out.println(">" + line);
					if (!errorDetected && exceptionDetected) {
						armed = true;
						if (line.startsWith("at")) {
							armed = true;
							executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));
						} else {
							executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, line));
						}
					} else if (line.startsWith("not ok")) {
						armed = true;
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, line));
					} else if (line.startsWith("at")) {
						armed = true;
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));
					} else if (line.startsWith("...")) {
						armed = false;
					} else if (armed) {
						executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, line));
					}
				}
	
				bufferedReader.close();
				fileInputStream.close();
			} catch (Exception e) {
				traceln(LOG_PRIORITY_SEVERE, "exception whle parsing the result file: " + e);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
				errorDetected = true;
			}
		}
		
		if (errorDetected || exceptionDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}

	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl toto = new CLauncherImpl();
		

		System.err.println("========================================================");
		
		File resultFile = new File("Y:/XStudio/src/com/xqual/xlauncher/tape/sample/tap_passed_test.txt");
		CReturnStatus returnStatus = toto.parseResultFile(resultFile, new Vector<CExecutionStep>());
		System.err.println("RESULT = " + CUtils.resultToUserString(returnStatus.getResult()));
		System.err.println(returnStatus);
	
		resultFile = new File("Y:/XStudio/src/com/xqual/xlauncher/tape/sample/tap_failed_test.txt");
		returnStatus = toto.parseResultFile(resultFile, new Vector<CExecutionStep>());
		System.err.println("RESULT = " + CUtils.resultToUserString(returnStatus.getResult()));
		System.err.println(returnStatus);

		resultFile = new File("Y:/XStudio/src/com/xqual/xlauncher/tape/sample/tap_exception_test.txt");
		returnStatus = toto.parseResultFile(resultFile, new Vector<CExecutionStep>());
		System.err.println("RESULT = " + CUtils.resultToUserString(returnStatus.getResult()));
		System.err.println(returnStatus);

		System.err.println("========================================================");

	}
	
}
