/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/

package com.xqual.xlauncher.testcomplete;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {
	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final private short TESTCOMPLETE_RESULT_SUCCESS      = 0;
	static final private short TESTCOMPLETE_RESULT_INDEFINITE   = 1;
	static final private short TESTCOMPLETE_RESULT_FAILURE      = 2;
	static final private short TESTCOMPLETE_RESULT_NOT_EXECUTED = 3;

	static final String TRACE_HEADER = "{testcomplete  }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String testCompleteInstallPath;
	private String testCompleteLogDirectoryPath;

	private static final String TESTCOMPLETE_INTERPRETER_FILE = "bin/TestComplete.exe";
	
	/*
	 * Example of correct command:
	 * "C:/Program Files/Automated QA/TestComplete 8/bin/TestComplete.exe" 
	 *      "C:/Program Files/Automated QA/TestComplete 8/Bin/New Folder/JavaTestProject1/JavaTestProject1/JavaTestProject1.mds" 
	 *      /run /exit
	 */
		
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			testRootPath                 = getStringParamValue("General", "Test root path");                       // i.e. C:/Program Files/Automated QA/TestComplete 8/Bin
			
			testCompleteInstallPath      = getStringParamValue("TestComplete", "TestComplete install path");       // i.e. C:/Program Files/Automated QA/TestComplete 8
			testCompleteLogDirectoryPath = getStringParamValue("TestComplete", "TestComplete log directory path"); // i.e. C:/Program Files/Automated QA/TestProject/Log
			
		} catch (CParamParsingException localCParamParsingException) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + localCParamParsingException.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		System.err.println("- - -  test root path = " + testRootPath);
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + " testName=" + testName + " testcaseIndex=" + testcaseIndex + "...");
		
		String commandLine = CFileUtils.quoteFilePath(testCompleteInstallPath + "/" + TESTCOMPLETE_INTERPRETER_FILE) + " " + 
		                     CFileUtils.quoteFilePath(CFileUtils.replaceSlashByBackslash(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + "/" + testcaseName + ".mds")) + " /run /exit";
				
		traceln(LOG_PRIORITY_INFO, "Running the command: " + commandLine);
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		Runtime runtime = null;
		Process process = null;
		try {
			runtime = Runtime.getRuntime();
			process = runtime.exec(commandLine);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Process started..."));
		} catch (IOException e) {
			traceln(LOG_PRIORITY_SEVERE, "Exception while running the process: " + e.getMessage());
			if (process != null)
				process.destroy();
			process = null;
			return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
		}

		short result = RESULT_FAILURE;
		try {
			result = (short)process.waitFor();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Process returned..."));
		} catch (InterruptedException e) {
			traceln(LOG_PRIORITY_SEVERE, "Exception while waiting for the end of the process: " + e.getMessage());
			if (process != null)
				process.destroy();
			process = null;
			return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
		}

		if (process != null)
			process.destroy();
		process = null;

		switch (result) {
			case TESTCOMPLETE_RESULT_SUCCESS:
				result = RESULT_SUCCESS;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test succeeded"));
				break;
			case TESTCOMPLETE_RESULT_INDEFINITE:
				result = RESULT_INDEFINITE;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test with indefinite status"));
				break;
			case TESTCOMPLETE_RESULT_NOT_EXECUTED:
				result = RESULT_NOT_EXECUTED;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test not executed"));
				break;
			case TESTCOMPLETE_RESULT_FAILURE:
			default:
				result = RESULT_FAILURE;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test failed"));
                File logFile = getLogFile();
                if (logFile != null && logFile.exists()) {
                    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "See logfile " + logFile + " in attachment!"));
                    addTestcaseMessageAttachment(logFile);
                } else {
                    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "logfile not found!"));
                }
				break;
		}
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Test execution ended."));
		return new CReturnStatus(result, executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTestcases(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

    private File getLogFile() {
        File logFolder = new File(testCompleteLogDirectoryPath);
        String[] childNames = logFolder.list();
        
        int childsNumber = childNames.length;
        if (childsNumber >= 2) {
            File parentFolder = new File(logFolder.getAbsolutePath() + "/" + childNames[childsNumber-2]); // pretty risky! isn't there a better way to identify the folder?
            String[] logFileNames = parentFolder.list();
            
            int nbLogFiles = logFileNames.length;
            if (nbLogFiles >= 3) {
                // copy and rename the log file
                File srcLogFile = new File(parentFolder.getAbsolutePath() + "/" + logFileNames[2]); // pretty risky! isn't there a better way to identify the logfile?
                File copyLogFile = null;
                
                try {
                    copyLogFile = File.createTempFile("TE9Log", ".mht");
                    CFileUtils.copyFiles(srcLogFile, copyLogFile, true);
                    return copyLogFile;
                } catch (Exception e) {
                    traceln(LOG_PRIORITY_INFO, "Logfile copy exception: " + e);
                }
            }
        }
        return null;
    }
}