/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     2.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/

package com.xqual.xlauncher.testcomplete_v2;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.CZipWriter;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {
	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final private short TESTCOMPLETE_RESULT_NO_ERROR_OR_WARNING          = 0;
	static final private short TESTCOMPLETE_RESULT_ONLY_WARNING                 = 1;
	static final private short TESTCOMPLETE_RESULT_ERRORS                       = 2;
	static final private short TESTCOMPLETE_RESULT_CANNOT_BE_EXECUTED           = 3;
	static final private short TESTCOMPLETE_RESULT_TESTCOMPLETE_ALREADY_RUNNING = 1000;
	static final private short TESTCOMPLETE_RESULT_NO_LICENSE                   = -1;

	static final String TRACE_HEADER = "{testcomplete_2}  ";

	// parameters impacting executing at run time set by the test operator
	private String testCompleteInstallPath;
	private String runType;
	private String projectSuiteFilePath;
	private String optionalArgs;
	private String projectSuiteExtension;

	private File logFile;
	private File errorFile;
	private File consoleFile;
	
	private static final String TESTCOMPLETE_INTERPRETER_FILE = "bin/TestComplete.exe";
	
	/*
	 * Example of correct command:
	 * "C:/Program Files/Automated QA/TestComplete 8/bin/TestComplete.exe" 
	 *      "C:/Program Files/Automated QA/TestComplete 8/Bin/New Folder/JavaTestProject1/JavaTestProject1/JavaTestProject1.mds" 
	 *      /run /exit
	 */
		
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			testCompleteInstallPath      = getStringParamValue("TestComplete", "TestComplete install path");                                        // i.e. "C:/Program Files (x86)/SmartBear/TestComplete 10"
			runType                      = getStringParamValue("TestComplete", "Run type");                                                         // i.e. "Selected project and tests"
			projectSuiteFilePath         = getStringParamValue("TestComplete", "Project suite file path (only with 'Selected project and tests')"); // i.e. "C:/Automation/automation_suite_1.mds" or .pjs
			projectSuiteExtension        = getStringParamValue("TestComplete", "Project suite extension (only with 'Full project suite')");         // i.e. ".mds"
			optionalArgs                 = getStringParamValue("TestComplete", "Optional arguments");                                               // i.e. "/ns /SilentMode"
			
		} catch (CParamParsingException localCParamParsingException) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + localCParamParsingException.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + " testName=" + testName + " testcaseIndex=" + testcaseIndex + "...");
		
		try {logFile     = File.createTempFile("TestCompleteLog_", ".tcLogX"); } catch (IOException e1) {traceln(LOG_PRIORITY_SEVERE, "Couldn't create log file!");}
		try {errorFile   = File.createTempFile("TestCompleteError_", ".txt");  } catch (IOException e1) {traceln(LOG_PRIORITY_SEVERE, "Couldn't create error file!");}
		try {consoleFile = File.createTempFile("TestCompleteConsole_", ".txt");} catch (IOException e1) {traceln(LOG_PRIORITY_SEVERE, "Couldn't create console file!");}
		
		CFileUtils.deleteFile(logFile); // TestComplete requires the file does not exist - we just need a unique available name in an accessible/writable path
		CFileUtils.deleteFile(errorFile);
		CFileUtils.deleteAndTouchFile(consoleFile);
		
		String commandLine = "";
		if (runType.equals("Full project suite")) {
			// The command executed will be:
			// <TestCompleteInstallPath>/bin/TestComplete.exe <TestPath>/<TestName>/<TestcaseName>.<projectSuiteExtension> /run /exit
			//
			// The launcher will parse the output of this run and should be able to generate detailed messages so that we have the details about the individual
			// tests statistics: how many tests suceeded, how many failed etc. in this project suite
			
			commandLine = CFileUtils.quoteFilePath(testCompleteInstallPath + "/" + TESTCOMPLETE_INTERPRETER_FILE) + " " + 
			                     CFileUtils.quoteFilePath(CFileUtils.replaceSlashByBackslash((testPath.length() > 0 ? testPath + "/" : "") + testName + "/" + testcaseName + projectSuiteExtension)) + " " + 
			                     "/ExportLog:\"" + logFile.getAbsolutePath() + "\" " +
			                     "/ErrorLog:\"" + errorFile.getAbsolutePath() + "\" " +
			                     "/run /exit " + optionalArgs;
		} else { // Selected project and tests
			//The mds/jps will be provided through the configuration only.
			//XStudio's "test name" will match TestComplete's"project name" and XStudio's "testcase name" will match TestComplete's "test name".
			//
			//The command executed will be:
			//<TestCompleteInstallPath>/bin/TestComplete.exe <TestCompleteProjectSuiteFile> /project:<TestName> /test:<TestcaseName> /run /exit

			commandLine = CFileUtils.quoteFilePath(testCompleteInstallPath + "/" + TESTCOMPLETE_INTERPRETER_FILE) + " " + 
			                     CFileUtils.quoteFilePath(CFileUtils.replaceSlashByBackslash(projectSuiteFilePath)) + " " + 
			                     "/project:\"" + testName + "\" " +
			                     "/test:\"" + testcaseName + "\" " +
			                     "/ExportLog:\"" + logFile.getAbsolutePath() + "\" " +
			                     "/ErrorLog:\"" + errorFile.getAbsolutePath() + "\" " +
			                     "/run /exit " + optionalArgs;
		}

		traceln(LOG_PRIORITY_INFO, "Running the command: " + commandLine);
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// +------------------------------------+
		// | Run TestComplete
		// +------------------------------------+

        CRunner testCompleteRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseName, commandLine);
        short result = testCompleteRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, consoleFile);
        
		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(logFile);
		addTestcaseMessageAttachment(errorFile);
        addTestcaseMessageAttachment(consoleFile);

        short globalStatus = testCompleteStatusToXStudioStatus(result);
		switch (result) {
			case TESTCOMPLETE_RESULT_NO_ERROR_OR_WARNING:
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "TestComplete Global Execution: no error or warning"));
				break;
			case TESTCOMPLETE_RESULT_ONLY_WARNING:
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "TestComplete Global Execution: only warning"));
				break;
			case TESTCOMPLETE_RESULT_CANNOT_BE_EXECUTED:
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "TestComplete Global Execution: cannot be executed"));
				break;
			case TESTCOMPLETE_RESULT_TESTCOMPLETE_ALREADY_RUNNING:
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "TestComplete Global Execution: cannot be executed (TestComplete already running)"));
				break;
			case TESTCOMPLETE_RESULT_NO_LICENSE:
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "TestComplete Global Execution: cannot be executed (no TestComplete license)"));
				break;
			case TESTCOMPLETE_RESULT_ERRORS:
			default:
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "TestComplete Global Execution: errors"));
				break;
		}

		parseTcLogFile(logFile, executionSteps); // log more details in the message about the execution

		return new CReturnStatus(globalStatus, executionSteps);
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

    	File testRootPathFolder = new File(testCompleteInstallPath.replaceAll("/", "\\\\"));
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTestcases(output,
				  testRootPathFolder,                       // folder to scan
				  "^.*$",                                   // any folder
				  "^.*\\x2e" + projectSuiteExtension + "$", // only *.<projectSuiteExtension> files
				  true,                                     // remove extension from the file name to determine the test name
				  "/",                                      // separator used in canonicalPath generated
				  "");                                      // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+
	public void parseTcLogFile(File tcLogXFile, Vector<CExecutionStep> executionSteps) {
		/*
		1) parse the tcLog file and retrieve a few statistics
		2) get the name of the data file
		3) parse the data file to get more details about each inner tests/steps execution
		*/
		
		if (!tcLogXFile.exists() || tcLogXFile.length() <= 1) { 
			traceln(LOG_PRIORITY_SEVERE, "tcLog file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: tcLogX file not found!"));
			return;
		}
		
		//executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: tcLogX file found"));
		
		traceln(LOG_PRIORITY_SEVERE, "unzipping .tcLogX file...");
		String baseName = CFileUtils.getBasename(tcLogXFile);
		File parentFolder = new File(tcLogXFile.getParent() + "/" + baseName);
		parentFolder.mkdirs();
		traceln(LOG_PRIORITY_SEVERE, "unzipping .tcLogX file in " + parentFolder.getAbsolutePath() + "...");
		CZipWriter.unzip(tcLogXFile.getAbsolutePath(), parentFolder.getAbsolutePath());
		
		// parse the tcLog file
		File tcLogFile = new File(tcLogXFile.getParent() + "/" + baseName + "/Description.tcLog");
		traceln(LOG_PRIORITY_INFO, "parsing tcLog file " + tcLogFile.getAbsolutePath() + "...");
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(tcLogFile);
	    
	    String computerName   = CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "//Prp[@name='computer name']/@value");
	    traceln(LOG_PRIORITY_INFO, "- reading computer name: " + computerName);
	    String userName       = CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "//Prp[@name='user name']/@value");
	    traceln(LOG_PRIORITY_INFO, "- reading user name: " + userName);
	    String dataFileName   = CXmlDocumentFactory.getNodeStringValueFromXPath(xmlDocument, "//Prp[@name='root file name']/@value");
	    traceln(LOG_PRIORITY_INFO, "- reading data file name: " + dataFileName);

	    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Executed by <b>" + userName + "</b> on computer: <b>" + computerName + "</b>"));

	    
	    
	    //int status            = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Prp[@name='status']/@value");
	    //short xstudioStatus   = testCompleteStatusToXStudioStatus(status);
	    //traceln(LOG_PRIORITY_INFO, "- reading status: " + CUtils.resultToString(xstudioStatus));

	    int errorCount        = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Prp[@name='error count']/@value");
	    int warningCount      = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Prp[@name='warning count']/@value");
	    
	    //int testCount         = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Prp[@name='test count']/@value");
	    int errorTestCount    = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Prp[@name='error test count']/@value");
	    int warningTestCount  = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Prp[@name='warning test count']/@value");
	    int successTestCount  = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Prp[@name='success test count']/@value");

	    String resultMessage = "Test statistics: <b>" + successTestCount + "</b> success, <b>" + warningTestCount + "</b> warnings and <b>" + errorTestCount + "</b> errors";
	    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, resultMessage)); // to not generate a wrong number of success, failure etc. (all tests are reported later)
	    
	    if (errorCount > errorTestCount) {
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Total errors: <b>" + errorCount + "</b>"));
	    }
	    if (warningCount > warningTestCount) {
	    	executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Total warnings: <b>" + warningCount + "</b>"));
	    }
	    
	    File datFile = new File(tcLogXFile.getParent() + "/" + baseName + "/" + dataFileName);
	    parseDatFile(datFile, executionSteps);
	}
	
	private void parseDatFile(File datFile, Vector<CExecutionStep> executionSteps) {
		if (!datFile.exists() || datFile.length() <= 1) { 
			traceln(LOG_PRIORITY_SEVERE, "dat file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: dat file not found!"));
			return;
		}
		
		//executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: dat file found"));
		
		// parse the tcLog file
		traceln(LOG_PRIORITY_INFO, "parsing dat file " + datFile.getAbsolutePath() + "...");
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(datFile);
	    NodeList nodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//Node[@name!='navigationinfo']");
	    traceln(LOG_PRIORITY_INFO, "- " + nodeList.getLength() + " node(s) detected...");
	    for (int i=0; i<nodeList.getLength(); i++) {
	    	Node node = nodeList.item(i);
	    	String name = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "Prp[@name='name']/@value");
	    	int status  = CXmlDocumentFactory.getNodeIntegerValueFromXPath(node, "Prp[@name='status']/@value");
	    	traceln(LOG_PRIORITY_INFO, "- node #" + i + " --> " + name + " : " + status);
	    	short xstudioStatus = testCompleteStatusToXStudioStatus(status);
	    	
	    	if (name != null && name.startsWith("Script Test Log ")) {
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), xstudioStatus, "<b>" + name.substring(17, name.length()-1) + "</b>"));
	    	}
	    }
	}
	
	private short testCompleteStatusToXStudioStatus(int status) {
		switch (status) {
			case TESTCOMPLETE_RESULT_NO_ERROR_OR_WARNING:
			case TESTCOMPLETE_RESULT_ONLY_WARNING:
				return RESULT_SUCCESS;
			case TESTCOMPLETE_RESULT_CANNOT_BE_EXECUTED:
			case TESTCOMPLETE_RESULT_TESTCOMPLETE_ALREADY_RUNNING:
			case TESTCOMPLETE_RESULT_NO_LICENSE:
				return RESULT_NOT_EXECUTED;
			case TESTCOMPLETE_RESULT_ERRORS:
			default:
				return RESULT_FAILURE;

		}
	}

	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl toto = new CLauncherImpl();
		toto.parseTcLogFile(new File("C:/TestCompleteLog_2602546476613663556.tcLogX"), executionSteps);
		
		for (int i=0; i<executionSteps.size(); i++) {
			System.err.println(executionSteps.get(i).getResult() + "  =  " + executionSteps.get(i).getMessage());
		}
	}
	
}