/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
|              Florent Morel-Isnard                                    |
| Version:     1.2                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/

package com.xqual.xlauncher.testexecute_v2;

import java.io.File;
import java.io.IOException;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {
    // +==============================================================+
    // | Attributes                                                   |
    // +==============================================================+

    static final private short TESTEXECUTE_RESULT_SUCCESS      = 0;
    static final private short TESTEXECUTE_RESULT_INDEFINITE   = 1;
    static final private short TESTEXECUTE_RESULT_FAILURE      = 2;
    static final private short TESTEXECUTE_RESULT_NOT_EXECUTED = 3;

    static final String TRACE_HEADER = "{testexecute_2}  ";

	// parameters impacting executing at run time set by the test operator
	private String testExecuteInstallPath;
	private String projectSuiteRootPath;
	private String testExecuteLogDirectoryPath;
	private String runType;
	private String optionalArgs;
	private String projectSuiteExtension;

    private static final String TESTEXECUTE_INTERPRETER_FILE = "bin/TestExecute.exe";
   
    // +==============================================================+
    // | Constructors                                                 |
    // +==============================================================+

    public CLauncherImpl() {
        super(TRACE_HEADER);
    }

    // +==============================================================+
    // | Methods                                                      |
    // +==============================================================+

    @Override
    public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
        setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
        // check the configuration sent by the manager
        printConfiguration();

        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        try {
			testExecuteInstallPath       = getStringParamValue("TestExecute", "TestExecute install path");                           // i.e. "C:/Program Files (x86)/SmartBear/TestExecute 11"
			projectSuiteRootPath         = getStringParamValue("TestExecute", "Project suite root path");                            // i.e. "C:/my_project_suites"
			testExecuteLogDirectoryPath  = getStringParamValue("TestExecute", "TestExecute log directory path");                     // i.e. "C:/Program Files (x86)/SmartBear/TestExecute 11/TestProject/Log"
			runType                      = getStringParamValue("TestExecute", "Run type");                                           // i.e. "Selected project and tests"
			projectSuiteExtension        = getStringParamValue("TestExecute", "Project suite extension (only with 'Full project')"); // i.e. ".mds"
			optionalArgs                 = getStringParamValue("TestExecute", "Optional arguments");                                 // i.e. "/SilentMode"
            
        } catch (CParamParsingException localCParamParsingException) {
            traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
            executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + localCParamParsingException.getMessage()));
            
            CUtils.showErrorMessage(null, 
                    "Error: Outdated configuration", 
                    "At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
                    "Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
                    JOptionPane.ERROR_MESSAGE);
            
            return new CReturnStatus(RESULT_FAILURE, executionSteps);
        }
        return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
    public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
        traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
    public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
        traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + " testName=" + testName + " testcaseIndex=" + testcaseIndex + "...");
        
		String commandLine = "";
		if (runType.equals("Full project")) {
			// <ProjectSuiteRootPath> + <test path> + <test name> + <projectSuiteExtension> will match TestExecute's project suite file path
			// <testcase name>                                                              will match TestExecute's project name (in the project suite)
			//
			// The command executed will be:
			// <TestExecuteInstallPath>/bin/TestExecute.exe <ProjectSuiteRootPath>/<TestPath>/<TestName>.<projectSuiteExtension> /run /project:<TestcaseName> /exit
			commandLine = CFileUtils.quoteFilePath(testExecuteInstallPath + "/" + TESTEXECUTE_INTERPRETER_FILE) + " " + 
			                     CFileUtils.quoteFilePath(CFileUtils.replaceSlashByBackslash((projectSuiteRootPath.length() > 0 ? projectSuiteRootPath + "/" : "") + 
			                    		 													 (testPath.length() > 0 ? testPath + "/" : "") + 
			                    		 													 testName + projectSuiteExtension)) + " " + 
			                     "/project:\"" + testcaseName + "\" " +
			                     "/run /exit " + optionalArgs;
		} else { // Selected project and tests
			// <ProjectSuiteRootPath> + <test path> will match TestExecute's project suite file path (testPath must include the extension i.e .pjs)
			// <test name>                          will match TestExecute's project name (in the project suite)
			// <testcase name>                      will match TestExecute's test name.
			//
			// The command executed will be:
			// <TestCompleteInstallPath>/bin/TestComplete.exe <ProjectSuiteRootPath>/<test path> /project:<TestName> /test:<TestcaseName> /run /exit

			commandLine = CFileUtils.quoteFilePath(testExecuteInstallPath + "/" + TESTEXECUTE_INTERPRETER_FILE) + " " + 
			                     CFileUtils.quoteFilePath(CFileUtils.replaceSlashByBackslash((projectSuiteRootPath.length() > 0 ? projectSuiteRootPath + "/" : "") + 
			                    		 													 testPath)) + " " + 
			                     "/project:\"" + testName + "\" " +
			                     "/test:\"" + testcaseName + "\" " +
			                     "/run /exit " + optionalArgs;
		}
        
        traceln(LOG_PRIORITY_INFO, "Running the command: " + commandLine);
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

        Runtime runtime = null;
        Process process = null;
        try {
            runtime = Runtime.getRuntime();
            process = runtime.exec(commandLine);
            executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Process started..."));
        } catch (IOException e) {
            traceln(LOG_PRIORITY_SEVERE, "Exception while running the process: " + e.getMessage());
            if (process != null)
                process.destroy();
            process = null;
            return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
        }

        short result = RESULT_FAILURE;
        try {
            result = (short)process.waitFor();
            executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Process returned..."));
        } catch (InterruptedException e) {
            traceln(LOG_PRIORITY_SEVERE, "Exception while waiting for the end of the process: " + e.getMessage());
            if (process != null)
                process.destroy();
            process = null;
            return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
        }

        if (process != null)
            process.destroy();
        process = null;

        switch (result) {
            case TESTEXECUTE_RESULT_SUCCESS:
                result = RESULT_SUCCESS;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test succeeded"));
                break;
            case TESTEXECUTE_RESULT_INDEFINITE:
                result = RESULT_INDEFINITE;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test succeeded but with warnings"));
                break;
            case TESTEXECUTE_RESULT_NOT_EXECUTED:
                result = RESULT_NOT_EXECUTED;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test not executed"));
                break;
            case TESTEXECUTE_RESULT_FAILURE:
            default:
                result = RESULT_FAILURE;
                executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), result, "Test failed"));
                
                File logFolder = new File(testExecuteLogDirectoryPath);
                File logFile = getLogFile(logFolder);
                if (logFile != null && logFile.exists() && logFile.isFile()) {
                    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "See logfile " + logFile + " in attachment!"));
                    addTestcaseMessageAttachment(logFile);
                } else {
                    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "logfile not found!"));
                }
                break;
        }
        executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test execution ended."));
        return new CReturnStatus(result, executionSteps);
    }

    @Override
    public CReturnStatus postRun(int testId, String testPath, String testName) {
        traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
        return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
    public CReturnStatus terminate() {
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
        return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	

    // +--------------------------+
    // | Utilities                |
    // +--------------------------+

    private File getLogFile(File logFolder) {
        if (logFolder != null && logFolder.exists()) {
        	if (!logFolder.isDirectory()) {
        		traceln(LOG_PRIORITY_SEVERE, "The file \"" + testExecuteLogDirectoryPath + "\" is NOT a folder!");
        		return null;
        	}
        	
	        String[] childNames = logFolder.list();
	        if (childNames.length <= 0) {
        		traceln(LOG_PRIORITY_SEVERE, "The folder \"" + testExecuteLogDirectoryPath + "\" does NOT contain any file!");
        		return null;
	        }
	        
	        File lastModifiedLog = null; // .mht file
	        long maxLastModified = 0;
	        for (int i=0; i<childNames.length; i++) {
	        	File logFile = new File(logFolder.getAbsolutePath() + "/" + childNames[i]);
	        	if (logFile != null && logFile.exists() && logFile.isFile()) {
	        		if (logFile.lastModified() > maxLastModified) {
	        			maxLastModified = logFile.lastModified();
	        			lastModifiedLog = logFile;
	        		}
	        	}
	        }
	        
	        traceln(LOG_PRIORITY_INFO, "The file \"" + lastModifiedLog + "\" seems to be the latest log generated");
	        
	        File copyLogFile = null;
	        if (lastModifiedLog != null && lastModifiedLog.exists()) {
		        try {
		            copyLogFile = File.createTempFile("TELog", ".mht");
		            CFileUtils.copyFiles(lastModifiedLog, copyLogFile, true);
		            return copyLogFile;
		        } catch (Exception e) {
		            traceln(LOG_PRIORITY_INFO, "Logfile copy exception: " + e);
		        }
	        }
        }
        return null;
    }

	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl launcher = new CLauncherImpl();

        File logFolder = new File("Y:/XStudio/src/com/xqual/xlauncher/testexecute/LogsErreurs");
        File logFile = launcher.getLogFile(logFolder);
	}

}