/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.8                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.testng;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for TestNG.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{testng        }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String additionalClasspath;
	private String javaInstallPath;
	private String testngJarPath;
	private String testngTestType;

	private String testngInterpreter;

	private File workingDir;

	private static final String TESTNG_RUNNER_CLASS = "org.testng.TestNG";

	private static final String JAVA_INTERPRETER_EXE = CUtils.getExecutableName("java");
	private static final char CLASSPATH_DELIMITER = CUtils.getClasspathDelimiterChar();

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath        = getStringParamValue("General", "Test root path");       // i.e. Y:/XStudio/build/tmp/eclipse_classes
			additionalClasspath = getStringParamValue("General", "Additional classpath"); // i.e. Y:/XStudio/build/lib/javaws-1_2-dev/jnlp.jar

			javaInstallPath = getStringParamValue("TestNG", "Java install path");         // i.e. C:/Progra~1/Java/jdk1.6.0_17
			testngJarPath   = getStringParamValue("TestNG", "TestNG jar path");           // i.e. Y:/Externals/testng-5.10/testng-5.10-jdk15.jar
			testngTestType  = getStringParamValue("TestNG", "TestNG test type");          // i.e. "TestNG suite (.xml)"

			if (testngTestType.equals("TestNG suite (.xml)")) {
				testngInterpreter = CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE) + " " +
	                                "-classpath " + CFileUtils.quoteFilePath(testngJarPath + CLASSPATH_DELIMITER + additionalClasspath) + " " +
				                    TESTNG_RUNNER_CLASS + " ";
				
			} else {
				testngInterpreter = CFileUtils.quoteFilePath(javaInstallPath + "/bin/" + JAVA_INTERPRETER_EXE) + " " +
                                    "-classpath " + CFileUtils.quoteFilePath(testngJarPath + CLASSPATH_DELIMITER + additionalClasspath + CLASSPATH_DELIMITER + testRootPath) + " " +
		                            TESTNG_RUNNER_CLASS + " ";
				
			}
			
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		workingDir = new File(testRootPath); // parent folder of all the test classes

		// +------------------------------------+
		// | console trace
		// +------------------------------------+
		// testpath is not a physical directory, it's a package so DON'T use it to store the traces
		// File testNGTraceFile = new File(testRootPath + "/" + /*(testPath.length()>0 ? testPath + "/" : "") +*/ "testng_traces.txt");
		// CFileUtils.deleteAndTouchFile(testNGTraceFile);
		File testNGTraceFile = CFileUtils.createTmpRandomFile("testng_");

		File testNGResultsFile = new File(testRootPath + "/test-output/testng-results.xml");
		CFileUtils.deleteAndTouchFile(testNGResultsFile);

		// +------------------------------------+
		// | Run the TestNG test class
		// +------------------------------------+
		CRunner testngRunner = null;
		// test = testsuite / no testcase
		if (testngTestType.equals("TestNG suite (.xml)")) {
			testngRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,
                    testngInterpreter.toString() + " " + testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".xml", // testPath is a / separated path (pointing to an xml)
                    workingDir);
		
		// test = class / no testcase
		} else if (testngTestType.equals("TestNG test (.class)")) {
			testngRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,
                    testngInterpreter.toString() + " -testclass " + (testPath.length()>0 ? testPath + "." : "") + testName, // testPath is a . separated path (pointing to a class)
                    workingDir);
		
		// test = class / testcase=method
		} else if (testngTestType.equals("TestNG test and method (.class)")) {
			testngRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,
                    testngInterpreter.toString() + " -methods " + (testPath.length()>0 ? testPath + "." : "") + testName + "." + testcaseName, // testPath is a . separated path (pointing to a class)
                    workingDir);
			
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "test type option not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		
		// here we redirect the output to a file for later parsing
		short result = testngRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, testNGTraceFile);

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(testNGTraceFile);
		addTestcaseMessageAttachment(new File(testRootPath + "/test-output/emailable-report.html"));
		addTestcaseMessageAttachment(new File(testRootPath + "/test-output/index.html"));
		addTestcaseMessageAttachment(testNGResultsFile);

		// parse the result file to get the result and the execution steps
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "execution process failed"));
			parseResultFile(testNGResultsFile, executionSteps); // to have the details anyway
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "execution process succeeded"));
			return parseResultFile(testNGResultsFile, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eclass$",        // only *.class files (or .xml?)
							  true,                    // remove extension from the file name to determine the test name
							  ".",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File resultFile, Vector<CExecutionStep> executionSteps) {

		if (!resultFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "run: result file found"));
		}
		
		//System.err.println("- - - result file: " + CFileUtils.FileToString(resultFile));

		// parse the result file to get the global result and the execution steps
	    Document xmlDocument = CXmlDocumentFactory.createXMLDoc(resultFile);
	    Node rootXmlNode = xmlDocument.getFirstChild();
	    if (rootXmlNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Invalid result file!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: invalid result file!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
	    }

	    int nbSkipped = 0;
		int nbFails   = 0;
		int nbSuccess = 0;
		try {
			nbSkipped = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "count(//test-method[@status='SKIP'])");
			nbFails = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "count(//test-method[@status='FAIL'])");
			nbSuccess = CXmlDocumentFactory.getNodeIntegerValueFromXPath(rootXmlNode, "count(//test-method[@status='PASS'])");
		} catch (NumberFormatException e) {
			traceln(LOG_PRIORITY_SEVERE, "result file parsing failed !");
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

	    short testcaseResult = RESULT_FAILURE;
	    if (nbFails == 0 && nbSkipped == 0 && nbSuccess > 0) {
	    	testcaseResult = RESULT_SUCCESS;
	    } else if (nbFails == 0 && nbSuccess == 0) { // whatever nbSkipped
	    	testcaseResult = RESULT_NOT_EXECUTED;
	    }
	    String testcaseResultMessage = "Statistics: <b>" + nbSuccess + "</b> success, <b>" + nbSkipped + "</b> skipped and <b>" + nbFails + "</b> failed";
	    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, testcaseResultMessage)); // to not generate a wrong number of success, failure etc. (all tests are reported later)
	    
	    
	    // Now parse each individual test separatly
	    NodeList stepNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(rootXmlNode, "//test");
	    for (int i=0; i<stepNodeList.getLength(); i++) {
	    	Node stepNode = stepNodeList.item(i);
	    	String stepName      = stepNode.getAttributes().getNamedItem("name").getNodeValue();
	    	System.err.println("========= " + stepName + " =======");
	    	String className = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "class/@name");
	    	
	    	int nbMethodSkipped = 0;
	    	int nbMethodFails = 0;
	    	int nbMethodSuccess = 0;
			try {
				nbMethodSkipped = CXmlDocumentFactory.getNodeIntegerValueFromXPath(stepNode, "count(class/test-method[@status='SKIP'])");
				System.err.println("    --> " + nbMethodSkipped + " skip");
				nbMethodFails = CXmlDocumentFactory.getNodeIntegerValueFromXPath(stepNode, "count(class/test-method[@status='FAIL'])");
				System.err.println("    --> " + nbMethodFails + " fail");
				nbMethodSuccess = CXmlDocumentFactory.getNodeIntegerValueFromXPath(stepNode, "count(class/test-method[@status='PASS'])");
				System.err.println("    --> " + nbMethodSuccess + " success");
			} catch (NumberFormatException e) {
				traceln(LOG_PRIORITY_SEVERE, "result file parsing failed !");
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			}

		    short stepResult = RESULT_FAILURE;
		    if (nbMethodFails == 0 && nbMethodSkipped == 0 && nbMethodSuccess > 0) {
		    	stepResult = RESULT_SUCCESS;
		    } else if (nbMethodFails == 0 && nbMethodSuccess == 0) { // whatever nbSkipped
		    	stepResult = RESULT_NOT_EXECUTED;
		    }
		    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), stepResult, "<b>" + stepName + "</b>")); // testNG TEST
		    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, className + "()"));
		    String resultMessage = "Test statistics: <b>" + nbMethodSuccess + "</b> success, <b>" + nbMethodSkipped + "</b> skipped and <b>" + nbMethodFails + "</b> failed";
		    executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, resultMessage)); // to not generate a wrong number of success, failure etc. (all tests are reported later)

		    
		    //look at all the test-method within the current test
		    NodeList resultNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(stepNode, "class/test-method");
		    for (int j=0; j<resultNodeList.getLength(); j++) {
		    	Node resultNode = resultNodeList.item(j);
		    	String status    = resultNode.getAttributes().getNamedItem("status").getNodeValue();
		    	String time      = resultNode.getAttributes().getNamedItem("started-at").getNodeValue();
		    	String name      = resultNode.getAttributes().getNamedItem("name").getNodeValue();
		    	String duration  = resultNode.getAttributes().getNamedItem("duration-ms").getNodeValue();
		    	String exception = CXmlDocumentFactory.getNodeStringValueFromXPath(resultNode, "exception/@class");
		    	Node messageNode = CXmlDocumentFactory.getNodeFromXPath(resultNode, "exception/message");
		    	String message = CXmlDocumentFactory.getNodeTextOrCDataContent(messageNode);
		    	
		    	if (status.equalsIgnoreCase("PASS")) {
		        	executionSteps.add(new CExecutionStep(time, RESULT_SUCCESS, "[" + status + "] " + name + " (" + duration + "ms)"));
	
		    	} else if (status.equalsIgnoreCase("FAIL")) {
		        	executionSteps.add(new CExecutionStep(time, RESULT_FAILURE, "[" + status + "] " + name + ": " + exception + "\n" + message + "\n(" + duration + "ms)"));
	
		    	} else {
		    		executionSteps.add(new CExecutionStep(time, RESULT_INFO, "[" + status + "] " + name + " (" + duration + "ms)"));
		    	}
		    }
	    }

		return new CReturnStatus(testcaseResult, executionSteps);
	}
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		CLauncherImpl toto = new CLauncherImpl();
		toto.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/testng/test-output/testng-results4.xml"), executionSteps);
		
		for (int i=0; i<executionSteps.size(); i++) {
			System.err.println(CUtils.resultToUserString(executionSteps.get(i).getResult()) + "  =  " + executionSteps.get(i).getMessage());
		}
	}
	

}
