/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Yaxiong Lin (TestOptimal)                               |
|              David (XQual)                                           |
| Version:     2.0                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.testoptimal;

import java.io.BufferedReader;
import java.io.File;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.net.URLConnection;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.json.JSONArray;
import org.json.JSONObject;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xcommon.CFileWriter;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{TestOptimal   }  ";

	static final String SUCCESS_RESP = "{\"alertMessage\":\"ok\"}";

	private String testOptimalServerUrl;
	private String username;
	private String password;
	private String sequencer;
	private String browser;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		try {
			// retrieve the parameters we need
			testOptimalServerUrl = getStringParamValue("Connection", "TestOptimal Server URL"); // i.e. http://localhost:8888/
			username             = getStringParamValue("Connection", "User (opt.)");            // required if security has been enabled in TestOptimal
			password             = getStringParamValue("Connection", "Password (opt.)");        // required if security has been enabled in TestOptimal
			
			sequencer            = getStringParamValue("Execution", "Sequencer");
			browser              = getStringParamValue("Execution", "Browser");
			
			if (!testOptimalServerUrl.endsWith("/")) {
				testOptimalServerUrl += "/";
			}
			
			return new CReturnStatus(RESULT_SUCCESS);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			
			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}


	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		
		String modelName = testName;
		String mCaseName = testcaseName;
		String statDesc = ""+CCalendarUtils.getCurrentTimeMillis();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		try {
			// +-------------------------------------------------+
			// | Start the test (model)                          |
			// +-------------------------------------------------+
			StringBuffer httpRequest = new StringBuffer(testOptimalServerUrl); // i.e. http://localhost:8888/
			httpRequest.append("MbtSvr/app=websvc");
			httpRequest.append("&action=exec");
			httpRequest.append((username == null  ? "" : "&userid="   + username));
			httpRequest.append((password == null  ? "" : "&password=" + password));
			httpRequest.append((modelName == null ? "" : "&mbtFile=" + modelName));
			httpRequest.append((mCaseName == null ? "" : "&mCase="   + mCaseName)); 
			httpRequest.append("&statDesc=" + statDesc); 
			httpRequest.append("&browser="  + browser); 
			httpRequest.append((sequencer == null ? "" : "&mbtMode=" + sequencer));
			httpRequest.append("&async=false"); 

			// read the response
			String response = readUrl(httpRequest.toString());
			if (response == null) {
				traceln(LOG_PRIORITY_SEVERE, "Invalid response");
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Execution command received a NULL response"));
				return new CReturnStatus(RESULT_FAILURE, executionSteps);
			} else {
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Stats retrieved successfully"));
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, response));
				
				// save the stats in a file and attach it
				File statsFile = CFileUtils.createTmpRandomFile("xstudio_testoptimal_");
				CFileWriter.write(statsFile, response);
				addTestcaseMessageAttachment(statsFile);
			}

			// +-------------------------------------------------+
			// | Parse the stats                                 |
			// +-------------------------------------------------+
			return parseStats(response, executionSteps);

		} catch (Exception e) {
			String errorMessage = "Error while executing the test: " + e.getMessage();
			traceln(LOG_PRIORITY_SEVERE, errorMessage);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, errorMessage));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}
	
	
	
	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		String modelName = testName;
		
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		try {
			// +-------------------------------------------------+
			// | Stop the test (model) in case it does not stop  |
			// | with multiple threads                           |
			// +-------------------------------------------------+
			// The following actions do not cause the execution to fail, but they can error and the error will be logged as messages.
			/*
			StringBuffer httpRequest = new StringBuffer(testOptimalServerUrl); // i.e. http://localhost:8888/
			httpRequest.append("MbtSvr/app=client");
			httpRequest.append("&action=stop");
			httpRequest.append((username == null  ? "" : "&userid="   + username));
			httpRequest.append((password == null  ? "" : "&password=" + password));
			httpRequest.append((modelName == null ? "" : "&mbtFile=" + modelName));
	
			String response = readUrl(httpRequest.toString());
			if (response == null || !response.contains(SUCCESS_RESP)) {
				String errorMessage = "Error STOPPING model execution: " + (response == null ? "NULL" : response);
				traceln(LOG_PRIORITY_SEVERE, errorMessage);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, errorMessage));
			}
			*/
	
			// +-------------------------------------------------+
			// | Close the model                                 |
			// +-------------------------------------------------+
			StringBuffer httpRequest = new StringBuffer(testOptimalServerUrl); // i.e. http://localhost:8888/
			httpRequest.append("MbtSvr/app=client");
			httpRequest.append("&action=close");
			httpRequest.append((username == null  ? "" : "&userid="   + username));
			httpRequest.append((password == null  ? "" : "&password=" + password));
			httpRequest.append((modelName == null ? "" : "&mbtFile=" + modelName));
	
			String response = readUrl(httpRequest.toString());
			if (response == null || !response.contains(SUCCESS_RESP)) {
				String errorMessage = "Error CLOSING model execution: " + (response == null ? "NULL" : response);
				traceln(LOG_PRIORITY_SEVERE, errorMessage);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, errorMessage));
			}
			
		} catch (Exception e) {
			String errorMessage = "Error while cleaning up the test: " + e.getMessage();
			traceln(LOG_PRIORITY_SEVERE, errorMessage);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, errorMessage));
		}
		
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		// maybe there is a call available to get the available models and mCases?
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+
	
	/**
	 * Calls TestOptimal via http request and receives response.
	 * @param urlAddress
	 * @return
	 * @throws Exception
	 */
	private String readUrl(String urlAddress) throws Exception {
		traceln(LOG_PRIORITY_INFO, "send request: " + urlAddress);
		URL url                          = new URL(urlAddress);
        URLConnection connection         = url.openConnection();
        InputStream inputStream          = connection.getInputStream();	
        BufferedReader inputBufferReader = new BufferedReader(new InputStreamReader(inputStream));
        
        // read the response
        String line = "";
        StringBuffer output = new StringBuffer();
        while ((line=inputBufferReader.readLine()) != null) {
        	output.append(line).append("\n");
        }
        
        traceln(LOG_PRIORITY_INFO, "response: " + output.toString());
        return output.toString();
	}

	/* not necessary in synchronous mode
	private String getSessionId(String jsonContent) {
		// parse the response to get the session id (if we want to use it in the future in an async version of this launcher
		// the response should look like: {"MbtSessionID": "SESS_0.5803864020400007"}
		try {
	    	JSONObject jsonObject = new JSONObject(jsonContent);
	        return jsonObject.getString("MbtSessionID");
    	} catch (Exception e) {
    		traceln(LOG_PRIORITY_SEVERE, "couldn't get session id: " + e);
			return null;
		}
	}
	*/
	
	private CReturnStatus parseStats(String jsonContent, Vector<CExecutionStep> executionSteps) {
		short status = RESULT_SUCCESS;
		
		//System.err.println("jsonContent = " + jsonContent);
		try {
	    	JSONObject jsonObject = new JSONObject(jsonContent);
	    	JSONObject modelExecProperties = jsonObject.getJSONObject("modelExec");
	    	if (modelExecProperties != null) {
		    	System.err.println("execId     = " + modelExecProperties.getInt("execID"));
		    	System.err.println("browser    = " + modelExecProperties.getString("browser"));
		    	System.err.println("plugins    = " + modelExecProperties.getString("plugins"));
		    	System.err.println("startTime  = " + modelExecProperties.getString("startTime"));
		    	System.err.println("endTime    = " + modelExecProperties.getString("endTime"));
		    	System.err.println("elapseTime = " + modelExecProperties.getString("elapseTime"));

		    	// optional
		    	String urlReport = "";
		    	try {urlReport = modelExecProperties.getString("urlReport");} catch (Exception e) {}
		    	System.err.println("urlReport  = " + urlReport);

	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "execId = " + modelExecProperties.getInt("execID")));
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "browser = " + modelExecProperties.getString("browser")));
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "plugins = " + modelExecProperties.getString("plugins")));
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "startTime = " + modelExecProperties.getString("startTime")));
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "endTime = " + modelExecProperties.getString("endTime")));
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "elapseTime = " + modelExecProperties.getString("elapseTime")));
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "report = [a]" + urlReport + "[/a]"));
	    	}
	    	
	    	JSONArray tcList = jsonObject.getJSONArray("tcList");
	    	if (tcList != null) {
		    	for (int i=0; i<tcList.length(); i++) {
	        		JSONObject tcObject = (JSONObject) tcList.get(i);
	        		if (tcObject != null) {
		        		String tcStatus = tcObject.getString("tcStatus");
		        		if (!tcStatus.equalsIgnoreCase("passed")) {
		        			status = RESULT_FAILURE;
		        		}
		
		    	    	System.err.println("   ---------------------");
		    	    	System.err.println("   tcId     = " + tcObject.getString("tcID"));
		    	    	System.err.println("   tcStatus = " + tcObject.getString("tcStatus"));
		
			    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "tc[" + tcObject.getString("tcID") + "]: " + tcObject.getString("tcStatus")));
		
		    	    	JSONArray stepList = tcObject.getJSONArray("StepList");
		    	    	if (stepList != null) {
			    	    	for (int j=0; j<stepList.length(); j++) {
			            		JSONObject stepObject = (JSONObject) stepList.get(j);
			            		if (stepObject != null) {
				            		String stepStatus = stepObject.getString("status");
				            		if (!stepStatus.equalsIgnoreCase("passed")) {
				            			status = RESULT_FAILURE;
				            		}
				
				        	    	System.err.println("      ---------------------");
				        	    	System.err.println("      stepState  = " + stepObject.getString("state"));
				        	    	System.err.println("      stepStatus = " + stepObject.getString("status"));
				
				    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "- step[" + stepObject.getString("state") + "]: " + stepObject.getString("status")));
			            		} else {// if (stepObject != null)
				    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No step found!"));
				    	    		//return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps); 
				        		}
			            		
			    	    	} // for each step
			    	    	
		    	    	} else { // if (stepList != null)
		    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No step found!"));
		    	    		//return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps);
		    	    	}
		    	    	
	        		} else { // if (tcObject != null)
	    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No test found!"));
	    	    		return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps); 
	        		}
	        		
		    	} // for each tc
		    	
	    	} else { // if (tcList != null)
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No test found!"));
	    		return new CReturnStatus(RESULT_NOT_EXECUTED, executionSteps); 
	    	}

    	} catch (Exception e) {
    		traceln(LOG_PRIORITY_SEVERE, "couldn't parse the stats: " + e);
			return null;
		}
		
		return new CReturnStatus(status, executionSteps); 
	}
	


	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl toto = new CLauncherImpl();
		
		/*
		System.err.println("========================================================");
		String jsonContentSession = "{\"MbtSessionID\": \"SESS_0.5803864020400007\"}";
		System.err.println("- - - -  session id = " + toto.getSessionId(jsonContentSession));
		*/
		
		/*
		System.err.println("========================================================");
		String jsonContentStats = "{ \"modelExec\": {\"execID\":2, \"modelName\": \"SystemHealth\", \"modelVer\": \"1.0\", \"AUTVer\": \"iteration 2\", \"ReqVer\": \"1.0\", \"TOVer\": \"5.0.1\", \"svrHost\": \"DESKTOP-LEVNM0N\", \"svrPort\": 8888, \"sequencer\": \"OptimalSequence\", \"keep\": true, \"browser\": \"firefox\", \"plugins\": \"DataGen,Selenium,Service\", \"startTime\": \"2015-10-01 10:09 AM\", \"endTime\": \"2015-10-01 10:11 AM\", \"elapseTime\": \"00:02:11\", \"numLoop\":1, \"numThread\":1, \"tcCount\":1, \"status\":\"failed\", \"failCount\":29, \"statDesc\": \"xyz\"}, \"tcList\": [{ \"tcID\": \"TC000001\", \"startTS\": \"2015-10-01 10:09:26.246\", \"endTS\": \"2015-10-01 10:11:35.013\", \"tcStatus\": \"failed\", \"thread\": 1, \"transCount\": 142, \"Requirements\": [\"1.1\" ,\"1.2\" ,\"2.2\" ,\"2.1\" ,\"PageTitle\" ,\"viewDetails\" ,\"1.3\" ], \"ReqStatusList\": {\"1.1\":\"passed\",\"1.2\":\"failed\",\"2.2\":\"passed\",\"2.1\":\"passed\",\"PageTitle\":\"passed\",\"viewDetails\":\"failed\",\"1.3\":\"passed\"}, \"StepList\":[{ \"state\":\"Start\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Start\", \"type\":\"trans\", \"trans\": \"start\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"336\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708573176@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708573176@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708573176@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"301\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"552\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"296\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"1891\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"281\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"297\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708583431@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708583431@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708583431@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"291\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"533\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"316\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"2140\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"285\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"317\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708593743@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708593743@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708593743@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"345\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"537\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"291\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"2005\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"300\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"285\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708604116@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708604116@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708604116@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"306\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"538\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"311\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"2052\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"408\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"384\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708614738@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708614738@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708614738@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"409\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"554\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"302\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"2065\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"311\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"335\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708625524@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708625524@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708625524@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"368\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"583\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"396\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"2129\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"338\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"334\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708636758@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708636758@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708636758@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"293\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"581\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"304\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"1986\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"295\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ViewDetail\", \"status\": \"failed\", \"perfMillis\": \"374\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed ProductDetail window title check\", \"lid\":127, \"reqTag\": \"PageTitle\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed to show details for prod code (must have a space around paranthesis): ( Toaster )@snapTS:1443708647557@\", \"lid\":128, \"reqTag\": \"viewDetails\"}]},{ \"state\":\"ProductDetail\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ProductDetail\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"passed\", \"perfMillis\": \"296\", \"itemList\": []},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"294\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"592\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"312\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"passed\", \"perfMillis\": \"311\", \"itemList\": []},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"300\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"2039\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"296\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"299\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708661881@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708661881@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708661881@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"292\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"602\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"RemoveItem\", \"status\": \"passed\", \"perfMillis\": \"1636\", \"itemList\": [{ \"assertID\": \"ShoppingCart_RemoveItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"1.3\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed removeItem check\", \"lid\":158, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"300\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"548\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"321\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"Pay\", \"status\": \"passed\", \"perfMillis\": \"1984\", \"itemList\": [{ \"assertID\": \"Checkout_Pay\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.1\"}]},{ \"state\":\"ThankYou\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ThankYou\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"318\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ViewDetail\", \"status\": \"failed\", \"perfMillis\": \"339\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed ProductDetail window title check\", \"lid\":127, \"reqTag\": \"PageTitle\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed to show details for prod code (must have a space around paranthesis): ( Toaster )@snapTS:1443708678932@\", \"lid\":128, \"reqTag\": \"viewDetails\"}]},{ \"state\":\"ProductDetail\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"ProductDetail\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"470\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"ShowShoppingCart\", \"status\": \"failed\", \"perfMillis\": \"322\", \"itemList\": [{ \"assertID\": \"ProductList_ShowShoppingCart\", \"status\": \"failed\", \"execMsg\": \"Transition Traversal@snapTS:1443708681896@\", \"lid\":0, \"reqTag\": \"1.2\"},{ \"assertID\": \"\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708681896@\", \"lid\":121, \"reqTag\": \"\"},{ \"assertID\": \"ProductList\", \"status\": \"failed\", \"execMsg\": \"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708681896@\", \"lid\":121, \"reqTag\": \"1.2\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"313\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"571\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"Order\", \"status\": \"passed\", \"perfMillis\": \"313\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []},{ \"state\":\"Checkout\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"495\", \"itemList\": [{ \"assertID\": \"Checkout_ContinueShopping\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"1.3\"}]},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"AddItem\", \"status\": \"passed\", \"perfMillis\": \"648\", \"itemList\": [{ \"assertID\": \"ProductList_AddItem\", \"status\": \"passed\", \"execMsg\": \"Transition Traversal\", \"lid\":0, \"reqTag\": \"2.2\"},{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"Passed addItem test\", \"lid\":87, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"\", \"status\": \"passed\", \"execMsg\": \"\", \"lid\":140, \"reqTag\": \"\"}]},{ \"state\":\"ShoppingCart\", \"type\":\"trans\", \"trans\": \"ContinueShopping\", \"status\": \"passed\", \"perfMillis\": \"309\", \"itemList\": []},{ \"state\":\"ProductList\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": [{ \"assertID\": \"ProductList\", \"status\": \"passed\", \"execMsg\": \"State Traversal\", \"lid\":74, \"reqTag\": \"1.1\"}]},{ \"state\":\"ProductList\", \"type\":\"trans\", \"trans\": \"end\", \"status\": \"passed\", \"perfMillis\": \"1294\", \"itemList\": []},{ \"state\":\"End\", \"type\":\"state\", \"status\": \"passed\", \"perfMillis\": \"\", \"itemList\": []}]}], \"reqList\": [{ \"tag\": \"1.1\", \"priority\": \"H\", \"failCount\":0, \"blockCount\":0, \"passCount\":24},{ \"tag\": \"1.2\", \"priority\": \"H\", \"failCount\":18, \"blockCount\":0, \"passCount\":0},{ \"tag\": \"1.3\", \"priority\": \"M\", \"failCount\":0, \"blockCount\":0, \"passCount\":2},{ \"tag\": \"2.1\", \"priority\": \"H\", \"failCount\":0, \"blockCount\":0, \"passCount\":9},{ \"tag\": \"2.2\", \"priority\": \"L\", \"failCount\":0, \"blockCount\":0, \"passCount\":12},{ \"tag\": \"PageTitle\", \"priority\": \"\", \"failCount\":0, \"blockCount\":0, \"passCount\":2},{ \"tag\": \"Security\", \"priority\": \"t\", \"failCount\":0, \"blockCount\":0, \"passCount\":0},{ \"tag\": \"viewDetails\", \"priority\": \"M\", \"failCount\":2, \"blockCount\":0, \"passCount\":0}], \"stateTransList\":[{\"state\": \"Checkout\", \"type\":\"state\", \"travCount\":11, \"reqTravCount\":0, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"Checkout\", \"type\":\"trans\", \"trans\":\"ContinueShopping\", \"travCount\":1, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":495, \"avg\":414, \"stdev\":113, \"min\":334, \"max\":495}},{\"state\": \"Checkout\", \"type\":\"trans\", \"trans\":\"Pay\", \"travCount\":9, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":2032, \"avg\":2358, \"stdev\":724, \"min\":1891, \"max\":4396}},{\"state\": \"Checkout\", \"type\":\"trans\", \"trans\":\"ShowShoppingCart\", \"travCount\":1, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":311, \"avg\":331, \"stdev\":28, \"min\":311, \"max\":351}},{\"state\": \"End\", \"type\":\"state\", \"travCount\":1, \"reqTravCount\":0, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"ProductDetail\", \"type\":\"state\", \"travCount\":2, \"reqTravCount\":0, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"ProductDetail\", \"type\":\"trans\", \"trans\":\"ContinueShopping\", \"travCount\":1, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":470, \"avg\":385, \"stdev\":120, \"min\":300, \"max\":470}},{\"state\": \"ProductDetail\", \"type\":\"trans\", \"trans\":\"ShowShoppingCart\", \"travCount\":1, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":296, \"avg\":292, \"stdev\":4, \"min\":289, \"max\":296}},{\"state\": \"ProductList\", \"type\":\"state\", \"travCount\":24, \"reqTravCount\":0, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"ProductList\", \"type\":\"trans\", \"trans\":\"AddItem\", \"travCount\":12, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":569, \"avg\":657, \"stdev\":260, \"min\":533, \"max\":1793}},{\"state\": \"ProductList\", \"type\":\"trans\", \"trans\":\"ShowShoppingCart\", \"travCount\":9, \"reqTravCount\":1, \"failCount\":9, \"L1\": [\"Transition Traversal@snapTS:1443708573176@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708573176@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708573176@ (lid:121)\",\"Transition Traversal@snapTS:1443708583431@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708583431@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708583431@ (lid:121)\",\"Transition Traversal@snapTS:1443708593743@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708593743@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708593743@ (lid:121)\",\"Transition Traversal@snapTS:1443708604116@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708604116@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708604116@ (lid:121)\",\"Transition Traversal@snapTS:1443708614738@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708614738@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708614738@ (lid:121)\",\"Transition Traversal@snapTS:1443708625524@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708625524@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708625524@ (lid:121)\",\"Transition Traversal@snapTS:1443708636758@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708636758@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708636758@ (lid:121)\",\"Transition Traversal@snapTS:1443708661881@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708661881@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708661881@ (lid:121)\",\"Transition Traversal@snapTS:1443708681896@\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart@snapTS:1443708681896@ (lid:121)\",\"Failed the check for page title, expecting ShoppingCart, found WebStore - Shopping Cart (lid:121)@snapTS:1443708681896@ (lid:121)\"], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":323, \"avg\":323, \"stdev\":0, \"min\":323, \"max\":323}},{\"state\": \"ProductList\", \"type\":\"trans\", \"trans\":\"ViewDetail\", \"travCount\":2, \"reqTravCount\":1, \"failCount\":2, \"L1\": [\"Failed to show details for prod code (must have a space around paranthesis): ( Toaster )@snapTS:1443708647557@ (lid:128)\",\"Failed to show details for prod code (must have a space around paranthesis): ( Toaster )@snapTS:1443708678932@ (lid:128)\"], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":356, \"avg\":356, \"stdev\":0, \"min\":356, \"max\":356}},{\"state\": \"ProductList\", \"type\":\"trans\", \"trans\":\"end\", \"travCount\":1, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":1294, \"avg\":1338, \"stdev\":62, \"min\":1294, \"max\":1382}},{\"state\": \"ShoppingCart\", \"type\":\"state\", \"travCount\":24, \"reqTravCount\":0, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"ShoppingCart\", \"type\":\"trans\", \"trans\":\"ContinueShopping\", \"travCount\":12, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":318, \"avg\":328, \"stdev\":37, \"min\":291, \"max\":427}},{\"state\": \"ShoppingCart\", \"type\":\"trans\", \"trans\":\"Order\", \"travCount\":11, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":314, \"avg\":360, \"stdev\":129, \"min\":291, \"max\":897}},{\"state\": \"ShoppingCart\", \"type\":\"trans\", \"trans\":\"RemoveItem\", \"travCount\":1, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":1636, \"avg\":1672, \"stdev\":51, \"min\":1636, \"max\":1709}},{\"state\": \"Start\", \"type\":\"state\", \"travCount\":1, \"reqTravCount\":0, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"Start\", \"type\":\"trans\", \"trans\":\"start\", \"travCount\":1, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"ThankYou\", \"type\":\"state\", \"travCount\":9, \"reqTravCount\":0, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":0, \"avg\":0, \"stdev\":0, \"min\":0, \"max\":0}},{\"state\": \"ThankYou\", \"type\":\"trans\", \"trans\":\"ContinueShopping\", \"travCount\":9, \"reqTravCount\":1, \"failCount\":0, \"L1\": [], \"L2\": [], \"L3\": [], \"L4\": [], \"L5\": [], \"slowCount\":0, \"rsp\":{ \"val\":314, \"avg\":337, \"stdev\":50, \"min\":281, \"max\":482}}]}";
		CReturnStatus returnStatus = toto.parseStats(jsonContentStats, new Vector<CExecutionStep>());
		System.err.println(returnStatus);
		*/
		
		/*
		System.err.println("========================================================");
		File stst = new File("Y:/XStudio/src/com/xqual/xlauncher/testoptimal/sample_stats.txt");
		String ststContent = CFileUtils.FileToString(stst);
		CReturnStatus returnStatus = toto.parseStats(ststContent, new Vector<CExecutionStep>());
		System.err.println(returnStatus);
		*/
		
		System.err.println("========================================================");
		File stst = new File("Y:/XStudio/src/com/xqual/xlauncher/testoptimal/sample/sample_stats2.txt");
		String ststContent = CFileUtils.FileToString(stst);
		CReturnStatus returnStatus = toto.parseStats(ststContent, new Vector<CExecutionStep>());
		System.err.println(returnStatus);
	
		System.err.println("========================================================");

	}
	
}
