/*
+----------------------------------------------------------------------+
|                    Class: CLauncherImpl                              |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.5                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.testpartner;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Compuware TestPartner.
 * The command line interface is as following:
 * C:\Program Files\TestPartner\tc.exe -d <dsn_name> -u <login> -p <password> -r <project> -s <script_name>
 * * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{test_partner  }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String projectName;
	private String username;
	private String password;
	private String dsn;
	private String detailedLogPath;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		try {
			// retrieve the parameters we need
			testRootPath    = getStringParamValue("TestPartner", "testRootPath");
			projectName     = getStringParamValue("TestPartner", "projectName");
			username        = getStringParamValue("TestPartner", "username");
			password        = getStringParamValue("TestPartner", "password");
			dsn             = getStringParamValue("TestPartner", "dsn");
			detailedLogPath = getStringParamValue("TestPartner", "detailedLogPath");
			return new CReturnStatus(RESULT_SUCCESS, null);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");

			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");

		printAttributes(attributes);

		return new CReturnStatus(RESULT_SUCCESS, null);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + ":" + testName + " testcaseIndex=" + testcaseIndex + "...");

		// +------------------------------------+
		// | run the test partner script        |
		// +------------------------------------+
		CRunner testPartnerRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                                CFileUtils.quoteFilePath("C:/Program Files/TestPartner/tc.exe") + " -d " + dsn + " -u " + username + " -p " + password + " -r " + projectName + " -s " + testRootPath + "/" + testName);
		short result = testPartnerRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);

		addTestcaseMessageAttachment(new File(detailedLogPath));

		return parseResultFile(new File(detailedLogPath));
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		return new CReturnStatus(RESULT_SUCCESS, null);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS, null);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File resultFile) {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// parse the result file to get the result and the execution steps
		if (!resultFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Result file not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "run: result file found"));
		}

		boolean errorDetected = false;

		Document xmlDocument = CXmlDocumentFactory.createXMLDoc(resultFile);
		NodeList nodeList = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "ExternalReferenceModel/Execution/ResultObject");
		for (int i=0; i<nodeList.getLength(); i++) {
			Node node = nodeList.item(i);
			String dateTime       = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "DateTime");
			String lineNumber     = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "LineNumber");
			String milliSeconds   = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "MilliSeconds");
			String stepNumber     = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "StepNumber");
			String step           = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "Step");
			String resultDetails  = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "ResultDetails");
			String commandDetails = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "CommandDetails");
			String checkName      = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "CheckName");
			String result         = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "Result");

			short testResult = RESULT_SUCCESS;
			if (result.equalsIgnoreCase("1")) {
				testResult = RESULT_FAILURE;
				errorDetected = true;
			}

			executionSteps.add(new CExecutionStep(testResult, dateTime + ":" + lineNumber + ":" + milliSeconds +
					                                          "  [" + stepNumber + "] " + step +
					                                          " / " + resultDetails + " / " + commandDetails + " / " + checkName));
		}

		if (errorDetected) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
}
