/*
+----------------------------------------------------------------------+
|                    Class: CLauncherImpl                              |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     0.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.teststand;

import java.io.File;
import java.io.FilenameFilter;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CFileWriter;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CTextUtils;
import com.xqual.xcommon.utils.CUtils;
import com.xqual.xlauncher.CTimeoutListener;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Jar packages.
 * @author Sanjeeth Nayak
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

    // +==============================================================+
    // | Attributes                                                   |
    // +==============================================================+

    static final String TRACE_HEADER = "{teststand     }  ";
    
	static final int MAX_DELAY_IN_SECONDS = 30;

    private String testRootPath;
    private String NITestStandInstallPath;
    private String NITestStandRunner;
    //private String stylesheetPath;
    private String entryPoint;
    private String options;
    private String archiveReportPath;
    private int timeout;
    private int initDelay;
    
    private File teststandHtmlReportFile;
    String currentTestcaseArchivePath;
    
	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

    public CLauncherImpl() {
            super(TRACE_HEADER);
    }

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
    
    @Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
        setSutDetails(sutId, sutName, sutVersion);
        
		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
        
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
        try {
        	// retrieve the parameters we need
        	testRootPath        	 = getStringParamValue("General",    "Test root path");                                 // i.e. C:\TestStand\tests
        	
        	NITestStandInstallPath   = getStringParamValue("TestStand",  "Test runner path");                               // i.e. C:\Program Files (x86)\National Instruments\TestStand 2010 SP1\UserInterfaces\Simple\LabVIEW
        	NITestStandRunner        = getStringParamValue("TestStand",  "Test runner");                                    // i.e. TestExec
        	//stylesheetPath         = getStringParamValue("TestStand",  "Stylesheet path");                                // i.e. C:\Program Files (x86)\National Instruments\TestStand 2012\Components\Models\TestStandModels\StyleSheets\horizontal.xsl
        	entryPoint               = getStringParamValue("TestStand",  "Default entry point");                            // i.e. Single Pass
        	options                  = getStringParamValue("TestStand",  "Default options");                                // i.e. /useExisting /editor /quit
        	archiveReportPath        = getStringParamValue("TestStand",  "Archive report path (Opt.)");                     // i.e. file://///shared_server/folder
        	timeout                  = getIntegerParamValue("TestStand", "Timeout (in seconds)");                           // i.e. 3600 (seconds)
        	initDelay                = getIntegerParamValue("TestStand", "Initial delay to search for .xml (in seconds)");  // i.e. 10 (seconds)
        	
        	NITestStandRunner = CUtils.getExecutableName(NITestStandRunner);
        	
        	System.err.println("- - - archiveReportPath = " + archiveReportPath);
        } catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
            return new CReturnStatus(RESULT_UNKNOWN, executionSteps);
        }

		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
    }

    @Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
        return new CReturnStatus(RESULT_SUCCESS);
    }


    @Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
        traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " Test:" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
        Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

        // create a file that has the session id as name so that TestStand can read that and know the session id so archive the report to the right path
		if (archiveReportPath.trim().length() > 0) {
	        File flagFile = new File(testRootPath + /*"/CS_" + getCampaignSessionId() +*/ "/" + testcaseName + ".txt");
	        CFileUtils.deleteAndTouchFile(flagFile);
	        System.err.println("- - - flagFile = " + flagFile);
	        
			currentTestcaseArchivePath = archiveReportPath;
			currentTestcaseArchivePath = currentTestcaseArchivePath.replaceAll("#session_id#",    "CS_" + getCampaignSessionId());
			currentTestcaseArchivePath = currentTestcaseArchivePath.replaceAll("#test_id#",       "T_" + testId);
			currentTestcaseArchivePath = currentTestcaseArchivePath.replaceAll("#test_name#",     testName);
			currentTestcaseArchivePath = currentTestcaseArchivePath.replaceAll("#testcase_id#",   "TC_" + testcaseId);
			currentTestcaseArchivePath = currentTestcaseArchivePath.replaceAll("#testcase_name#", testcaseName);
			System.err.println("- - - currentTestcaseArchivePath = " + currentTestcaseArchivePath);
			
	        CFileWriter.write(flagFile, currentTestcaseArchivePath);
		}
		
		File workingDir = new File(NITestStandInstallPath); // parent folder of the runner

		// check if the testcase has a parameter "teststand.entrypoint" set to overwrite the default entry point
		for (CTestcaseParam param : params) {
			if (param.getName().equalsIgnoreCase("teststand.entrypoint")) {
				entryPoint = param.getStringValue();
				break;
			}
		}

		// check if the testcase has a parameter "teststand.options" set to overwrite the default options
		for (CTestcaseParam param : params) {
			if (param.getName().equalsIgnoreCase("teststand.options")) {
				options = param.getStringValue();
				break;
			}
		}

		// +------------------------------------+
		// | Run the teststand test class
		// +------------------------------------+
		/*
		File testCompletedFile = new File(testRootPath + "/" + testPath + "/Reports/test_completed.txt");
		if (testCompletedFile != null && testCompletedFile.exists() && testCompletedFile.isFile()) {
			testCompletedFile.delete();
		}
		*/
		
		String runnerAbsolutePath   = NITestStandInstallPath + "\\" + NITestStandRunner;
		String sequenceAbsolutePath = testRootPath + "\\" + testPath.replaceAll("/", "\\\\") + "\\" + testcaseName + ".seq"; // for some odd reason, one single slash and the test is executed but the .xml not generated
		
        CRunner testStandRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
                                              CFileUtils.quoteFilePath(runnerAbsolutePath) + " -runEntryPoint " + CFileUtils.quoteFilePath(entryPoint) + " " + 
                                              CFileUtils.quoteFilePath(sequenceAbsolutePath) + " " + 
                                              options, 
                                              workingDir);
        short result = testStandRunner.requestAction(IRunner.START_PROCESS, IRunner.DO_NOT_WAIT_END_OF_EXECUTION); // asynchronous
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "TestStand sequence execution failed"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		CUtils.sleep(initDelay*1000); // wait for 10 seconds so that the environment has the time to start and create an empty .xml file
		
		// at this point, the result file should be created with a size of 0 bytes
		// let's search for it
		// search for 2 different places for the result files:
		// 1) search for the last modified <testcasename>_Report*.xml file in the Reports folder
		// 2) if does not exist, search for the last modified <testcasename>_Report*.xml in the current folder
		File mostRecentResultFile;
		long mostRecentResultFile1LastModified = 0;
		long mostRecentResultFile2LastModified = 0;
		
		File mostRecentResultFile1 = getTheMostRecentResultFile(testRootPath + "\\" + testPath + "\\Reports", testcaseName); // may be null
		traceln(LOG_PRIORITY_INFO, "Most recent result file in " + testRootPath + "\\" + testPath + "\\Reports\\: " + mostRecentResultFile1);

		File mostRecentResultFile2 = getTheMostRecentResultFile(testRootPath + "\\" + testPath, testcaseName); // may be null
		traceln(LOG_PRIORITY_INFO, "Most recent result file in " + testRootPath + "\\" + testPath + "\\: " + mostRecentResultFile2);

		if (mostRecentResultFile1 != null && mostRecentResultFile1.exists()) mostRecentResultFile1LastModified = mostRecentResultFile1.lastModified();
		if (mostRecentResultFile2 != null && mostRecentResultFile2.exists()) mostRecentResultFile2LastModified = mostRecentResultFile2.lastModified();
		if (mostRecentResultFile1LastModified > mostRecentResultFile2LastModified) {
			mostRecentResultFile = mostRecentResultFile1;
		} else {
			mostRecentResultFile = mostRecentResultFile2;
		}
		traceln(LOG_PRIORITY_INFO, "Most recent result file = " + mostRecentResultFile);
		
		if (mostRecentResultFile1LastModified + mostRecentResultFile2LastModified == 0 || mostRecentResultFile == null) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "No result file found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		// Wait until the result file contains actual data - which will mean the test actually completed
		traceln(LOG_PRIORITY_INFO, "looking for result file to contain actual data: " + mostRecentResultFile.getAbsolutePath() + "...");
		short resultTimeout = CTimeoutListener.waitForFileWithData(mostRecentResultFile, 50, 1500, timeout); // bigger than 50 bytes, constant size after a delay of 1.5s and by default 1 hour timeout
		if (resultTimeout != RESULT_SUCCESS) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "timeout of " + timeout + " seconds to execute the test case expired"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		// parse the result file to generate the messages
		if (mostRecentResultFile !=null && mostRecentResultFile.exists() && mostRecentResultFile.isFile()) {
			return parseResultFile(mostRecentResultFile, executionSteps, testcaseName);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Results file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
    }

    
	private File getTheMostRecentResultFile(String rootFolderPath, String testcaseName) {
		return getTheMostRecentResultFile(rootFolderPath, testcaseName, false);
	}
	
	private File getTheMostRecentResultFile(String rootFolderPath, String testcaseName, boolean skipTimeVerification) {
		traceln(LOG_PRIORITY_INFO, "searching for the result file in " + rootFolderPath + "...");

		File rootFolder = new File(rootFolderPath);
		if (rootFolder == null || !rootFolder.exists() || !rootFolder.isDirectory())
			return null;
		
		File[] childResultFiles = rootFolder.listFiles(new CResultFileFilter(testcaseName));
		traceln(LOG_PRIORITY_INFO, "number of candidates: " + childResultFiles.length + "...");
		if (childResultFiles.length == 0) return null;
			
		// search for the most recent .data file
		long lastModifiedMillis = 0;
		File mostRecentDataFile = null;
		for (File dataFile : childResultFiles) {
			if (dataFile.lastModified() > lastModifiedMillis) {
				mostRecentDataFile = dataFile;
				lastModifiedMillis = dataFile.lastModified();
			}
		}
		
		if (mostRecentDataFile == null) {
			return null;
		}

		traceln(LOG_PRIORITY_INFO, "most recent candidate: " + mostRecentDataFile);

		// check the file is not too old (that's why this method is not 100% reliable)
		if (!skipTimeVerification && mostRecentDataFile.lastModified() < System.currentTimeMillis()-(1000*MAX_DELAY_IN_SECONDS)) {
			traceln(LOG_PRIORITY_INFO,"most recent candidate is too old!");
			return null;
		}
		
		return mostRecentDataFile;
	}
    
    
    @Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
        return new CReturnStatus(RESULT_SUCCESS);
    }

	@Override
	public CReturnStatus terminate() {
		// Kill the GUI if it is up
		/*
		File workingDir = new File(NITestStandInstallPath); // parent folder of all the test classes

        CRunner testStandRunner = new CRunner("Cleanup",
                                              NITestStandRunner + " /quit", 
                                              workingDir);
        testStandRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
        */
		return new CReturnStatus(RESULT_SUCCESS);
	}
	
	
	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
									  testRootPathFolder,      // folder to scan
									  "^.*$",                  // any folder
									  "^.*\\x2eseq$",          // only *.seq files
									  true,                    // remove extension from the file name to determine the test name
									  "/",                     // separator used in canonicalPath generated
									  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)

		return output;
	}
	

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	// parser based on this TestStand's Report Options (that can be set from the Sequence editor):
	// * Contents:
	//   - Report Format:           XML Document
	//   - style Sheet:             horizontal.xsl
	//   - Default Numeric format:  %$.13f
	//   - Include Step Results:    true
	//   - Include Test Limits:     true
	//   - Include Execution Times: true
	// * Report File Pathname:
	//   - Type of Model:               Sequential
	//   - File Name/Directory Options: Specify Report File Path By Expression
	//   - Report File Path:            "<ClientFileDir>\\Reports\\<ClientFileName>_Report[<FileTime>][<FileDate>]<Unique>.<FileExtension>" (<ClientFileDir> being the parent folder of the testcase sequence)
	//                              or  "<ClientFileDir>\\<ClientFileName>_Report[<FileTime>][<FileDate>]<Unique>.<FileExtension>"          (<ClientFileDir> being the parent folder of the testcase sequence)
	
	private CReturnStatus parseResultFile(File resultsFile, Vector<CExecutionStep> executionSteps, String testcaseName) {
		short testcaseStatus = RESULT_SUCCESS;
		
		String xmlContent = CFileUtils.FileToString(resultsFile);
		Document xmlOutputDocument = CXmlDocumentFactory.createXMLDoc(xmlContent);
		Node rootNode = CXmlDocumentFactory.getNodeFromXPath(xmlOutputDocument, "/Reports");
		if (rootNode == null) {
			traceln(LOG_PRIORITY_SEVERE, "Invalid result file!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "run: Invalid result file!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		

		// ---------------------------------- stats -------------------------------------

		String reportTitle = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/@Title");

		String uUTResult = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/@UUTResult");
		String stepCount = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/@StepCount");
		
		String startDate = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Name='StartDate']/Prop[@Name='Text']/Value");
		String startTime = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Name='StartTime']/Prop[@Name='Text']/Value");

		String stationId = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Name='StationInfo']/Prop[@Name='StationID']/Value");
		String operator = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Name='StationInfo']/Prop[@Name='LoginName']/Value");

		String batchSerialNumber = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Name='UUT']/Prop[@Name='BatchSerialNumber']/Value");
		if (batchSerialNumber == null) batchSerialNumber = "-";
		
		String testSocketIndex = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Name='UUT']/Prop[@Name='TestSocketIndex']/Value");
		if (testSocketIndex == null) testSocketIndex = "-";
		
		String serialNumber = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Name='UUT']/Prop[@Name='SerialNumber']/Value");

		String executionTime = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop/Prop[@Name='TS']/Prop[@Name='TotalTime']/Value");

		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "[h1][b][u]" + reportTitle + "[/u][/b][/h1]"));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Station Id: [b]" + stationId + "[/b]"));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Batch Serial Number: " + batchSerialNumber));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Test Socket Index: " + testSocketIndex));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Serial Number: " + serialNumber));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "operator: [b]" + operator + "[/b]"));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Start: [b]" + startDate + " " + startTime + "[/b]"));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Execution Time: [b]" + executionTime + " seconds[/b]"));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Number of results: [b]" + stepCount + "[/b]"));
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "UUT Result: " + colorResult(uUTResult)));
		
		if (uUTResult.equalsIgnoreCase("Failed"))
			testcaseStatus = RESULT_FAILURE;

		// ---------------------------------- Failure Chain -------------------------------------
		
		Node criticalFailureStackNode = CXmlDocumentFactory.getNodeFromXPath(rootNode, "Report/Prop[@Name='UUT']/Prop[@Name='CriticalFailureStack']/Value");
		if (criticalFailureStackNode != null) {
			StringBuffer failureChainTable = new StringBuffer();
			failureChainTable.append("[h1][u]Failure Chain:[/u][/h1]");
			failureChainTable.append("[table]");
			failureChainTable.append("[tr][th]Step[/th][th]Sequence[/th][th]Sequence File[/th][th]Result Id[/th][/tr]\n");
			
			NodeList failureNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(rootNode, "Report/Prop[@Name='UUT']/Prop[@Name='CriticalFailureStack']/Value");
			for (int i=0; i<failureNodeList.getLength(); i++) {
				Node failureNode = failureNodeList.item(i);
				
				String step          = CXmlDocumentFactory.getNodeTextOrCDataContent(failureNode, "Prop/Prop[1]/Value"); // Don't know why but the search on attribute does not work in this case !!!???
				String sequence      = CXmlDocumentFactory.getNodeTextOrCDataContent(failureNode, "Prop/Prop[@Name='SequenceName']/Value");
				String sequenceFile  = CXmlDocumentFactory.getNodeTextOrCDataContent(failureNode, "Prop/Prop[@Name='SequenceFileName']/Value");
				String resultId      = CXmlDocumentFactory.getNodeTextOrCDataContent(failureNode, "Prop/Prop[@Name='ResultId']/Value");
				failureChainTable.append("[tr][td]" + step + "[/td][td]" + sequence + "[/td][td]" + sequenceFile + "[/td][td]" + resultId + "[/td][/tr]");
			}
			failureChainTable.append("[/table]");
			executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, failureChainTable.toString()));
		}
		
		
		// ---------------------------------- Sequence details -------------------------------------
		
		String sequenceFile   = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='SequenceCall']/Prop[@Name='SequenceFile']/Value");
		String sequence       = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='SequenceCall']/Prop[@Name='Sequence']/Value");
		String sequenceStatus = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='SequenceCall']/Prop[@Name='Status']/Value");
		String threadId       = CXmlDocumentFactory.getNodeStringValueFromXPath(rootNode, "Report/Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='SequenceCall']/Prop[@Name='ThreadId']/Value");

		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "ThreadId: [b]" + threadId + "[/b]"));

		if (archiveReportPath.trim().length() > 0) {
			System.err.println("- - - currentTestcaseArchivePath = " + currentTestcaseArchivePath);
			System.err.println("- - - escaped currentTestcaseArchivePath = " + CTextUtils.escapeURLString(currentTestcaseArchivePath));
			executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, "Archived Report: [a]" + CTextUtils.escapeURLString(currentTestcaseArchivePath) + "[/a]"));
		}
		
		if (sequenceStatus.equalsIgnoreCase("Failed")) {
			executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_FAILURE, "Status: " + colorResult(sequenceStatus)));
			testcaseStatus = RESULT_FAILURE;
		} else {
			executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_SUCCESS, "Status: " + colorResult(sequenceStatus)));
		}

		Node TEResultNode = CXmlDocumentFactory.getNodeFromXPath(rootNode, "Report/Prop[@Type='TEResult']");
		StringBuffer detailedStepsTable = new StringBuffer();

		// +----------------------------+
		// | Recursive parsing          |
		// +----------------------------+
		recursivelyParseTEResultNode(TEResultNode, detailedStepsTable);

		
		executionSteps.add(new CExecutionStep(/*CCalendarUtils.getCurrentTime(),*/ RESULT_INFO, detailedStepsTable.toString()));
		
		// Just for debugging
		/*
		StringBuffer messageXml = new StringBuffer();
		messageXml.append(com.xqual.xstudio_agent.CAgentXMLEncoder.encodeMessageStart());
		for (int i=0; i<executionSteps.size(); i++) {
			messageXml.append(com.xqual.xstudio_agent.CAgentXMLEncoder.encodeMessageStep(executionSteps.elementAt(i)));
		}
		messageXml.append(com.xqual.xstudio_agent.CAgentXMLEncoder.encodeMessageStop());
		System.err.println("\n---> " + messageXml + "\n");
		*/
		return new CReturnStatus(testcaseStatus, executionSteps);
    }
	
	
	// RECURSIVE METHODE TO PARSE A TERESULT NODE
	private void recursivelyParseTEResultNode(Node node, StringBuffer output) {
		String sequenceFile   = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "Prop[@Name='TS']/Prop[@Name='SequenceCall']/Prop[@Name='SequenceFile']/Value");
		String sequence       = CXmlDocumentFactory.getNodeStringValueFromXPath(node, "Prop[@Name='TS']/Prop[@Name='SequenceCall']/Prop[@Name='Sequence']/Value");
		if (sequence != null) output.append("[b]" + sequence + "[/b]\n");
		if (sequenceFile != null) output.append("[i]" + sequenceFile.replaceAll("\\\\", "/") + "[/i]\n");

		NodeList valueNodeList = CXmlDocumentFactory.getNodeObjListFromXPath(node,    "Prop[@Name='TS']/Prop[@Name='SequenceCall']/Prop[@Name='ResultList']/Value");
		if (valueNodeList.getLength() > 0) {
			output.append("[table]");
			output.append("[tr][th]Group[/th][th]Step[/th][th]Status[/th][th]Measurement[/th][th]Units[/th][th]Low Limit[/th][th]High Limit[/th][th]Comparison Type[/th][/tr]\n");
		}
		
		for (int i=0; i<valueNodeList.getLength(); i++) {
			Node valueNode = valueNodeList.item(i);
			//System.err.println("----> " + CTextUtils.left(CXmlDocumentFactory.nodeToString(valueNode), 100));
			
			String errorCode  = CXmlDocumentFactory.getNodeStringValueFromXPath(valueNode,                                "Prop[@Type='TEResult']/Prop[@Name='Error']/Prop[@Name='Code']/Value");
			String errorMsg   = CXmlDocumentFactory.getNodeStringValueFromXPath(valueNode,                                "Prop[@Type='TEResult']/Prop[@Name='Error']/Prop[@Name='Msg']/Value");
			if (errorMsg == null) errorMsg = "-";

			String status     = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                  "Prop[@Type='TEResult']/Prop[@Name='Status']/Value");
			//System.err.println("      **** status = " + status);
			if (status == null) status = "-";
			
			String exitcode   = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                  "Prop[@Type='TEResult']/Prop[@Name='ExitCode']/Value");
			if (exitcode == null) exitcode = "-";

			String reportText = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                  "Prop[@Type='TEResult']/Prop[@Name='ReportText']/Value");
			if (reportText == null) reportText = "";

			String stepName   = CTextUtils.escapeAttributeString(CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode, "Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='StepName']/Value").trim());
			String stepGroup  = CTextUtils.escapeAttributeString(CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode, "Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='StepGroup']/Value").trim());
			String stepId     = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                  "Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='StepId']/Value");
			String steptype   = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                  "Prop[@Type='TEResult']/Prop[@Name='TS']/Prop[@Name='StepType']/Value");

			String measurement = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                 "Prop[@Type='TEResult']/Prop[@Name='String' and @Flags='0x2400']/Value");
			if (measurement == null) measurement = "";
			
			String units = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                       "Prop[@Type='TEResult']/Prop[@Name='TOTO']/Value");
			if (units == null) units = "";
			
			String lowLimit = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                    "Prop[@Type='TEResult']/Prop[@Name='Limits']/Prop[1]/Value");
			if (lowLimit == null) lowLimit = "";
			
			String highLimit = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                   "Prop[@Type='TEResult']/Prop[@Name='Limits']/Prop[2]/Value");
			if (highLimit == null) highLimit = "";
			
			String comparison = CXmlDocumentFactory.getNodeTextOrCDataContent(valueNode,                                  "Prop[@Type='TEResult']/Prop[@Name='Comp']/Value");
			if (comparison == null) comparison = "";

			/*
			short thisStatus = RESULT_SUCCESS;
			if (status.equalsIgnoreCase("Failed")) {
				thisStatus = RESULT_FAILURE;
			} else if (status.equalsIgnoreCase("Done")) {
				thisStatus = RESULT_INFO;
			} else if (status.equalsIgnoreCase("Skipped")) {
				thisStatus = RESULT_NOT_EXECUTED;
			}
			*/
			
			output.append("[tr]");
			output.append("[td]" + stepGroup + "[/td]");
			output.append("[td]");
			output.append(stepName + "\n");
			if (reportText.length()>0) output.append("[chocolate]" + CTextUtils.escapeAttributeString(reportText.replaceAll("\\\\", "/")) + "[/chocolate]\n");
			recursivelyParseTEResultNode(CXmlDocumentFactory.getNodeFromXPath(valueNode, "Prop[@Type='TEResult']"), output);
			output.append("[/td]");
			output.append("[td]" + colorResult(status) + "[/td]");
			output.append("[td]" + measurement + "[/td]");
			output.append("[td]" + units + "[/td]");
			output.append("[td]" + lowLimit + "[/td]");
			output.append("[td]" + highLimit + "[/td]");
			output.append("[td]" + comparison + "[/td]");
			output.append("[/tr]\n");
		}

		if (valueNodeList.getLength() > 0) {
			output.append("[/table]");
		}
	}
	
	private String colorResult(String input) {
		if (input.equalsIgnoreCase("Failed")) {
			return "[firebrick][b][u]" + input + "[/u][/b][/firebrick]";
					
		} else if (input.equalsIgnoreCase("Skipped")) {
			return "[dimgray][b][u]" + input + "[/u][/b][/dimgray]";
			
		} else if (input.equalsIgnoreCase("Passed")) {
			return "[darkseagreen][b][u]" + input + "[/u][/b][/darkseagreen]";
			
		} else {
			return input;
			
		}
	}
	
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		CReturnStatus returnStatus = null;
		
		CLauncherImpl toto = new CLauncherImpl();
		toto.archiveReportPath = "file://///shared_server/folder/";
		
		// search for 2 different places for the result files:
		// 1) search for the last modified <testcasename>_Report*.xml file in the Reports folder
		// 2) if does not exist, search for the last modified <testcasename>_Report*.xml in the current folder
		//File mostRecentResultFile = new File("Y:/XStudio/src/com/xqual/xlauncher/teststand/sample/TestCase1_Report[3 01 26 PM][6 29 2016].xml");
		//File mostRecentResultFile = new File("Y:/XStudio/src/com/xqual/xlauncher/teststand/sample2/GS Beatup Base_Report[4 59 47 PM][8 29 2014] BUILD 2_0_28.xml");
		File mostRecentResultFile = new File("Y:/XStudio/src/com/xqual/xlauncher/teststand/sample3/GS Beatup Base_Report[4 16 25 PM][7 7 2016].xml");
		
		// generate the HTML report
		// We tried generating the HTML report using any of the 3 stylesheets NI provide (horizontal.xsl, expand.xsl and report.xsl) but this will not work as they all include some Javascript and VB (!) functions
		// whose namespace is "http://www.ni.com/TestStand/" which is not supported by Xalan (the XSLT processor included by default in Java - so that the launchers use).
		// NI very likely use a specific XSLT processor that supports their own namespace
		/*
		File teststandHtmlReportFile = null;
		try {teststandHtmlReportFile = File.createTempFile("teststand_report", ".html"); } catch (IOException e1) {toto.traceln(LOG_PRIORITY_SEVERE, "Couldn't create html report file!");}
		System.err.println("- - - - - teststandHtmlReportFile = " + teststandHtmlReportFile);
		CFileUtils.deleteAndTouchFile(teststandHtmlReportFile);
		//File xslTransform = new File("Y:/XStudio/src/com/xqual/xlauncher/teststand/sample/StyleSheets/horizontal.xsl");
		//File xslTransform = new File("Y:/XStudio/src/com/xqual/xlauncher/teststand/sample/StyleSheets/expand.xsl");
		File xslTransform = new File("Y:/XStudio/src/com/xqual/xlauncher/teststand/sample/StyleSheets/report.xsl");
		System.err.println("- - - - - xslTransform = " + xslTransform);
		String htmlReportContent = CXmlDocumentFactory.getTransformedXML(CFileUtils.FileToString(mostRecentResultFile), CFileUtils.FileToString(xslTransform));
		CFileWriter.write(teststandHtmlReportFile, htmlReportContent);
		*/
		
		// parsing
		toto.parseResultFile(mostRecentResultFile, executionSteps, "TestSequence1");

		System.err.println(executionSteps);
	}
	
	
	/*
	+----------------------------------------------------------------------+
	|                    Class: CResultFileFilter                          |
	|                                                                      |
	| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
	| Version:     0.1                                                     |
	| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
	+----------------------------------------------------------------------+
	*/
	public class CResultFileFilter implements FilenameFilter {
		private final String testcaseName;
		
		public CResultFileFilter(String testcaseName) {
			this.testcaseName= testcaseName;
		}
		
		@Override
		public boolean accept(File dir, String name) {
			return name.startsWith(testcaseName) && name.endsWith(".xml");
		}
	}
	
	
	
	
	
}
