/*
+----------------------------------------------------------------------+
|                    Class: CLauncherimpl                              |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.9                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.visualstudio_codedui;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Microsoft VisualStudio.
 * * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{coded ui      }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String runnerPath;
	private String options;
	private String dllName;

	private File workingDir;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		try {
			// retrieve the parameters we need

			// section related to the SUT, independant from VisualStudio
			testRootPath    = getStringParamValue("General", "Test root path");         // i.e. C:/my_test_repository
			dllName         = getStringParamValue("General", "Library name");           // i.e. my_test.dll

			// section related to VisualStudio
			runnerPath      = getStringParamValue("VisualStudio", "Runner path");       // i.e. C:/Program Files (x86)/Microsoft Visual Studio 12.0/Common7/IDE/CommonExtensions/Microsoft/TestWindow/vstest.console.exe
			options         = getStringParamValue("VisualStudio", "Options");           // i.e. /InIsolation

			return new CReturnStatus(RESULT_SUCCESS, null);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");

			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");

		printAttributes(attributes);

		return new CReturnStatus(RESULT_SUCCESS, null);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + ":" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		workingDir = new File(testRootPath);
		File resultsFolder = new File(testRootPath + "/TestResults");
		
		// we should maybe empty this folder at this point !?
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		File visualStudioTraceFile = new File(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + "codedui_traces.txt");
		CFileUtils.deleteAndTouchFile(visualStudioTraceFile);
		
		// +------------------------------------+
		// | run the visual studio script       |
		// +------------------------------------+
		// C:/Program Files (x86)/Microsoft Visual Studio 12.0/Common7/IDE/CommonExtensions/Microsoft/TestWindow/vstest.console.exe  my_test.dll   /Tests:test1       /Logger:trx
		// <--------------------------------------------- runnerPath ------------------------------------------------------------->  <- dllName ->        <testName>
		CRunner visualStudioRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
		                                         CFileUtils.quoteFilePath(runnerPath) + " " +
		                                         CFileUtils.quoteFilePath((testRootPath.length()>0 ? testRootPath + "/" : "") + dllName) + " " +
		                                         "/tests:" + testName + " " +
		                                         options + " " +
		                                         "/Logger:trx",
		                                         workingDir);
		short result = visualStudioRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, visualStudioTraceFile);

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(visualStudioTraceFile);

		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "execution process failed"));
			parseResultFile(resultsFolder, executionSteps); // to have the details anyway
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "execution process succeeded"));
			return parseResultFile(resultsFolder, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		return new CReturnStatus(RESULT_SUCCESS, null);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS, null);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  ".",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}
	

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	/*
	private String convertPathSeparator(String input) {
		return input.replaceAll("/", "\\\\");
	}
	*/

	private CReturnStatus parseResultFile(File resultsFolder, Vector<CExecutionStep> executionSteps) {
		// search for the .trx file in the result folder
		if (resultsFolder == null || !resultsFolder.exists() || !resultsFolder.isDirectory()) {
			traceln(LOG_PRIORITY_INFO, "couldn't find the folder " + resultsFolder + "...");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "couldn't find the folder"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		File[] childFiles = resultsFolder.listFiles();
		if (childFiles == null) {
			traceln(LOG_PRIORITY_INFO, "empty test folder " + resultsFolder);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "empty results folder"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		File trxFile = null;
		boolean trxFound = false;
		for (File childFile: childFiles) {
			System.out.println("-> " + childFile);
			if (childFile.isFile() && CFileUtils.getExtension(childFile).equals("trx")) {
				trxFound = true;
				trxFile = childFile;
				traceln(LOG_PRIORITY_INFO, "found .trx file " + trxFile);
				break;
			}
		}
		
		if (!trxFound) {
			traceln(LOG_PRIORITY_INFO, ".trx report missing for this result folder");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, ".trx report missing for this result folder: " + resultsFolder));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		
		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		addTestcaseMessageAttachment(trxFile);

		// parse the result file to get the result and the execution steps
		boolean errorDetected = false;

        Document xmlDocument = CXmlDocumentFactory.createXMLDoc(trxFile);
        //Node rootXmlNode = xmlDocument.getFirstChild();

        // unknown
		int nbTests                    = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@total");
		int nbTestsExecuted            = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@executed");
		int nbTestsNotExecuted         = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@notExecuted");
		int nbTestsInProgress          = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@inProgress");
		int nbTestsPending             = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@pending");

		// success
		int nbTestsPassed              = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@passed");
		int nbTestsCompleted           = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@completed");

		// failure
		int nbTestsError               = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@error");
		int nbTestsFailed              = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@failed");
		int nbTestsTimeout             = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@timeout");
		int nbTestsAborted             = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@aborted");
		int nbTestsInconclusive        = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@inconclusive");
		int nbTestsPassedButRunAborted = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@passedButRunAborted");
		int nbTestsNotRunnable         = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@notRunnable");
		int nbTestsDisconnected        = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@disconnected");
		int nbTestsWarning             = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Counters/@warning");

		System.err.println("- nbTests                    = " + nbTests);
		System.err.println("- nbTestsExecuted            = " + nbTestsExecuted);
		System.err.println("- nbTestsNotExecuted         = " + nbTestsNotExecuted);
		System.err.println("- nbTestsInProgress          = " + nbTestsInProgress);
		System.err.println("- nbTestsPending             = " + nbTestsPending);

		System.err.println("- nbTestsPassed              = " + nbTestsPassed);
		System.err.println("- nbTestsCompleted           = " + nbTestsCompleted);

		System.err.println("- nbTestsError               = " + nbTestsError);
		System.err.println("- nbTestsFailed              = " + nbTestsFailed);
		System.err.println("- nbTestsTimeout             = " + nbTestsTimeout);
		System.err.println("- nbTestsAborted             = " + nbTestsAborted);
		System.err.println("- nbTestsInconclusive        = " + nbTestsInconclusive);
		System.err.println("- nbTestsPassedButRunAborted = " + nbTestsPassedButRunAborted);
		System.err.println("- nbTestsNotRunnable         = " + nbTestsNotRunnable);
		System.err.println("- nbTestsDisconnected        = " + nbTestsDisconnected);
		System.err.println("- nbTestsWarning             = " + nbTestsWarning);

		int timeToExecute   = CXmlDocumentFactory.getNodeIntegerValueFromXPath(xmlDocument, "//Times/@finish");

		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "timeToExecute:" + timeToExecute));

		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "nbTests:" + nbTests + " " +
													          "Executed:" + nbTestsExecuted + " " +
													          "NotExecuted:" + nbTestsNotExecuted + " " +
													          "InProgress:" + nbTestsInProgress + " " +
													          "Pending:" + nbTestsPending));

		if (nbTestsPassed+nbTestsCompleted>0) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Passed:" + nbTestsPassed + " " +
	                                                              "Completed:" + nbTestsCompleted));
		}

		if (nbTestsError+nbTestsFailed+nbTestsTimeout+nbTestsAborted+nbTestsInconclusive+nbTestsPassedButRunAborted+nbTestsNotRunnable+nbTestsDisconnected+nbTestsWarning>0) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "nbTestsError:" + nbTestsError + " " +
																  "Failed:" + nbTestsFailed + " " +
																  "Timeout:" + nbTestsTimeout + " " +
																  "Aborted:" + nbTestsAborted + " " +
																  "Inconclusive:" + nbTestsInconclusive + " " +
																  "PassedButRunAborted:" + nbTestsPassedButRunAborted + " " +
																  "NotRunnable:" + nbTestsNotRunnable + " " +
																  "Disconnected:" + nbTestsDisconnected + " " +
	                                                              "Warning:" + nbTestsWarning));
			errorDetected = true;
		}

		// grab each test
		NodeList testNodelist = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "//UnitTestResult");
		for (int i=0; i< testNodelist.getLength(); i++) {
			Node TestNode = testNodelist.item(i);
			String testId        = TestNode.getAttributes().getNamedItem("testId").getNodeValue();
    		String testName      = TestNode.getAttributes().getNamedItem("testName").getNodeValue();
    		String computerName  = TestNode.getAttributes().getNamedItem("computerName").getNodeValue();
    		String startTime     = TestNode.getAttributes().getNamedItem("startTime").getNodeValue();
    		Node stdoutNode      = CXmlDocumentFactory.getNodeFromXPath(TestNode, "Output/StdOut");
    		String stdout        = CXmlDocumentFactory.getNodeTextOrCDataContent(stdoutNode);

			executionSteps.add(new CExecutionStep(startTime, RESULT_INFO,
					"id:" + testId + " " +
					"name:" + testName + " " +
					"computer:" + computerName + " " +
					"stdout:" + stdout));
		}

		if (errorDetected) {
			System.err.println("==> FAILURE");
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			System.err.println("==> SUCCESS");
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl launcher = new CLauncherImpl();
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		
		launcher.parseResultFile(new File("Y:/XStudio/src/com/xqual/xlauncher/visualstudio_codedui/sample"), executionSteps);
		
		System.out.println(executionSteps);
	}
	
}
