/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     0.6                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.wapt;

import java.io.File;
import java.util.Vector;
import java.util.regex.Pattern;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for AutoIt.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{wapt          }  ";

	static String WAPT_EXE;
	
	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String waptInstallPath;
	private boolean waptPro;

	private File workingDir;

	private File reportHtmlFolder;
	private File resultWprFolder;
	private File resultCsvFolder;
	private File logFolder;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath    = getStringParamValue("General", "Test root path");    // i.e. Y:/Externals/wapt/scripts

			waptInstallPath = getStringParamValue("WAPT", "WAPT install path");    // i.e. C:/Program Files/wapt
			waptPro         = getBooleanParamValue("WAPT", "WAPT Pro");            // i.e. true/false
			
			workingDir = new File(waptInstallPath);

		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		if (waptPro) {
			WAPT_EXE = CUtils.getExecutableName("waptpro");
		} else {
			WAPT_EXE = CUtils.getExecutableName("wapt");
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		reportHtmlFolder = new File(testRootPath + "/" + testPath + "/reports");
		resultWprFolder  = new File(testRootPath + "/" + testPath + "/results/wpr");
		resultCsvFolder  = new File(testRootPath + "/" + testPath + "/results/csv");
		logFolder        = new File(testRootPath + "/" + testPath + "/logs");
		
		CFileUtils.deleteAndTouchFolder(reportHtmlFolder);
		CFileUtils.deleteAndTouchFolder(resultWprFolder);
		CFileUtils.deleteAndTouchFolder(resultCsvFolder);
		CFileUtils.deleteAndTouchFolder(logFolder);
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
        CRunner waptTestcaseRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseName,
								           CFileUtils.quoteFilePath(waptInstallPath + "\\" + WAPT_EXE) + " -run " + CFileUtils.quoteFilePath(testRootPath + "\\" + testPath + "\\" + testName + "_" + testcaseName + ".wps") + " " +
										   "-hr " + CFileUtils.quoteFilePath(reportHtmlFolder.getAbsolutePath()) + " " +
								           "-sr " + CFileUtils.quoteFilePath(resultWprFolder.getAbsolutePath()) + " " +
								           "-csv " + CFileUtils.quoteFilePath(resultCsvFolder.getAbsolutePath()) + " " +
								           "-log " + CFileUtils.quoteFilePath(logFolder.getAbsolutePath()),
                                           workingDir);
        short result = waptTestcaseRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
        if (result == RESULT_SUCCESS) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "script returned a success"));
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "script returned a failure"));
		}
        
        addTestcaseMessageAttachmentFolder(reportHtmlFolder);
        addTestcaseMessageAttachmentFolder(resultWprFolder);
        addTestcaseMessageAttachmentFolder(resultCsvFolder);
        addTestcaseMessageAttachmentFolder(logFolder);
        
        return new CReturnStatus(RESULT_INDEFINITE, executionSteps); // no way to return simple success/failure (need to be analyzed by an analyst)
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForWaptTestcases(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2ewps$",          // only *.wps files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	
	protected void recursivelyScanFolderForWaptTestcases(Vector<CScannedTest> output, File folder, String regexFolder, String regexFile, boolean removeExtension, String separator, String currentRelativePath) {
		if (folder.isDirectory()) {
			File[] children = folder.listFiles();
			CScannedTest scannedTest = null;
			if (children != null) {
				for (int i = 0; i < children.length; i++) {
					File child = children[i];
					if (child.isDirectory()) {
						if (Pattern.matches(regexFolder, child.getName())) {
							recursivelyScanFolderForWaptTestcases(output, child, regexFolder, regexFile, removeExtension, separator, 
												  currentRelativePath.trim().length() > 0 ? currentRelativePath + separator + child.getName() : child.getName());
						} else {
							System.err.println("skipping folder \"" + currentRelativePath + separator + child.getName() + "\"...");
						}
						
					} else { // file
						if (Pattern.matches(regexFile, child.getName())) {
							String testName     = "";
							String testcaseName = "";
					        int j = child.getName().lastIndexOf('_');
					        if (j > 0 &&  j < child.getName().length() - 1) {
					        	testName    = child.getName().substring(0, j);                            // left part
					        	testcaseName = child.getName().substring(j+1, child.getName().length());  // right part
					        }
							
							if (scannedTest == null) {
								scannedTest = new CScannedTest(testName, currentRelativePath);
								System.out.println("> found test \"" + testName + "\" in " + currentRelativePath);
							}
							
							scannedTest.addScannedTestcase((i+1), CFileUtils.getBasename(new File(testcaseName)));
							System.out.println("> found testcase \"" + CFileUtils.getBasename(new File(testcaseName)) + "\"");
						} else {
							//System.out.println("> skipping file \"" + currentRelativePath + separator + child.getName() + "\"");
						}
					}
				}
			}
			if (scannedTest != null)
				output.add(scannedTest);
		} else { // should never happen!
		}
	}
}
