/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     0.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.webdriverio;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.json.JSONArray;
import org.json.JSONObject;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{WebdriverIO   }  ";

	static final String WEBDRIVERIO_RUNNER = "wdio";
	
	private String testRootPath;
	private String testLevel;
	private String webdriverIOInstallPath;
	private String webdriverIOConfig;
	private String webdriverIOOptions;
	private String framework;
	
	private String reporter;
	private String options;

	private File workingDir;
	private File webdriverIOTraceFile;
	
	private final String jsonContent = null;

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();
		
		try {
			// retrieve the parameters we need
			testRootPath           = getStringParamValue("General", "Test root path");                // i.e. "/home/webdriverio"
			testLevel              = getStringParamValue("General", "Test level");                    // i.e. "Spec"
			
			webdriverIOInstallPath = getStringParamValue("WebdriverIO", "WebdriverIO install path");  // i.e. "/wd"
			webdriverIOConfig      = getStringParamValue("WebdriverIO", "WebdriverIO conf");          // i.e. "wdio.conf.js"
			webdriverIOOptions     = getStringParamValue("WebdriverIO", "WebdriverIO options");       // i.e. ""
			framework              = getStringParamValue("WebdriverIO", "Framework");                 // i.e. "Mocha"

			workingDir = new File(webdriverIOInstallPath + "/node_modules/.bin/");
			if (!workingDir.exists() || !workingDir.isDirectory()) {
				traceln(LOG_PRIORITY_SEVERE, "WebdriverIO install path does not exist or is not a folder: " + webdriverIOInstallPath + "/node_modules/.bin/ not found!");
				return new CReturnStatus(RESULT_FAILURE);
			}
			
			return new CReturnStatus(RESULT_SUCCESS);
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			
			Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}


	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// +------------------------------------+
		// | Run the WebdriverIO test           |
		// +------------------------------------+
		CRunner webdriverIORunner = null;
		short result = RESULT_SUCCESS;

		// +--------------------------+
		// |           Test           |
		// +--------------------------+
		if (testLevel.equalsIgnoreCase("Suite")) { // XStudio test = WebdriverIO suite
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST LEVEL = Suite");
			System.err.println("- - - RUNNING THE .JS FILE " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + ".js"));
			System.err.println("+----------------------------------------------+");

			webdriverIOTraceFile = CFileUtils.createTmpRandomFile("webdriverIO_");
			System.err.println("- - - WebdriverIO TRACE FILE: " + webdriverIOTraceFile);
			webdriverIORunner = new CRunner("[" + testId + "] "+ testPath + testName,
											WEBDRIVERIO_RUNNER + " --framework " + framework + " --reporters dot,json " + webdriverIOOptions + " --suite " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName),
											workingDir);
			
			// here we redirect the output to a file for later parsing
			result = webdriverIORunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, webdriverIOTraceFile);

		// +--------------------------+
		// |        Test case         |
		// +--------------------------+
		} else if (testLevel.equalsIgnoreCase("Spec")) { // XStudio testcase = WebdriverIO spec
			System.err.println("+----------------------------------------------+");
			System.err.println("- - - TEST LEVEL = Spec");
			System.err.println("- - - RUNNING THE .JS FILE " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "/" + testcaseName + ".js"));
			System.err.println("+----------------------------------------------+");

			webdriverIOTraceFile = CFileUtils.createTmpRandomFile("webdriverIO_");
			System.err.println("- - - TESTCASE MOCHA TRACE FILE: " + webdriverIOTraceFile);
			webdriverIORunner = new CRunner("[" + testId + "] "+ testPath + testName + testcaseName,
											WEBDRIVERIO_RUNNER + " --framework " + framework + " --reporters dot,json " + webdriverIOOptions + " --spec " + CFileUtils.quoteFilePath(testRootPath + "/" + (testPath.length()>0 ? testPath + "/" : "") + testName + "/" + testcaseName + ".js"),
											workingDir);
			
			// here we redirect the output to a file for later parsing
			result = webdriverIORunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, webdriverIOTraceFile);
		}

		// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+
		if (webdriverIOTraceFile.exists())
			addTestcaseMessageAttachment(webdriverIOTraceFile); // if "Test and Test cases" mode is used, this will be upload only once (on the first testcase which is ok)!
		
		File jsonFile = new File(testRootPath + "/reporter.json");
		addTestcaseMessageAttachment(jsonFile);


		return parseResultFile(jsonFile, executionSteps);
	}
	
	
	
	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		return new CReturnStatus(RESULT_SUCCESS);
	}

	@Override
	public CReturnStatus terminate() {
		return new CReturnStatus(RESULT_SUCCESS);
	}
	

	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		// maybe there is a call available to get the available specs or Suites?
		return false;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/*
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolder(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2emds$",          // only *.mds files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		*/
		return output;
	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(File jsonFile, Vector<CExecutionStep> executionSteps) {
		String jsonContent = CFileUtils.FileToString(jsonFile);
		System.out.println("jsonContent = " + jsonContent);

		boolean allSuccess        = true;
		boolean atLeastOneFailure = false;

		try {
	    	JSONObject jsonObject = new JSONObject(jsonContent);
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "host = "      + jsonObject.getString("host")));
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "port = "      + jsonObject.getInt("port")));
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "base url = "  + jsonObject.getString("baseUrl")));
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "framework = " + jsonObject.getString("framework")));
    		
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "platform = " + jsonObject.getJSONObject("capabilities").getString("platform")));
    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "browser = " + jsonObject.getJSONObject("capabilities").getString("browserName")));

	    	
	    	// search for FAILED tests (those in the "failures" array)
	    	JSONArray suiteList = jsonObject.getJSONArray("suites");
	    	if (suiteList != null) {
		    	for (int i=0; i<suiteList.length(); i++) {
	        		JSONObject suite = (JSONObject) suiteList.get(i);
	        		if (suite != null) {
	            		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "[b][Suite] " + suite.getString("name") + " (duration=" + suite.getInt("duration") + ")[/b]"));

	            		JSONArray testList = suite.getJSONArray("tests");
	            		if (testList != null) {
	        		    	for (int j=0; j<testList.length(); j++) {
	        	        		JSONObject test = (JSONObject) testList.get(j);
	        	        		if (test != null) {
	        	        			short status = stateToStatus(test.getString("state"));
	        	        			switch (status) {
	        	        			case RESULT_SUCCESS:
	        	        				break;
	        	        				
	        	        			case RESULT_FAILURE:
	        	        				atLeastOneFailure = true;
	        	        				allSuccess = false;
	        	        				break;
	        	        				
	        	        			case RESULT_INDEFINITE:
	        	        			default:
	        	        				allSuccess = false;
	        	        				break;
	        	        			}
	        	        			
	        	        			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), status, "[Test] " + test.getString("name") + " (duration=" + test.getInt("duration") + ")"));
	        	        			
	        	        			// optionals
	        	        			String error=null, errorType=null, standardError=null;
	        	        			try {error         = test.getString("error");} catch (Exception e) {}
	        	        			try {errorType     = test.getString("errorType");} catch (Exception e) {}
	        	        			try {standardError = test.getString("standardError");} catch (Exception e) {}

	        	        			if (error != null         && error.trim().length()>0)         executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Error: "          + error.trim()));
	        	        			if (errorType != null     && errorType.trim().length()>0)     executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Error Type: "     + errorType.trim()));
	        	        			if (standardError != null && standardError.trim().length()>0) executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Standard Error: " + standardError.trim()));
	        	        			
	        	        		} else {
	        	        			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No test found!"));
	        	        		}
	        		    	} // for each test
	            		} else {
	            			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No test found!"));
	            		}
	        		} else {
	    	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No suite found!"));
	        		}
		    	} // for each suite
	    	} else {
	    		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "No suite found!"));
	    	}
	    	
    	} catch (Exception e) {
    		traceln(LOG_PRIORITY_SEVERE, "couldn't parse the json report: " + e);
			return null;
		}
		
		if (atLeastOneFailure) {
			return new CReturnStatus(RESULT_FAILURE, executionSteps); 

		} else if (allSuccess) {
			return new CReturnStatus(RESULT_SUCCESS, executionSteps); 
		}
		
		return new CReturnStatus(RESULT_INDEFINITE, executionSteps); 
	}

	private short stateToStatus(String input) {
		if (input.equalsIgnoreCase("pass")) {
			return RESULT_SUCCESS;
		} else if (input.equalsIgnoreCase("fail")) {
			return RESULT_SUCCESS;
		} else {
			return RESULT_INDEFINITE;
		}
	}
	
	// +------------------------------------------+
	// |        ONLY FOR TESTING PURPOSES         |
	// +------------------------------------------+
	public static void main(String[] args) {
		CLauncherImpl toto = new CLauncherImpl();

		System.err.println("========================================================");
		File jsonFile = new File("Y:/XStudio/src/com/xqual/xlauncher/webdriverIO/sample/sample.json");
		CReturnStatus returnStatus = toto.parseResultFile(jsonFile, new Vector<CExecutionStep>());
		System.err.println(returnStatus);
	
		System.err.println("========================================================");

	}
	
}
