/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.1                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.webui;

import java.io.File;
import java.util.Vector;

import javax.swing.JOptionPane;

import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.CXmlDocumentFactory;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;





/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for WebUI.
 * @author egavaldo
 * developed based on:
 * - http://blogs.telerik.com/testing/posts/10-07-01/a_look_under_the_hood_at_webui_test_studio_s_execution_engine_part_1_of_2.aspx
 * - http://blogs.telerik.com/testing/posts/10-07-06/a_look_under_the_hood_at_webui_test_studio_s_execution_engine_part_2_of_2.aspx
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{webui         }  ";

	// parameters impacting executing at run time set by the test operator
	private String testRootPath;
	private String webUIInstallPath;

	private File workingDir;

	private static final String WEBUI_RUNNER = CUtils.getExecutableName("ArtOfTest.Runner");

	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+
	
	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) { 
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");
		
		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need
			testRootPath     = getStringParamValue("General",  "Test root path"); 
			webUIInstallPath = getStringParamValue("WebUI",    "WebUI install path");
		} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));
			
			CUtils.showErrorMessage(null, 
					"Error: Outdated configuration", 
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.", 
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);
			
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + " testPath=" + testRootPath + "/" + testPath + "/" + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		String scriptParentFolderPath = testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/";
		workingDir = new File(scriptParentFolderPath);

		deleteResultfiles(workingDir);
		
		
		// +------------------------------------+
		// | Interpret the script
		// +------------------------------------+
		CRunner webUIRunner = new CRunner("[" + testId + "] "+ testPath + ":" + testName + "." + testcaseIndex,
		                                   CFileUtils.quoteFilePath(webUIInstallPath + "/Bin/" + WEBUI_RUNNER) + " " +
		                                   "aii=" + CFileUtils.quoteFilePath(testRootPath + (testPath.length() > 0 ? "/" + testPath : "") + "/" + testName + ".aii") + " " +
		                                   "out=" + CFileUtils.quoteFilePath(testRootPath + (testPath.length() > 0 ? "/" + testPath : "")) + " " + // where the .aiiresult file is generated
                                           workingDir);
		short result = webUIRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION);
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "script interpretation failed"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}

		File resultFile = searchResultfiles(workingDir);
		if (resultFile != null) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "result file found: " + resultFile));
			addTestcaseMessageAttachment(resultFile);
			return parseResultFile(resultFile, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "result file not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}

	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+
	
	@Override
	public boolean canScanPhysicalTests() {
		return true;
	}
	
	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		Vector<CScannedTest> output = new Vector<CScannedTest>();

		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2eaii$",          // only *.aii files
							  true,                    // remove extension from the file name to determine the test name
							  "/",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
		return output;
	}
	

	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private void deleteResultfiles(File folder) {
		if (folder.isDirectory()) {
			File[] files = folder.listFiles();
			for (int i=0; i<files.length; i++) {
				File file = files[i];
				if (file.isFile()) {
					if (CFileUtils.getExtension(file).equalsIgnoreCase("aairesult")) {
						traceln(LOG_PRIORITY_INFO, "deleting former result file: " + file);
						CFileUtils.deleteFile(file);
					}
				}
			}
		}
	}
	
	private File searchResultfiles(File folder) {
		// search for the result file
		File resultFile = null;
		if (folder.isDirectory()) {
			File[] files = folder.listFiles();
			for (int i=0; i<files.length; i++) {
				File file = files[i];
				if (file.isFile()) {
					if (CFileUtils.getExtension(file).equalsIgnoreCase("aairesult")) {
						resultFile = file;
						break;
					}
				}
			}
		}
		return resultFile;
	}
	
	public static CReturnStatus parseResultFile(File resultFile, Vector<CExecutionStep> executionSteps) {	
		Document xmlDocument = CXmlDocumentFactory.createXMLDoc(resultFile);	
		boolean errorDetected = false;
		System.out.println("xml = " + CFileUtils.FileToString(resultFile));
		
		NodeList testNodes = CXmlDocumentFactory.getNodeObjListFromXPath(xmlDocument, "/RunResult/TestResults/Item");
		//System.err.println("- - - - - nb tests: " + testNodes.getLength());
		for (int i = 0; i < testNodes.getLength(); i++) {
            Node testNode = testNodes.item(i);
            
            String testResult = CXmlDocumentFactory.getNodeStringValueFromXPath(testNode, "ResultType[@Type='ArtOfTest.Common.Design.ResultType']");
            if (!testResult.equalsIgnoreCase("Pass")) errorDetected = true;
            executionSteps.add(new CExecutionStep(testResult.equalsIgnoreCase("Pass") ? RESULT_SUCCESS : RESULT_FAILURE, testResult));
            
            NodeList stepNodes = CXmlDocumentFactory.getNodeObjListFromXPath(testNode, "AutomationStepResults/Item[@Type='ArtOfTest.WebAii.Design.AutomationStepResult']");
            //System.err.println("- - - - - nb steps: " + stepNodes.getLength());
            for (int j = 0; j < stepNodes.getLength(); j++) {
            	Node stepNode = stepNodes.item(j);

            	String stepStatus     = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "ResultType[@Type='ArtOfTest.Common.Design.ResultType']");
            	String stepDescription = CXmlDocumentFactory.getNodeStringValueFromXPath(stepNode, "StepDescription");
            	if (!stepStatus.equalsIgnoreCase("Pass")) errorDetected = true;
                executionSteps.add(new CExecutionStep(stepStatus.equalsIgnoreCase("true") ? RESULT_SUCCESS : RESULT_FAILURE, " > " + stepDescription)); 
            }
		}

		System.out.println("executionSteps=" + executionSteps);
		return new CReturnStatus(errorDetected ? RESULT_FAILURE : RESULT_SUCCESS, executionSteps);
	}	
}
