/*
+----------------------------------------------------------------------+
|                    Class: CLauncher                                  |
|                                                                      |
| Developper:  Eric Gavaldo (eric.gavaldo@xqual.com)                   |
| Version:     1.10                                                     |
| License:     GPL v2 (https://www.gnu.org/licenses/gpl-2.0.html)      |
+----------------------------------------------------------------------+
*/
package com.xqual.xlauncher.xci;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;
import java.util.Vector;

import javax.swing.JOptionPane;

import com.xqual.xagent.CScannedTest;
import com.xqual.xagent.launcher.CExecutionStep;
import com.xqual.xagent.launcher.CLauncher;
import com.xqual.xagent.launcher.CReturnStatus;
import com.xqual.xagent.launcher.runner.CRunner;
import com.xqual.xagent.launcher.runner.IRunner;
import com.xqual.xcommon.IConstantsResults;
import com.xqual.xcommon.type.CParamParsingException;
import com.xqual.xcommon.type.CTestAttribute;
import com.xqual.xcommon.type.CTestcaseParam;
import com.xqual.xcommon.utils.CCalendarUtils;
import com.xqual.xcommon.utils.CFileUtils;
import com.xqual.xcommon.utils.CUtils;



/**
 * The <code>CLauncherImpl</code> implementation of <code>ILauncher</code> for Rapise.
 * @author egavaldo
 */
public class CLauncherImpl extends CLauncher implements IConstantsResults {

	// +==============================================================+
	// | Attributes                                                   |
	// +==============================================================+

	static final String TRACE_HEADER = "{xci        }  ";

	// parameters impacting executing at run time set by the test manager in the configuration
	// general parameters
	private String windowsCommand;


	//Parameters driving teh xContinuousIntegration
	private String xciInstalPath;
	private boolean xciSynchonous;
	private boolean xciGenerateReport;
	private String xciReportTransform;
	private String xciReportStyle;
	private String xciReportOutputPath;
	private String xciEngine;

	//Parameters to decide on success failure
	private int minPctSuccess;
	private int maxPctFailures;
	private int maxPctNotExecuted;

// is a string  for conaining the prefix of the interpreter
	private String xciInterpreter;
// is a string that contains the
	private String xciReportArguments;

	private File workingDir;


	File xciTraceFile;
	File junitOutputFile;


	// +==============================================================+
	// | Constructors                                                 |
	// +==============================================================+

	public CLauncherImpl() {
		super(TRACE_HEADER);
	}

	// +==============================================================+
	// +==============================================================+
	// | Methods                                                      |
	// +==============================================================+

	@Override
	public CReturnStatus initialize(int sutId, String sutName, String sutVersion) {
		setSutDetails(sutId, sutName, sutVersion);

		traceln(LOG_PRIORITY_INFO, "launcher executing in " + CUtils.getCurrentLocation() + "...");

		// check the configuration sent by the manager
		printConfiguration();

		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		try {
			// retrieve the parameters we need from the configuration

			//Get default general configuration parameters
			windowsCommand = getStringParamValue("General", "Windows command (leave blank if not executing under Windows)");

			//Get configuration parameters specific to this session
			// this one should have have / at the end
			xciInstalPath = getStringParamValue("Xci", "xci install path");
			xciSynchonous = getBooleanParamValue("Xci", "synchronous");
			xciGenerateReport = getBooleanParamValue("Xci", "generateReport");
			xciReportTransform = getStringParamValue("Xci", "reportTransform");
			xciReportStyle = getStringParamValue("Xci", "reportStyle");
			xciReportOutputPath = getStringParamValue("Xci", "reportOutputPath");
			// we avoid having to manage space within path name
			xciReportOutputPath = xciReportOutputPath.replace(" " , "_");
			xciEngine = getStringParamValue("Xci", "xci Engine");

//getting parameters to decide if we are n a sycces or failure state
			minPctSuccess = getIntegerParamValue("Promotion", "Min % of success");
			maxPctFailures = getIntegerParamValue("Promotion", "Max % of fails");
			maxPctNotExecuted = getIntegerParamValue("Promotion", "Max % of Not Executed");

			//create the default interpreter for all tests
			xciInterpreter = (windowsCommand.length()>0 ? windowsCommand + " /C " : "") + CFileUtils.quoteFilePath (xciInstalPath + "/" + xciEngine) + " ";

			//The arguments are the  full session - all invocation fo the XCI will work the same in the session
			//The is a choice because we aim to use this mostly to create release pipelines that start sucessive campaigns
			xciReportArguments =
				(xciSynchonous ? "--synchronous " : "") +
				(xciGenerateReport ? "--generateReport" +
						" --reportTransform " + xciReportTransform +
						" --reportStyle " + xciReportStyle +
						" --reportOutputPath " + xciReportOutputPath : "");


} catch (CParamParsingException e) {
			traceln(LOG_PRIORITY_SEVERE, "parsing error during initialization");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during initialize: " + e.getMessage()));

			CUtils.showErrorMessage(null,
					"Error: Outdated configuration",
					"At least one launcher is requiring some missing configuration parameters.<br/>This may happen when a launcher is updated and needs new or modified parameters.",
					"Please go to the <b>Configuration</b> tab and create a new configuration for the indicated category<br/>(and of course select this new configuration)",
					JOptionPane.ERROR_MESSAGE);

			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}

	@Override
	public CReturnStatus preRun(int testId, String testPath, String testName, Vector<CTestAttribute> attributes, String additionalInfo) {
		traceln(LOG_PRIORITY_INFO, "preRun testId=" + testId + " testPath=" + testPath + " [" + testName + "]...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();

		// in case their are attributes that overide teh default configuration - and there should often be
		int testMinPctSuccess;
		int testMaxPctFailures;
		int testMaxPctNotExecuted;

// we need to provide the ids used to cerate a session  - they are provided as attributes of each test
		int xciSutId = -1;
		int xciCampaingId = -1;
		String xciAgentIds = "";
		String xciConfigurationIds = "";

		// teh session argument contains those needed ids by XCI
		String xciSessionArguments = "";

		// we create the handle teh working id - note that we force a / at the end
		workingDir = new File(CFileUtils.quoteFilePath (xciInstalPath + "/")); // parent folder of all the test classes

		traceln(LOG_PRIORITY_INFO, "Getting promotion for SLA attributes from test");

		//data extract from the attribute to decide on success may overide those of the configuration
		testMinPctSuccess = getIntegerAttributeValue(attributes,"com.xqual.promotion.minPctSuccess");
		testMaxPctFailures = getIntegerAttributeValue(attributes,"com.xqual.promotion.maxPctFailures");
		testMaxPctNotExecuted = getIntegerAttributeValue(attributes,"com.xqual.promotion.maxPctNotExecuted");

		// we are working with percentages so value must be between 0 and 100 - clusive
		// by convention if the api , if the attributes are not provided , it renders a value of -1, meaning it's empty
		testMinPctSuccess = (testMinPctSuccess > 100 ? -1 : testMinPctSuccess);
		testMaxPctFailures = (testMaxPctFailures > 100 ? -1 : testMaxPctFailures);
		testMaxPctNotExecuted = (testMaxPctNotExecuted > 100 ? -1 : testMaxPctNotExecuted);

		// if the values provided in te attrutes of the test are > -1 then they are valid data and overide the ones from  he configuration
		minPctSuccess = (testMinPctSuccess > -1 ? testMinPctSuccess : minPctSuccess);
		maxPctFailures = (testMinPctSuccess > -1 ? testMaxPctFailures : maxPctFailures);
		maxPctNotExecuted = (testMinPctSuccess > -1 ? testMaxPctNotExecuted : maxPctNotExecuted);


		// get the data for teh XCI  about campaign, configuration, agents  and SUT
		xciSutId = getIntegerAttributeValue(attributes,"com.xqual.xci.sutId");
		xciCampaingId = getIntegerAttributeValue(attributes,"com.xqual.xci.campaingId");
		xciAgentIds = getStringAttributeValue(attributes, "com.xqual.xci.agentIds");
		xciConfigurationIds = getStringAttributeValue(attributes, "com.xqual.xci.configurationsIds");

		//we then ensure that have all the needed idsfor XCi to create a new session
		if (xciSutId > -1 && xciCampaingId > -1 && xciAgentIds.length() > 0 && xciConfigurationIds.length() > 0)  {
	xciSessionArguments =  " --campaignId " + xciCampaingId + " --agents " + xciAgentIds + " --sutId " + xciSutId  + " --configurations " + xciConfigurationIds + " ";
}
		else {
			//otherwise  this is not possible and we have to stop
			traceln(LOG_PRIORITY_SEVERE, "test attributes do not contain necessary values " +
											" com.xqual.xci.sutId = " + xciSutId +
											" com.xqual.xci.campaingId = " + xciCampaingId +
											" com.xqual.xci.agentIds = " + xciAgentIds +
											" com.xqual.xci.configurationsIds = " + xciConfigurationIds );
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception during run: test attributes do not contain necessary values " ));
		return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		//we identify where eth test root path is  - in there's no test , but the file results will end up in a subdir of the rootpath with the testname  name
		File testReportPath;
		testReportPath = new File(xciReportOutputPath + "/" + testName.replace(" " , "_") + "/");
		CFileUtils.deleteFile(testReportPath);

		// | console trace file that will be used during the run of the interpreter
		xciTraceFile = new File(xciReportOutputPath + "/" + testName.replace(" " , "_") + ".log");
		CFileUtils.deleteAndTouchFile(xciTraceFile);

// Junit file that will ve parsed
		junitOutputFile = new File(xciReportOutputPath + "/" + testName.replace(" " , "_") + "/" + testName.replace(" " , "_") + ".xml");
						traceln(LOG_PRIORITY_INFO, "-#-#-#- creating the junit file handle " + junitOutputFile.toString() );

		// +------------------------------------+
		// | Run the  xci test
		// +------------------------------------+
		CRunner xciRunner = new CRunner("[" + testId + "] "+ testPath + "." + testName,xciInterpreter.toString() + xciSessionArguments.toString() + xciReportArguments.toString() + " --reportName " + testName.replace(" " , "_") + "");


		// here we redirect the output to a file for later parsing
		short result = xciRunner.requestAction(IRunner.START_PROCESS, IRunner.WAIT_END_OF_EXECUTION, xciTraceFile);



		// return code
		if (result == RESULT_FAILURE) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Session creation failed"));
// on return we just an execution result step - we will parse the result file as part of the deault step of teh test
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		} else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Session creation succeeded"));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		}
	}


	@Override
	//There's a defulat testcase  in whcih we grab the 2 files (console trace ena xml junit results) and we parse that one -
	public CReturnStatus run(int testId, String testPath, String testName, int testcaseId, int testcaseIndex, String testcaseName, Vector<CTestcaseParam> params, String additionalInfo) {

		traceln(LOG_PRIORITY_INFO, "run testId=" + testId + "Session to create " + testName + " testcaseIndex=" + testcaseIndex + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
				// +------------------------------------+
		// | Add the output console to the test
		// +------------------------------------+


		addTestcaseMessageAttachment(xciTraceFile);

		addTestcaseMessageAttachment(junitOutputFile);

		// note that teh parse result decide of the success or failure of this default testcase , hence of teh entire test
		return parseResultFile(executionSteps);
		}


// we don't do any postrun test work
	@Override
	public CReturnStatus postRun(int testId, String testPath, String testName) {
		// the post may decide if the test is or not successfull
		traceln(LOG_PRIORITY_INFO, "postRun testId=" + testId + " testPath=" + testPath + ":" + testName + "...");
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "postRun: succeeded"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}



// We don't do any clean up of the session so that all result can be kepted
	@Override
	public CReturnStatus terminate() {
		Vector<CExecutionStep> executionSteps = new Vector<CExecutionStep>();
		executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Terminate"));
		return new CReturnStatus(RESULT_SUCCESS, executionSteps);
	}


	// +---------------------------------------------+
	// |               SCANNER FEATURE               |
	// +---------------------------------------------+

	@Override
	public boolean canScanPhysicalTests() {
		return false;
	}

	@Override
	public Vector<CScannedTest> scanPhysicalTests() {
		//2018-03-17 : we do not expect to have to use scanning for If the XCI launcher -

		Vector<CScannedTest> output = new Vector<CScannedTest>();
		/**
		File testRootPathFolder = new File(CFileUtils.getFilePathWithValidSeparator(testRootPath)); // OS dependant path
		if (testRootPathFolder == null || !testRootPathFolder.exists() || !testRootPathFolder.isDirectory()) {
			traceln(LOG_PRIORITY_SEVERE, "Test root path does not exist or is not a directory!");
			return output;
		}

		recursivelyScanFolderForTests(output,
							  testRootPathFolder,      // folder to scan
							  "^.*$",                  // any folder
							  "^.*\\x2esstest$",        // only *.class files
							  true,                    // remove extension from the file name to determine the test name
							  ".",                     // separator used in canonicalPath generated
							  "");                     // canonical path of the folders to start to scan (from testRootPathFolder)
											**/
		return output;

	}


	// +--------------------------+
	// | Utilities                |
	// +--------------------------+

	private CReturnStatus parseResultFile(Vector<CExecutionStep> executionSteps) {
		// parse the result file to get the result and the execution steps


		boolean errorDetected = true;
		String line = "";

		int errorCount = 0;
		int testcaseTried = 0;
		int testcaseSkipped = 0;
		int testcaseFailures = 0;

		// be default we expect that the session will be a success
		boolean succeededOnSLA= true;

// we check taht the result file  si available
		if (!junitOutputFile.exists()) {
			traceln(LOG_PRIORITY_SEVERE, "Junit file from session execution not found!");
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Junit file from session execution not found!"));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
		else {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Found the Junit file from session execution : " + junitOutputFile.toString()));
				}
// then parse it

		//  we search somthing like <testsuite errors="0" time="0.001" tests="14" skipped="0" failures="14" name="[CS_40] ci_2018-03-16 16:05:33">))
		try {
			FileInputStream fileInputStream = new FileInputStream(junitOutputFile);
			BufferedReader bufferedReader = new BufferedReader(new InputStreamReader(fileInputStream));

			while ((line = bufferedReader.readLine()) != null) {
				line = line.trim();

				//We loook for a pattern of <testsuite as showed below
				// <?xml version="1.0" encoding="UTF-8"?><testsuite errors="0" time="0.001" tests="14" skipped="0" failures="14" name="[CS_40] ci_2018-03-16 16:05:33">
				if (line.indexOf("<testsuite") > -1) {
					System.out.println("Found the summary line " + line);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Summary line is : " + line));
					errorDetected = false;

					//Then if found we extract and cast to int the number of errors, triies, failed and skipped testcases
					errorCount = extractCountsFromResultLine(line, "errors=", "time=");
					System.out.println("Test case errors --> " + errorCount);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case errors --> " + errorCount));

					testcaseTried = extractCountsFromResultLine(line, "tests=", "skipped=");
					System.out.println("Test case tried  --> " + testcaseTried);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case tried  --> " + testcaseTried));

					testcaseSkipped = extractCountsFromResultLine(line, "skipped=", "failures=");
						System.out.println("Test case skipped  --> " + testcaseSkipped);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case skipped  --> " + testcaseSkipped));

						testcaseFailures = extractCountsFromResultLine(line, "failures=", "name=");
					System.out.println("Test case failed  --> " + testcaseFailures);
					executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_INFO, "Test case failed  --> " + testcaseFailures));
				}
			}

			// the below resource clousre should go into Finaly but we shoudl don't care as the launcher is  instanciated and killed  rapidly
			bufferedReader.close();
			fileInputStream.close();


			// if we have tried some testcases
			if (testcaseTried > 0){
				System.out.println("Entering SLA analyis for these tried tests cases --> " + testcaseTried);

				//then we ensure that we have enough succes as requested  in the test attributes
			if ((((testcaseTried - testcaseSkipped - testcaseFailures) / testcaseTried ) * 100 ) < minPctSuccess){
				succeededOnSLA = false;
				System.out.println("SLA failed on minPctSuccess   --> " + minPctSuccess);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Need at least  " + minPctSuccess + "%" + " successes"));
			}
			else {
				System.out.println("SLA succeeded on minPctSuccess   --> " + minPctSuccess);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Got over  " + minPctSuccess + "%" + " successes"));
			}

				// and that we have no more a certain percentage of failures
			if (((testcaseFailures / testcaseTried ) * 100) > maxPctFailures) {
				succeededOnSLA = false;
				System.out.println("SLA failed on maxPctFailures   --> " + maxPctFailures);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Need no more than " + maxPctFailures + "%" + " failures"));
			}
			else {
				System.out.println("SLA succeeded on maxPctFailures   --> " + maxPctFailures);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Got less than   " + maxPctFailures + "%" + " failures"));
			}

				// and that we what no more than a % of skipped
				if (((testcaseSkipped / testcaseTried ) *  100)> maxPctNotExecuted) {
				succeededOnSLA = false;
				System.out.println("SLA failed on maxPctNotExecuted   --> " + maxPctNotExecuted);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Need no more than " + maxPctNotExecuted + "%" + " skipped test cases"));
			}
			else {
				System.out.println("SLA succeeded on maxPctNotExecuted   --> " + maxPctNotExecuted);
				executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "Got less than   " + maxPctNotExecuted + "%" + " skipped test cases"));
			}
			}

			}
		 catch (Exception e) {
			traceln(LOG_PRIORITY_SEVERE, "exception while parsing the result file: " + e);
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "Exception while parsing the result file: " + e));
			errorDetected = true;
		}

		//unless we met some errors and till the SLA is match then teh test case ( hence teh test in our context) is a success
		if (!errorDetected && succeededOnSLA) {
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_SUCCESS, "The session was executed and matches the SLA "));
			return new CReturnStatus(RESULT_SUCCESS, executionSteps);
		} else {
			//otherwise the the test case is failed
			executionSteps.add(new CExecutionStep(CCalendarUtils.getCurrentTime(), RESULT_FAILURE, "The session was executed but does not match the SLA "));
			return new CReturnStatus(RESULT_FAILURE, executionSteps);
		}
	}



// utility to eactract and parse int an integrer value between 0 and whatever
private int extractCountsFromResultLine(String line, String firstSubString, String nextSubString) {

	return (Integer.parseInt(line.substring(line.indexOf(firstSubString) + (firstSubString).length() + 1, line.indexOf(nextSubString) - 2)));

}

}
